// (C) 2001-2022 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


`timescale 1ps/1ps

module alt_cpriphy_c3_soft 
import alt_cpriphy_c3_package::*;
#(
    parameter sl_num_lanes  = 1,
    parameter SIM_EMULATE   = 0,
    parameter enable_external_aib_clocking = 1,
    parameter ehip_mode_sl_0 = "ehip_disable",
    parameter cpri_include_alternate_ports = 0,
    parameter COUNT_WIDTH        = 20,
    parameter DL_TOT_MEASURE_CNT = 128,
    parameter N                  = COUNT_WIDTH - $clog2(DL_TOT_MEASURE_CNT),
    parameter M                  = $clog2(DL_TOT_MEASURE_CNT) + 1,
    parameter CALC_WIDTH         = N + M
) (
    // Clock
    input wire                       i_reconfig_clk,
    input wire                       i_aib_clk,
    input wire [sl_num_lanes-1:0]    i_sl_clk_tx,
    input wire [sl_num_lanes-1:0]    i_clk_pll_div66,
    input wire [sl_num_lanes-1:0]    i_clk_rec_div66,
    input wire                       i_sampling_clk,
    
    // Reset
    input wire [sl_num_lanes-1:0]    i_sl_csr_rst_n, // Resets the entire Ethernet core. Active low.
    input wire [sl_num_lanes-1:0]    i_sl_tx_rst_n, // Resets the TX digital path (Ethernet MAC and Ethernet PCS)
    input wire [sl_num_lanes-1:0]    i_sl_rx_rst_n, // Resets the RX digital path (Ethernet MAC, PCS)

    input wire [sl_num_lanes-1:0]    i_sl_soft_csr_rst,
    input wire [sl_num_lanes-1:0]    i_sl_soft_tx_rst,
    input wire [sl_num_lanes-1:0]    i_sl_soft_rx_rst,

    input wire [sl_num_lanes-1:0]    i_sl_soft_dl_rst,

    // Reset Controller
    output wire [sl_num_lanes-1:0]   o_sl_tx_aib_reset,
    output wire [sl_num_lanes-1:0]   o_sl_rx_aib_reset,
    output wire [sl_num_lanes-1:0]   o_sl_tx_pcs_reset,
    output wire [sl_num_lanes-1:0]   o_sl_rx_pcs_reset,
    // output wire [sl_num_lanes-1:0]   o_sl_pmad_reset,
    output wire [sl_num_lanes-1:0]   o_sl_tx_reset,
    output wire [sl_num_lanes-1:0]   o_sl_rx_reset,
    input wire [sl_num_lanes-1:0]    i_sl_tx_pma_ready,
    input wire [sl_num_lanes-1:0]    i_sl_rx_pma_ready,
    input wire [sl_num_lanes-1:0]    i_sl_tx_dll_lock,
    input wire [sl_num_lanes-1:0]    i_sl_rx_is_lockedtodata,
    input wire [sl_num_lanes-1:0]    i_sl_rx_transfer_ready,
    input wire [sl_num_lanes-1:0]    i_sl_tx_transfer_ready,
    input wire [sl_num_lanes-1:0]    i_sl_rx_pcs_fully_aligned,
    input wire [sl_num_lanes-1:0]    i_sl_rx_fifo_ready,
    input wire [sl_num_lanes-1:0]    i_sl_tx_fifo_ready,
    input wire [sl_num_lanes-1:0]    i_sl_rx_nphy_ready,
    input wire [sl_num_lanes-1:0]    i_sl_tx_nphy_ready,

    input wire [sl_num_lanes-1:0]    i_sl_ehip_ready,
    input wire [sl_num_lanes-1:0]    i_sl_ehip_reset_stat,
    input wire [sl_num_lanes-1:0]    i_sl_ehip_tx_reset_stat,
    input wire [sl_num_lanes-1:0]    i_sl_ehip_rx_reset_stat,
    output wire [sl_num_lanes-1:0]   o_sl_ehip_reset,
    output wire [sl_num_lanes-1:0]   o_sl_ehip_tx_reset,
    output wire [sl_num_lanes-1:0]   o_sl_ehip_rx_reset,
    output wire [sl_num_lanes-1:0]   o_sl_ehip_pld_ready,
    output wire [sl_num_lanes-1:0]   o_sl_ehip_signal_ok,
    output wire [sl_num_lanes-1:0]   o_sl_ehip_ready,   

    output wire [sl_num_lanes-1:0]   o_sl_rsfec_reset,
    output wire [sl_num_lanes-1:0]   o_sl_rsfec_tx_reset,
    output wire [sl_num_lanes-1:0]   o_sl_rsfec_rx_reset,

    output wire [sl_num_lanes-1:0]   o_sl_tx_lanes_stable, //Asserted when TX MAC is ready to send data
    output wire [sl_num_lanes-1:0]   o_sl_rx_pcs_ready, //Asserted when RX PCS is ready to receive data
    
    input  wire [64*sl_num_lanes-1:0]        i_sl_tx_mii_d,
    input  wire [8*sl_num_lanes-1:0]         i_sl_tx_mii_c,
    input  wire [sl_num_lanes-1:0]           i_sl_tx_mii_valid,
    input  wire [sl_num_lanes-1:0]           i_sl_tx_mii_ready,
    output wire [64*sl_num_lanes-1:0]        o_sl_tx_mii_d,
    output wire [8*sl_num_lanes-1:0]         o_sl_tx_mii_c,
    output wire [sl_num_lanes-1:0]           o_sl_tx_mii_valid,
    output  wire [sl_num_lanes-1:0]          o_sl_tx_mii_ready,
    output wire [sl_num_lanes-1:0]           o_sl_tx_mii_am,
    input  wire [64*sl_num_lanes-1:0]        i_sl_rx_mii_d,
    input  wire [8*sl_num_lanes-1:0]         i_sl_rx_mii_c,
    input  wire [sl_num_lanes-1:0]           i_sl_rx_mii_valid,
    input  wire [sl_num_lanes-1:0]           i_sl_rx_mii_am_valid,
    output wire [64*sl_num_lanes-1:0]        o_sl_rx_mii_d,
    output wire [8*sl_num_lanes-1:0]         o_sl_rx_mii_c,
    output wire [sl_num_lanes-1:0]           o_sl_rx_mii_valid,
    
    input  wire [sl_num_lanes-1:0]           i_sl_rx_block_lock,
    input  wire [sl_num_lanes-1:0]           i_sl_txfifo_pfull,
    input  wire [4*sl_num_lanes-1:0]         i_sl_cpri_rate_sel,
    input  wire [sl_num_lanes-1:0]           i_sl_cpri_fec_en,
    
    output wire [sl_num_lanes-1:0]           o_sl_mii_txfifo_pfull,
    output wire [sl_num_lanes-1:0]           o_sl_mii_txfifo_pempty,
    output wire [sl_num_lanes-1:0]           o_sl_mii_txfifo_overflow,
    output wire [sl_num_lanes-1:0]           o_sl_mii_txfifo_underflow,
    
    // PMA Direct 8b10b Interface
    input  wire [2*sl_num_lanes-1:0]         i_sl_tx_ctrlenable,
    input  wire [16*sl_num_lanes-1:0]        i_sl_tx_datain,
    output wire [80*sl_num_lanes-1:0]        o_sl_tx_pma,
    input  wire [80*sl_num_lanes-1:0]        i_sl_rx_pma,
    input wire  [1*sl_num_lanes-1:0]         i_sl_tx_hard_fifo_full,
    input wire  [1*sl_num_lanes-1:0]         i_sl_tx_hard_fifo_pfull,
    input wire  [1*sl_num_lanes-1:0]         i_sl_rx_hard_fifo_empty,
    input wire  [1*sl_num_lanes-1:0]         i_sl_rx_hard_fifo_pempty,
    output wire [1*sl_num_lanes-1:0]         o_sl_hard_fifo_rd_en,
    output wire [2*sl_num_lanes-1:0]         o_sl_rx_ctrldetect,
    output wire [16*sl_num_lanes-1:0]        o_sl_rx_dataout,
    output wire [1*sl_num_lanes-1:0]         o_sl_rx_patterndetect,
    output wire [5*sl_num_lanes-1:0]         o_sl_rx_bitslipboundary_sel,
    output wire [2*sl_num_lanes-1:0]         o_sl_rx_disperr,
    output wire [2*sl_num_lanes-1:0]         o_sl_rx_errdetect,
    output wire [1*sl_num_lanes-1:0]         o_sl_tx_lspcs_ready,
    output wire [1*sl_num_lanes-1:0]         o_sl_rx_lspcs_ready,
    
    // Deterministic Latency Interface
    input  wire [sl_num_lanes-1:0]           i_sl_tx_dl_async_pulse,
    input  wire [sl_num_lanes-1:0]           i_sl_rx_dl_async_pulse,
    output wire [sl_num_lanes-1:0]           o_sl_latency_sclk,
    output wire [sl_num_lanes-1:0]           o_sl_tx_rx_dl_measure_sel,
    
    output wire [CALC_WIDTH*sl_num_lanes-1:0]   o_sl_tx_delay_sclk,
    output wire [CALC_WIDTH*sl_num_lanes-1:0]   o_sl_rx_delay_sclk,
    output wire [COUNT_WIDTH*sl_num_lanes-1:0]  o_sl_tx_sync_count,
    output wire [COUNT_WIDTH*sl_num_lanes-1:0]  o_sl_tx_async_count,
    output wire [COUNT_WIDTH*sl_num_lanes-1:0]  o_sl_rx_sync_count,
    output wire [COUNT_WIDTH*sl_num_lanes-1:0]  o_sl_rx_async_count,
    output wire [sl_num_lanes-1:0]              o_sl_measure_valid
);

    localparam TX_EFIFO_DEPTH = 16;
    //localparam RX_EFIFO_DEPTH = 32; // getting it works with the size of 16 is very tight for 24G because the rx cadence output is on average of 1s:0s=80:8
    localparam RX_EFIFO_DEPTH = 128; // getting it works with the size of 16 is very tight for 24G because the rx cadence output is on average of 1s:0s=80:8
    localparam CSR_RST_HOLD_PERIOD = 800; // 800 ns based on i_reconfig_clk of 100Mhz for worst case scenario at 2.4GHz
    localparam TX_RST_HOLD_PERIOD  = 100; // 100ns
    localparam RX_RST_HOLD_PERIOD  = 150; // 150ns

    wire sl_tx_rst_n_sync;

    wire        sl_csr_rst_hold;
    wire        sl_csr_rst;
    wire        sl_csr_rst_async;

    wire        sl_tx_rst_hold;
    wire        sl_tx_rst;
    wire        sl_tx_rst_async;

    wire        sl_rx_rst_hold;
    wire        sl_rx_rst;
    wire        sl_rx_rst_async;
    
    // DL Logic
    wire        sl_tx_mii_am;
    wire        sl_rx_mii_am_valid;
    wire        sl_rx_mii_am_valid_mux;
    wire        latency_measure_i_init_txclk_sync;
    wire        blklock_sclk_sync;
    wire        restart_sync_sclk;
    reg         restart_sync;
    wire        i_blklock_n_sclk_sync;
    wire        lspcs_rx_block_lock_n_sync;
    wire        rx_block_lock_n_sync_mux;
    wire        i_blklock_n_txclk_sync;
    wire        lspcs_rx_block_lock_n_txclk_sync;
    wire        i_blklock_n_rxclk_sync;
    wire        rx_block_lock_n_txclk_sync_mux;
    reg         rx_block_lock_n_txclk_sync_mux_reg;
    wire        rx_block_lock_n_rxclk_sync_mux;
    reg         rx_block_lock_n_rxclk_sync_mux_reg;

    // Soft PCS
    wire        lspcs_tx_soft_fifo_clk;
    wire        lspcs_rx_soft_fifo_clk;
    wire        lspcs_tx_hard_fifo_clk;
    wire        lspcs_rx_hard_fifo_clk;
    
    wire        lspcs_tx_pcs_rst_n;
    wire        lspcs_rx_pcs_rst_n;
    wire        lspcs_tx_fifo_rst_n;
    wire        lspcs_rx_fifo_rst_n;
	
    wire        lspcs_tx_soft_pcs_rst_n_sync;
    wire        lspcs_rx_soft_pcs_rst_n_sync;
    reg         lspcs_tx_soft_pcs_rst_n_sync_reg;
    reg         lspcs_rx_soft_pcs_rst_n_sync_reg;
    wire        lspcs_tx_soft_pcs_rst_n_async;
    wire        lspcs_rx_soft_pcs_rst_n_async;
    // wire        lspcs_tx_hard_pcs_rst_n;
    // wire        lspcs_rx_hard_pcs_rst_n;
	
    // wire        lspcs_tx_soft_fifo_rst_n_sync;
    // wire        lspcs_rx_soft_fifo_rst_n_sync;
    wire        lspcs_tx_soft_fifo_rst_n_async;
    wire        lspcs_rx_soft_fifo_rst_n_async;
    wire        lspcs_tx_hard_fifo_rst_n;
    wire        lspcs_rx_hard_fifo_rst_n;
	
    wire        lspcs_rx_mii_am_valid;
    reg         lspcs_rx_block_lock;
    
    wire        lspcs_tx_ready_rst_n_async;
    wire        lspcs_rx_ready_rst_n_async;
    wire        sl_rx_block_lock_async;
    wire        sl_ehip_ready_async;
    
    wire        sl_tx_nphy_ready;
    wire        sl_rx_nphy_ready;

    wire        dl_rst_sclk_sync;
    wire        dl_rst_txclk_sync;
    wire        dl_rst_rxclk_sync;
    
    alt_xcvr_resync_std #(
        .SYNC_CHAIN_LENGTH  (3),
        .WIDTH              (1),
        .INIT_VALUE         (0)
    ) ehip_tx_rst_n_sync (
        .clk                (i_reconfig_clk),
        .reset              (1'b0),
        .d                  (i_sl_tx_rst_n),
        .q                  (sl_tx_rst_n_sync)
    );
    
    //////////////////////////////////////////////////////////////
    // Workaround for TX realignment after receiving AM pulse
    localparam WAIT_AM_PULSE = 2; // Number of AM pulses to wait
    
    wire [sl_num_lanes-1:0] sl_ehip_ready;
    wire [sl_num_lanes-1:0] sl_ehip_ready_clk_tx_sync;
    wire [sl_num_lanes-1:0] am_gate_sync;
    reg [sl_num_lanes-1:0][WAIT_AM_PULSE:0] am_gate;
    reg [sl_num_lanes-1:0] sl_tx_mii_am_reg;
    reg [sl_num_lanes-1:0] tx_mii_am_pulse;
    reg [sl_num_lanes-1:0] ehip_ready;
    

    genvar i;
    generate
    if (ehip_mode_sl_0 != "ehip_disable") begin: AM_GATED_EHIP_READY_PCS_ONLY
    for (i=0; i<sl_num_lanes; i++) begin: AM_GATED_EHIP_READY
        alt_xcvr_resync_std #(
            .SYNC_CHAIN_LENGTH (2),  // Number of flip-flops for retiming
            .INIT_VALUE        (1)
        ) i_clk_tx_rst_sync_inst(
            .clk                       (i_sl_clk_tx[i]),
            .reset                     (o_sl_ehip_tx_reset[i]),
            .d                         (sl_ehip_ready[i]),
            .q                         (sl_ehip_ready_clk_tx_sync[i])
        );

        always @(posedge i_sl_clk_tx[i]) begin
            if(~sl_ehip_ready_clk_tx_sync[i]) begin
                am_gate[i]         <= {(WAIT_AM_PULSE+1){1'b0}};
            end
            else begin
                am_gate[i][0]   <= 1'b1;
                sl_tx_mii_am_reg[i] <= o_sl_tx_mii_am[i];
                tx_mii_am_pulse[i] <= !o_sl_tx_mii_am[i] & sl_tx_mii_am_reg[i];
                if(tx_mii_am_pulse[i] == 1'b1) begin
                    am_gate[i][WAIT_AM_PULSE:1] <= am_gate[i][WAIT_AM_PULSE-1:0];
                end
            end
        end

        alt_xcvr_resync_std #(
            .SYNC_CHAIN_LENGTH(2),    
            .WIDTH(1),  
            .INIT_VALUE(0)	
        ) am_gate_reconfig_clk_sync_inst (
            .clk                        (i_reconfig_clk),
            .reset                      (o_sl_ehip_reset[i]),
            .d                          (am_gate[i][WAIT_AM_PULSE]),
            .q                          (am_gate_sync[i])
        );

        always @(posedge i_reconfig_clk) begin
            if(!sl_ehip_ready[i]) begin
                ehip_ready[i]      <= 1'b0;
            end
            else begin
                if(am_gate_sync[i] == 1'b1) begin
                    ehip_ready[i] <= sl_ehip_ready[i];
                end else begin
                    ehip_ready[i] <= 1'b0;
                end
            end
        end
        assign o_sl_ehip_ready[i] = ehip_ready[i];
    end
    end else begin
        assign o_sl_ehip_ready = sl_ehip_ready;
    end
    endgenerate
    // End workaround for TX realignment after receiving AM pulse
    //////////////////////////////////////////////////////////////
    
    // ------------------------------------------------------------------------------------
    // Reset controller IP
    alt_cpriphy_c3_reset_controller #(
        .CLK_FREQ_IN_HZ(100000000),
        .NUM_CHANNELS(1),
        .SIM_EMULATE(SIM_EMULATE),
        .ENABLE_RSFEC(0),
        .TILE("etile")
    ) i_sl_reset_controller (
        .csr_rst_in                (~i_sl_csr_rst_n),
        .tx_rst_in                 (~sl_tx_rst_n_sync),
        .rx_rst_in                 (~i_sl_rx_rst_n),
        .soft_tx_rst_in            (i_sl_soft_tx_rst),
        .soft_rx_rst_in            (i_sl_soft_rx_rst),
        .soft_csr_rst_in           (i_sl_soft_csr_rst),

        .tx_pma_ready_in           (i_sl_tx_pma_ready),
        .rx_pma_ready_in           (i_sl_rx_pma_ready),
        .tx_transfer_ready_in      (i_sl_tx_transfer_ready),
        .rx_transfer_ready_in      (i_sl_rx_transfer_ready),
        .tx_dll_lock_in            (i_sl_tx_dll_lock),
        .tx_cal_busy               (1'b0),
        .rx_cal_busy               (1'b0),
        .rx_is_lockedtoref_in      (1'b1),
        .rx_is_lockedtodata_in     (i_sl_rx_is_lockedtodata),

        .tx_aib_reset_out          (o_sl_tx_aib_reset),
        .tx_pcs_reset_out          (o_sl_tx_pcs_reset),
        .rx_aib_reset_out          (o_sl_rx_aib_reset),
        .rx_pcs_reset_out          (o_sl_rx_pcs_reset),

        .ehip_ready_in             (i_sl_ehip_ready),
        .ehip_reset_ack_in         (~i_sl_ehip_reset_stat),
        .ehip_tx_reset_ack_in      (~i_sl_ehip_tx_reset_stat),
        .ehip_rx_reset_ack_in      (~i_sl_ehip_rx_reset_stat),
        .ehip_reset_out            (o_sl_ehip_reset),
        .ehip_tx_reset_out         (o_sl_ehip_tx_reset),
        .ehip_rx_reset_out         (o_sl_ehip_rx_reset),
        .ehip_signal_ok_out        (o_sl_ehip_signal_ok),
        .ehip_pld_ready_out        (o_sl_ehip_pld_ready),

        .rsfec_tx_reset_ack_in     (o_sl_rsfec_tx_reset),
        .rsfec_rx_reset_ack_in     (o_sl_rsfec_rx_reset),
        .rsfec_reset_out           (o_sl_rsfec_reset),
        .rsfec_tx_reset_out        (o_sl_rsfec_tx_reset),
        .rsfec_rx_reset_out        (o_sl_rsfec_rx_reset),

        .csr_rst_stat              (),
        .tx_rst_stat               (),
        .rx_rst_stat               (),

        .kr_mode_in                (1'b0),
        .kr_rst_req                (1'b0),
        .kr_rst_stat               (),

        .clk_status                (i_reconfig_clk),
        .pll_locked_in             (1'b1),
        .rx_pcs_fully_aligned_in   (i_sl_rx_pcs_fully_aligned),

        .tx_lanes_stable           (o_sl_tx_lanes_stable),
        .tx_eth_ready              (),
        .rx_eth_ready              (o_sl_rx_pcs_ready),
        .ehip_ready_out            (sl_ehip_ready)
    
    );

    // NPHY general reset (only used for PMA Direct mode)
    // assign o_sl_pmad_reset = ~i_sl_csr_rst_n;

    
    // ------------------------------------------------------------------------------------
    // EFIFO for 64/66 data path
    generate
    if (enable_external_aib_clocking == 1 && ehip_mode_sl_0 != "ehip_disable") begin: EXT_AIB_CLK
        // EFIFO
        wire        tx_rst_i_clk_aib_sync;
        wire        rx_rst_i_clk_aib_sync;
        wire        tx_rst_pll_div66_sync;
        wire        rx_rst_rec_div66_sync;
        wire        rx_efifo_nvalid;
        wire [9:0]  nc_float;
        wire        sl_txfifo_pfull_403mhz_sync;
        wire [6:0] TX_POR_FIFO_CNT_EXT, RX_POR_FIFO_CNT_EXT;
        wire [($clog2(TX_EFIFO_DEPTH)-1):0] TX_R_PFULL_PORT, TX_R_PEMPTY_PORT;
        wire [($clog2(RX_EFIFO_DEPTH)-1):0] RX_R_PFULL_PORT, RX_R_PEMPTY_PORT;

        // FIFO will always fill until PEMPTY level even if POR_CNT = 0.
        assign TX_POR_FIFO_CNT_EXT = 7'd0;
        
        // If mii_valid > actual line rate, set this close to middle of FIFO depth, else set it to (FIFO depth - 4)
        assign TX_R_PFULL_PORT = (i_sl_cpri_rate_sel == CPRI_RATE_24G) ? 4'd9 : 4'd11;
        
        // If mii_valid < actual line rate, set this close to middle of FIFO depth, else set it to 4
        assign TX_R_PEMPTY_PORT = (i_sl_cpri_rate_sel == CPRI_RATE_24G) ? 4'd4 : 4'd6;
        
        // PFULL and PEMPTY has no effect in RX
        //assign RX_R_PFULL_PORT = 5'd27;
        assign RX_R_PFULL_PORT = 7'd123; // with FIFO depth set to 128 to cater for 10G/12G FEC variant
        assign RX_R_PEMPTY_PORT = 7'd4;
        
        /////////////////////////////////////////////////////////////////////////
        // 
        // RX POR_FIFO_CNT = Target fill level * (aib_clk/parallel_clk)
        // 
        // where aib_clk (MHz)      = 402.83203125
        //       parallel_clk (MHz) = 368.64 (24G)
        //                          = 184.32 (12G)
        //                          = 153.6 (10G)
        // We are targetting around 12-13 fill level.
        // 
        //////////////////////////////////////////////////////////////////////////
        assign RX_POR_FIFO_CNT_EXT = (i_sl_cpri_rate_sel == CPRI_RATE_24G) ? 7'd12 : 7'd31;

        // Prevent warning
        reg  [4:0]  dummy_reg /* synthesis noprune */;
        always @ (posedge i_aib_clk) begin
            dummy_reg <= nc_float[4:0];
        end

        // Prevent warning
        reg  [4:0]  dummy_reg2 /* synthesis noprune */;
        always @ (posedge i_clk_rec_div66) begin
            dummy_reg2 <= nc_float[9:5];
        end

        alt_xcvr_resync_std #(
            .SYNC_CHAIN_LENGTH (2),  // Number of flip-flops for retiming
            .INIT_VALUE        (1)
        ) i_clk_tx_rst_sync_inst(
            .clk                       (i_aib_clk),
            .reset                     (o_sl_ehip_tx_reset),
            .d                         (1'b0),
            .q                         (tx_rst_i_clk_aib_sync)
        );

        alt_xcvr_resync_std #(
            .SYNC_CHAIN_LENGTH (2),  // Number of flip-flops for retiming
            .INIT_VALUE        (1)
        ) i_clk_rx_rst_sync_inst(
            .clk                       (i_aib_clk),
            .reset                     (o_sl_ehip_rx_reset),
            .d                         (1'b0),
            .q                         (rx_rst_i_clk_aib_sync)
        );
       
        alt_xcvr_resync_std #(
            .SYNC_CHAIN_LENGTH (2),  // Number of flip-flops for retiming
            .INIT_VALUE        (1)
        ) pll_div66_rst_sync_inst(
            .clk                       (i_clk_pll_div66),
            .reset                     (o_sl_ehip_tx_reset),
            .d                         (1'b0),
            .q                         (tx_rst_pll_div66_sync)
        );
        alt_xcvr_resync_std #(
            .SYNC_CHAIN_LENGTH (2),  // Number of flip-flops for retiming
            .INIT_VALUE        (1)
        ) rec_div66_rst_sync_inst(
            .clk                       (i_clk_rec_div66),
            .reset                     (o_sl_ehip_rx_reset),
            .d                         (1'b0),
            .q                         (rx_rst_rec_div66_sync)
        );

        alt_xcvr_resync_std #(
            .SYNC_CHAIN_LENGTH (2),  
            .INIT_VALUE        (1)
        ) i_tx_sfifo_pfull_403mhzclk_sync_inst(
            .clk                       (i_aib_clk),  
            .reset                     (o_sl_ehip_reset), // in management clock
            .d                         (i_sl_txfifo_pfull),
            .q                         (sl_txfifo_pfull_403mhz_sync)
        );

        alt_cpriphy_c3_efifo_top #(
            .PMA_MODE (1),
            .SOFT_FIFO_DEPTH(TX_EFIFO_DEPTH)
        ) tx_efifo (
            .hard_fifo_clk (i_aib_clk),
            .hard_fifo_rst_n (~tx_rst_i_clk_aib_sync),
            .soft_fifo_clk (i_clk_pll_div66),
            .soft_fifo_rst_n (~tx_rst_pll_div66_sync),
            .DATA_RATE_CONFIG (i_sl_cpri_rate_sel),
            .POR_FIFO_CNT(TX_POR_FIFO_CNT_EXT),
            .R_FULL (4'd15),
            .R_PFULL (TX_R_PFULL_PORT),
            .R_EMPTY (4'd0),
            .R_PEMPTY (TX_R_PEMPTY_PORT),
            .tunneling_en (1'b0),
            .soft_fifo_pfull (o_sl_mii_txfifo_pfull),
            .soft_fifo_full (o_sl_mii_txfifo_overflow),
            .soft_fifo_empty (o_sl_mii_txfifo_underflow),
            .soft_fifo_pempty (o_sl_mii_txfifo_pempty),
            .hard_fifo_pfull (1'b0),
            .hard_fifo_full (1'b0),
            .hard_fifo_empty (1'b0),
            .hard_fifo_pempty (1'b0),
            .soft_fifo_wr_en (1'b1),
            .soft_fifo_wr_data ({5'd0,sl_tx_mii_am,i_sl_tx_mii_c,i_sl_tx_mii_d}),
            .soft_fifo_rd_en (1'b0),
            .soft_fifo_rd_data (),
            .hard_fifo_wr_en (),
            .hard_fifo_wr_data_valid (o_sl_tx_mii_valid),
            .hard_fifo_wr_data ({nc_float[4:0],o_sl_tx_mii_am,o_sl_tx_mii_c,o_sl_tx_mii_d}),
            .hard_fifo_rd_en (),
            .hard_fifo_rd_data_valid (1'b0),
            .hard_fifo_rd_data (78'b0)
        );

        alt_cpriphy_c3_efifo_top #(
            .PMA_MODE (0),
            .SOFT_FIFO_DEPTH(RX_EFIFO_DEPTH)
        ) rx_efifo (
            .hard_fifo_clk (i_aib_clk),
            .hard_fifo_rst_n (~rx_rst_i_clk_aib_sync),
            .soft_fifo_clk (i_clk_rec_div66),
            .soft_fifo_rst_n (~rx_rst_rec_div66_sync),
            .DATA_RATE_CONFIG (i_sl_cpri_rate_sel),
            .POR_FIFO_CNT(RX_POR_FIFO_CNT_EXT),
            .R_FULL (7'd127),
            .R_PFULL (RX_R_PFULL_PORT),
            .R_EMPTY (7'd0),
            .R_PEMPTY (RX_R_PEMPTY_PORT),
            .tunneling_en (1'b0),
            .soft_fifo_pfull (),
            .soft_fifo_full (),
            .soft_fifo_empty (rx_efifo_nvalid), // error flag telling the fifo is empty and no data available
            .soft_fifo_pempty (),
            .hard_fifo_pfull (1'b0),
            .hard_fifo_full (1'b0),
            .hard_fifo_empty (1'b0),
            .hard_fifo_pempty (1'b0),
            .soft_fifo_wr_en (1'b0),
            .soft_fifo_wr_data (78'b0),
            .soft_fifo_rd_en (1'b1),
            .soft_fifo_rd_data ({nc_float[9:5],sl_rx_mii_am_valid,o_sl_rx_mii_c,o_sl_rx_mii_d}),
            .hard_fifo_wr_en (),
            .hard_fifo_wr_data_valid (),
            .hard_fifo_wr_data (),
            .hard_fifo_rd_en (),
            .hard_fifo_rd_data_valid (i_sl_rx_mii_valid & i_sl_rx_block_lock),
            .hard_fifo_rd_data ({5'b0,i_sl_rx_mii_am_valid, i_sl_rx_mii_c,i_sl_rx_mii_d})
        );
        assign o_sl_tx_mii_ready = ~o_sl_mii_txfifo_pfull;
        assign o_sl_rx_mii_valid = 'd0;
    end
    else begin
        assign o_sl_tx_mii_d = i_sl_tx_mii_d;
        assign o_sl_tx_mii_c = i_sl_tx_mii_c;
        assign o_sl_tx_mii_valid = i_sl_tx_mii_valid;
        assign o_sl_tx_mii_am = sl_tx_mii_am;
        assign o_sl_tx_mii_ready = i_sl_tx_mii_ready;
        
        assign o_sl_rx_mii_d = i_sl_rx_mii_d;
        assign o_sl_rx_mii_c = i_sl_rx_mii_c;
        assign o_sl_rx_mii_valid = i_sl_rx_mii_valid;
        assign sl_rx_mii_am_valid = (ehip_mode_sl_0 != "ehip_disable") ? i_sl_rx_mii_am_valid: 1'b0;

        assign o_sl_mii_txfifo_pfull = 'd0;
        assign o_sl_mii_txfifo_pempty = 'd0;
        assign o_sl_mii_txfifo_overflow = 'd0;
        assign o_sl_mii_txfifo_underflow = 'd0;
    end
    endgenerate

    // ------------------------------------------------------------------------------------
    // EFIFO and soft PCS for 8b/10b pma direct data path    
    generate
    if (ehip_mode_sl_0 == "ehip_disable" || cpri_include_alternate_ports == 1) begin: PCS_8B10B
        assign lspcs_tx_soft_fifo_clk = i_clk_pll_div66;  // 495 / 245 / 122 Mhz
        assign lspcs_rx_soft_fifo_clk = i_clk_rec_div66;  // 495 / 245 / 122 Mhz
        assign lspcs_tx_hard_fifo_clk = i_sl_clk_tx;      // 403 MHz
        assign lspcs_rx_hard_fifo_clk = i_sl_clk_tx;      // 403 MHz

        //////////////////////////////////////////////////////////////////////////////
		
        assign sl_csr_rst = ~i_sl_csr_rst_n | i_sl_soft_csr_rst;
        assign sl_tx_rst  = ~i_sl_tx_rst_n | i_sl_soft_tx_rst;
        assign sl_rx_rst  = ~i_sl_rx_rst_n | i_sl_soft_rx_rst;
        
        alt_xcvr_resync_std #(
            .SYNC_CHAIN_LENGTH  (3),
            .WIDTH              (1),
            .INIT_VALUE         (1)
        ) csr_rst_sync (
            .clk                (i_reconfig_clk),
            .reset              (sl_csr_rst),
            .d                  (1'b0),
            .q                  (sl_csr_rst_async)
        );
        
        alt_xcvr_resync_std #(
            .SYNC_CHAIN_LENGTH  (3),
            .WIDTH              (1),
            .INIT_VALUE         (1)
        ) tx_rst_sync (
            .clk                (i_reconfig_clk),
            .reset              (sl_tx_rst),
            .d                  (1'b0),
            .q                  (sl_tx_rst_async)
        );
        
        alt_xcvr_resync_std #(
            .SYNC_CHAIN_LENGTH  (3),
            .WIDTH              (1),
            .INIT_VALUE         (1)
        ) rx_rst_sync (
            .clk                (i_reconfig_clk),
            .reset              (sl_rx_rst),
            .d                  (1'b0),
            .q                  (sl_rx_rst_async)
        );

        //////////////////////////////////////////////////////////////////////////////
        
        alt_xcvr_reset_counter_s10_etile #(
            .CLKS_PER_SEC   (100000000),            // Clock frequency in Hz
            .RESET_PER_NS   (CSR_RST_HOLD_PERIOD),  // Reset period in ns
            .RESET_COUNT    (0),                    // Overrides RESET_PER_NS
            .ACTIVE_LEVEL   (1)                     //
        ) csr_rst_hold_counter (
            .clk            (i_reconfig_clk),
            .async_req      (sl_csr_rst_async),     // asynchronous reset request (restart sequence)
            .sync_req       (1'b0),                 // synchronous reset request
            .reset_or       (1'b0),                 // auxilliary reset override (assert only)

            .reset          (sl_csr_rst_hold),      // synchronous reset out
            .reset_n        (),                     // negation of reset
            .reset_stat     ()                      // reset status (intended for control logic)
        );

        //////////////////////////////////////////////////////////////////////////////
        
        alt_xcvr_reset_counter_s10_etile #(
            .CLKS_PER_SEC   (100000000),            // Clock frequency in Hz
            .RESET_PER_NS   (TX_RST_HOLD_PERIOD),   // Reset period in ns
            .RESET_COUNT    (0),                    // Overrides RESET_PER_NS
            .ACTIVE_LEVEL   (1)                     //
        ) tx_rst_hold_counter (
            .clk            (i_reconfig_clk),
            .async_req      (sl_tx_rst_async),      // asynchronous reset request (restart sequence)
            .sync_req       (1'b0),                 // synchronous reset request
            .reset_or       (1'b0),                 // auxilliary reset override (assert only)

            .reset          (sl_tx_rst_hold),       // synchronous reset out
            .reset_n        (),                     // negation of reset
            .reset_stat     ()                      // reset status (intended for control logic)
        );
        
        //////////////////////////////////////////////////////////////////////////////
        
        alt_xcvr_reset_counter_s10_etile #(
            .CLKS_PER_SEC   (100000000),            // Clock frequency in Hz
            .RESET_PER_NS   (RX_RST_HOLD_PERIOD),   // Reset period in ns
            .RESET_COUNT    (0),                    // Overrides RESET_PER_NS
            .ACTIVE_LEVEL   (1)                     //
        ) rx_rst_hold_counter (
            .clk            (i_reconfig_clk),
            .async_req      (sl_rx_rst_async),      // asynchronous reset request (restart sequence)
            .sync_req       (1'b0),                 // synchronous reset request
            .reset_or       (1'b0),                 // auxilliary reset override (assert only)

            .reset          (sl_rx_rst_hold),       // synchronous reset out
            .reset_n        (),                     // negation of reset
            .reset_stat     ()                      // reset status (intended for control logic)
        );

        //////////////////////////////////////////////////////////////////////////////

        assign o_sl_tx_reset = sl_csr_rst_hold;
        assign o_sl_rx_reset = sl_csr_rst_hold;

        assign lspcs_tx_pcs_rst_n = ~(sl_csr_rst_hold | sl_tx_rst_hold);
        assign lspcs_rx_pcs_rst_n = ~(sl_csr_rst_hold | sl_rx_rst_hold);
        assign lspcs_tx_fifo_rst_n = ~sl_csr_rst_hold;
        assign lspcs_rx_fifo_rst_n = ~sl_csr_rst_hold;
            
        //////////////////////////////////////////////////////////////////////////////

        alt_xcvr_resync_std #(
            .SYNC_CHAIN_LENGTH(3),    .WIDTH(1),  .INIT_VALUE(0)	
        ) sync_lspcs_tx_soft_pcs_rst_n_sync (
            .clk    (lspcs_tx_soft_fifo_clk),
            .reset  (~(lspcs_tx_pcs_rst_n & i_sl_tx_pma_ready & i_sl_tx_transfer_ready)),
            .d  (1'b1),
            .q  (lspcs_tx_soft_pcs_rst_n_sync) // active low
        );
        
        alt_xcvr_resync_std #(
            .SYNC_CHAIN_LENGTH(3),    .WIDTH(1),  .INIT_VALUE(0)	
        ) sync_lspcs_tx_soft_pcs_rst_n_async (
            .clk    (lspcs_tx_soft_fifo_clk),
            .reset  (~(lspcs_tx_pcs_rst_n & i_sl_tx_pma_ready & i_sl_tx_transfer_ready)),
            .d  (1'b1),
            .q  (lspcs_tx_soft_pcs_rst_n_async) // active low
        );
        
        // alt_xcvr_resync_std #(
            // .SYNC_CHAIN_LENGTH(3),    .WIDTH(1),  .INIT_VALUE(0)	
        // ) sync_lspcs_tx_hard_pcs_rst_n (
            // .clk    (lspcs_tx_hard_fifo_clk),
            // .reset  (~(lspcs_tx_pcs_rst_n & i_sl_tx_pma_ready & i_sl_tx_transfer_ready)),
            // .d  (1'b1),
            // .q  (lspcs_tx_hard_pcs_rst_n) // active low
        // );
        
        alt_xcvr_resync_std #(
            .SYNC_CHAIN_LENGTH(3),    .WIDTH(1),  .INIT_VALUE(0)	
        ) sync_lspcs_rx_soft_pcs_rst_n_sync (
            .clk    (lspcs_rx_soft_fifo_clk),
            .reset  (~(lspcs_rx_pcs_rst_n & i_sl_rx_pma_ready & i_sl_rx_is_lockedtodata & i_sl_rx_transfer_ready)),
            .d  (1'b1),
            .q  (lspcs_rx_soft_pcs_rst_n_sync) // active low
        );
        
        alt_xcvr_resync_std #(
            .SYNC_CHAIN_LENGTH(3),    .WIDTH(1),  .INIT_VALUE(0)	
        ) sync_lspcs_rx_soft_pcs_rst_n_async (
            .clk    (lspcs_rx_soft_fifo_clk),
            .reset  (~(lspcs_rx_pcs_rst_n & i_sl_rx_pma_ready & i_sl_rx_is_lockedtodata & i_sl_rx_transfer_ready)),
            .d  (1'b1),
            .q  (lspcs_rx_soft_pcs_rst_n_async) // active low
        );
        
        // alt_xcvr_resync_std #(
            // .SYNC_CHAIN_LENGTH(3),    .WIDTH(1),  .INIT_VALUE(0)	
        // ) sync_lspcs_rx_hard_pcs_rst_n (
            // .clk    (lspcs_rx_hard_fifo_clk),
            // .reset  (~(lspcs_rx_pcs_rst_n & i_sl_rx_pma_ready & i_sl_rx_is_lockedtodata & i_sl_rx_transfer_ready)),
            // .d  (1'b1),
            // .q  (lspcs_rx_hard_pcs_rst_n) // active low
        // );
		
        ////////////////////////////////////////////////////////////
		
        // alt_xcvr_resync_std #(
            // .SYNC_CHAIN_LENGTH(3),    .WIDTH(1),  .INIT_VALUE(0)	
        // ) sync_lspcs_tx_soft_fifo_rst_n_sync (
            // .clk    (lspcs_tx_soft_fifo_clk),
            // .reset  (~(lspcs_tx_fifo_rst_n & i_sl_tx_fifo_ready)),
            // .d  (1'b1),
            // .q  (lspcs_tx_soft_fifo_rst_n_sync) // active low
        // );
        
        alt_xcvr_resync_std #(
            .SYNC_CHAIN_LENGTH(3),    .WIDTH(1),  .INIT_VALUE(0)	
        ) sync_lspcs_tx_soft_fifo_rst_n_async (
            .clk    (lspcs_tx_soft_fifo_clk),
            .reset  (~(lspcs_tx_fifo_rst_n & i_sl_tx_pma_ready & i_sl_tx_transfer_ready)),
            .d  (1'b1),
            .q  (lspcs_tx_soft_fifo_rst_n_async) // active low
        );
        
        alt_xcvr_resync_std #(
            .SYNC_CHAIN_LENGTH(3),    .WIDTH(1),  .INIT_VALUE(0)	
        ) sync_lspcs_tx_hard_fifo_rst_n (
            .clk    (lspcs_tx_hard_fifo_clk),
            .reset  (~(lspcs_tx_fifo_rst_n & i_sl_tx_pma_ready & i_sl_tx_transfer_ready)),
            .d  (1'b1),
            .q  (lspcs_tx_hard_fifo_rst_n) // active low
        );
        
        // alt_xcvr_resync_std #(
            // .SYNC_CHAIN_LENGTH(3),    .WIDTH(1),  .INIT_VALUE(0)	
        // ) sync_lspcs_rx_soft_fifo_rst_n_sync (
            // .clk    (lspcs_rx_soft_fifo_clk),
            // .reset  (~(lspcs_rx_fifo_rst_n & i_sl_rx_fifo_ready & i_sl_rx_is_lockedtodata)),
            // .d  (1'b1),
            // .q  (lspcs_rx_soft_fifo_rst_n_sync) // active low
        // );
        
        alt_xcvr_resync_std #(
            .SYNC_CHAIN_LENGTH(3),    .WIDTH(1),  .INIT_VALUE(0)	
        ) sync_lspcs_rx_soft_fifo_rst_n_async (
            .clk    (lspcs_rx_soft_fifo_clk),
            .reset  (~(lspcs_rx_fifo_rst_n & i_sl_rx_pma_ready & i_sl_rx_is_lockedtodata & i_sl_rx_fifo_ready)),
            .d  (1'b1),
            .q  (lspcs_rx_soft_fifo_rst_n_async) // active low
        );
        
        alt_xcvr_resync_std #(
            .SYNC_CHAIN_LENGTH(3),    .WIDTH(1),  .INIT_VALUE(0)	
        ) sync_lspcs_rx_hard_fifo_rst_n (
            .clk    (lspcs_rx_hard_fifo_clk),
            .reset  (~(lspcs_rx_fifo_rst_n & i_sl_rx_pma_ready & i_sl_rx_is_lockedtodata & i_sl_rx_fifo_ready)),
            .d  (1'b1),
            .q  (lspcs_rx_hard_fifo_rst_n) // active low
        );
		
        ////////////////////////////////////////////////////////////

        alt_xcvr_resync_std #(
            .SYNC_CHAIN_LENGTH(3),    
            .WIDTH(1),  
            .INIT_VALUE(1)	
        ) lspcs_rx_blocklock_sclk_sync_inst (
            .clk                        (i_sampling_clk),
            .reset                      (o_sl_ehip_reset),
            .d                          (~lspcs_rx_block_lock),
            .q                          (lspcs_rx_block_lock_n_sync) // active high
        );

        alt_xcvr_resync_std #(
            .SYNC_CHAIN_LENGTH(3),    
            .WIDTH(1),  
            .INIT_VALUE(1)	
        ) lspcs_rx_blocklock_txclk_sync_inst (
            .clk             (i_clk_pll_div66),
            .reset           (o_sl_ehip_reset),
            .d               (~lspcs_rx_block_lock),
            .q               (lspcs_rx_block_lock_n_txclk_sync) // active high
        );
        
        always @ (posedge lspcs_tx_soft_fifo_clk) begin
            lspcs_tx_soft_pcs_rst_n_sync_reg <= lspcs_tx_soft_pcs_rst_n_sync;
        end

        always @ (posedge lspcs_rx_soft_fifo_clk) begin
            lspcs_rx_soft_pcs_rst_n_sync_reg <= lspcs_rx_soft_pcs_rst_n_sync;
        end

        alt_cpriphy_ls_pcs_top cpriphy_ls_pcs_top_0(
            // clock and reset
            .i_tx_soft_fifo_clk                 (lspcs_tx_soft_fifo_clk),
            .i_rx_soft_fifo_clk                 (lspcs_rx_soft_fifo_clk),
            .i_tx_hard_fifo_clk                 (lspcs_tx_hard_fifo_clk),
            .i_rx_hard_fifo_clk                 (lspcs_rx_hard_fifo_clk),
            
            .i_tx_soft_pcs_rst_n_sync           (lspcs_tx_soft_pcs_rst_n_sync_reg),
            .i_rx_soft_pcs_rst_n_sync           (lspcs_rx_soft_pcs_rst_n_sync_reg),
            .i_tx_soft_fifo_rst_n_async         (lspcs_tx_soft_fifo_rst_n_async),
            .i_rx_soft_fifo_rst_n_async         (lspcs_rx_soft_fifo_rst_n_async),
            .i_tx_hard_fifo_rst_n               (lspcs_tx_hard_fifo_rst_n),
            .i_rx_hard_fifo_rst_n               (lspcs_rx_hard_fifo_rst_n),
			
            // FIFO configuration
            .DATA_RATE_CONFIG                   (i_sl_cpri_rate_sel),
            .i_tunneling_en                     (1'b0),
            
            // Ethernet 1GE TX tbi2x interface
            .i_sl_tbi2x_tx_d                    (20'd0),
            // TX soft 32-bit interface
            .i_tx_datain_32b                    (32'd0),
            // TX soft 16-bit interface
            .i_tx_ctrlenable                    (i_sl_tx_ctrlenable),
            .i_tx_datain                        (i_sl_tx_datain),
            .i_tx_mii_am                        (sl_tx_mii_am),
            // TX hard 80-bit interface
            .i_tx_hard_fifo_pfull               (i_sl_tx_hard_fifo_pfull),
            .i_tx_hard_fifo_full                (i_sl_tx_hard_fifo_full),
            .o_tx_parallel_data                 (o_sl_tx_pma),
        
            // Ethernet 1GE RX tbi2x interface
            .o_sl_tbi2x_rx_d                    (),
            // RX soft 32-bit interface
            .o_rx_dataout_32b                   (),
            // RX hard 80-bit interface
            .i_rx_hard_fifo_pempty              (i_sl_rx_hard_fifo_pempty),
            .i_rx_hard_fifo_empty               (i_sl_rx_hard_fifo_empty),
            .o_rx_hard_fifo_rd_en               (o_sl_hard_fifo_rd_en),
            .i_rx_parallel_data                 (i_sl_rx_pma),
            
            .o_rx_mii_am_valid                  (lspcs_rx_mii_am_valid),
            .o_rx_ctrldetect                    (o_sl_rx_ctrldetect),
            .o_rx_dataout                       (o_sl_rx_dataout),
            .o_rx_patterndetect                 (o_sl_rx_patterndetect),
            .o_rx_bitslipboundary_sel           (o_sl_rx_bitslipboundary_sel),
            .o_rx_disperr                       (o_sl_rx_disperr),
            .o_rx_errdetect                     (o_sl_rx_errdetect)
        );

        always @ (posedge lspcs_rx_soft_fifo_clk or negedge lspcs_rx_soft_pcs_rst_n_async) begin
            if (~lspcs_rx_soft_pcs_rst_n_async) begin
                lspcs_rx_block_lock <= 1'b0;
            end
            else begin
                if ((o_sl_rx_patterndetect) && (o_sl_rx_ctrldetect!=2'b00) && (o_sl_rx_disperr==2'b00) && (o_sl_rx_errdetect==2'b00)) begin
                    lspcs_rx_block_lock <= 1'b1;
                end
                else if ((o_sl_rx_disperr!=2'b00) || (o_sl_rx_errdetect!=2'b00)) begin
                    lspcs_rx_block_lock <= 1'b0;
                end
            end
        end
		
        ////////////////////////////////////////////////////////////
    
        alt_xcvr_resync_std #(
            .SYNC_CHAIN_LENGTH(3),    .WIDTH(1),  .INIT_VALUE(0)	
        ) sync_lspcs_tx_ready_rst_n_async (
            .clk    (i_reconfig_clk),
            .reset  (~(lspcs_tx_pcs_rst_n & i_sl_tx_pma_ready & i_sl_tx_transfer_ready)),
            .d  (1'b1),
            .q  (lspcs_tx_ready_rst_n_async) // active low
        );
            
        alt_xcvr_resync_std #(
            .SYNC_CHAIN_LENGTH(3),    .WIDTH(1),  .INIT_VALUE(0)	
        ) sync_lspcs_rx_ready_rst_n_async (
            .clk    (i_reconfig_clk),
            .reset  (~(lspcs_rx_pcs_rst_n & i_sl_rx_pma_ready & i_sl_rx_is_lockedtodata & i_sl_rx_transfer_ready)),
            .d  (1'b1),
            .q  (lspcs_rx_ready_rst_n_async) // active low
        );

    end
    else begin
        assign o_sl_tx_pma = 'd0;
        assign o_sl_hard_fifo_rd_en = 'd0;
        assign o_sl_rx_ctrldetect = 'd0;
        assign o_sl_rx_dataout = 'd0;
        assign o_sl_rx_patterndetect = 'd0;
        assign o_sl_rx_bitslipboundary_sel = 'd0;
        assign o_sl_rx_disperr = 'd0;
        assign o_sl_rx_errdetect = 'd0;
        assign lspcs_rx_mii_am_valid = 'd0;
        assign lspcs_rx_block_lock_n_sync = 'd0;
        assign lspcs_rx_block_lock_n_txclk_sync = 'd0;
        // assign o_sl_tx_lspcs_ready = 'd0;
        // assign o_sl_rx_lspcs_ready = 'd0;

        assign o_sl_tx_reset = 'd0;
        assign o_sl_rx_reset = 'd0;
        assign lspcs_tx_pcs_rst_n = 'd1;
        assign lspcs_rx_pcs_rst_n = 'd1;
        assign lspcs_tx_fifo_rst_n = 'd1;
        assign lspcs_rx_fifo_rst_n = 'd1;
        assign lspcs_tx_ready_rst_n_async = 'd1;
        assign lspcs_rx_ready_rst_n_async = 'd1;

        always @ (posedge i_clk_rec_div66) begin
            lspcs_rx_block_lock <= 1'b0;
        end
    end
    endgenerate

    // ------------------------------------------------------------------------------------
    // TX and RX ready for low-speed and high-speed mode
        
    generate
    if (ehip_mode_sl_0 == "ehip_disable")
    begin: LSPCS_READY_8B10B
        alt_xcvr_resync_std #(
            .SYNC_CHAIN_LENGTH (2),  
            .INIT_VALUE        (0)
        ) lspcs_tx_nphy_ready_sync_inst(
            .clk                       (i_reconfig_clk),
            .reset                     (~lspcs_tx_ready_rst_n_async),
            .d                         (i_sl_tx_nphy_ready),
            .q                         (sl_tx_nphy_ready)
        );
        
        alt_xcvr_resync_std #(
            .SYNC_CHAIN_LENGTH (2),  
            .INIT_VALUE        (0)
        ) lspcs_rx_nphy_ready_sync_inst(
            .clk                       (i_reconfig_clk),
            .reset                     (~lspcs_rx_ready_rst_n_async),
            .d                         (i_sl_rx_nphy_ready),
            .q                         (sl_rx_nphy_ready)
        );
        
        assign o_sl_tx_lspcs_ready = sl_tx_nphy_ready;
        assign o_sl_rx_lspcs_ready = sl_rx_nphy_ready;
    end
    else if (cpri_include_alternate_ports == 1)
    begin: LSPCS_READY_AIB
        alt_xcvr_resync_std #(
            .SYNC_CHAIN_LENGTH (2),  
            .INIT_VALUE        (0)
        ) lspcs_ehip_ready_sync_inst(
            .clk                       (i_reconfig_clk),
            .reset                     (~lspcs_tx_ready_rst_n_async),
            .d                         (i_sl_ehip_ready),
            .q                         (sl_ehip_ready_async)
        );
        
        alt_xcvr_resync_std #(
            .SYNC_CHAIN_LENGTH (2),  
            .INIT_VALUE        (0)
        ) lspcs_rx_blocklock_sync_inst(
            .clk                       (i_reconfig_clk),
            .reset                     (~lspcs_rx_ready_rst_n_async),
            .d                         (i_sl_rx_block_lock),
            .q                         (sl_rx_block_lock_async)
        );
        
        assign o_sl_tx_lspcs_ready = (i_sl_cpri_rate_sel[3] == 1'b0) ? sl_ehip_ready_async : 1'b0;
        assign o_sl_rx_lspcs_ready = (i_sl_cpri_rate_sel[3] == 1'b0) ? sl_rx_block_lock_async : 1'b0;
    end
    else begin
        assign o_sl_tx_lspcs_ready = 'd0;
        assign o_sl_rx_lspcs_ready = 'd0;
    end
    endgenerate

    // ------------------------------------------------------------------------------------
    // Deterministic Latency
    alt_xcvr_resync_std #(
        .SYNC_CHAIN_LENGTH (2),  
        .INIT_VALUE        (1)
    ) rx_blocklock_sclk_sync_inst(
        .clk                       (i_sampling_clk),
        .reset                     (o_sl_ehip_reset), // in management clock
        .d                         (~i_sl_rx_block_lock),
        .q                         (i_blklock_n_sclk_sync)
    );

    //alt_xcvr_resync_std #(
        //.SYNC_CHAIN_LENGTH (2),  
        //.INIT_VALUE        (1)
    //) restart_sclksync_inst(
        //.clk                       (i_sampling_clk),
        //.reset                     (o_sl_ehip_reset),
        //.d                         (restart_sync),
        //.q                         (restart_sync_sclk)
    //);    

    alt_xcvr_resync_std #(
        .SYNC_CHAIN_LENGTH(3),    .WIDTH(1),  .INIT_VALUE(1)	
    ) sync_latency_measure_i_init_sync_inst (
        .clk                        (i_clk_pll_div66),
        .reset                      (~i_sl_tx_transfer_ready),
        .d                          (1'b0), //i_sl_tx_pma_ready & i_sl_tx_transfer_ready
        .q                          (latency_measure_i_init_txclk_sync)
    );

    alt_xcvr_resync_std #(
        .SYNC_CHAIN_LENGTH (2),  
        .INIT_VALUE        (1)
    ) rx_blocklock_txclk_sync_inst (
        .clk             (i_clk_pll_div66),
        .reset           (o_sl_ehip_reset),
        .d               (~i_sl_rx_block_lock),
        .q               (i_blklock_n_txclk_sync)
    );

    alt_xcvr_resync_std #(
        .SYNC_CHAIN_LENGTH (2),  
        .INIT_VALUE        (1)
    ) rx_blocklock_rxclk_sync_inst (
        .clk             (i_clk_rec_div66),
        .reset           (o_sl_ehip_reset),
        .d               (~i_sl_rx_block_lock),
        .q               (i_blklock_n_rxclk_sync)
    );

    // ---------------

    alt_xcvr_resync_std #(
        .SYNC_CHAIN_LENGTH(3),    
        .WIDTH(1),  
        .INIT_VALUE(1)	
    ) dl_rst_sclk_sync_inst (
        .clk             (i_sampling_clk),
        .reset           (o_sl_ehip_reset),
        .d               (i_sl_soft_dl_rst),
        .q               (dl_rst_sclk_sync)
    );

    alt_xcvr_resync_std #(
        .SYNC_CHAIN_LENGTH (2),  
        .INIT_VALUE        (1)
    ) dl_rst_txclk_sync_inst (
        .clk             (i_clk_pll_div66),
        .reset           (o_sl_ehip_reset),
        .d               (i_sl_soft_dl_rst),
        .q               (dl_rst_txclk_sync)
    );

    alt_xcvr_resync_std #(
        .SYNC_CHAIN_LENGTH (2),  
        .INIT_VALUE        (1)
    ) dl_rst_rxclk_sync_inst (
        .clk             (i_clk_rec_div66),
        .reset           (o_sl_ehip_reset),
        .d               (i_sl_soft_dl_rst),
        .q               (dl_rst_rxclk_sync)
    );
    
    assign sl_rx_mii_am_valid_mux = (i_sl_cpri_rate_sel[3] == 1'b0) ? lspcs_rx_mii_am_valid : sl_rx_mii_am_valid;
    assign rx_block_lock_n_sync_mux = (i_sl_cpri_rate_sel[3] == 1'b0) ? lspcs_rx_block_lock_n_sync : i_blklock_n_sclk_sync;
    assign rx_block_lock_n_txclk_sync_mux = (i_sl_cpri_rate_sel[3] == 1'b0) ? lspcs_rx_block_lock_n_txclk_sync : i_blklock_n_txclk_sync;
    assign rx_block_lock_n_rxclk_sync_mux = (i_sl_cpri_rate_sel[3] == 1'b0) ? ~lspcs_rx_block_lock : i_blklock_n_rxclk_sync;

    always @ (posedge i_clk_pll_div66) begin
        rx_block_lock_n_txclk_sync_mux_reg <= rx_block_lock_n_txclk_sync_mux;
    end

    always @ (posedge i_clk_rec_div66) begin
        rx_block_lock_n_rxclk_sync_mux_reg <= rx_block_lock_n_rxclk_sync_mux;
    end    
    
    alt_cpriphy_c3_latency_measure #(
        .DL_SYNC_PULSE_PERIOD(319), //do not modify
        .DL_ASYNC_PULSE_WIDTH (11), // do not modify
        .DL_TOT_MEASURE_CNT (DL_TOT_MEASURE_CNT),
        .COUNT_WIDTH(COUNT_WIDTH)
    ) latency_measure_inst (
        //.i_reset(rx_block_lock_n_sync_mux | restart_sync_sclk),
        .i_reset_sync_sclk(rx_block_lock_n_sync_mux | dl_rst_sclk_sync),
        .i_reset_sync_txclk(rx_block_lock_n_txclk_sync_mux_reg | dl_rst_txclk_sync), 
        .i_reset_sync_rxclk(rx_block_lock_n_rxclk_sync_mux_reg | dl_rst_rxclk_sync),
        .i_reset_async(o_sl_ehip_reset | i_sl_soft_dl_rst),
        //.i_init_sync(i_sl_tx_pma_ready & i_sl_tx_transfer_ready),
        .i_init_sync(~latency_measure_i_init_txclk_sync),
        .i_tx_parallel_clk(i_clk_pll_div66),
        .i_rx_parallel_clk(i_clk_rec_div66),
        .i_smpl_clk(i_sampling_clk), 
        .i_datarate_config(i_sl_cpri_rate_sel),
        .i_cpri_fec_en(i_sl_cpri_fec_en),
        .i_rx_sync_pulse(sl_rx_mii_am_valid_mux),
        .i_rx_async_pulse(i_sl_rx_dl_async_pulse), 
        .i_tx_async_pulse(i_sl_tx_dl_async_pulse),
        .o_tx_sync_pulse(sl_tx_mii_am),
        .o_async_pulse(o_sl_latency_sclk),
        .o_tx_rx_measure_sel(o_sl_tx_rx_dl_measure_sel),
        .o_tx_delay_sclk(o_sl_tx_delay_sclk),
        .o_rx_delay_sclk(o_sl_rx_delay_sclk),
        .o_measure_valid(o_sl_measure_valid),
        .tx_sync_count(o_sl_tx_sync_count),
        .tx_async_count(o_sl_tx_async_count),
        .rx_sync_count(o_sl_rx_sync_count),
        .rx_async_count(o_sl_rx_async_count)
    );

endmodule // alt_cpriphy_c3_soft
