# (C) 2001-2022 Intel Corporation. All rights reserved.
# Your use of Intel Corporation's design tools, logic functions and other 
# software and tools, and its AMPP partner logic functions, and any output 
# files from any of the foregoing (including device programming or simulation 
# files), and any associated documentation or information are expressly subject 
# to the terms and conditions of the Intel Program License Subscription 
# Agreement, Intel FPGA IP License Agreement, or other applicable 
# license agreement, including, without limitation, that your use is for the 
# sole purpose of programming logic devices manufactured by Intel and sold by 
# Intel or its authorized distributors.  Please refer to the applicable 
# agreement for further details.



proc alt_cpriphy_c3_constraint_net_delay {from_reg to_reg max_net_delay {check_exist 0}} {
    
    # Check for instances
    set inst [get_registers -nowarn ${to_reg}]
    
    # Check number of instances
    set inst_num [llength [query_collection -report -all $inst]]
    if {$inst_num > 0} {
        # Uncomment line below for debug purpose
        #puts "${inst_num} ${to_reg} instance(s) found"
    } else {
        # Uncomment line below for debug purpose
        #puts "No ${to_reg} instance found"
    }
    
    if {($check_exist == 0) || ($inst_num > 0)} {
        if { [string equal "quartus_sta" $::TimeQuestInfo(nameofexecutable)] } {
            set_max_delay -from [get_registers ${from_reg}] -to [get_registers ${to_reg}] 200ns
            set_min_delay -from [get_registers ${from_reg}] -to [get_registers ${to_reg}] -200ns
        } else {
            set_net_delay -from [get_pins -compatibility_mode ${from_reg}|q] -to [get_registers ${to_reg}] -max $max_net_delay
            
            # Relax the fitter effort
            set_max_delay -from [get_registers ${from_reg}] -to [get_registers ${to_reg}] 200ns
            set_min_delay -from [get_registers ${from_reg}] -to [get_registers ${to_reg}] -200ns
        }
    }
}

proc alt_cpriphy_c3_constraint_ptr {from_path from_reg to_path to_reg max_skew max_net_delay} {
    
    if { [string equal "quartus_sta" $::TimeQuestInfo(nameofexecutable)] } {
        # Check for instances
        set inst [get_registers -nowarn *${from_path}|${from_reg}*]
        
        # Check number of instances
        set inst_num [llength [query_collection -report -all $inst]]
        if {$inst_num > 0} {
            # Uncomment line below for debug purpose
            #puts "${inst_num} ${from_path}|${from_reg} instance(s) found"
        } else {
            # Uncomment line below for debug purpose
            #puts "No ${from_path}|${from_reg} instance found"
        }
        
        set inst_list [query_collection -list -all $inst]
        foreach each_inst $inst_list {
            # Get the path to instance
            regexp "(.*${from_path})(.*|)(${from_reg})" $each_inst reg_path inst_path inst_name reg_name
            
            set_max_skew -from [get_registers ${inst_path}${inst_name}${from_reg}[*]] -to [get_registers *${to_path}|${to_reg}*] $max_skew
            
            set_max_delay -from [get_registers ${inst_path}${inst_name}${from_reg}[*]] -to [get_registers *${to_path}|${to_reg}*] 200ns
            set_min_delay -from [get_registers ${inst_path}${inst_name}${from_reg}[*]] -to [get_registers *${to_path}|${to_reg}*] -200ns
        }
    } else {
        set_net_delay -from [get_pins -compatibility_mode *${from_path}|${from_reg}*|q] -to [get_registers *${to_path}|${to_reg}*] -max $max_net_delay
        
        # Relax the fitter effort
        set_max_delay -from [get_registers *${from_path}|${from_reg}*] -to [get_registers *${to_path}|${to_reg}*] 200ns
        set_min_delay -from [get_registers *${from_path}|${from_reg}*] -to [get_registers *${to_path}|${to_reg}*] -200ns
    }
}

## get current IP instance 
set ip_inst_name [get_current_instance]
#post_message -type info "Debug IP SDC: ip_inst_name $ip_inst_name"

alt_cpriphy_c3_constraint_net_delay * $ip_inst_name|*|resync_chains[*].synchronizer_nocut|* \
                                   2ns 1


alt_cpriphy_c3_constraint_net_delay $ip_inst_name|cpri_rate_sel[*] * \
                                   2ns 1

alt_cpriphy_c3_constraint_net_delay $ip_inst_name|cpri_fec_en[*] * \
                                   2ns 1

alt_cpriphy_c3_constraint_net_delay * $ip_inst_name|alt_cpriphy_c3_hard_inst|sl_rsfec_pld_ready \
                                   2ns 1

alt_cpriphy_c3_constraint_net_delay $ip_inst_name|SL_SOFT.SL_SOFT_I[*].sl_soft|SL_RST_CTRL.i_sl_reset_controller|ehip_rst_seqinst|ehip_tx_reset_out_stage \
                                   $ip_inst_name|SL_SOFT.SL_SOFT_I[*].sl_soft|SL_RST_CTRL.tx_reset_sync_tx_clk|* \
                                   2ns 1

alt_cpriphy_c3_constraint_net_delay $ip_inst_name|SL_SOFT.SL_SOFT_I[*].sl_soft|SL_RST_CTRL.i_sl_reset_controller|ehip_rst_seqinst|ehip_rx_reset_out_stage \
                                   $ip_inst_name|SL_SOFT.SL_SOFT_I[*].sl_soft|SL_RST_CTRL.rx_reset_sync_rx_clk|* \
                                   2ns 1


# efifo (common)
alt_cpriphy_c3_constraint_ptr alt_cpriphy_c3_efifo_async_fifo_0|dcfifo_componenet|auto_generated  delayed_wrptr_g \
                              alt_cpriphy_c3_efifo_async_fifo_0|dcfifo_componenet|auto_generated  rdemp_eq_comp_*_aeb \
                              2ns 2ns

alt_cpriphy_c3_constraint_ptr alt_cpriphy_c3_efifo_async_fifo_0|dcfifo_componenet|auto_generated  rdptr_g\
                              alt_cpriphy_c3_efifo_async_fifo_0|dcfifo_componenet|auto_generated  wrfull_eq_comp_*_mux_reg \
                              2ns 2ns

alt_cpriphy_c3_constraint_ptr alt_cpriphy_c3_efifo_async_fifo_0|dcfifo_componenet|auto_generated  delayed_wrptr_g \
                              alt_cpriphy_c3_efifo_async_fifo_0|dcfifo_componenet|auto_generated|rs_dgwp|dffpipe*  dffe* \
                              2ns 2ns

alt_cpriphy_c3_constraint_ptr alt_cpriphy_c3_efifo_async_fifo_0|dcfifo_componenet|auto_generated  rdptr_g \
                              alt_cpriphy_c3_efifo_async_fifo_0|dcfifo_componenet|auto_generated|ws_dgrp|dffpipe*  dffe* \
                              2ns 2ns

alt_cpriphy_c3_constraint_net_delay * $ip_inst_name|SL_SOFT_I[*].sl_soft|*|altera_std_synchronizer_nocut_por|din_s1 \
                                    2ns 1

# False path to dummy/unused registers
set_false_path -to [get_registers $ip_inst_name|SL_SOFT_I[*].sl_soft|*dummy_reg*[*]]

# efifo (64b/66b)
alt_cpriphy_c3_constraint_net_delay $ip_inst_name|SL_SOFT_I[*].sl_soft|EXT_AIB_CLK.pll_div66_rst_sync_inst|resync_chains[0].synchronizer_nocut|dreg[0] \
                                    $ip_inst_name|SL_SOFT_I[*].sl_soft|EXT_AIB_CLK.tx_efifo|alt_cpriphy_c3_efifo_async_fifo_0|dcfifo_componenet|auto_generated|rdaclr|*[0] \
                                    2ns 1

alt_cpriphy_c3_constraint_net_delay $ip_inst_name|SL_SOFT_I[*].sl_soft|EXT_AIB_CLK.rec_div66_rst_sync_inst|resync_chains[0].synchronizer_nocut|dreg[0] \
                                    $ip_inst_name|SL_SOFT_I[*].sl_soft|EXT_AIB_CLK.rx_efifo|alt_cpriphy_c3_efifo_async_fifo_0|dcfifo_componenet|auto_generated|wraclr|*[0] \
                                    2ns 1

alt_cpriphy_c3_constraint_net_delay $ip_inst_name|SL_SOFT_I[*].sl_soft|EXT_AIB_CLK.i_clk_rx_rst_sync_inst|resync_chains[0].synchronizer_nocut|dreg[0] \
                                    $ip_inst_name|SL_SOFT_I[*].sl_soft|EXT_AIB_CLK.rx_efifo|alt_cpriphy_c3_efifo_async_fifo_0|dcfifo_componenet|auto_generated|rdaclr|*[0] \
                                    2ns 1

alt_cpriphy_c3_constraint_net_delay $ip_inst_name|SL_SOFT_I[*].sl_soft|EXT_AIB_CLK.i_clk_tx_rst_sync_inst|resync_chains[0].synchronizer_nocut|dreg[0] \
                                    $ip_inst_name|SL_SOFT_I[*].sl_soft|EXT_AIB_CLK.tx_efifo|alt_cpriphy_c3_efifo_async_fifo_0|dcfifo_componenet|auto_generated|wraclr|*[0] \
                                    2ns 1


# Deterministic Latency
alt_cpriphy_c3_constraint_net_delay $ip_inst_name|SL_SOFT_I[*].sl_soft|latency_measure_inst|calc_delay_0|calc_delay_valid \
                                    $ip_inst_name|gen_avmm_rcfg[*].soft_csr_sl_inst|readdata[*] \
                                    2ns 1

alt_cpriphy_c3_constraint_net_delay $ip_inst_name|SL_SOFT_I[*].sl_soft|latency_measure_inst|calc_delay_0|*_delay_sclk[*] \
                                    $ip_inst_name|gen_avmm_rcfg[*].soft_csr_sl_inst|readdata[*] \
                                    2ns 1



