// (C) 2001-2022 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


`timescale 1ps/1ps

module ex_10G_alt_cpriphy_c3_1961_7djwtgq
import alt_cpriphy_c3_package::*;
  #(
    parameter sl_num_lanes = 4,
    parameter max_refclk = 5,
    parameter max_ch = 4,
    parameter max_xcvr_ch = 4,
    parameter max_clkout = 4,
    parameter max_clkout_ch = 4,
    parameter enable_rsfec_rst_ports = 0,
    parameter sim_mode                              = "disable" , //                    {"disable" "enable"}
    parameter ENABLE_RSFEC            = 0,
    parameter enable_external_aib_clocking = 0,
    parameter cpri_include_alternate_ports = 0,
    parameter cpri_rate_sl_0 = "10G",
    parameter ehip_mode_sl_0        = "ehip_disable" //            {"ehip_lb_txmac_rxmac" "ehip_lb_txmac_rxmac_ptp" "ehip_lb_txpcs_rxaib" "ehip_lb_txpcs_rxmac" "ehip_lb_txpcs_rxpcs" "ehip_lb_tx_rx" "ehip_lb_tx_rx_ptp" "ehip_mac" "ehip_mac_no_fec" "ehip_mac_ptp" "ehip_mac_ptp_no_fec" "ehip_otn" "ehip_otn_no_fec" "ehip_pcs" "ehip_pcs66" "ehip_pcs66_no_fec" "ehip_pcs_no_fec" "ehip_standby"}
    )
   (


    // Clock and Resets
    input wire [max_refclk-1:0]              i_clk_ref, //Native PHY IP reference clock

    input  wire [sl_num_lanes-1:0]           i_sl_clk_tx, //TX Avalon-ST interface clock, in synchronous mode connect o_tx_clkout to this
    input  wire [sl_num_lanes-1:0]           i_sl_clk_rx, //TX Avalon-ST interface clock, in synchronous mode connect o_tx_clkout to this
    input                                    i_aib_clk,       // External clock input for AIB
    input                                    i_aib_2x_clk,    // External clock input for AIB at 2x frequency of i_aib_clk
    output wire [max_clkout-1:0]             o_tx_clkout, // this is /64 clock (402.83 MHz) EHIP system clock
    output wire [max_clkout-1:0]             o_tx_clkout2, // this is /66 clock (390.625 MHz) EHIP system clock times 64/66
    output wire [max_clkout-1:0]             o_rx_clkout, // this is /64 clock (402.83 MHz), recovered clock
    output wire [max_clkout-1:0]             o_rx_clkout2, // this is /66 clock (390.625 MHz), recovered clock
    input  wire                              i_sampling_clk,
    input  wire [sl_num_lanes-1:0]           i_sl_csr_rst_n, // Resets the entire Ethernet core. Active low.
    input  wire [sl_num_lanes-1:0]           i_sl_tx_rst_n, //Resets the TX digital path (Ethernet MAC and Ethernet PCS)
    input  wire [sl_num_lanes-1:0]           i_sl_rx_rst_n, // Resets the RX digital path (Ethernet MAC, PCS)
    input  wire                              i_rsfec_rst_n, //Resets the RSFEC block
    input  wire                              i_rsfec_tx_rst_n, //Resets the TX digital path (RSFEC)
    input  wire                              i_rsfec_rx_rst_n, // Resets the RX digital path (RSFEC)

    // High speed serial Pins
    input  wire [max_xcvr_ch-1:0]            i_rx_serial,
    output wire [max_xcvr_ch-1:0]            o_tx_serial,
    input  wire [max_xcvr_ch-1:0]            i_rx_serial_n,
    output wire [max_xcvr_ch-1:0]            o_tx_serial_n,


   // Status
    output wire [max_ch-1:0]                 o_cdr_lock, //Signal to indicate that the PMA has locked to data.
    output wire [max_ch-1:0]                 o_tx_pll_locked, //Signal to indicate that the TX PLL has locked.

    output wire [sl_num_lanes-1:0]           o_sl_tx_lanes_stable,       // Asserted when TX MAC is ready to send data
    output wire [sl_num_lanes-1:0]           o_sl_rx_pcs_ready,          // Asserted when RX PCS is ready to receive data
    output wire [sl_num_lanes-1:0]           o_sl_rx_block_lock,         // Asserted when 66b block alignment is finished on all PCS lanes
    output wire [sl_num_lanes-1:0]           o_sl_rx_hi_ber,             // Indicates RX PCS Hi BER state in the selected channel
    output wire [sl_num_lanes-1:0]           o_sl_ehip_ready,            //Signal to indicate that EHIP CSRs are ready.

    // MII Interface
    input  wire [64*sl_num_lanes-1:0]        i_sl_tx_mii_d, // TX MII data
    input  wire [8*sl_num_lanes-1:0]         i_sl_tx_mii_c, // Tx MII control
    input  wire [sl_num_lanes-1:0]           i_sl_tx_mii_valid, // Indicates valid data ports, must follow o_sl_tx_mii_ready
    // input  wire [sl_num_lanes-1:0]           i_sl_tx_mii_am, // Alignment marker insertion bit
    output wire [sl_num_lanes-1:0]           o_sl_tx_mii_ready, // indicates PCS is ready to receive new data
    output wire [64*sl_num_lanes-1:0]        o_sl_rx_mii_d, // RX MII data
    output wire [8*sl_num_lanes-1:0]         o_sl_rx_mii_c, // RX MII control
    output wire [sl_num_lanes-1:0]           o_sl_rx_mii_valid, // Indicates valid datain MII ports
    // output wire [sl_num_lanes-1:0]           o_sl_rx_mii_am_valid, // Indicates valid datain MII ports


    // PMA Direct 8b10b Interface
    output wire [sl_num_lanes-1:0]           o_sl_tx_ready,
    output wire [sl_num_lanes-1:0]           o_sl_rx_ready,
    input  wire [2*sl_num_lanes-1:0]         i_sl_tx_c,
    input  wire [16*sl_num_lanes-1:0]        i_sl_tx_d,
    output wire [2*sl_num_lanes-1:0]         o_sl_rx_c,
    output wire [16*sl_num_lanes-1:0]        o_sl_rx_d,
    output wire [1*sl_num_lanes-1:0]         o_sl_rx_patterndetect,
    output wire [2*sl_num_lanes-1:0]         o_sl_rx_disperr,
    output wire [2*sl_num_lanes-1:0]         o_sl_rx_errdetect,

    // RX AIB deskew debug
    input  wire [sl_num_lanes-1:0]           i_sl_aib_rxfifo_latency_adj_ena,

    // Custom Rate Interface
    input  wire [sl_num_lanes-1:0]           i_sl_custom_cadence,

    // XCVR FIFO Interface
    output wire [sl_num_lanes-1:0]           o_sl_txfifo_pfull,
    output wire [sl_num_lanes-1:0]           o_sl_txfifo_pempty,
    output wire [sl_num_lanes-1:0]           o_sl_txfifo_overflow,
    output wire [sl_num_lanes-1:0]           o_sl_txfifo_underflow,

    // EFIFO Interface
    output wire [sl_num_lanes-1:0]           o_sl_mii_txfifo_pfull,
    output wire [sl_num_lanes-1:0]           o_sl_mii_txfifo_pempty,
    output wire [sl_num_lanes-1:0]           o_sl_mii_txfifo_overflow,
    output wire [sl_num_lanes-1:0]           o_sl_mii_txfifo_underflow,
    
    // // Deterministic Latency Interface
    // output wire [sl_num_lanes-1:0]           o_sl_tx_dl_async_pulse,
    // output wire [sl_num_lanes-1:0]           o_sl_rx_dl_async_pulse,
    // input  wire [sl_num_lanes-1:0]           i_sl_latency_sclk,
    // input  wire [sl_num_lanes-1:0]           i_sl_tx_dl_measure_sel,
    // input  wire [sl_num_lanes-1:0]           i_sl_rx_dl_measure_sel,

    //Reconfig interface
    input wire                               i_reconfig_clk,
    input wire                               i_reconfig_reset,

    input  wire [19*sl_num_lanes-1:0]        i_sl_cpri_reconfig_addr,
    input  wire [sl_num_lanes-1:0]           i_sl_cpri_reconfig_read,
    input  wire [sl_num_lanes-1:0]           i_sl_cpri_reconfig_write,
    input  wire [32*sl_num_lanes-1:0]        i_sl_cpri_reconfig_writedata,
    output wire [32*sl_num_lanes-1:0]        o_sl_cpri_reconfig_readdata,
    output wire [sl_num_lanes-1:0]           o_sl_cpri_reconfig_readdata_valid,
    output wire [sl_num_lanes-1:0]           o_sl_cpri_reconfig_waitrequest,
    input  wire [sl_num_lanes-1:0]           i_sl_stats_snapshot, // Record snapshot of current state of statistics registers.

    //XCVR Reconfig Interface, seperate one for each lane
    input wire                               i_rsfec_reconfig_write,
    input wire                               i_rsfec_reconfig_read,
    input wire [11-1:0]                      i_rsfec_reconfig_addr,
    input wire [8-1:0]                       i_rsfec_reconfig_writedata,
    output wire [8-1:0]                      o_rsfec_reconfig_readdata,
    output wire                              o_rsfec_reconfig_readdata_valid,
    output wire                              o_rsfec_reconfig_waitrequest,


    // Seperate XCVR reconfig Interfaces for each xcvr lane
    input wire [19*max_xcvr_ch-1:0]          i_xcvr_reconfig_address,
    input wire [max_xcvr_ch-1:0]             i_xcvr_reconfig_read,
    input wire [max_xcvr_ch-1:0]             i_xcvr_reconfig_write,
    input wire [8*max_xcvr_ch-1:0]           i_xcvr_reconfig_writedata,
    output wire [8*max_xcvr_ch-1:0]          o_xcvr_reconfig_readdata,
    output wire [max_xcvr_ch-1:0]            o_xcvr_reconfig_waitrequest    //AVMM stalling signal. The read/write cycle is only complete when this signal goes low

   );


   // For synthesis disable sim_mode and pass sim_mode parameter only for simulation
`ifdef ALTERA_RESERVED_QIS
   localparam SIM_EMULATE = 1'b0;
`else
   localparam SIM_EMULATE = (sim_mode == "enable") ? 1'b1:1'b0;
`endif

    localparam ENABLE_RSFEC_MODE = ((ehip_mode_sl_0 == "ehip_mac") ||
                                    (ehip_mode_sl_0 == "ehip_mac_ptp") ||
                                    (ehip_mode_sl_0 == "ehip_pcs") ||
                                    (ehip_mode_sl_0 == "ehip_pcs66") ||
                                    (ehip_mode_sl_0 == "ehip_otn")) ? 1'b1: 1'b0;
    localparam LOCATION_SEL                     = 0;

    localparam SL_DP_SEL_RESET_VALUE            = (ehip_mode_sl_0 == "ehip_mac_no_fec")     ? 3'd0 :
                                                  (ehip_mode_sl_0 == "ehip_mac")            ? 3'd0 :
                                                  (ehip_mode_sl_0 == "ehip_mac_ptp_no_fec") ? 3'd1 :
                                                  (ehip_mode_sl_0 == "ehip_mac_ptp")        ? 3'd1 :
                                                  (ehip_mode_sl_0 == "ehip_pcs_no_fec")     ? 3'd2 :
                                                  (ehip_mode_sl_0 == "ehip_pcs")            ? 3'd2 :
                                                  (ehip_mode_sl_0 == "ehip_pcs66_no_fec")   ? 3'd4 :
                                                  (ehip_mode_sl_0 == "ehip_pcs66")          ? 3'd4 :
                                                  (ehip_mode_sl_0 == "ehip_otn_no_fec")     ? 3'd3 :
                                                  (ehip_mode_sl_0 == "ehip_otn")            ? 3'd3 :
                                                                                              3'd0;

    localparam CPRI_RATE_SEL_RESET_VALUE = (cpri_rate_sl_0 == "0_6G") ? CPRI_RATE_0_6G :
                                           (cpri_rate_sl_0 == "1_2G") ? CPRI_RATE_1_2G :
                                           (cpri_rate_sl_0 == "2_4G") ? CPRI_RATE_2_4G :
                                           (cpri_rate_sl_0 == "3G")   ? CPRI_RATE_3G :
                                           (cpri_rate_sl_0 == "4_9G") ? CPRI_RATE_4_9G :
                                           (cpri_rate_sl_0 == "6G")   ? CPRI_RATE_6G :
                                           (cpri_rate_sl_0 == "9_8G") ? CPRI_RATE_9_8G :
                                           (cpri_rate_sl_0 == "8G")   ? CPRI_RATE_8G :
                                           (cpri_rate_sl_0 == "10G")  ? CPRI_RATE_10G :
                                           (cpri_rate_sl_0 == "12G")  ? CPRI_RATE_12G :
                                           (cpri_rate_sl_0 == "24G")  ? CPRI_RATE_24G :
                                                                        CPRI_RATE_10G;

    wire [sl_num_lanes-1:0]           sl_rx_is_lockedtodata;
    wire [sl_num_lanes-1:0]           sl_rx_transfer_ready;
    wire [sl_num_lanes-1:0]           sl_tx_transfer_ready;
    wire [sl_num_lanes-1:0]           o_sl_rx_pma_ready;
    wire [sl_num_lanes-1:0]           o_sl_tx_pma_ready;
    wire [sl_num_lanes-1:0]           sl_rx_fifo_ready;
    wire [sl_num_lanes-1:0]           sl_tx_fifo_ready;
    wire [sl_num_lanes-1:0]           sl_rx_nphy_ready;
    wire [sl_num_lanes-1:0]           sl_tx_nphy_ready;

    wire [sl_num_lanes-1:0]           sl_tx_aib_reset;
    wire [sl_num_lanes-1:0]           sl_rx_aib_reset;
    wire [sl_num_lanes-1:0]           sl_tx_pcs_reset;
    wire [sl_num_lanes-1:0]           sl_rx_pcs_reset;
    // wire [sl_num_lanes-1:0]           sl_pmad_reset;
    wire [sl_num_lanes-1:0]           sl_tx_reset;
    wire [sl_num_lanes-1:0]           sl_rx_reset;
    wire                              reconfig_reset_sync;

    wire                              ehip_rst;
    wire                              ehip_tx_rst;
    wire                              ehip_rx_rst;
    wire                              ehip_pld_ready;
    wire                              ehip_signal_ok;
    wire                              ehip_signal_ok_p;

    wire [sl_num_lanes-1:0]           sl_ehip_reset_stat;
    wire [sl_num_lanes-1:0]           sl_ehip_tx_reset_stat;
    wire [sl_num_lanes-1:0]           sl_ehip_rx_reset_stat;
    wire [sl_num_lanes-1:0]           sl_ehip_reset;
    wire [sl_num_lanes-1:0]           sl_ehip_tx_reset;
    wire [sl_num_lanes-1:0]           sl_ehip_rx_reset;
    wire [sl_num_lanes-1:0]           sl_ehip_pld_ready;
    wire [sl_num_lanes-1:0]           sl_ehip_signal_ok;

    wire                              sl_rsfec_reset;
    wire                              sl_rsfec_tx_reset;
    wire                              sl_rsfec_rx_reset;

    wire [sl_num_lanes-1:0]           sl_soft_rsfec_reset;
    wire [sl_num_lanes-1:0]           sl_soft_rsfec_tx_reset;
    wire [sl_num_lanes-1:0]           sl_soft_rsfec_rx_reset;

    wire [sl_num_lanes-1:0]           sl_tx_dll_lock;


    wire   [sl_num_lanes-1:0]         sl_soft_csr_rst;
    wire   [sl_num_lanes-1:0]         sl_soft_tx_rst;
    wire   [sl_num_lanes-1:0]         sl_soft_rx_rst;

    wire   [sl_num_lanes-1:0]         sl_soft_dl_rst;

    wire [sl_num_lanes-1:0]           sl_rx_pcs_fully_aligned;


    wire [max_clkout_ch-1:0]          tx_clkout_div64;
    wire [max_clkout_ch-1:0]          rx_clkout_div64;
    wire [max_clkout_ch-1:0]          tx_clkout_div66;
    wire [max_clkout_ch-1:0]          rx_clkout_div66;

    // ELANE: 2D array wire declaration
    wire [sl_num_lanes-1:0][63:0]     i_sl_tx_data_2d;
    wire [sl_num_lanes-1:0][2:0]      i_sl_tx_empty_2d;
    wire [sl_num_lanes-1:0][7:0]      i_sl_tx_pfc_2d;
    wire [sl_num_lanes-1:0][63:0]     o_sl_rx_data_2d;
    wire [sl_num_lanes-1:0][2:0]      o_sl_rx_empty_2d;
    wire [sl_num_lanes-1:0][5:0]      o_sl_rx_error_2d;
    wire [sl_num_lanes-1:0][39:0]     o_sl_rxstatus_data_2d;
    wire [sl_num_lanes-1:0][7:0]      o_sl_rx_pfc_2d;
    wire [sl_num_lanes-1:0][63:0]     i_sl_tx_mii_d_2d;
    wire [sl_num_lanes-1:0][7:0]      i_sl_tx_mii_c_2d;
    wire [sl_num_lanes-1:0][63:0]     o_sl_rx_mii_d_2d;
    wire [sl_num_lanes-1:0][7:0]      o_sl_rx_mii_c_2d;
    wire [sl_num_lanes-1:0][65:0]     i_sl_tx_pcs66_d_2d;
    wire [sl_num_lanes-1:0][65:0]     o_sl_rx_pcs66_d_2d;
    wire [sl_num_lanes-1:0][18:0]     i_sl_cpri_reconfig_addr_2d;
    wire [sl_num_lanes-1:0][31:0]     i_sl_cpri_reconfig_writedata_2d;
    reg  [sl_num_lanes-1:0][31:0]     o_sl_cpri_reconfig_readdata_2d;
    wire [sl_num_lanes-1:0][1:0]      i_sl_tx_ctrlenable_2d;
    wire [sl_num_lanes-1:0][15:0]     i_sl_tx_datain_2d;
    wire [sl_num_lanes-1:0][1:0]      o_sl_rx_ctrldetect_2d;
    wire [sl_num_lanes-1:0][15:0]     o_sl_rx_dataout_2d;
    wire [sl_num_lanes-1:0][1:0]      o_sl_rx_disperr_2d;
    wire [sl_num_lanes-1:0][1:0]      o_sl_rx_errdetect_2d;
    wire [sl_num_lanes-1:0]           o_sl_rx_patterndetect_2d;
    
    wire [sl_num_lanes-1:0][63:0]     sl_tx_mii_d;
    wire [sl_num_lanes-1:0][7:0]      sl_tx_mii_c;
    wire [sl_num_lanes-1:0]           sl_tx_mii_valid;
    wire [sl_num_lanes-1:0]           sl_tx_mii_am;
    wire [sl_num_lanes-1:0]           sl_tx_mii_ready;
    wire [sl_num_lanes-1:0][63:0]     sl_rx_mii_d;
    wire [sl_num_lanes-1:0][7:0]      sl_rx_mii_c;
    wire [sl_num_lanes-1:0]           sl_rx_mii_valid;
    wire [sl_num_lanes-1:0]           sl_rx_mii_am_valid;

    reg  [sl_num_lanes-1:0][2:0]      sl_dp_sel;
    reg  [sl_num_lanes-1:0]           sl_rsfec_mode;
    reg  [sl_num_lanes-1:0][3:0]      cpri_rate_sel;
    reg  [sl_num_lanes-1:0]           cpri_fec_en;
    reg  [sl_num_lanes-1:0]           sl_custom_cadence;

    // Deterministic Latency Interface
    wire [sl_num_lanes-1:0]           sl_tx_dl_async_pulse;
    wire [sl_num_lanes-1:0]           sl_rx_dl_async_pulse;
    wire [sl_num_lanes-1:0]           sl_latency_sclk;
    wire [sl_num_lanes-1:0]           sl_tx_rx_dl_measure_sel;
    wire [sl_num_lanes-1:0][20:0]     sl_tx_delay_sclk;
    wire [sl_num_lanes-1:0][20:0]     sl_rx_delay_sclk;
    wire [sl_num_lanes-1:0]           sl_measure_valid;
    wire [sl_num_lanes-1:0]           sl_measure_valid_sync;

    // between pma_direct phy and soft pcs
    wire [sl_num_lanes-1:0][79:0]    int_sl_tx_pma;
    wire [sl_num_lanes-1:0][79:0]    int_sl_rx_pma;
    wire [sl_num_lanes-1:0]          int_sl_aib_txfifo_empty;
    wire [sl_num_lanes-1:0]          int_sl_aib_txfifo_full;
    wire [sl_num_lanes-1:0]          int_sl_aib_txfifo_pempty;
    wire [sl_num_lanes-1:0]          int_sl_aib_txfifo_pfull;
    wire [sl_num_lanes-1:0]          int_sl_aib_rxfifo_empty;
    wire [sl_num_lanes-1:0]          int_sl_aib_rxfifo_full;
    wire [sl_num_lanes-1:0]          int_sl_aib_rxfifo_pempty;
    wire [sl_num_lanes-1:0]          int_sl_aib_rxfifo_pfull;
    wire [sl_num_lanes-1:0]          int_sl_aib_rxfifo_rd_en;
   
    wire [max_refclk-1:0]         i_clk_ref_m; //Native PHY IP reference clock
    
    wire [sl_num_lanes-1:0][4:0]     sl_rx_bitslipboundary_sel;
    wire [sl_num_lanes-1:0][4:0]     sl_rx_bitslipboundary_sel_sync;

    assign i_clk_ref_m = i_clk_ref;

    assign sl_custom_cadence = (enable_external_aib_clocking) ? sl_tx_mii_valid : i_sl_custom_cadence;

    assign int_sl_aib_rxfifo_rd_en = {sl_num_lanes{1'b1}};
    
    // ELANE: 1D to 2D array remapping
    genvar elane_remap_i;
    generate
        for (elane_remap_i=0; elane_remap_i<sl_num_lanes; elane_remap_i=elane_remap_i+1) begin : ELANE_REMAP
            // input ports
            assign i_sl_tx_mii_d_2d               [elane_remap_i]   = i_sl_tx_mii_d               [64*elane_remap_i+63:64*elane_remap_i];
            assign i_sl_tx_mii_c_2d               [elane_remap_i]   = i_sl_tx_mii_c               [8*elane_remap_i+7:8*elane_remap_i];
            assign i_sl_cpri_reconfig_addr_2d     [elane_remap_i]   = i_sl_cpri_reconfig_addr     [19*elane_remap_i+18:19*elane_remap_i];
            assign i_sl_cpri_reconfig_writedata_2d[elane_remap_i]   = i_sl_cpri_reconfig_writedata[32*elane_remap_i+31:32*elane_remap_i];
            assign i_sl_tx_datain_2d              [elane_remap_i]   = i_sl_tx_d                   [16*elane_remap_i+15:16*elane_remap_i];
            assign i_sl_tx_ctrlenable_2d          [elane_remap_i]   = i_sl_tx_c                   [2*elane_remap_i+1:2*elane_remap_i];

    
            // output ports
            assign o_sl_rx_mii_d              [64*elane_remap_i+63:64*elane_remap_i]    = o_sl_rx_mii_d_2d              [elane_remap_i];
            assign o_sl_rx_mii_c              [8*elane_remap_i+7:8*elane_remap_i]       = o_sl_rx_mii_c_2d              [elane_remap_i];
            assign o_sl_cpri_reconfig_readdata[32*elane_remap_i+31:32*elane_remap_i]    = o_sl_cpri_reconfig_readdata_2d[elane_remap_i];
            assign o_sl_rx_d                  [16*elane_remap_i+15:16*elane_remap_i]    = o_sl_rx_dataout_2d            [elane_remap_i];
            assign o_sl_rx_c                  [2*elane_remap_i+1:2*elane_remap_i]       = o_sl_rx_ctrldetect_2d         [elane_remap_i];
            assign o_sl_rx_disperr            [2*elane_remap_i+1:2*elane_remap_i]       = o_sl_rx_disperr_2d            [elane_remap_i];
            assign o_sl_rx_errdetect          [2*elane_remap_i+1:2*elane_remap_i]       = o_sl_rx_errdetect_2d          [elane_remap_i];
            assign o_sl_rx_patterndetect      [elane_remap_i]                           = o_sl_rx_patterndetect_2d      [elane_remap_i];
            
        end
    endgenerate

    assign o_cdr_lock      = sl_rx_is_lockedtodata;
    assign o_tx_pll_locked = o_sl_tx_pma_ready;

    // wire int_tx_clk;
    // wire int_rx_clk;

    // assign int_tx_clk = tx_clkout_div64[0];
    // assign int_rx_clk = tx_clkout_div64[0]; // rx and tx sourced from tx-side

    //PLL_DIV64
    assign o_tx_clkout  = tx_clkout_div64[max_clkout-1:0];
    assign o_tx_clkout2 = tx_clkout_div66[max_clkout-1:0];
    assign o_rx_clkout  = rx_clkout_div64[max_clkout-1:0];
    assign o_rx_clkout2 = rx_clkout_div66[max_clkout-1:0];


    genvar          ig;

    //XCVR Reconfig Interface
    wire [8*max_xcvr_ch-1:0]  xcvr_reconfig_readdata;
    wire [max_xcvr_ch-1:0]    xcvr_reconfig_waitrequest;

    //10G,25G ELANE XCVR Reconfig Interface
    wire [sl_num_lanes-1:0] sl_xcvr_reconfig_write;
    wire [sl_num_lanes-1:0] sl_xcvr_reconfig_read;
    wire [19*sl_num_lanes-1:0] sl_xcvr_reconfig_address;
    wire [8*sl_num_lanes-1:0]  sl_xcvr_reconfig_writedata;
    wire [8*sl_num_lanes-1:0]  sl_xcvr_reconfig_readdata;
    wire [sl_num_lanes-1:0]    sl_xcvr_reconfig_waitrequest;

    wire         rsfec_reconfig_write;
    wire         rsfec_reconfig_read;
    wire  [10:0] rsfec_reconfig_address;
    wire  [7:0]  rsfec_reconfig_writedata;
    wire  [7:0]  rsfec_reconfig_readdata;
    wire         rsfec_reconfig_waitrequest;


    // sync reconfig_reset as in module avmm_32_8 and alt_ehipc2_csr.v and alt_ehipc2_adapt_pma it is used as a synchronous reset
    alt_xcvr_resync_std #(
        .SYNC_CHAIN_LENGTH(3),
        .WIDTH(1),
        .INIT_VALUE(0)
    ) sync_reconfig_reset (
        .clk    (i_reconfig_clk),
        .reset  (1'b0),
        .d  (i_reconfig_reset),
        .q  (reconfig_reset_sync)
    );

    wire   [32-1:0] o_rsfec_channel_ssr; //[0]:rsfec_lane_rx_stat.not_locked, [1]: rsfec_lane_rx_stat.hi_ser, [2]: o_pcs_rx_sf(signal failed from rsfec_core)

    generate
        if (ENABLE_RSFEC_MODE) begin : ehip_signal_ok_rsfec
            assign ehip_signal_ok_p= ehip_signal_ok && (!o_rsfec_channel_ssr[2]);
        end else begin  : ehip_signal_ok_norsfec
            assign ehip_signal_ok_p= ehip_signal_ok;
        end
    endgenerate


    //*********************** PAUSE Work Arround *********************
    wire [sl_num_lanes-1:0] sl_ehip_ready;

    //*********************** AVMM Decoding ****************************

    wire  [sl_num_lanes-1:0][31:0]              cpri_reconfig_addr_int;
    wire  [sl_num_lanes-1:0]                    cpri_reconfig_read_int;
    wire  [sl_num_lanes-1:0]                    cpri_reconfig_write_int;
    wire  [sl_num_lanes-1:0][31:0]              cpri_reconfig_writedata_int;
    wire  [sl_num_lanes-1:0][31:0]              cpri_reconfig_readdata_int;
    wire  [sl_num_lanes-1:0]                    cpri_reconfig_readdata_valid_int;
    wire  [sl_num_lanes-1:0]                    cpri_reconfig_waitrequest_int;

    wire  [sl_num_lanes-1:0][1:0]               ext_addr;
    wire  [sl_num_lanes-1:0]                    ext_addr_2;
    wire  [sl_num_lanes-1:0]                    o_xcvr_reconfig_readdata_valid;
    wire  [sl_num_lanes-1:0][31:0]              o_xcvr_reconfig_readdata_int;
    wire  [sl_num_lanes-1:0][31:0]              xcvr_reconfig_writedata_int;
    wire  [sl_num_lanes-1:0]                    o_sl_cpri_reconfig_readdata_valid_int;
    wire  [sl_num_lanes-1:0]                    o_sl_cpri_reconfig_waitrequest_int;
    wire  [sl_num_lanes-1:0][31:0]              o_sl_cpri_reconfig_readdata_int;
    wire  [sl_num_lanes-1:0]                    o_reconfig_readdata_valid_int;


    wire  [sl_num_lanes-1:0][31:0]              sl_khz_ref_phy;
    wire  [sl_num_lanes-1:0][31:0]              sl_khz_rx_clk64;
    wire  [sl_num_lanes-1:0][31:0]              sl_khz_tx_clk64;
    wire  [sl_num_lanes-1:0][15:0]              sl_int_khz_ref_phy;
    wire  [sl_num_lanes-1:0][15:0]              sl_int_khz_rx_clk64;
    wire  [sl_num_lanes-1:0][15:0]              sl_int_khz_tx_clk64;
    wire  [sl_num_lanes-1:0]                    sl_csr_rx_is_lockedtodata_sync;
    wire  [sl_num_lanes-1:0] [3:0]              sl_csr_tx_pll_locked;
    wire  [sl_num_lanes-1:0] [3:0]              sl_csr_rx_is_lockedtodata;

    wire  [sl_num_lanes-1:0][31:0]              sl_soft_csr_phy_read_data;
    wire  [sl_num_lanes-1:0]                    sl_soft_csr_phy_read_valid;
    wire  [sl_num_lanes-1:0]                    sl_soft_csr_phy_waitrequest;
    wire  [sl_num_lanes-1:0]                    sl_soft_csr_phy_sel;
    wire  [sl_num_lanes-1:0][18:0]              cpri_sl_addr;
    reg   [sl_num_lanes-1:0]                    soft_csr_write_req;
    reg   [sl_num_lanes-1:0]                    soft_csr_read_req;
    reg   [sl_num_lanes-1:0]                    soft_csr_read_req_r;
    wire  [sl_num_lanes-1:0]                    cpri_sl_sel;
    reg   [sl_num_lanes-1:0]                    cpri_sl_sel_r;

    genvar i;
    generate
    for (i=0; i<sl_num_lanes; i++) begin: gen_soft_csr_sel
        assign cpri_sl_addr[i]      = i_sl_cpri_reconfig_addr_2d[i];
        assign sl_soft_csr_phy_sel[i] = ((cpri_sl_addr[i] >= 19'h310) && (cpri_sl_addr[i] <= 19'h322)) ||
                                        ((cpri_sl_addr[i] >= 19'h340) && (cpri_sl_addr[i] <= 19'h342)) ||
                                        ((cpri_sl_addr[i] >= 19'hC00) && (cpri_sl_addr[i] <= 19'hC0F));
        assign cpri_sl_sel[i] =   (cpri_sl_addr[i] >= 19'h300) && (cpri_sl_addr[i] <= 19'hFFF) && (cpri_sl_addr[i][18:4] != 15'h00b1) && !sl_soft_csr_phy_sel[i];

        always@(posedge i_reconfig_clk)
        begin
            soft_csr_write_req[i]  <= i_sl_cpri_reconfig_write[i] & sl_soft_csr_phy_sel[i];
            soft_csr_read_req[i]   <= i_sl_cpri_reconfig_read[i] & sl_soft_csr_phy_sel[i];
            soft_csr_read_req_r[i] <= soft_csr_read_req[i] & sl_soft_csr_phy_sel[i];
            cpri_sl_sel_r[i]       <= cpri_sl_sel[i];
        end
    end
    endgenerate

    genvar j;
    generate
    for (j=0; j<sl_num_lanes; j++) begin: gen_avmm_rcfg
        alt_cpriphy_c3_avmm_32to8 #(
            .TIMEOUT(1000)
        ) alt_cpriphy_c3_avmm_32to8_inst (
            .clk                   (i_reconfig_clk),                           //input
            .reset                 (reconfig_reset_sync),                       //input

            .master_read           (cpri_sl_sel[j] && i_sl_cpri_reconfig_read[j]),                //input
            .master_write          (cpri_sl_sel[j] && i_sl_cpri_reconfig_write[j]),               //input
            .master_address        ({13'h0,i_sl_cpri_reconfig_addr_2d[j][18:0]}),  //input       [31:0]
            .master_write_data     (i_sl_cpri_reconfig_writedata_2d[j]),           //input       [31:0]
            .master_byte_enable    (4'b1111),                                  //input       [3:0]
            .master_read_valid     (o_sl_cpri_reconfig_readdata_valid_int[j]),      //output reg
            .master_wait           (o_sl_cpri_reconfig_waitrequest_int[j]),         //output reg
            .master_read_data      (o_sl_cpri_reconfig_readdata_int[j]),            //output reg  [31:0]

            .slave_read            (cpri_reconfig_read_int[j]),                 //output reg
            .slave_write           (cpri_reconfig_write_int[j]),                //output reg
            .slave_address         (cpri_reconfig_addr_int[j]),                 //output reg  [31:0]
            .slave_write_data      (cpri_reconfig_writedata_int[j][7:0]),       //output reg  [ 7:0]
            .slave_read_valid      (cpri_reconfig_readdata_valid_int[j]),       //input
            .slave_wait            (cpri_reconfig_waitrequest_int[j]),          //input
            .slave_read_data       (cpri_reconfig_readdata_int[j][7:0])         //input       [ 7:0]
        );

        // Generated from alt_cpriphy_c3_sl_avmm_rcfg_hw.tcl
        alt_cpriphy_c3_sl_avmm_rcfg avmm_rcfg_inst (
            .clk_clk                                         (i_reconfig_clk),
            .elane_avalon_universal_slave_0_address          ({cpri_reconfig_addr_int[j][16:8],3'b100,cpri_reconfig_addr_int[j][7:0],2'b00}),
            .elane_avalon_universal_slave_0_burstcount       (1'b1),
            .elane_avalon_universal_slave_0_read             (cpri_reconfig_read_int[j]),
            .elane_avalon_universal_slave_0_write            (cpri_reconfig_write_int[j]),
            .elane_avalon_universal_slave_0_waitrequest      (cpri_reconfig_waitrequest_int[j]),
            .elane_avalon_universal_slave_0_readdatavalid    (),
            .elane_avalon_universal_slave_0_byteenable       (4'b1111),
            .elane_avalon_universal_slave_0_readdata         (cpri_reconfig_readdata_int[j]),
            .elane_avalon_universal_slave_0_writedata        ({4{cpri_reconfig_writedata_int[j][7:0]}}),
            .elane_avalon_universal_slave_0_lock             (1'b0),
            .elane_avalon_universal_slave_0_debugaccess      (1'b0),
            .master_avalon_universal_master_0_address        ({ext_addr_2[j],sl_xcvr_reconfig_address[19*j+18:19*j],ext_addr[j][1:0]}),
            .master_avalon_universal_master_0_burstcount     (),
            .master_avalon_universal_master_0_read           (sl_xcvr_reconfig_read[j]),
            .master_avalon_universal_master_0_write          (sl_xcvr_reconfig_write[j]),
            .master_avalon_universal_master_0_waitrequest    (xcvr_reconfig_waitrequest[j]),
            .master_avalon_universal_master_0_readdatavalid  (o_reconfig_readdata_valid_int[j]),
            .master_avalon_universal_master_0_byteenable     (),
            .master_avalon_universal_master_0_readdata       ({24'b000000000000000000000000, xcvr_reconfig_readdata[8*j+7:8*j]}),
            .master_avalon_universal_master_0_writedata      (xcvr_reconfig_writedata_int[j]),
            .master_avalon_universal_master_0_lock           (),
            .master_avalon_universal_master_0_debugaccess    (),
            .reset_reset                                     (reconfig_reset_sync),
            .xcvr_avalon_universal_slave_0_address           ({1'b0,i_xcvr_reconfig_address[19*j+18:19*j],2'b00}),
            .xcvr_avalon_universal_slave_0_burstcount        (1'b1),
            .xcvr_avalon_universal_slave_0_read              (i_xcvr_reconfig_read[j]),
            .xcvr_avalon_universal_slave_0_write             (i_xcvr_reconfig_write[j]),
            .xcvr_avalon_universal_slave_0_waitrequest       (sl_xcvr_reconfig_waitrequest[j]),
            .xcvr_avalon_universal_slave_0_readdatavalid     (o_xcvr_reconfig_readdata_valid[j]),
            .xcvr_avalon_universal_slave_0_byteenable        (4'b1111),
            .xcvr_avalon_universal_slave_0_readdata          (o_xcvr_reconfig_readdata_int[j]),
            .xcvr_avalon_universal_slave_0_writedata         ({24'b000000000000000000000000, i_xcvr_reconfig_writedata[8*j+7:8*j]}),
            .xcvr_avalon_universal_slave_0_lock              (1'b0),
            .xcvr_avalon_universal_slave_0_debugaccess       (1'b0)
        );
        assign cpri_reconfig_readdata_valid_int[j] = cpri_reconfig_read_int[j] & ~cpri_reconfig_waitrequest_int[j];
        assign sl_xcvr_reconfig_readdata[8*j+7:8*j] = o_xcvr_reconfig_readdata_int[j][7:0];
        assign sl_xcvr_reconfig_writedata[8*j+7:8*j] = xcvr_reconfig_writedata_int[j][7:0];
        assign o_reconfig_readdata_valid_int[j] = sl_xcvr_reconfig_read[j] & ~xcvr_reconfig_waitrequest[j];


        //*********************** EHIP Soft CSR ****************************

         assign sl_khz_ref_phy[j]  = {16'd0, sl_int_khz_ref_phy[j]};
         assign sl_khz_rx_clk64[j] = {16'd0, sl_int_khz_rx_clk64[j]};
         assign sl_khz_tx_clk64[j] = {16'd0, sl_int_khz_tx_clk64[j]};

         assign sl_csr_tx_pll_locked[j]      = {{3{1'b0}},1'b1};
         assign sl_csr_rx_is_lockedtodata[j] = {4{sl_rx_is_lockedtodata[j]}};

         alt_xcvr_resync_std #(
             .SYNC_CHAIN_LENGTH(3),
             .WIDTH(1),
             .INIT_VALUE(0)
         ) sync_sl_csr_rx_is_lockedtodata (
             .clk    (i_reconfig_clk),
             .reset  (1'b0),
             .d  (&sl_csr_rx_is_lockedtodata[j]),
             .q  (sl_csr_rx_is_lockedtodata_sync[j])
         );

         alt_xcvr_resync_std #(
             .SYNC_CHAIN_LENGTH(3),
             .WIDTH(1),
             .INIT_VALUE(0)
         ) sync_sl_measure_valid (
             .clk    (i_reconfig_clk),
             .reset  (1'b0),
             .d  (sl_measure_valid[j]),
             .q  (sl_measure_valid_sync[j])
         );

         alt_xcvr_resync_std #(
             .SYNC_CHAIN_LENGTH(3),
             .WIDTH(5),
             .INIT_VALUE(0)
         ) sync_sl_rx_bitslipboundary_sel (
             .clk    (i_reconfig_clk),
             .reset  (1'b0),
             .d  (sl_rx_bitslipboundary_sel[j]),
             .q  (sl_rx_bitslipboundary_sel_sync[j])
         );

         alt_cpriphy_c3_clock_mon clk_mon_sl_inst (
            .csr_clk(i_reconfig_clk),
            .cdr_ref_clk(i_clk_ref_m[0]),
            // Inputs
            .tx_clk64(o_tx_clkout[0]),
            .rx_clk64(o_rx_clkout[0]),
            .tx_clk66(o_tx_clkout2[0]),
            .rx_clk66(o_rx_clkout2[0]),
            // outputs
            .khz_ref_phy(sl_int_khz_ref_phy[j]),
            .khz_rx_clk64(sl_int_khz_rx_clk64[j]),
            .khz_tx_clk64(sl_int_khz_tx_clk64[j]),
            .khz_rx_clk66(),
            .khz_tx_clk66()
        );

        alt_cpriphy_c3_csr soft_csr_sl_inst (
            .clk                (i_reconfig_clk),
            .reset              (reconfig_reset_sync),
            .writedata          (i_sl_cpri_reconfig_writedata_2d[j]),
            .read               (soft_csr_read_req[j]&sl_soft_csr_phy_sel[j]),
            .write              (i_sl_cpri_reconfig_write[j]&sl_soft_csr_phy_sel[j]),
            .byteenable         (4'b1111),
            .readdata           (sl_soft_csr_phy_read_data[j]),
            .readdatavalid      (sl_soft_csr_phy_read_valid[j]),
            .address            (i_sl_cpri_reconfig_addr_2d[j][11:0]),

            .mac_rx_adapt_dropped_frame_lo  (32'd0),
            .mac_rx_adapt_dropped_frame_hi  (32'd0),
            .mac_rx_adapt_clear_dropped     (/* unused */),

            .phy_config_eio_sys_rst                 (sl_soft_csr_rst[j]),
            .phy_config_soft_tx_rst                 (sl_soft_tx_rst[j]),
            .phy_config_soft_rx_rst                 (sl_soft_rx_rst[j]),
            .phy_config_set_ref_lock                (),
            .phy_config_set_data_lock               (),
            .phy_pma_sloop_sloop                    (),
            // Need to add frequency counters
            .phy_txclk_khz_khz_tx_i                 (sl_khz_tx_clk64[j]),
            .phy_recclk_khz_khz_rx_i                (sl_khz_rx_clk64[j]),
            .phy_refclk_khz_khz_ref_i               (sl_khz_ref_phy[j]),
            .phy_tx_pll_locked_tx_pll_locked_i      (sl_csr_tx_pll_locked[j]),
            .phy_tx_corepll_locked_tx_pcs_ready_i   (o_sl_tx_lanes_stable[j]),
            .phy_eiofreq_locked_eio_freq_lock_i     ({4{sl_csr_rx_is_lockedtodata_sync[j]}}),
            .txAdpt_to_rxAdpt_loopback              (),
            
            .i_sl_cpri_rate_sel                     (cpri_rate_sel[j]),
            .i_sl_cpri_fec_en                       (cpri_fec_en[j]),
            .i_sl_rx_bitslipboundary_sel            (sl_rx_bitslipboundary_sel_sync[j]),
            .i_sl_tx_delay_sclk                     (sl_tx_delay_sclk[j]),
            .i_sl_rx_delay_sclk                     (sl_rx_delay_sclk[j]),
            .i_sl_measure_valid                     (sl_measure_valid_sync[j]),

            .sl_dl_master_reset                     (sl_soft_dl_rst[j])
        );

        assign sl_soft_csr_phy_waitrequest[j]      = (i_sl_cpri_reconfig_read[j] & sl_soft_csr_phy_sel[j] & ~soft_csr_read_req[j]);

        assign o_sl_cpri_reconfig_waitrequest[j] = (reconfig_reset_sync) ? 1'b1 : 
                                                   (sl_soft_csr_phy_sel[j]) ? sl_soft_csr_phy_waitrequest[j] :
                                                   (cpri_sl_sel_r[j]) ? o_sl_cpri_reconfig_waitrequest_int[j] :
                                                   1'b1;
        // data valid
        assign o_sl_cpri_reconfig_readdata_valid[j]   = o_sl_cpri_reconfig_readdata_valid_int[j] | sl_soft_csr_phy_read_valid[j];

        // read data
        always_comb begin
            case (1'b1)
                sl_soft_csr_phy_read_valid[j] : o_sl_cpri_reconfig_readdata_2d[j]   = sl_soft_csr_phy_read_data[j];
                default                       : o_sl_cpri_reconfig_readdata_2d[j]   = o_sl_cpri_reconfig_readdata_int[j];
            endcase
        end

    end
    endgenerate

// HARD IP instantiation

    alt_cpriphy_c3_hard_7djwtgq #(
        .ENABLE_RSFEC(ENABLE_RSFEC),
        .ENABLE_RSFEC_MODE(ENABLE_RSFEC_MODE),
        .MAX_REFCLK(max_refclk),
        .MAX_XCVR_CH(max_xcvr_ch),
        .MAX_CLKOUT_CH(max_clkout_ch),
        .SL_LANES(sl_num_lanes),
        .enable_external_aib_clocking      (enable_external_aib_clocking      ),
        .ehip_mode_sl_0                    (ehip_mode_sl_0                    ),
        .cpri_include_alternate_ports      (cpri_include_alternate_ports      ),
        .LOCATION_SEL                      (LOCATION_SEL                      ),
        .FIRST_LANE                        (0                       )
    ) alt_cpriphy_c3_hard_inst (
        .i_refclk(i_clk_ref_m),
        .i_reconfig_clk(i_reconfig_clk),
        .i_sl_clk_rx(i_sl_clk_rx),
        .i_sl_clk_tx(i_sl_clk_tx),
        .i_aib_clk(i_aib_clk),
        .i_aib_2x_clk(i_aib_2x_clk),
        .o_tx_clk_div64(tx_clkout_div64),
        .o_rx_clk_div64(rx_clkout_div64),
        .o_tx_clk_div66(tx_clkout_div66),
        .o_rx_clk_div66(rx_clkout_div66),

        .i_sl_enable_rsfec(sl_rsfec_mode),

        .i_rx_serial_data(i_rx_serial),
        .o_tx_serial_data(o_tx_serial),
        .i_rx_serial_data_n(i_rx_serial_n),
        .o_tx_serial_data_n(o_tx_serial_n),

        .i_ehip_rst_n(~ehip_rst), // this will
        .i_ehip_rx_rst_n(~ehip_rx_rst),
        .i_ehip_tx_rst_n(~ehip_tx_rst),
        .i_ehip_pld_ready(ehip_pld_ready),
        .i_ehip_signal_ok(ehip_signal_ok_p),
        .i_ehip_clear_internal_err(1'b0),


        .i_sl_rx_pmaif_reset(sl_rx_pcs_reset),
        .i_sl_tx_pmaif_reset(sl_tx_pcs_reset),
        .i_sl_rx_aib_reset(sl_rx_aib_reset),
        .i_sl_tx_aib_reset(sl_tx_aib_reset),
        // .i_sl_pmad_reset(sl_pmad_reset),
        .i_sl_tx_reset(sl_tx_reset),
        .i_sl_rx_reset(sl_rx_reset),

        .o_sl_rx_is_lockedtodata(sl_rx_is_lockedtodata),
        .o_sl_rx_transfer_ready(sl_rx_transfer_ready),
        .o_sl_tx_transfer_ready(sl_tx_transfer_ready),
        .o_sl_tx_dll_lock(sl_tx_dll_lock),
        .o_sl_rx_pma_ready(o_sl_rx_pma_ready),
        .o_sl_tx_pma_ready(o_sl_tx_pma_ready),

        .i_sl_tx_data(),
        .i_sl_tx_error(),
        .i_sl_tx_empty(),
        .i_sl_tx_inframe(),
        .i_sl_tx_skip_crc(),
        .i_sl_tx_aib_dsk({sl_num_lanes{1'b0}}),

        .o_sl_rx_data(),
        .o_sl_rx_empty(),
        .o_sl_rx_fcs_error(),
        .o_sl_rx_error(),
        .o_sl_rx_status(),
        .o_sl_rx_inframe(),

        .i_sl_tx_valid               ({sl_num_lanes{1'b0}}),
        .o_sl_rx_valid               (),
        .o_sl_tx_ready               (),

        // Elane Interface
        .o_sl_rx_pcs_fully_aligned   (sl_rx_pcs_fully_aligned),
        .o_sl_rx_block_lock          (o_sl_rx_block_lock),
        .o_sl_rx_hi_ber              (o_sl_rx_hi_ber),
        .o_sl_local_fault_status     (),
        .o_sl_rx_pause               (),
        .o_sl_rx_pfc                 (),
        .o_sl_remote_fault_status    (),
        .i_sl_ehip_signal_ok         (sl_ehip_signal_ok),
        .i_sl_tx_pause               ({sl_num_lanes{1'b0}}),
        .i_sl_tx_pfc                 ({(sl_num_lanes*8){1'b0}}),
        .o_sl_xcvr_rxfifo_overflow   (), // TODO
        .o_sl_xcvr_txfifo_pempty     (o_sl_txfifo_pempty),
        .o_sl_xcvr_txfifo_pfull      (o_sl_txfifo_pfull),
        .o_sl_xcvr_txfifo_overflow   (o_sl_txfifo_overflow),
        .o_sl_xcvr_txfifo_underflow  (o_sl_txfifo_underflow),
        .i_sl_clear_internal_err     ({sl_num_lanes{1'b0}}), // TODO
        .o_sl_internal_err           (), // TODO
        .i_sl_force_rcfg_reset       ({sl_num_lanes{1'b1}}),
        .o_sl_cfg_load_done          (), // TODO
        .i_sl_stats_snapshot         (i_sl_stats_snapshot),
        .i_sl_custom_cadence         (sl_custom_cadence),

        .i_sl_ehip_reset             (sl_ehip_reset),
        .i_sl_ehip_rx_reset          (sl_ehip_rx_reset),
        .i_sl_ehip_tx_reset          (sl_ehip_tx_reset),
        .o_sl_ehip_ready             (sl_ehip_ready),
        .i_sl_ehip_pld_ready         (sl_ehip_pld_ready),
        .o_sl_ehip_reset_stat        (sl_ehip_reset_stat),
        .o_sl_ehip_rx_reset_stat     (sl_ehip_rx_reset_stat),
        .o_sl_ehip_tx_reset_stat     (sl_ehip_tx_reset_stat),

        .i_sl_rsfec_reset            (sl_rsfec_reset),
        .i_sl_rsfec_tx_reset         (sl_rsfec_tx_reset),
        .i_sl_rsfec_rx_reset         (sl_rsfec_rx_reset),

        .o_sl_rx_fifo_ready          (sl_rx_fifo_ready),
        .o_sl_tx_fifo_ready          (sl_tx_fifo_ready),
        .o_sl_rx_nphy_ready          (sl_rx_nphy_ready),
        .o_sl_tx_nphy_ready          (sl_tx_nphy_ready),


        // mii specific interface
        .i_sl_tx_mii_d(sl_tx_mii_d),
        .i_sl_tx_mii_c(sl_tx_mii_c),
        .i_sl_tx_mii_valid(sl_tx_mii_valid),
        .i_sl_tx_mii_am(sl_tx_mii_am),
        .o_sl_tx_mii_ready(sl_tx_mii_ready),
        .o_sl_rx_mii_d(sl_rx_mii_d),
        .o_sl_rx_mii_c(sl_rx_mii_c),
        .o_sl_rx_mii_valid(sl_rx_mii_valid),
        .o_sl_rx_mii_am_valid(sl_rx_mii_am_valid),

        // pcs66 (OTN, FlexE) specific interface
        .i_sl_tx_pcs66_d         (),
        .i_sl_tx_pcs66_valid     (),
        .i_sl_tx_pcs66_am        (),
        .o_sl_tx_pcs66_ready     (),
        .o_sl_rx_pcs66_d         (),
        .o_sl_rx_pcs66_valid     (),
        .o_sl_rx_pcs66_am_valid  (),

        // PMA Direct Interface
        .i_sl_tx_pma             (int_sl_tx_pma),
        .o_sl_rx_pma             (int_sl_rx_pma),
        .o_sl_aib_txfifo_empty   (int_sl_aib_txfifo_empty),
        .o_sl_aib_txfifo_full    (int_sl_aib_txfifo_full),
        .o_sl_aib_txfifo_pempty  (int_sl_aib_txfifo_pempty),
        .o_sl_aib_txfifo_pfull   (int_sl_aib_txfifo_pfull),
        .o_sl_aib_rxfifo_empty   (int_sl_aib_rxfifo_empty),
        .o_sl_aib_rxfifo_full    (int_sl_aib_rxfifo_full),
        .o_sl_aib_rxfifo_pempty  (int_sl_aib_rxfifo_pempty),
        .o_sl_aib_rxfifo_pfull   (int_sl_aib_rxfifo_pfull),
        .i_sl_aib_rxfifo_rd_en   (int_sl_aib_rxfifo_rd_en),

        // RX AIB deskew debug
        .i_sl_aib_rxfifo_latency_adj_ena     (i_sl_aib_rxfifo_latency_adj_ena),

        // Datapath mux select
        .i_sl_dp_sel             (sl_dp_sel),

        // Deterministic Latency Interface
        .o_sl_tx_dl_async_pulse  (sl_tx_dl_async_pulse),
        .o_sl_rx_dl_async_pulse  (sl_rx_dl_async_pulse),
        .i_sl_latency_sclk       (sl_latency_sclk),
        .i_sl_tx_dl_measure_sel  (sl_tx_rx_dl_measure_sel),
        .i_sl_rx_dl_measure_sel  (sl_tx_rx_dl_measure_sel),


        // EHIP Interface: Reference Timestamp
        .i_sl_sclk               ({sl_num_lanes{1'b0}}),
        .i_sl_tx_sel_sclk_return ({sl_num_lanes{1'b0}}),
        .i_sl_rx_sel_sclk_return ({sl_num_lanes{1'b0}}),
        .o_sl_tx_sclk_return     (),
        .o_sl_rx_sclk_return     (),

        // AVMM Interface
        .i_reconfig_reset(i_reconfig_reset),
        .i_rsfec_reconfig_read(rsfec_reconfig_read),
        .i_rsfec_reconfig_addr(rsfec_reconfig_address),
        .i_rsfec_reconfig_write(rsfec_reconfig_write),
        .i_rsfec_reconfig_writedata(rsfec_reconfig_writedata),
        .o_rsfec_reconfig_readdata(rsfec_reconfig_readdata),
        .o_rsfec_reconfig_waitrequest(rsfec_reconfig_waitrequest),

        .i_xcvr_reconfig_reset(i_reconfig_reset),
        .i_xcvr_reconfig_write(sl_xcvr_reconfig_write),
        .i_xcvr_reconfig_read(sl_xcvr_reconfig_read),
        .i_xcvr_reconfig_address(sl_xcvr_reconfig_address),
        .i_xcvr_reconfig_writedata(sl_xcvr_reconfig_writedata),
        .o_xcvr_reconfig_readdata(xcvr_reconfig_readdata),
        .o_xcvr_reconfig_waitrequest(xcvr_reconfig_waitrequest),

        .o_rsfec_channel_ssr(o_rsfec_channel_ssr)
    );

    assign o_xcvr_reconfig_readdata     = sl_xcvr_reconfig_readdata;
    assign o_xcvr_reconfig_waitrequest  = sl_xcvr_reconfig_waitrequest;

    assign var_i_csr_rst                =  &i_sl_csr_rst_n;

    assign rsfec_reconfig_write = i_rsfec_reconfig_write;
    assign rsfec_reconfig_writedata = i_rsfec_reconfig_writedata;
    assign rsfec_reconfig_read = i_rsfec_reconfig_read;
    assign rsfec_reconfig_address = i_rsfec_reconfig_addr;
    assign o_rsfec_reconfig_readdata = rsfec_reconfig_readdata;
    assign o_rsfec_reconfig_waitrequest = rsfec_reconfig_waitrequest;
    assign o_rsfec_reconfig_readdata_valid = rsfec_reconfig_read & ~rsfec_reconfig_waitrequest;

    assign ehip_rst = &sl_ehip_reset;
    assign ehip_tx_rst = &sl_ehip_tx_reset;
    assign ehip_rx_rst = &sl_ehip_rx_reset;
    assign ehip_pld_ready = |sl_ehip_pld_ready;
    assign ehip_signal_ok = |sl_ehip_signal_ok;


    //alt_cpriphy_c3_std_synchronizer_nocut ehip_csr_rst_sync_inst (
    //    .clk        (i_reconfig_clk),
    //   .reset_n    (1'b1),
    //    .din        (ehip_rst),
    //    .dout       (ehip_rst_sync)
    //);


    // RSFEC reset for single lane
    assign sl_rsfec_reset    = enable_rsfec_rst_ports ? !i_rsfec_rst_n    : &sl_soft_rsfec_reset;
    assign sl_rsfec_tx_reset = enable_rsfec_rst_ports ? !i_rsfec_tx_rst_n : &sl_soft_rsfec_tx_reset;
    assign sl_rsfec_rx_reset = enable_rsfec_rst_ports ? !i_rsfec_rx_rst_n : &sl_soft_rsfec_rx_reset;

    // Soft logic for single lane
    genvar sl_soft_i;
    generate
    for(sl_soft_i = 0; sl_soft_i < sl_num_lanes; sl_soft_i++) begin : SL_SOFT_I

        alt_cpriphy_c3_soft #(
            .SIM_EMULATE                (SIM_EMULATE),
            .enable_external_aib_clocking (enable_external_aib_clocking),
            // .DL_TOT_MEASURE_CNT           (SIM_EMULATE ? 32 : 128),
            .DL_TOT_MEASURE_CNT           (128),
            .ehip_mode_sl_0               (ehip_mode_sl_0),
            .cpri_include_alternate_ports (cpri_include_alternate_ports)
        ) sl_soft (
            // Clock
            .i_reconfig_clk             (i_reconfig_clk),
            .i_aib_clk                  (i_aib_clk),
            .i_sl_clk_tx                (i_sl_clk_tx[sl_soft_i]),
            .i_clk_pll_div66            (o_tx_clkout2[sl_soft_i]),
            .i_clk_rec_div66            (o_rx_clkout2[sl_soft_i]),
            .i_sampling_clk             (i_sampling_clk),

            // Reset
            .i_sl_csr_rst_n             (i_sl_csr_rst_n[sl_soft_i]),
            .i_sl_tx_rst_n              (i_sl_tx_rst_n[sl_soft_i]),
            .i_sl_rx_rst_n              (i_sl_rx_rst_n[sl_soft_i]),

            .i_sl_soft_csr_rst          (sl_soft_csr_rst[sl_soft_i]),
            .i_sl_soft_tx_rst           (sl_soft_tx_rst[sl_soft_i]),
            .i_sl_soft_rx_rst           (sl_soft_rx_rst[sl_soft_i]),

            .i_sl_soft_dl_rst           (sl_soft_dl_rst[sl_soft_i]),

            // Reset Controller
            .o_sl_tx_aib_reset          (sl_tx_aib_reset[sl_soft_i]),
            .o_sl_rx_aib_reset          (sl_rx_aib_reset[sl_soft_i]),
            .o_sl_tx_pcs_reset          (sl_tx_pcs_reset[sl_soft_i]),
            .o_sl_rx_pcs_reset          (sl_rx_pcs_reset[sl_soft_i]),
            // .o_sl_pmad_reset            (sl_pmad_reset[sl_soft_i]),
            .o_sl_tx_reset              (sl_tx_reset[sl_soft_i]),
            .o_sl_rx_reset              (sl_rx_reset[sl_soft_i]),
            .i_sl_tx_pma_ready          (o_sl_tx_pma_ready[sl_soft_i]),
            .i_sl_rx_pma_ready          (o_sl_rx_pma_ready[sl_soft_i]),
            .i_sl_tx_dll_lock           (sl_tx_dll_lock[sl_soft_i]),
            .i_sl_rx_is_lockedtodata    (sl_rx_is_lockedtodata[sl_soft_i]),
            .i_sl_rx_transfer_ready     (sl_rx_transfer_ready[sl_soft_i]),
            .i_sl_tx_transfer_ready     (sl_tx_transfer_ready[sl_soft_i]),
            .i_sl_rx_pcs_fully_aligned  (sl_rx_pcs_fully_aligned[sl_soft_i]),
            .i_sl_rx_fifo_ready         (sl_rx_fifo_ready[sl_soft_i]),
            .i_sl_tx_fifo_ready         (sl_tx_fifo_ready[sl_soft_i]),
            .i_sl_rx_nphy_ready         (sl_rx_nphy_ready[sl_soft_i]),
            .i_sl_tx_nphy_ready         (sl_tx_nphy_ready[sl_soft_i]),

            .i_sl_ehip_ready            (sl_ehip_ready[sl_soft_i]),
            .i_sl_ehip_reset_stat       (sl_ehip_reset_stat[sl_soft_i]),
            .i_sl_ehip_tx_reset_stat    (sl_ehip_tx_reset_stat[sl_soft_i]),
            .i_sl_ehip_rx_reset_stat    (sl_ehip_rx_reset_stat[sl_soft_i]),
            .o_sl_ehip_reset            (sl_ehip_reset[sl_soft_i]),
            .o_sl_ehip_tx_reset         (sl_ehip_tx_reset[sl_soft_i]),
            .o_sl_ehip_rx_reset         (sl_ehip_rx_reset[sl_soft_i]),
            .o_sl_ehip_pld_ready        (sl_ehip_pld_ready[sl_soft_i]),
            .o_sl_ehip_signal_ok        (sl_ehip_signal_ok[sl_soft_i]),

            .o_sl_rsfec_reset           (sl_soft_rsfec_reset[sl_soft_i]),
            .o_sl_rsfec_tx_reset        (sl_soft_rsfec_tx_reset[sl_soft_i]),
            .o_sl_rsfec_rx_reset        (sl_soft_rsfec_rx_reset[sl_soft_i]),

            .o_sl_tx_lanes_stable       (o_sl_tx_lanes_stable[sl_soft_i]),
            .o_sl_rx_pcs_ready          (o_sl_rx_pcs_ready[sl_soft_i]),

            .o_sl_ehip_ready            (o_sl_ehip_ready[sl_soft_i]),
            
            .i_sl_tx_mii_d              (i_sl_tx_mii_d_2d[sl_soft_i]),
            .i_sl_tx_mii_c              (i_sl_tx_mii_c_2d[sl_soft_i]),
            .i_sl_tx_mii_valid          (i_sl_tx_mii_valid[sl_soft_i]),
            .i_sl_tx_mii_ready          (sl_tx_mii_ready[sl_soft_i]),
            .o_sl_tx_mii_d              (sl_tx_mii_d[sl_soft_i]),
            .o_sl_tx_mii_c              (sl_tx_mii_c[sl_soft_i]),
            .o_sl_tx_mii_valid          (sl_tx_mii_valid[sl_soft_i]),
            .o_sl_tx_mii_am             (sl_tx_mii_am[sl_soft_i]),
            .o_sl_tx_mii_ready          (o_sl_tx_mii_ready[sl_soft_i]),
            .i_sl_rx_mii_d              (sl_rx_mii_d[sl_soft_i]),
            .i_sl_rx_mii_c              (sl_rx_mii_c[sl_soft_i]),
            .i_sl_rx_mii_valid          (sl_rx_mii_valid[sl_soft_i]),
            .i_sl_rx_mii_am_valid       (sl_rx_mii_am_valid[sl_soft_i]),
            .o_sl_rx_mii_d              (o_sl_rx_mii_d_2d[sl_soft_i]),
            .o_sl_rx_mii_c              (o_sl_rx_mii_c_2d[sl_soft_i]),
            .o_sl_rx_mii_valid          (o_sl_rx_mii_valid[sl_soft_i]),
            
            .o_sl_mii_txfifo_pempty     (o_sl_mii_txfifo_pempty[sl_soft_i]),
            .o_sl_mii_txfifo_pfull      (o_sl_mii_txfifo_pfull[sl_soft_i]),
            .o_sl_mii_txfifo_overflow   (o_sl_mii_txfifo_overflow[sl_soft_i]),
            .o_sl_mii_txfifo_underflow  (o_sl_mii_txfifo_underflow[sl_soft_i]),
        
            .i_sl_rx_block_lock         (o_sl_rx_block_lock[sl_soft_i]),
            .i_sl_txfifo_pfull          (o_sl_txfifo_pfull[sl_soft_i]),
            .i_sl_cpri_rate_sel         (cpri_rate_sel[sl_soft_i]),
            .i_sl_cpri_fec_en           (cpri_fec_en[sl_soft_i]),
            
            // PMA Direct 8b10b Interface
            .i_sl_tx_datain             (i_sl_tx_datain_2d[sl_soft_i]),
            .i_sl_tx_ctrlenable         (i_sl_tx_ctrlenable_2d[sl_soft_i]),
            .o_sl_tx_pma                (int_sl_tx_pma[sl_soft_i]),
            .i_sl_rx_pma                (int_sl_rx_pma[sl_soft_i]),
            .i_sl_tx_hard_fifo_full        (int_sl_aib_txfifo_full[sl_soft_i]),
            .i_sl_tx_hard_fifo_pfull       (int_sl_aib_txfifo_pfull[sl_soft_i]),
            .i_sl_rx_hard_fifo_empty       (int_sl_aib_rxfifo_empty[sl_soft_i]),
            .i_sl_rx_hard_fifo_pempty      (int_sl_aib_rxfifo_pempty[sl_soft_i]),
//            .o_sl_hard_fifo_rd_en       (int_sl_aib_rxfifo_rd_en[sl_soft_i]),
            .o_sl_hard_fifo_rd_en       (),
            .o_sl_rx_dataout            (o_sl_rx_dataout_2d[sl_soft_i]),
            .o_sl_rx_ctrldetect         (o_sl_rx_ctrldetect_2d[sl_soft_i]),
            .o_sl_rx_disperr            (o_sl_rx_disperr_2d[sl_soft_i]),
            .o_sl_rx_errdetect          (o_sl_rx_errdetect_2d[sl_soft_i]),
            .o_sl_rx_patterndetect      (o_sl_rx_patterndetect_2d[sl_soft_i]),
            .o_sl_rx_bitslipboundary_sel(sl_rx_bitslipboundary_sel[sl_soft_i]),
            .o_sl_rx_lspcs_ready        (o_sl_rx_ready[sl_soft_i]),
            .o_sl_tx_lspcs_ready        (o_sl_tx_ready[sl_soft_i]),
            
            .i_sl_tx_dl_async_pulse     (sl_tx_dl_async_pulse[sl_soft_i]),
            .i_sl_rx_dl_async_pulse     (sl_rx_dl_async_pulse[sl_soft_i]),
            .o_sl_latency_sclk          (sl_latency_sclk[sl_soft_i]),
            .o_sl_tx_rx_dl_measure_sel  (sl_tx_rx_dl_measure_sel[sl_soft_i]),

            .o_sl_tx_delay_sclk         (sl_tx_delay_sclk[sl_soft_i]),
            .o_sl_rx_delay_sclk         (sl_rx_delay_sclk[sl_soft_i]),
            .o_sl_tx_sync_count         (),
            .o_sl_tx_async_count        (),
            .o_sl_rx_sync_count         (),
            .o_sl_rx_async_count        (),
            .o_sl_measure_valid         (sl_measure_valid[sl_soft_i])
        );


        if(cpri_include_alternate_ports) begin : sl_dp_sel_gen
            always @(posedge i_reconfig_clk) begin
                if(reconfig_reset_sync) begin
                    sl_dp_sel[sl_soft_i] <= SL_DP_SEL_RESET_VALUE;
                end
                else begin
                    if((i_sl_cpri_reconfig_addr[sl_soft_i*19+:12] == 12'h350) && i_sl_cpri_reconfig_write[sl_soft_i]) begin
                        sl_dp_sel[sl_soft_i] <= i_sl_cpri_reconfig_writedata[sl_soft_i*32+:3];
                    end
                end
            end
        end
        else begin
            assign sl_dp_sel[sl_soft_i] = SL_DP_SEL_RESET_VALUE;
        end

        always @(posedge i_reconfig_clk) begin
            if(reconfig_reset_sync) begin
                sl_rsfec_mode[sl_soft_i]        <= ((ehip_mode_sl_0 == "ehip_mac") || (ehip_mode_sl_0 == "ehip_mac_ptp")) ? 1'b1 : 1'b0;
            end
            else begin
                if((i_sl_cpri_reconfig_addr[sl_soft_i*19+:12] == 12'h40B) && i_sl_cpri_reconfig_write[sl_soft_i]) begin
                    sl_rsfec_mode[sl_soft_i]        <= (i_sl_cpri_reconfig_writedata[sl_soft_i*32+5:sl_soft_i*32+3] == 3'd4) ? 1'b1 : 1'b0;
                end
            end
        end
    
        always @(posedge i_reconfig_clk) begin
            if(reconfig_reset_sync) begin
                {cpri_fec_en[sl_soft_i],cpri_rate_sel[sl_soft_i]} <= {ENABLE_RSFEC_MODE,CPRI_RATE_SEL_RESET_VALUE};
            end
            else begin
                if((i_sl_cpri_reconfig_addr[sl_soft_i*19+:12] == 12'hC00) && i_sl_cpri_reconfig_write[sl_soft_i]) begin
                    {cpri_fec_en[sl_soft_i],cpri_rate_sel[sl_soft_i]} <= i_sl_cpri_reconfig_writedata[sl_soft_i*32+:5];
                end
            end
        end

    end
    endgenerate

endmodule // alt_cpriphy_c3_top


