module ex_10G #(
		parameter sim_mode = "enable"
	) (
		output wire [3:0]   o_cdr_lock,                        //                        o_cdr_lock.o_cdr_lock,                        RX CDR Locked
		output wire [3:0]   o_tx_pll_locked,                   //                   o_tx_pll_locked.o_tx_pll_locked,                   TX PLL Locked
		input  wire [4:0]   i_clk_ref,                         //                         i_clk_ref.i_clk_ref,                         Reference clock used by the RX PHY
		output wire [3:0]   o_tx_clkout,                       //                       o_tx_clkout.o_tx_clkout,                       CPRI PHY system clock, this is /64 clock (380.16 MHz)
		output wire [3:0]   o_tx_clkout2,                      //                      o_tx_clkout2.o_tx_clkout2,                      This is /66 clock (368.64 MHz)
		output wire [3:0]   o_rx_clkout,                       //                       o_rx_clkout.o_rx_clkout,                       This is /64 recovered clock (380.16 MHz)
		output wire [3:0]   o_rx_clkout2,                      //                      o_rx_clkout2.o_rx_clkout2,                      This is /66 recovered clock (368.64 MHz)
		input  wire         i_sampling_clk,                    //                    i_sampling_clk.i_sampling_clk,                    Deterministic Latency sampling clock
		output wire [3:0]   o_tx_serial,                       //                          serial_p.o_tx_serial,                       TX XCVR serial pins
		input  wire [3:0]   i_rx_serial,                       //                                  .i_rx_serial,                       RX XCVR serial pins
		input  wire         i_reconfig_clk,                    //                    i_reconfig_clk.clk,                               XCVR reconfiguration bus clock
		input  wire         i_reconfig_reset,                  //                  i_reconfig_reset.i_reconfig_reset,                  XCVR reconfig reset
		input  wire [75:0]  i_xcvr_reconfig_address,           //           i_xcvr_reconfig_address.i_xcvr_reconfig_address,           reconfig access address
		input  wire [3:0]   i_xcvr_reconfig_read,              //              i_xcvr_reconfig_read.i_xcvr_reconfig_read,              read signal asserted to start a read cycle
		input  wire [3:0]   i_xcvr_reconfig_write,             //             i_xcvr_reconfig_write.i_xcvr_reconfig_write,             write signal asserted to write data on reconfig_writedata bus
		output wire [31:0]  o_xcvr_reconfig_readdata,          //          o_xcvr_reconfig_readdata.o_xcvr_reconfig_readdata,          data that was read by a read cycle
		input  wire [31:0]  i_xcvr_reconfig_writedata,         //         i_xcvr_reconfig_writedata.i_xcvr_reconfig_writedata,         data to be written on a write cycle
		output wire [3:0]   o_xcvr_reconfig_waitrequest,       //       o_xcvr_reconfig_waitrequest.o_xcvr_reconfig_waitrequest,       AVMM stalling signal. The read/write cycle is only complete when this signal goes low
		output wire [3:0]   o_sl_rx_hi_ber,                    //                    o_sl_rx_hi_ber.o_sl_rx_hi_ber,                    High BER
		input  wire [75:0]  i_sl_cpri_reconfig_addr,           //           i_sl_cpri_reconfig_addr.i_sl_cpri_reconfig_addr,           CSR access address
		input  wire [3:0]   i_sl_cpri_reconfig_read,           //           i_sl_cpri_reconfig_read.i_sl_cpri_reconfig_read,           read signal asserted to start a read cycle
		input  wire [3:0]   i_sl_cpri_reconfig_write,          //          i_sl_cpri_reconfig_write.i_sl_cpri_reconfig_write,          write signal asserted to write data on status_writedata bus
		output wire [127:0] o_sl_cpri_reconfig_readdata,       //       o_sl_cpri_reconfig_readdata.o_sl_cpri_reconfig_readdata,       data that was read by a read cycle
		output wire [3:0]   o_sl_cpri_reconfig_readdata_valid, // o_sl_cpri_reconfig_readdata_valid.o_sl_cpri_reconfig_readdata_valid, signal to indicate a valid read
		input  wire [127:0] i_sl_cpri_reconfig_writedata,      //      i_sl_cpri_reconfig_writedata.i_sl_cpri_reconfig_writedata,      data to be written on a write cycle
		output wire [3:0]   o_sl_cpri_reconfig_waitrequest,    //    o_sl_cpri_reconfig_waitrequest.o_sl_cpri_reconfig_waitrequest,    AVMM stalling signal. The read/write cycle is only complete when this signal goes low
		output wire [3:0]   o_sl_tx_lanes_stable,              //              o_sl_tx_lanes_stable.o_sl_tx_lanes_stable,              Asserted when TX MAC is ready to send data
		output wire [3:0]   o_sl_rx_pcs_ready,                 //                 o_sl_rx_pcs_ready.o_sl_rx_pcs_ready,                 Asserted when RX PCS is ready to receive data
		output wire [3:0]   o_sl_ehip_ready,                   //                   o_sl_ehip_ready.o_sl_ehip_ready,                   Signal to indicate EHIP CSRs are ready to access
		output wire [3:0]   o_sl_rx_block_lock,                //                o_sl_rx_block_lock.o_sl_rx_block_lock,                Asserted when 66b block alignment is finished on all PCS lanes
		input  wire [3:0]   i_sl_clk_tx,                       //                       i_sl_clk_tx.i_sl_clk_tx,                       TX Avalon-ST interface clock, in synchronous mode connect o_tx_clkout to this clock
		input  wire [3:0]   i_sl_clk_rx,                       //                       i_sl_clk_rx.i_sl_clk_rx,                       RX Avalon-ST interface clock, in synchronous mode connect o_tx_clkout to this clock
		input  wire [3:0]   i_sl_csr_rst_n,                    //                    i_sl_csr_rst_n.i_sl_csr_rst_n,                    Resets the entire CPRI core. Active low.
		input  wire [3:0]   i_sl_tx_rst_n,                     //                     i_sl_tx_rst_n.i_sl_tx_rst_n,                     Resets the TX digital path (MAC, PCS)
		input  wire [3:0]   i_sl_rx_rst_n,                     //                     i_sl_rx_rst_n.i_sl_rx_rst_n,                     Resets the RX digital path (MAC, PCS)
		input  wire         i_aib_clk,                         //                     aib_clk_ports.i_aib_clk
		input  wire         i_aib_2x_clk,                      //                                  .i_aib_2x_clk
		input  wire [255:0] i_sl_tx_mii_d,                     //                      sl_mii_ports.i_sl_tx_mii_d
		input  wire [31:0]  i_sl_tx_mii_c,                     //                                  .i_sl_tx_mii_c
		output wire [255:0] o_sl_rx_mii_d,                     //                                  .o_sl_rx_mii_d
		output wire [31:0]  o_sl_rx_mii_c                      //                                  .o_sl_rx_mii_c
	);
endmodule

