// (C) 2001-2022 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


`timescale 1ps/1ps

module alt_cpriphy_c3_top #(
    parameter sim_mode     = "enable",
    parameter NUM_CHANNELS = 4,
    parameter COUNT_WIDTH        = 20,
    parameter DL_TOT_MEASURE_CNT = 32,
    parameter N                  = COUNT_WIDTH - $clog2(DL_TOT_MEASURE_CNT),
    parameter M                  = $clog2(DL_TOT_MEASURE_CNT) + 1,
    parameter CALC_WIDTH         = N + M

) (
    input  wire [5-1:0]        i_clk_ref,

    output wire [NUM_CHANNELS-1:0]         o_tx_serial,
    input  wire [NUM_CHANNELS-1:0]         i_rx_serial,
    input  wire                            i_reconfig_clk,
    output wire [NUM_CHANNELS-1:0]         o_tx_clkout2,
    output wire [NUM_CHANNELS-1:0]         o_rx_clkout,
    output wire [NUM_CHANNELS-1:0]         o_rx_clkout2,
    output wire [NUM_CHANNELS-1:0]         o_tx_clkout,

    input wire [NUM_CHANNELS-1:0]          i_xcvr_reconfig_write,
    input wire [NUM_CHANNELS-1:0]          i_xcvr_reconfig_read,
    input wire [(NUM_CHANNELS*19)-1:0]     i_xcvr_reconfig_address,
    input wire [(NUM_CHANNELS*8)-1:0]      i_xcvr_reconfig_writedata,
    output wire [(NUM_CHANNELS*8)-1:0]     o_xcvr_reconfig_readdata,
    output wire [NUM_CHANNELS-1:0]         o_xcvr_reconfig_waitrequest,
    
    input wire [NUM_CHANNELS-1:0]          i_sl_cpri_reconfig_write,
    input wire [NUM_CHANNELS-1:0]          i_sl_cpri_reconfig_read,
    input wire [(NUM_CHANNELS*19)-1:0]     i_sl_cpri_reconfig_addr,
    input wire [(NUM_CHANNELS*32)-1:0]     i_sl_cpri_reconfig_writedata,
    output wire [(NUM_CHANNELS*32)-1:0]    o_sl_cpri_reconfig_readdata,
    output wire [NUM_CHANNELS-1:0]         o_sl_cpri_reconfig_readdata_valid,
    output wire [NUM_CHANNELS-1:0]         o_sl_cpri_reconfig_waitrequest,



    output wire                            channel_pll_locked,
    output wire                            tx_serial_data,
    input  wire                            i_clk_pll_ref,
    input wire [NUM_CHANNELS-1:0]          i_sl_csr_rst_n,
    input wire [NUM_CHANNELS-1:0]          i_sl_tx_rst_n,
    input wire [NUM_CHANNELS-1:0]          i_sl_rx_rst_n,
    input wire                             i_reconfig_reset,
    
    //ISSP or STP outputs
    output wire [NUM_CHANNELS-1:0]         o_cdr_lock,
    output wire [NUM_CHANNELS-1:0]         o_tx_pll_locked,
    output wire [NUM_CHANNELS-1:0]         o_sl_rx_hi_ber,
    output wire [NUM_CHANNELS-1:0]         o_sl_tx_lanes_stable,
    output wire [NUM_CHANNELS-1:0]         o_sl_rx_pcs_ready,
    output wire [NUM_CHANNELS-1:0]         o_sl_ehip_ready,
    output wire [NUM_CHANNELS-1:0]         o_sl_rx_block_lock,
    output wire                            iopll_sclk_locked,
    output wire [NUM_CHANNELS-1:0]         checker_pass,
    input wire                             restart,
    output wire [NUM_CHANNELS-1:0][COUNT_WIDTH-1:0]          rt_count,

    input           ehip_mode_sl_0,  // 1'b0: ehip_disable, 1'b1: ehip_pcs
    input  [3:0]    cpri_speed       // 4'b1011 : 24G
                                     // 4'b1010 : 12G
                                     // 4'b1001 : 10G
                                     // 4'b0110 : 9_8G
                                     // 4'b0101 : 6G
                                     // 4'b0100 : 4_9G
                                     // 4'b0011 : 3G
                                     // 4'b0010 : 2_4G
                                     // 4'b0001 : 1_2G
);

    // MII Interface
    wire [NUM_CHANNELS-1:0][63:0]          i_sl_tx_mii_d;
    wire [NUM_CHANNELS-1:0][7:0]           i_sl_tx_mii_c;
    wire [NUM_CHANNELS-1:0][63:0]          o_sl_rx_mii_d;
    wire [NUM_CHANNELS-1:0][7:0]           o_sl_rx_mii_c;


    wire [NUM_CHANNELS-1:0]                i_sl_clk_tx;
    wire [NUM_CHANNELS-1:0]                i_sl_clk_rx;
    wire                                   i_sampling_clk;

    //---------------------------------------------------------------------------------------------
    // 250 MHz clock for DL   
    alt_cpriphy_c3_iopll_sclk iopll_sclk_inst (
        .refclk          (i_reconfig_clk),
        .rst             (1'b0),
        .locked          (iopll_sclk_locked),
        .outclk_0        (i_sampling_clk)
    );


    wire i_aib_clk;
    wire i_aib_2x_clk;
    wire pll_refclk0;
    wire pll_clkout2;
    wire pll_clkout1;
    wire pll_locked;

    //---------------------------------------------------------------------------------------------
    // 402.83203125 MHz / 805.6640625 MHz clock for aib/aib_2x
    alt_cpriphy_c3_channel_pll i_channel_pll (
        .pll_locked     (pll_locked),     //  output,  width = 1,     pll_locked.tx_pma_ready
        .tx_serial_data (tx_serial_data), //  output,  width = 1, tx_serial_data.tx_serial_data
        .pll_refclk0    (pll_refclk0),    //   input,  width = 1,    pll_refclk0.clk
        .pll_clkout1    (pll_clkout1),    //  output,  width = 1,    pll_clkout1.clk
        .pll_clkout2    (pll_clkout2)     //  output,  width = 1,    pll_clkout2.clk
    );

    assign channel_pll_locked  = pll_locked;
    assign pll_refclk0 = i_clk_pll_ref;
    assign i_aib_clk = pll_clkout2;    // 402.83203125 MHz
    assign i_aib_2x_clk = pll_clkout1; // 805.6640625 MHz

    assign i_sl_clk_tx = {NUM_CHANNELS{i_aib_clk}};
    assign i_sl_clk_rx = {NUM_CHANNELS{i_aib_clk}};

    //---------------------------------------------------------------------------------------------
    // E-TILE INTERFACE IP   
    ex_10G #(
        .sim_mode                            (sim_mode)
    ) dut (
       .i_reconfig_clk(i_reconfig_clk),
       .i_reconfig_reset(i_reconfig_reset),
       .i_sampling_clk(i_sampling_clk),
       .i_clk_ref(i_clk_ref),
       .o_tx_clkout(o_tx_clkout),
       .o_tx_clkout2(o_tx_clkout2),
       .o_rx_clkout(o_rx_clkout),
       .o_rx_clkout2(o_rx_clkout2),
       .o_cdr_lock(o_cdr_lock),
       .o_tx_pll_locked(o_tx_pll_locked),
       .o_tx_serial(o_tx_serial),
       .i_rx_serial(i_rx_serial),
       .i_xcvr_reconfig_address(i_xcvr_reconfig_address),
       .i_xcvr_reconfig_read(i_xcvr_reconfig_read),
       .i_xcvr_reconfig_write(i_xcvr_reconfig_write),
       .o_xcvr_reconfig_readdata(o_xcvr_reconfig_readdata),
       .i_xcvr_reconfig_writedata(i_xcvr_reconfig_writedata),
       .o_xcvr_reconfig_waitrequest(o_xcvr_reconfig_waitrequest),
       .i_sl_cpri_reconfig_addr(i_sl_cpri_reconfig_addr),
       .i_sl_cpri_reconfig_read(i_sl_cpri_reconfig_read),
       .i_sl_cpri_reconfig_write(i_sl_cpri_reconfig_write),
       .o_sl_cpri_reconfig_readdata(o_sl_cpri_reconfig_readdata),
       .o_sl_cpri_reconfig_readdata_valid(o_sl_cpri_reconfig_readdata_valid),
       .i_sl_cpri_reconfig_writedata(i_sl_cpri_reconfig_writedata),
       .o_sl_cpri_reconfig_waitrequest(o_sl_cpri_reconfig_waitrequest),
       .o_sl_rx_hi_ber(o_sl_rx_hi_ber),
       .o_sl_tx_lanes_stable(o_sl_tx_lanes_stable),
       .o_sl_rx_pcs_ready(o_sl_rx_pcs_ready),
       .o_sl_ehip_ready(o_sl_ehip_ready),
       .o_sl_rx_block_lock(o_sl_rx_block_lock),
       .i_sl_clk_tx(i_sl_clk_tx),
       .i_sl_clk_rx(i_sl_clk_rx),
       .i_sl_csr_rst_n(i_sl_csr_rst_n),
       .i_sl_tx_rst_n(i_sl_tx_rst_n),
       .i_sl_rx_rst_n(i_sl_rx_rst_n),
       .i_aib_clk(i_aib_clk),
       .i_aib_2x_clk(i_aib_2x_clk),
       .i_sl_tx_mii_d(i_sl_tx_mii_d),
       .i_sl_tx_mii_c(i_sl_tx_mii_c),
       .o_sl_rx_mii_d(o_sl_rx_mii_d),
       .o_sl_rx_mii_c(o_sl_rx_mii_c)
    );


    // DL Logic
    
    wire [NUM_CHANNELS-1:0]         start_pulse;
    reg  [NUM_CHANNELS-1:0]         start_pulse_reg;
    wire [NUM_CHANNELS-1:0]         start_pulse_xgmii;
    wire [NUM_CHANNELS-1:0]         start_pulse_8b10b;
    wire [NUM_CHANNELS-1:0]         start_pulse_sclk;
    wire [NUM_CHANNELS-1:0]         stop_pulse;
    reg  [NUM_CHANNELS-1:0]         stop_pulse_reg;
    wire [NUM_CHANNELS-1:0]         stop_pulse_xgmii;
    wire [NUM_CHANNELS-1:0]         stop_pulse_8b10b;
    wire [NUM_CHANNELS-1:0]         stop_pulse_sclk;
    wire [NUM_CHANNELS-1:0]         rt_recount;
    wire [NUM_CHANNELS-1:0]         rt_cnt_done;
    wire [NUM_CHANNELS-1:0]         wa_status_sclk_sync;
    wire [NUM_CHANNELS-1:0]         restart_sync_sclk;
    wire [NUM_CHANNELS-1:0]         restart_txsync;
    wire [NUM_CHANNELS-1:0]         restart_rxsync;
    wire                            restart_sync;
    wire [NUM_CHANNELS-1:0]         verifier_lock_int;
    wire [NUM_CHANNELS-1:0]         verifier_error_xgmii_int;
    wire [NUM_CHANNELS-1:0]         verifier_error_8b10b_int;
    wire [NUM_CHANNELS-1:0]         error_flag;
    wire [NUM_CHANNELS-1:0][4:0]    error_count_xgmii;
    wire [NUM_CHANNELS-1:0][4:0]    error_count_8b10b;
    wire [NUM_CHANNELS-1:0]         rx_rst_pll_div66_sync;
    wire [NUM_CHANNELS-1:0]         o_sl_ehip_ready_plldiv66_sync;
    wire [NUM_CHANNELS-1:0]         o_sl_tx_ready_txclkout2_sync;
    wire [NUM_CHANNELS-1:0]         o_blklock_recdiv66_sync;
    wire [NUM_CHANNELS-1:0]         wa_8b10b_status_sclk_sync;
    wire [NUM_CHANNELS-1:0]         rt_reset;
    reg  [NUM_CHANNELS-1:0]         rx_pattern_aligned;
    // wire [NUM_CHANNELS-1:0][4:0]    cpri_rate_sel;
    wire [NUM_CHANNELS-1:0][3:0]    cpri_speed_txclk_sync;
    wire [NUM_CHANNELS-1:0][3:0]    cpri_speed_rxclk_sync;
    wire [NUM_CHANNELS-1:0]         cpri_speed_sclk_sync;
    wire [NUM_CHANNELS-1:0]         hyperframe_sync;
    wire [NUM_CHANNELS-1:0]         hyperframe_sync_xgmii;
    wire [NUM_CHANNELS-1:0]         hyperframe_sync_8b10b;
    wire [NUM_CHANNELS-1:0]         hyperframe_sync_sclk_sync;
    // wire [NUM_CHANNELS-1:0][CALC_WIDTH-1:0]           o_tx_delay_sclk;
    // wire [NUM_CHANNELS-1:0][CALC_WIDTH-1:0]           o_rx_delay_sclk;
    // real [NUM_CHANNELS-1:0]                           tx_delay_real;
    // real [NUM_CHANNELS-1:0]                           rx_delay_real;
    // real [NUM_CHANNELS-1:0]                           rt_delay_real;

    // assign                          tx_delay_real = (o_tx_delay_sclk / 2.0**M) * 4.0;
    // assign                          rx_delay_real = (o_rx_delay_sclk / 2.0**M) * 4.0;
    // assign                          rt_delay_real = ((rt_count / DL_TOT_MEASURE_CNT) * 4.0) - ((reconfig_to_cpri_10g_sync)? 6.79652722:2.831840181);
    
    //*********************** Reset synchonizer ****************************
    alt_cpriphy_c3_reset_synchronizer #(.CLKS_PER_SEC (100000000  ),
           .RESET_PER_NS (25       )
    ) restart_inst(
          .clk        (i_reconfig_clk  ),
          .reset_req  (restart   ),  // asynchronous reset request
          .reset      (restart_sync      ),  // synchronous reset out
          .reset_n    (/*unused*/     )   // synchronous reset_n out
    );

    genvar i;
    generate
    for (i=0; i<NUM_CHANNELS; i=i+1) begin:gen_check      
        alt_xcvr_resync_std #(
            .SYNC_CHAIN_LENGTH (2),  
            .INIT_VALUE        (1)
        ) restart_txsync_inst(
            .clk                       (o_tx_clkout2[i]),
            .reset                     (~i_sl_csr_rst_n[i]),
            .d                         (restart_sync),
            .q                         (restart_txsync[i]) // active high
        );    

        alt_xcvr_resync_std #(
            .SYNC_CHAIN_LENGTH (2),  
            .INIT_VALUE        (1)
        ) restart_rxsync_inst(
            .clk                       (o_rx_clkout2[i]),
            .reset                     (~i_sl_csr_rst_n[i]),
            .d                         (restart_sync),
            .q                         (restart_rxsync[i]) // active high
        );

        // alt_xcvr_resync_std #(
            // .SYNC_CHAIN_LENGTH (2),  // Number of flip-flops for retiming
            // .INIT_VALUE        (1)
        // ) rec_div66_rst_sync_inst(
            // .clk                       (o_tx_clkout2[i]),
            // .reset                     (~i_sl_rx_rst_n[i]),
            // .d                         (1'b0),
            // .q                         (rx_rst_pll_div66_sync[i]) // active high
        // );

        alt_xcvr_resync_std #(
            .SYNC_CHAIN_LENGTH (2),
            .INIT_VALUE        (0)
        ) stoppulse_sclksync_inst(
            .clk                       (i_sampling_clk),
            .reset                     (~i_sl_csr_rst_n[i]),
            .d                         (stop_pulse_reg[i]),
            .q                         (stop_pulse_sclk[i])
        );

        alt_xcvr_resync_std #(
            .SYNC_CHAIN_LENGTH (2),  
            .INIT_VALUE        (0)
        ) startpulse_sclksync_inst(
            .clk                       (i_sampling_clk),
            .reset                     (~i_sl_csr_rst_n[i]),
            .d                         (start_pulse_reg[i]),
            .q                         (start_pulse_sclk[i])
        );        

        alt_xcvr_resync_std #(
            .SYNC_CHAIN_LENGTH (2),  
            .INIT_VALUE        (1)
        ) restart_sclksync_inst(
            .clk                       (i_sampling_clk),
            .reset                     (~i_sl_csr_rst_n[i]),
            .d                         (restart_sync),
            .q                         (restart_sync_sclk[i]) // active high
        );    

        alt_xcvr_resync_std #(
            .SYNC_CHAIN_LENGTH (2),  
            .INIT_VALUE        (1)
        ) hyperframe_sync_sclksync_inst(
            .clk                       (i_sampling_clk),
            .reset                     (~hyperframe_sync[i]),
            .d                         (1'b0),
            .q                         (hyperframe_sync_sclk_sync[i])  // active high
        );

        alt_xcvr_resync_std #(
           .SYNC_CHAIN_LENGTH (2),  
           .INIT_VALUE        (0)
        ) ehip_ready_plldiv66_sync_inst(
           .clk                        (o_tx_clkout2[i]),
           .reset                      (~o_sl_ehip_ready[i]),
           .d                          (1'b1),
           .q                          (o_sl_ehip_ready_plldiv66_sync[i]) // active low
        );

        alt_xcvr_resync_std #(
           .SYNC_CHAIN_LENGTH (2),  
           .INIT_VALUE        (0)
        ) rx_blocklock_plldiv66_sync_inst(
           .clk                        (o_rx_clkout2[i]),
           .reset                      (~o_sl_rx_block_lock[i]),
           .d                          (1'b1),
           .q                          (o_blklock_recdiv66_sync[i]) // active low
        );

        alt_xcvr_resync_std #(
            .SYNC_CHAIN_LENGTH (2),  
            .INIT_VALUE        (1)
        ) rtcnt_rst_sclksync_inst(
            .clk                       (i_sampling_clk),
            .reset                     (~o_sl_rx_pcs_ready[i]),
            .d                         (1'b0),
            .q                         (wa_status_sclk_sync[i]) // active high
        );        

        alt_cpriphy_c3_pat_gen_xgmii pattern_gen_xgmii_inst (
            .clk                (o_tx_clkout2[i]),
            .reset              (~o_sl_ehip_ready_plldiv66_sync[i] | restart_txsync[i]),
            .cpri_rate_sel      (cpri_speed_txclk_sync[i]),
            .tx_dataout         (i_sl_tx_mii_d[i][63:0]),
            .tx_ctrlout         (i_sl_tx_mii_c[i][7:0]),
            .start              (start_pulse_xgmii[i])
        );

        alt_cpriphy_c3_pat_chk_xgmii #(
            .DATA_WIDTH(64)
        ) pattern_checker_xgmii_inst (
            .errorflag     (verifier_error_xgmii_int[i]), 
            .err_cnt       (error_count_xgmii[i]),
            .clk           (o_rx_clkout2[i]),
            .nreset        (o_blklock_recdiv66_sync[i] & ~restart_rxsync[i]),
            .datain        (o_sl_rx_mii_d[i][63:0]),
            .ctrl          (o_sl_rx_mii_c[i][7:0]),
            .cpri_rate_sel (cpri_speed_rxclk_sync[i]),
            .hyperframe_sync(hyperframe_sync_xgmii[i]),
            .stop          (stop_pulse_xgmii[i])
        );

        assign start_pulse_8b10b[i] = 1'b0;
        assign stop_pulse_8b10b[i] = 1'b0;
        assign verifier_error_8b10b_int[i] = 1'b0;
        assign error_count_8b10b[i] = 1'b0;
        assign wa_8b10b_status_sclk_sync[i] = 1'b0;
        assign hyperframe_sync_8b10b[i] = 1'b0;


        alt_cpriphy_c3_rt_acc_counter #(  //update rt_counter
            .DL_TOT_MEASURE_CNT (DL_TOT_MEASURE_CNT),
            .COUNT_WIDTH(COUNT_WIDTH)        
        ) rt_counter_inst     (
            .reset (rt_reset[i] | restart_sync_sclk[i] | hyperframe_sync_sclk_sync[i]), 
            .clk (i_sampling_clk),
            .start(start_pulse_sclk[i]),
            .stop(stop_pulse_sclk[i]),
            .trigger_recount(rt_recount[i]),
            .counter_out(rt_count[i]),
            .measure_done(rt_cnt_done[i])
        );    
    
        alt_xcvr_resync_std #(
                .SYNC_CHAIN_LENGTH (2),
                .WIDTH(4),
                .INIT_VALUE        (0)
        ) __syncdata__cpri_speed_txclksync_inst(
                .clk                       (o_tx_clkout2[i]),
                .reset                     (),
                .d                         (cpri_speed),
                .q                         (cpri_speed_txclk_sync[i])
        );
		 
        alt_xcvr_resync_std #(
                .SYNC_CHAIN_LENGTH (2),
                .WIDTH(4),
                .INIT_VALUE        (0)
        ) __syncdata__cpri_speed_rxclksync_inst(
                .clk                       (o_rx_clkout2[i]),
                .reset                     (),
                .d                         (cpri_speed),
                .q                         (cpri_speed_rxclk_sync[i])
        );
		 
        alt_xcvr_resync_std #(
                .SYNC_CHAIN_LENGTH (2),
                .INIT_VALUE        (0)
        ) __syncdata__cpri_speed_sclksync_inst(
                .clk                       (i_sampling_clk),
                .reset                     (),
                .d                         (cpri_speed[3]),
                .q                         (cpri_speed_sclk_sync[i])
        );
        
//        assign cpri_rate_sel[i] = (SPEED == "24G")  ? 4'b1011 : 
//                                  (SPEED == "12G")  ? 4'b1010 : 
//                                  (SPEED == "10G")  ? 4'b1001 : 
//                                  (SPEED == "9_8G") ? 4'b0110 : 
//                                  (SPEED == "6G")   ? 4'b0101 : 
//                                  (SPEED == "4_9G") ? 4'b0100 : 
//                                  (SPEED == "3G")   ? 4'b0011 : 
//                                  (SPEED == "2_4G") ? 4'b0010 : 
//                                                      4'b1001;
        assign rt_reset[i] = (cpri_speed_sclk_sync[i] == 1'b0) ? wa_8b10b_status_sclk_sync[i] : wa_status_sclk_sync[i];
        assign start_pulse[i] = (cpri_speed_txclk_sync[i][3] == 1'b0) ? start_pulse_8b10b[i] : start_pulse_xgmii[i];
        assign stop_pulse[i] = (cpri_speed_rxclk_sync[i][3] == 1'b0) ? stop_pulse_8b10b[i] : stop_pulse_xgmii[i];
        assign error_flag[i] = (cpri_speed_rxclk_sync[i][3] == 1'b0) ? verifier_error_8b10b_int[i] : verifier_error_xgmii_int[i];
        assign hyperframe_sync[i] = (cpri_speed_rxclk_sync[i][3] == 1'b0) ? hyperframe_sync_8b10b[i] : hyperframe_sync_xgmii[i];
        assign checker_pass[i]     = ((error_flag[i] == 1'b0) && (rt_cnt_done[i] == 1'b1)) ? 1'b1:1'b0;

        always @ (posedge o_tx_clkout2[i]) begin
                start_pulse_reg[i] <= start_pulse[i];
        end
		 
        always @ (posedge o_rx_clkout2[i]) begin
                stop_pulse_reg[i] <= stop_pulse[i];
        end
    end
    endgenerate

endmodule

