// (C) 2001-2022 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.


`timescale 1 ps / 1 fs
module basic_avl_tb_top ();
    localparam NUM_CHANNELS  = 4;
    localparam MAX_NUM_CHANNELS = 4;
    localparam COUNT_WIDTH        = 20;
    localparam DL_TOT_MEASURE_CNT = 128;
    localparam N                  = COUNT_WIDTH - $clog2(DL_TOT_MEASURE_CNT);
    localparam M                  = $clog2(DL_TOT_MEASURE_CNT) + 1;
    localparam CALC_WIDTH         = N + M;
    localparam DL_TOT_MEASURE_CNT_FOR_RT = 32;

    reg                             i_clk_ref = '0;
    wire [NUM_CHANNELS-1:0]         o_tx_clkout;
    wire [NUM_CHANNELS-1:0]         o_tx_clkout2;
    wire [NUM_CHANNELS-1:0]         o_rx_clkout;
    wire [NUM_CHANNELS-1:0]         o_rx_clkout2;

    reg                             i_reconfig_clk = 1'b0;
    reg                             i_reconfig_reset;
    
    wire [MAX_NUM_CHANNELS-1:0]     i_xcvr_reconfig_write;
    wire [MAX_NUM_CHANNELS-1:0]     i_xcvr_reconfig_read;
    wire [19*MAX_NUM_CHANNELS-1:0]  i_xcvr_reconfig_address;
    wire [8*MAX_NUM_CHANNELS-1:0]   i_xcvr_reconfig_writedata;
    wire [8*MAX_NUM_CHANNELS-1:0]   o_xcvr_reconfig_readdata;
    wire [MAX_NUM_CHANNELS-1:0]     o_xcvr_reconfig_waitrequest;
    
    wire [MAX_NUM_CHANNELS-1:0]     i_sl_cpri_reconfig_write;
    wire [MAX_NUM_CHANNELS-1:0]     i_sl_cpri_reconfig_read;
    wire [19*MAX_NUM_CHANNELS-1:0]  i_sl_cpri_reconfig_addr;
    wire [32*MAX_NUM_CHANNELS-1:0]  i_sl_cpri_reconfig_writedata;
    wire [32*MAX_NUM_CHANNELS-1:0]  o_sl_cpri_reconfig_readdata;
    wire [MAX_NUM_CHANNELS-1:0]     o_sl_cpri_reconfig_readdata_valid;
    wire [MAX_NUM_CHANNELS-1:0]     o_sl_cpri_reconfig_waitrequest;
        
    wire [NUM_CHANNELS-1:0]         o_tx_serial;
    wire [NUM_CHANNELS-1:0]         i_rx_serial;
    
    // Reset
    reg  [NUM_CHANNELS-1:0]         i_sl_csr_rst_n;
    reg  [NUM_CHANNELS-1:0]         i_sl_tx_rst_n;
    reg  [NUM_CHANNELS-1:0]         i_sl_rx_rst_n;
    
    reg                             ref_clk156MHz = 1'b0;
    wire                            tx_serial_data;
    wire                            channel_pll_locked;
    
    
    wire [NUM_CHANNELS-1:0]         o_tx_pll_locked;
    wire [NUM_CHANNELS-1:0]         o_sl_tx_lanes_stable;
    wire [NUM_CHANNELS-1:0]         o_sl_rx_block_lock;
    wire [NUM_CHANNELS-1:0]         o_sl_rx_pcs_ready;
    wire [NUM_CHANNELS-1:0]         o_sl_ehip_ready;
    wire [NUM_CHANNELS-1:0]         o_sl_rx_hi_ber;
    wire [NUM_CHANNELS-1:0]         o_cdr_lock;
    
    wire                            iopll_sclk_locked;
    wire [NUM_CHANNELS-1:0]         checker_pass;
    reg                             polling = 1'b0;
    reg                             restart = 1'b0;
    reg [NUM_CHANNELS-1:0][COUNT_WIDTH-1:0]          rt_count;
    reg [NUM_CHANNELS-1:0][CALC_WIDTH-1:0]           tx_delay_sclk;
    reg [NUM_CHANNELS-1:0][CALC_WIDTH-1:0]           rx_delay_sclk;
    reg [NUM_CHANNELS-1:0][CALC_WIDTH-1:0]           bitslip;
    reg [NUM_CHANNELS-1:0][4:0]                      bitslip_8b10b;
    reg [NUM_CHANNELS-1:0]          ch_reset_queue[0:NUM_CHANNELS-1];
	 
    wire [NUM_CHANNELS-1:0][3:0]    cpri_speed;
	 
    assign cpri_speed = 4'b1001; // 10.1376G

    // DUT
    alt_cpriphy_c3_top #(
        .sim_mode                            ("enable"),
        .DL_TOT_MEASURE_CNT                  (DL_TOT_MEASURE_CNT_FOR_RT) // For RT counter only
    ) dut_wrapper (
        
        // Clocks
        .i_clk_ref                           ({4'b0,i_clk_ref}),
        .i_reconfig_clk                      (i_reconfig_clk),
        .o_tx_clkout                         (o_tx_clkout),
        .o_rx_clkout                         (o_rx_clkout),
        .o_tx_clkout2                        (o_tx_clkout2),
        .o_rx_clkout2                        (o_rx_clkout2),

        // Resets
        .i_sl_tx_rst_n                       (i_sl_tx_rst_n),
        .i_sl_rx_rst_n                       (i_sl_rx_rst_n),
        .i_sl_csr_rst_n                      (i_sl_csr_rst_n),
        
        .i_reconfig_reset                    (i_reconfig_reset),  //active high
        
        
        .i_clk_pll_ref                       (ref_clk156MHz),
        .tx_serial_data                      (tx_serial_data),
        .channel_pll_locked                  (channel_pll_locked),

        // Tranceiver - 24.33024Gbps for 24G, 10.1376 for 10G
        .i_rx_serial                         (i_rx_serial),
        .o_tx_serial                         (o_tx_serial),
        
        .i_sl_cpri_reconfig_addr             (i_sl_cpri_reconfig_addr[19*NUM_CHANNELS-1:0]),
        .i_sl_cpri_reconfig_read             (i_sl_cpri_reconfig_read[NUM_CHANNELS-1:0]),
        .i_sl_cpri_reconfig_write            (i_sl_cpri_reconfig_write[NUM_CHANNELS-1:0]),
        .i_sl_cpri_reconfig_writedata        (i_sl_cpri_reconfig_writedata[32*NUM_CHANNELS-1:0]),
        .o_sl_cpri_reconfig_readdata         (o_sl_cpri_reconfig_readdata[32*NUM_CHANNELS-1:0]),
        .o_sl_cpri_reconfig_readdata_valid   (o_sl_cpri_reconfig_readdata_valid[NUM_CHANNELS-1:0]),
        .o_sl_cpri_reconfig_waitrequest      (o_sl_cpri_reconfig_waitrequest[NUM_CHANNELS-1:0]),
        
        // XCVR CSR AVMM
        .i_xcvr_reconfig_address             (i_xcvr_reconfig_address[19*NUM_CHANNELS-1:0]),
        .i_xcvr_reconfig_read                (i_xcvr_reconfig_read[NUM_CHANNELS-1:0]),
        .i_xcvr_reconfig_write               (i_xcvr_reconfig_write[NUM_CHANNELS-1:0]),
        .i_xcvr_reconfig_writedata           (i_xcvr_reconfig_writedata[8*NUM_CHANNELS-1:0]),
        .o_xcvr_reconfig_readdata            (o_xcvr_reconfig_readdata[8*NUM_CHANNELS-1:0]),
        .o_xcvr_reconfig_waitrequest         (o_xcvr_reconfig_waitrequest[NUM_CHANNELS-1:0]),
        
        // Core Status
        .o_tx_pll_locked                     (o_tx_pll_locked),
        .o_cdr_lock                          (o_cdr_lock),
        .o_sl_tx_lanes_stable                (o_sl_tx_lanes_stable),
        .o_sl_rx_block_lock                  (o_sl_rx_block_lock),
        .o_sl_rx_pcs_ready                   (o_sl_rx_pcs_ready),
        .o_sl_ehip_ready                     (o_sl_ehip_ready),
        .o_sl_rx_hi_ber                      (o_sl_rx_hi_ber),
        
        .iopll_sclk_locked                   (iopll_sclk_locked),
        .checker_pass                        (checker_pass),
        .restart                             (restart),
        .rt_count                            (rt_count),
        .ehip_mode_sl_0                      (cpri_speed[3]),
        .cpri_speed                          (cpri_speed)
    );
    
    //---------------------------------------------------------------------------------------------
    // JTAG2AVALON MASTER AND AVMM'S SLAVE SPLITTER BLOCK
    reg    [31:0]    avmm_mst_writedata = 'h0;
    reg              avmm_mst_read = 'h0;
    reg    [31:0]    avmm_mst_address = 'h0;
    reg              avmm_mst_write = 'h0;
    wire   [31:0]    avmm_mst_readdata;
    wire             avmm_mst_waitrequest;
    wire             avmm_mst_readdata_valid;
    
    wire    [24*MAX_NUM_CHANNELS-1:0]   i_unused_xcvr_reconfig_writedata;

    alt_cpriphy_c3_avmm_if alt_cpriphy_c3_avmm_if (
       .i_reconfig_clk                       (i_reconfig_clk), 

       .i_avmm_resetn                        (~i_reconfig_reset),
       
       .avmm_mst_writedata                   (avmm_mst_writedata),
       .avmm_mst_read                        (avmm_mst_read),
       .avmm_mst_address                     (avmm_mst_address),
       .avmm_mst_write                       (avmm_mst_write),
       .avmm_mst_readdata                    (avmm_mst_readdata),
       .avmm_mst_waitrequest                 (avmm_mst_waitrequest),
       .avmm_mst_readdata_valid              (avmm_mst_readdata_valid),

       .o_ch0_cpri_reconfig_addr             (i_sl_cpri_reconfig_addr[(0+1)*19-1:(0)*19]),                            //  output,  width = 19, 
       .o_ch0_cpri_reconfig_write            (i_sl_cpri_reconfig_write[0]),                                 //  output,   width = 1, 
       .o_ch0_cpri_reconfig_read             (i_sl_cpri_reconfig_read[0]),                                  //  output,   width = 1, 
       .o_ch0_cpri_reconfig_writedata        (i_sl_cpri_reconfig_writedata[(0+1)*32-1:(0)*32]),                             //  output,  width = 32, 
       .i_ch0_cpri_reconfig_readdata         (o_sl_cpri_reconfig_readdata[(0+1)*32-1:(0)*32]),                              //   input,  width = 32, 
       .i_ch0_cpri_reconfig_readdata_valid   (o_sl_cpri_reconfig_readdata_valid[0]), //   input,   width = 1,
       .i_ch0_cpri_reconfig_waitrequest      (o_sl_cpri_reconfig_waitrequest[0]),                           //   input,   width = 1,

       .o_ch1_cpri_reconfig_addr             (i_sl_cpri_reconfig_addr[(1+1)*19-1:(1)*19]),                            //  output,  width = 19, 
       .o_ch1_cpri_reconfig_write            (i_sl_cpri_reconfig_write[1]),                                 //  output,   width = 1, 
       .o_ch1_cpri_reconfig_read             (i_sl_cpri_reconfig_read[1]),                                  //  output,   width = 1, 
       .o_ch1_cpri_reconfig_writedata        (i_sl_cpri_reconfig_writedata[(1+1)*32-1:1*32]),                             //  output,  width = 32, 
       .i_ch1_cpri_reconfig_readdata         (o_sl_cpri_reconfig_readdata[(1+1)*32-1:1*32]),                              //   input,  width = 32, 
       .i_ch1_cpri_reconfig_readdata_valid   (o_sl_cpri_reconfig_readdata_valid[1]), //   input,   width = 1,
       .i_ch1_cpri_reconfig_waitrequest      (o_sl_cpri_reconfig_waitrequest[1]),                           //   input,   width = 1,

       .o_ch2_cpri_reconfig_addr             (i_sl_cpri_reconfig_addr[(2+1)*19-1:(2)*19]),                            //  output,  width = 19, 
       .o_ch2_cpri_reconfig_write            (i_sl_cpri_reconfig_write[2]),                                 //  output,   width = 1, 
       .o_ch2_cpri_reconfig_read             (i_sl_cpri_reconfig_read[2]),                                  //  output,   width = 1, 
       .o_ch2_cpri_reconfig_writedata        (i_sl_cpri_reconfig_writedata[(2+1)*32-1:(2)*32]),                             //  output,  width = 32, 
       .i_ch2_cpri_reconfig_readdata         (o_sl_cpri_reconfig_readdata[(2+1)*32-1:(2)*32]),                              //   input,  width = 32, 
       .i_ch2_cpri_reconfig_readdata_valid   (o_sl_cpri_reconfig_readdata_valid[2]), //   input,   width = 1,
       .i_ch2_cpri_reconfig_waitrequest      (o_sl_cpri_reconfig_waitrequest[2]),                           //   input,   width = 1,

       .o_ch3_cpri_reconfig_addr             (i_sl_cpri_reconfig_addr[(3+1)*19-1:(3)*19]),                            //  output,  width = 19, 
       .o_ch3_cpri_reconfig_write            (i_sl_cpri_reconfig_write[3]),                                 //  output,   width = 1, 
       .o_ch3_cpri_reconfig_read             (i_sl_cpri_reconfig_read[3]),                                  //  output,   width = 1, 
       .o_ch3_cpri_reconfig_writedata        (i_sl_cpri_reconfig_writedata[(3+1)*32-1:(3)*32]),                             //  output,  width = 32, 
       .i_ch3_cpri_reconfig_readdata         (o_sl_cpri_reconfig_readdata[(3+1)*32-1:(3)*32]),                              //   input,  width = 32, 
       .i_ch3_cpri_reconfig_readdata_valid   (o_sl_cpri_reconfig_readdata_valid[3]), //   input,   width = 1,
       .i_ch3_cpri_reconfig_waitrequest      (o_sl_cpri_reconfig_waitrequest[3]),                           //   input,   width = 1,

       .ch0_pkt_avl_mm_read                  (),
       .ch0_pkt_avl_mm_write                 (),
       .ch0_pkt_avl_mm_waitrequest           ('h0),
       .ch0_pkt_avl_mm_baddress              (),
       .ch0_pkt_avl_mm_readdata              ('h0),
       .ch0_pkt_avl_mm_writedata             (),

       .ch1_pkt_avl_mm_read                  (),
       .ch1_pkt_avl_mm_write                 (),
       .ch1_pkt_avl_mm_waitrequest           ('h0),
       .ch1_pkt_avl_mm_baddress              (),
       .ch1_pkt_avl_mm_readdata              ('h0),
       .ch1_pkt_avl_mm_writedata             (),

       .ch2_pkt_avl_mm_read                  (),
       .ch2_pkt_avl_mm_write                 (),
       .ch2_pkt_avl_mm_waitrequest           ('h0),
       .ch2_pkt_avl_mm_baddress              (),
       .ch2_pkt_avl_mm_readdata              ('h0),
       .ch2_pkt_avl_mm_writedata             (),

       .ch3_pkt_avl_mm_read                  (),
       .ch3_pkt_avl_mm_write                 (),
       .ch3_pkt_avl_mm_waitrequest           ('h0),
       .ch3_pkt_avl_mm_baddress              (),
       .ch3_pkt_avl_mm_readdata              ('h0),
       .ch3_pkt_avl_mm_writedata             (),

       .o_ch0_xcvr_reconfig_addr             (i_xcvr_reconfig_address[(0+1)*19-1:(0)*19]),
       .o_ch0_xcvr_reconfig_read             (i_xcvr_reconfig_read[0]),
       .o_ch0_xcvr_reconfig_write            (i_xcvr_reconfig_write[0]),
       .o_ch0_xcvr_reconfig_writedata        ({i_unused_xcvr_reconfig_writedata[(0+1)*24-1:(0)*24],i_xcvr_reconfig_writedata[(0+1)*8-1:(0)*8]}),
       .i_ch0_xcvr_reconfig_readdata         ({24'b0,o_xcvr_reconfig_readdata[(0+1)*8-1:(0)*8]}),
       .i_ch0_xcvr_reconfig_readdata_valid   (i_xcvr_reconfig_read[0] & ~o_xcvr_reconfig_waitrequest[0]),
       .i_ch0_xcvr_reconfig_waitrequest      (o_xcvr_reconfig_waitrequest[0]),
 
       .o_ch1_xcvr_reconfig_addr             (i_xcvr_reconfig_address[(1+1)*19-1:(1)*19]),
       .o_ch1_xcvr_reconfig_read             (i_xcvr_reconfig_read[1]),
       .o_ch1_xcvr_reconfig_write            (i_xcvr_reconfig_write[1]),
       .o_ch1_xcvr_reconfig_writedata        ({i_unused_xcvr_reconfig_writedata[(1+1)*24-1:(1)*24],i_xcvr_reconfig_writedata[(1+1)*8-1:(1)*8]}),
       .i_ch1_xcvr_reconfig_readdata         ({24'b0,o_xcvr_reconfig_readdata[(1+1)*8-1:(1)*8]}),
       .i_ch1_xcvr_reconfig_readdata_valid   (i_xcvr_reconfig_read[1] & ~o_xcvr_reconfig_waitrequest[1]),
       .i_ch1_xcvr_reconfig_waitrequest      (o_xcvr_reconfig_waitrequest[1]),

       .o_ch2_xcvr_reconfig_addr             (i_xcvr_reconfig_address[(2+1)*19-1:(2)*19]),
       .o_ch2_xcvr_reconfig_read             (i_xcvr_reconfig_read[2]),
       .o_ch2_xcvr_reconfig_write            (i_xcvr_reconfig_write[2]),
       .o_ch2_xcvr_reconfig_writedata        ({i_unused_xcvr_reconfig_writedata[(2+1)*24-1:(2)*24],i_xcvr_reconfig_writedata[(2+1)*8-1:(2)*8]}),
       .i_ch2_xcvr_reconfig_readdata         ({24'b0,o_xcvr_reconfig_readdata[(2+1)*8-1:(2)*8]}),
       .i_ch2_xcvr_reconfig_readdata_valid   (i_xcvr_reconfig_read[2] & ~o_xcvr_reconfig_waitrequest[2]),
       .i_ch2_xcvr_reconfig_waitrequest      (o_xcvr_reconfig_waitrequest[2]),

       .o_ch3_xcvr_reconfig_addr             (i_xcvr_reconfig_address[(3+1)*19-1:(3)*19]),
       .o_ch3_xcvr_reconfig_read             (i_xcvr_reconfig_read[3]),
       .o_ch3_xcvr_reconfig_write            (i_xcvr_reconfig_write[3]),
       .o_ch3_xcvr_reconfig_writedata        ({i_unused_xcvr_reconfig_writedata[(3+1)*24-1:(3)*24],i_xcvr_reconfig_writedata[(3+1)*8-1:(3)*8]}),
       .i_ch3_xcvr_reconfig_readdata         ({24'b0,o_xcvr_reconfig_readdata[(3+1)*8-1:(3)*8]}),
       .i_ch3_xcvr_reconfig_readdata_valid   (i_xcvr_reconfig_read[3] & ~o_xcvr_reconfig_waitrequest[3]),
       .i_ch3_xcvr_reconfig_waitrequest      (o_xcvr_reconfig_waitrequest[3]),
 
       .o_rsfec_reconfig_addr                (i_rsfec_reconfig_addr),
       .o_rsfec_reconfig_read                (i_rsfec_reconfig_read),
       .o_rsfec_reconfig_write               (i_rsfec_reconfig_write),
       .i_rsfec_reconfig_readdata            (o_rsfec_reconfig_readdata),
       .o_rsfec_reconfig_writedata           (i_rsfec_reconfig_writedata),
       .i_rsfec_reconfig_waitrequest         (o_rsfec_reconfig_waitrequest),
    
       .pio_stat0                            ('h0),
       .pio_ctrl0                            ()
    );

    // wait for RX alignment then enable TX
    initial begin
        #2000000;
        // Print clock info
        $display("Ref clock is 184.32 MHz");
        
        $display ("waiting for EHIP Ready....");
        wait (&o_sl_ehip_ready === 1'b1);
        $display ("EHIP READY is 1 at time  ", $time);
        
        // Turnon internal serial loopback for all channels
        $display ("Enable internal serial loopback...");
        for (int ch=0; ch<NUM_CHANNELS; ch++)
        begin
           enable_serial_loopback (ch);
        end
        $display ("Internal serial loopback is enabled");
        
        $display ("Waiting for RX Block Lock");
        wait (&o_sl_rx_block_lock === 1'b1);
        $display ("RX Block Lock is high at time  ", $time);
        
        $display ("Waiting for RX ready");
        while (&o_sl_rx_pcs_ready !== 1'b1) @(negedge i_reconfig_clk);
        $display ("RX is ready is high at time  ", $time);

        
        // send_packets_10G24G_cpri(3);
        dl_10G24G_cpri;

        $display("**");
        $display("*****************************************");
        #10000;
        $finish();
    end
    
    // Periods rounded for simulation simplification
    // Multiply reported frequencies by (33/32) to get correct frequencies
    always begin
        // #2712.67361111111111111111111111 i_clk_ref = ~i_clk_ref;
        // 184.32 MHz clock is used for simulation
        #2713 i_clk_ref = ~i_clk_ref;
    end

    // 156.25 MHz clock is used for channel pll
    parameter refclk_freq = 3200; 
    always begin
        #refclk_freq  ref_clk156MHz = ~ref_clk156MHz;
    end

    always begin
        #5000 i_reconfig_clk = ~i_reconfig_clk;
    end
    
    localparam  BASE_ADDR_S0_CH0_CPRI_RECONFIG     = 32'h00000000;
    localparam  BASE_ADDR_S1_CH1_CPRI_RECONFIG     = 32'h01000000;
    localparam  BASE_ADDR_S2_CH2_CPRI_RECONFIG     = 32'h02000000;
    localparam  BASE_ADDR_S3_CH3_CPRI_RECONFIG     = 32'h03000000;

    localparam  BASE_ADDR_S4_CH0_XCVR_RECONFIG     = 32'h04000000;
    localparam  BASE_ADDR_S5_CH1_XCVR_RECONFIG     = 32'h05000000;
    localparam  BASE_ADDR_S6_CH2_XCVR_RECONFIG     = 32'h06000000;
    localparam  BASE_ADDR_S7_CH3_XCVR_RECONFIG     = 32'h07000000;

    localparam  BASE_ADDR_S8_CH0_PKTGEN_RECONFIG   = 32'h08000000;
    localparam  BASE_ADDR_S9_CH1_PKTGEN_RECONFIG   = 32'h09000000;
    localparam  BASE_ADDR_S10_CH2_PKTGEN_RECONFIG  = 32'h0A000000;
    localparam  BASE_ADDR_S11_CH3_PKTGEN_RECONFIG  = 32'h0B000000;

    localparam  BASE_ADDR_S12_RSFEC_RECONFIG       = 32'h0C000000;

    initial begin
        i_sl_csr_rst_n        = 0;
        i_sl_tx_rst_n         = 0;
        i_sl_rx_rst_n         = 0;
        i_reconfig_reset      = 1;

        wait (channel_pll_locked == 1'b1 && iopll_sclk_locked == 1'b1)
        i_sl_csr_rst_n[0]     = 1;
        i_sl_tx_rst_n[0]      = 1;
        i_sl_rx_rst_n[0]      = 1;
        i_reconfig_reset      = 0;
       
        wait (o_sl_ehip_ready[0] == 1'b1)
        i_sl_csr_rst_n[1]     = 1;
        i_sl_tx_rst_n[1]      = 1;
        i_sl_rx_rst_n[1]      = 1;
        i_sl_csr_rst_n[2]     = 1;
        i_sl_tx_rst_n[2]      = 1;
        i_sl_rx_rst_n[2]      = 1;
        i_sl_csr_rst_n[3]     = 1;
        i_sl_tx_rst_n[3]      = 1;
        i_sl_rx_rst_n[3]      = 1;

    end

    initial begin
       #2500000ns;
       $display("TIMEOUT!! @ %.2f ps", $time);
       $finish;
    end
  
    //---------------------------------------------------------------------------------------------
    // AVMM Master Write Task
    task avmm_write;
        input logic [31:0] addr;
        input logic [31:0] data;
        avmm_mst_address  = addr;

        @(posedge i_reconfig_clk);
        avmm_mst_write     = 1'b1;
        avmm_mst_writedata = data;
        $display("** Address offset = 0x%0h, WriteData = 0x%h", avmm_mst_address[19:2], avmm_mst_writedata);
        @(posedge i_reconfig_clk);
        while (avmm_mst_waitrequest !== 1'b0) @(posedge i_reconfig_clk);
        avmm_mst_write = 1'b0;
    endtask

    //---------------------------------------------------------------------------------------------
    // AVMM Master Read Task
    task avmm_read;
        input logic [31:0]      addr;
        input logic [31:0]      refdout;
        input logic             byte0_only;

        avmm_mst_address  = addr;
        @(posedge i_reconfig_clk);
        avmm_mst_read  = 1'b1;
        @(negedge i_reconfig_clk);
        while (avmm_mst_waitrequest !== 1'b0) @(posedge i_reconfig_clk);
        avmm_mst_read = 1'b0;
        while (avmm_mst_readdata_valid !== 1'b1) @(posedge i_reconfig_clk);
        if (refdout[31:0]==avmm_mst_readdata[31:0]) begin 
          $display("** Address offset = 0x%0h, ReadData  = 0x%h", avmm_mst_address[19:2], avmm_mst_readdata);
          polling = 0;
        end else if (byte0_only & (refdout[7:0]==avmm_mst_readdata[7:0])) begin
          $display("** Address offset = 0x%0h, ReadData  = 0x%h", avmm_mst_address[19:2], avmm_mst_readdata);
          polling = 0;
        end else begin
          // $display("** ERROR - Base Address = %h, Address offset = %h, ReadData = %h, Expected_ReadData = %h", {avmm_mst_address[31:20],20'h0}, avmm_mst_address[19:2], avmm_mst_readdata, refdout[31:0]);
          // csr_err = 1;
          // $display("** Polling... Address offset = %h, ReadData  = %h", avmm_mst_address[19:2], avmm_mst_readdata);
          polling = 1;
        end
    endtask

    task avmm_readdata;
        input logic [31:0]      addr;
        output logic [31:0]     readdata;

        avmm_mst_address  = addr;
        @(posedge i_reconfig_clk);
        avmm_mst_read  = 1'b1;
        @(negedge i_reconfig_clk);
        while (avmm_mst_waitrequest !== 1'b0) @(posedge i_reconfig_clk);
        while (avmm_mst_readdata_valid !== 1'b1) @(posedge i_reconfig_clk);
        avmm_mst_read = 1'b0;
        $display("** Address offset = 0x%0h, ReadData  = 0x%h", avmm_mst_address[19:2], avmm_mst_readdata);
        readdata = avmm_mst_readdata;
        while (avmm_mst_readdata_valid === 1'b1) @(posedge i_reconfig_clk);
    endtask
   
    task avmm_read_bit;
        input logic [31:0]      addr;
        input logic [31:0]      refdout;
        input logic [31:0]      byte0_only;

        avmm_mst_address  = addr;
        @(posedge i_reconfig_clk);
        avmm_mst_read  = 1'b1;
        @(negedge i_reconfig_clk);
        while (avmm_mst_waitrequest !== 1'b0) @(posedge i_reconfig_clk);
        while (avmm_mst_readdata_valid !== 1'b1) @(posedge i_reconfig_clk);
        avmm_mst_read = 1'b0;
        if (refdout[byte0_only]==avmm_mst_readdata[byte0_only]) begin
          $display("** Address offset = 0x%0h[%0h], ReadData  = 0x%h", avmm_mst_address[19:2], byte0_only, avmm_mst_readdata[byte0_only]);
          polling = 0;
        end else begin
          // $display("** ERROR - Base Address = %h, Address offset = %h, ReadData[%h] = %h, Expected_ReadData = %h", {avmm_mst_address[31:20],20'h0}, avmm_mst_address[19:2], byte0_only, avmm_mst_readdata[byte0_only], refdout[byte0_only]);
          // $display("** Polling... Address offset = %h, ReadData[%h]  = %h", avmm_mst_address[19:2], byte0_only, avmm_mst_readdata[byte0_only]);
          polling = 1;
        end
        while (avmm_mst_readdata_valid === 1'b1) @(posedge i_reconfig_clk);
    endtask
    
    task enable_serial_loopback;
        input logic [31:0]      ch;
        logic [31:0]            base_addr;
        base_addr =  (ch == 0) ? BASE_ADDR_S4_CH0_XCVR_RECONFIG:
                     (ch == 1) ? BASE_ADDR_S5_CH1_XCVR_RECONFIG:
                     (ch == 2) ? BASE_ADDR_S6_CH2_XCVR_RECONFIG:
                                 BASE_ADDR_S7_CH3_XCVR_RECONFIG;

        //PMA serial internal loopback
        avmm_write (base_addr | 32'h84<<2, 32'h1);
        avmm_write (base_addr | 32'h85<<2, 32'h1);
        avmm_write (base_addr | 32'h86<<2, 32'h8);
        avmm_write (base_addr | 32'h87<<2, 32'h0);
        avmm_write (base_addr | 32'h90<<2, 32'h1);
        // avmm_write (base_addr | 32'h8a<<2, 32'h80);
        #2us;
        $display("** Reading address 0x8a[7] until it changes to 1...");
        do begin
            avmm_read_bit  (base_addr | 32'h8a<<2, 32'h80, 7);
        end while (polling == 1);
        #2us
        $display("** Reading address 0x8b[0] until it changes to 0...");
        do begin
            avmm_read_bit  (base_addr | 32'h8b<<2, 32'h0, 0);
        end while (polling == 1);
        // $display("** Reading address 0x88 until it changes to 0x8...");
        // do begin
            // avmm_read  (base_addr | 32'h88<<2, 32'h8, 1);
        // end while (polling == 1);
        // avmm_read  (base_addr | 32'h89<<2, 32'h0, 1);
        avmm_write (base_addr | 32'h8a<<2, 32'h80);
    endtask

    task wait_serdes_powerup;
        input logic [31:0]      ch;
        logic [31:0]            base_addr;
        base_addr =  (ch == 0) ? BASE_ADDR_S4_CH0_XCVR_RECONFIG:
                     (ch == 1) ? BASE_ADDR_S5_CH1_XCVR_RECONFIG:
                     (ch == 2) ? BASE_ADDR_S6_CH2_XCVR_RECONFIG:
                                 BASE_ADDR_S7_CH3_XCVR_RECONFIG;

        // Wait for serdes powerup to complete
        do begin
            avmm_read_bit  (base_addr | 32'h8b<<2, 32'h0, 0);
        end while (polling == 1);
    endtask

    task dl_10G24G_cpri;
        integer i;
        logic [31:0]            base_addr;
        logic [31:0]            base_addr_xcvr;
        logic [31:0]            readdata;
        real                    tx_delay_real[NUM_CHANNELS-1:0];
        real                    rx_delay_real[NUM_CHANNELS-1:0];
        real                    rt_delay_real[NUM_CHANNELS-1:0];
    
        begin
            $display("*** sending packets in progress, waiting for checker pass ***");
            fork
                for (i = 0; i < NUM_CHANNELS; i = i +1) begin
                    base_addr =  (i == 0) ? BASE_ADDR_S0_CH0_CPRI_RECONFIG:
                                 (i == 1) ? BASE_ADDR_S1_CH1_CPRI_RECONFIG:
                                 (i == 2) ? BASE_ADDR_S2_CH2_CPRI_RECONFIG:
                                            BASE_ADDR_S3_CH3_CPRI_RECONFIG;

                    base_addr_xcvr =  (i == 0) ? BASE_ADDR_S4_CH0_XCVR_RECONFIG:
                                      (i == 1) ? BASE_ADDR_S5_CH1_XCVR_RECONFIG:
                                      (i == 2) ? BASE_ADDR_S6_CH2_XCVR_RECONFIG:
                                                 BASE_ADDR_S7_CH3_XCVR_RECONFIG;

                    // Poll for measure_valid
                    $display("*** waiting for measure_valid to assert...");
                    do begin
                        avmm_read_bit  (base_addr | 32'hC01<<2, 32'h1, 0);
                    end while (polling == 1);
                    $display("** measure_valid is asserted.");
                    avmm_readdata (base_addr | 32'hC02<<2, tx_delay_sclk[i]);
                    avmm_readdata (base_addr | 32'hC03<<2, rx_delay_sclk[i]);
                    // non FEC bitslip
                    avmm_readdata (base_addr_xcvr | 32'h28<<2, readdata);
                    bitslip[i] = readdata & 'h7f;
                    $display("*** waiting for hyperframe sync to assert...");
                    wait (dut_wrapper.hyperframe_sync[i]  == 1'b1);
                    $display("** hyperframe sync is asserted.");
                    $display("*** waiting for round trip measure...");
                    wait (dut_wrapper.rt_cnt_done[i]  == 1'b1)
                    $display("-> %0t: Channel %0d: Round trip measure done with count %0d",$time,i,rt_count[i]);
                    tx_delay_real[i] = (tx_delay_sclk[i] / 2.0**M) * 4.0;
                    rx_delay_real[i] = (rx_delay_sclk[i] / 2.0**M) * 4.0;
                    rt_delay_real[i] = ((rt_count[i] / 2.0**($clog2(DL_TOT_MEASURE_CNT_FOR_RT))) * 4.0) ;
                    // $display("-> %0t: Channel %0d: Latency Measured\n\t-> TX latency = %0f ns\n\t-> RX latency = %0f ns\n\t-> Round trip latency = %0f ns\n\t-> RX Bitslip = %0d ui",$time,i,tx_delay_real[i],rx_delay_real[i],rt_delay_real[i],bitslip[i]);
                    #10000
                    if (checker_pass[i] == 1'b1) begin
                        $display("** Channel %0d: RX checker has received packets correctly!",i);
                        $display("** PASSED");
                    end else begin
                        $display("** Error: Channel %0d: RX checker has not received packets correctly!",i);
                        $display("** FAILED");
                    end
                end
            join
        end
    endtask

endmodule // basic_avl_tb_top

