// (C) 2001-2022 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.



module alt_cpriphy_c3_avmm_interconnect #(
    parameter BYPASS = 1,
    parameter NUM_SLAVE = 13,
    parameter TIMEOUT = 2000,
    parameter SLAVE0_BASE_ADDR   = ('h0000000),
    parameter SLAVE1_BASE_ADDR   = ('h1000000),
    parameter SLAVE2_BASE_ADDR   = ('h2000000),
    parameter SLAVE3_BASE_ADDR   = ('h3000000),
    parameter SLAVE4_BASE_ADDR   = ('h4000000),
    parameter SLAVE5_BASE_ADDR   = ('h5000000),
    parameter SLAVE6_BASE_ADDR   = ('h6000000),
    parameter SLAVE7_BASE_ADDR   = ('h7000000),
    parameter SLAVE8_BASE_ADDR   = ('h8000000),
    parameter SLAVE9_BASE_ADDR   = ('h9000000),
    parameter SLAVE10_BASE_ADDR  = ('hA000000),
    parameter SLAVE11_BASE_ADDR  = ('hB000000),
    parameter SLAVE12_BASE_ADDR  = ('hC000000)


) (                          
    input                           clk,
    input                           rst_n,
    // master
    input      [31:0]               avmm_mst_address,
    input                           avmm_mst_read,
    input                           avmm_mst_write,
    input      [31:0]               avmm_mst_writedata,
    output reg [31:0]               avmm_mst_readdata,
    output reg                      avmm_mst_readdata_valid,
    output reg                      avmm_mst_waitrequest,
    // slave port
    output reg [31:0]               avmm_slv_address,
    output reg [NUM_SLAVE-1:0]      avmm_slv_read,
    output reg [NUM_SLAVE-1:0]      avmm_slv_write,
    output reg [31:0]               avmm_slv_writedata,
    input      [(NUM_SLAVE*32)-1:0] avmm_slv_readdata,
    input      [NUM_SLAVE-1:0]      avmm_slv_readdata_valid,
    input      [NUM_SLAVE-1:0]      avmm_slv_waitrequest,
    output reg [NUM_SLAVE-1:0]      slave_sel
);

`ifdef AVMM_SPLITTER_BYPASS
  always @(*) begin
    slave_sel                = {12'b0, 1'b1};// always select default slave
    avmm_slv_address         = avmm_mst_address;
    avmm_slv_read            = {3'b0, avmm_mst_read};
    avmm_slv_write           = {3'b0, avmm_mst_write};
    avmm_slv_writedata       = avmm_mst_writedata;
    avmm_mst_readdata        = avmm_slv_readdata[31:0];
    avmm_mst_readdata_valid  = avmm_slv_readdata_valid[0];
    avmm_mst_waitrequest     = avmm_slv_waitrequest[0];
  end
`else
  localparam STATE_IDLE=2'b00;
  localparam STATE_READ=2'b01;
  localparam STATE_WRITE=2'b10;
  localparam STATE_WAIT=2'b11;

  reg [1:0] present_state, next_state;
  reg [31:0] address_reg;
  reg [31:0] writedata_reg;
  reg waitrequest_r;
  wire waitrequest_w;
  wire cmd_accept_w;
  wire readdata_valid_w;

  localparam TIMER_BITS = $clog2(TIMEOUT + 1);
  reg [TIMER_BITS-1:0]    timer;
  reg time_out;

  // Timeout handler and calculation 
  always @(posedge clk or negedge rst_n)
  begin
  if (!rst_n) begin
      timer    <= {TIMER_BITS{1'b0}};
      time_out <= 1'b1;
  end 
  else if ((present_state == STATE_READ) || (present_state == STATE_WRITE) 
                                         || (present_state == STATE_WAIT)) begin
      timer   <= timer + 1'b1;
      if (timer == TIMEOUT[TIMER_BITS-1:0]) begin
          time_out <= 1'b1;
      end
  end 
  else begin
      time_out <= 1'b0;
      timer    <= {TIMER_BITS{1'b0}};
  end
  end
            
  always @(posedge clk or negedge rst_n)
  begin
    if (!rst_n)
       slave_sel <= {NUM_SLAVE{1'b0}};
    else if (present_state == STATE_IDLE) begin
       if (avmm_mst_address >= SLAVE12_BASE_ADDR)
           slave_sel <= {  1'b1, 
                           1'b0, 1'b0, 1'b0, 1'b0,
                           1'b0, 1'b0, 1'b0, 1'b0,
                           1'b0, 1'b0, 1'b0, 1'b0
                        };
       else if (avmm_mst_address >= SLAVE11_BASE_ADDR)
           slave_sel <= {  1'b0, 
                           1'b1, 1'b0, 1'b0, 1'b0,
                           1'b0, 1'b0, 1'b0, 1'b0,
                           1'b0, 1'b0, 1'b0, 1'b0
                        };
       else if (avmm_mst_address >= SLAVE10_BASE_ADDR)
           slave_sel <= {  1'b0, 
                           1'b0, 1'b1, 1'b0, 1'b0,
                           1'b0, 1'b0, 1'b0, 1'b0,
                           1'b0, 1'b0, 1'b0, 1'b0
                        };
       else if (avmm_mst_address >= SLAVE9_BASE_ADDR)
           slave_sel <= {  1'b0, 
                           1'b0, 1'b0, 1'b1, 1'b0,
                           1'b0, 1'b0, 1'b0, 1'b0,
                           1'b0, 1'b0, 1'b0, 1'b0
                        };
       else if (avmm_mst_address >= SLAVE8_BASE_ADDR)
           slave_sel <= {  1'b0, 
                           1'b0, 1'b0, 1'b0, 1'b1,
                           1'b0, 1'b0, 1'b0, 1'b0,
                           1'b0, 1'b0, 1'b0, 1'b0
                        };
       else if (avmm_mst_address >= SLAVE7_BASE_ADDR)
           slave_sel <= {  1'b0, 
                           1'b0, 1'b0, 1'b0, 1'b0,
                           1'b1, 1'b0, 1'b0, 1'b0,
                           1'b0, 1'b0, 1'b0, 1'b0
                        };
       else if (avmm_mst_address >= SLAVE6_BASE_ADDR)
           slave_sel <= {  1'b0, 
                           1'b0, 1'b0, 1'b0, 1'b0,
                           1'b0, 1'b1, 1'b0, 1'b0,
                           1'b0, 1'b0, 1'b0, 1'b0
                        };
       else if (avmm_mst_address >= SLAVE5_BASE_ADDR)
           slave_sel <= {  1'b0, 
                           1'b0, 1'b0, 1'b0, 1'b0,
                           1'b0, 1'b0, 1'b1, 1'b0,
                           1'b0, 1'b0, 1'b0, 1'b0
                        };
       else if (avmm_mst_address >= SLAVE4_BASE_ADDR)
           slave_sel <= {  1'b0, 
                           1'b0, 1'b0, 1'b0, 1'b0,
                           1'b0, 1'b0, 1'b0, 1'b1,
                           1'b0, 1'b0, 1'b0, 1'b0
                        };
       else if (avmm_mst_address >= SLAVE3_BASE_ADDR)
           slave_sel <= {  1'b0, 
                           1'b0, 1'b0, 1'b0, 1'b0,
                           1'b0, 1'b0, 1'b0, 1'b0,
                           1'b1, 1'b0, 1'b0, 1'b0
                        };
       else if (avmm_mst_address >= SLAVE2_BASE_ADDR)
           slave_sel <= {  1'b0, 
                           1'b0, 1'b0, 1'b0, 1'b0,
                           1'b0, 1'b0, 1'b0, 1'b0,
                           1'b0, 1'b1, 1'b0, 1'b0
                        };
       else if (avmm_mst_address >= SLAVE1_BASE_ADDR)
           slave_sel <= {  1'b0, 
                           1'b0, 1'b0, 1'b0, 1'b0,
                           1'b0, 1'b0, 1'b0, 1'b0,
                           1'b0, 1'b0, 1'b1, 1'b0
                        };
       else if (avmm_mst_address >= SLAVE0_BASE_ADDR)
           slave_sel <= {  1'b0, 
                           1'b0, 1'b0, 1'b0, 1'b0,
                           1'b0, 1'b0, 1'b0, 1'b0,
                           1'b0, 1'b0, 1'b0, 1'b1
                        };
       else // default slave
           slave_sel <= {  1'b0, 
                           1'b0, 1'b0, 1'b0, 1'b0,
                           1'b0, 1'b0, 1'b0, 1'b0,
                           1'b0, 1'b0, 1'b0, 1'b1
                        };

    end
    else
       slave_sel <= slave_sel;
  end

  always @(posedge clk or negedge rst_n)
  begin
    if (!rst_n)
       address_reg <= {32{1'b0}};
    else if (present_state == STATE_IDLE)
       address_reg <= avmm_mst_address;
    else
       address_reg <= address_reg;
  end

  always @(posedge clk or negedge rst_n)
  begin
    if (!rst_n)
       writedata_reg <= {32{1'b0}};
    else if (present_state == STATE_IDLE)
       writedata_reg <= avmm_mst_writedata;
    else
       writedata_reg <= writedata_reg;
  end


  assign waitrequest_w = (avmm_mst_read | avmm_mst_write) & |(avmm_slv_waitrequest & slave_sel);
  assign readdata_valid_w = |(avmm_slv_readdata_valid & slave_sel);

  always @(posedge clk or negedge rst_n)
  begin
    if (!rst_n)
       waitrequest_r <= 1'b1;
    else
       waitrequest_r <= waitrequest_w;
  end
  
  assign cmd_accept_w = !waitrequest_w;

  always @(posedge clk or negedge rst_n)
  begin
    if (!rst_n)
       present_state <= STATE_IDLE;
    else
       present_state <= next_state;
  end

  always @(*)
  begin
    case (present_state)
      STATE_IDLE:  begin
                    if (waitrequest_r)
                       next_state = present_state;
                    else if (avmm_mst_read)
                       next_state = STATE_READ;
                    else if (avmm_mst_write)
                       next_state = STATE_WRITE;
                    else
                       next_state = present_state;
                   end
      STATE_READ:  begin
                    if (((!waitrequest_w & readdata_valid_w) | time_out))
                       next_state = STATE_IDLE;
                    else if (readdata_valid_w)
                       next_state = STATE_WAIT;
                    else
                       next_state = present_state;
                   end
      STATE_WRITE: begin
                    if (!waitrequest_w | time_out)
                       next_state = STATE_IDLE;
                    else
                       next_state = present_state;
                   end
      STATE_WAIT:  begin
                   if (waitrequest_w | time_out)
                       next_state = STATE_IDLE;
                   else
                       next_state = present_state;
                   end
    endcase
  end

  always @(*)
  begin
    case (present_state)
       STATE_IDLE:  begin
                      avmm_slv_address = avmm_mst_address;
                      avmm_slv_writedata = avmm_mst_writedata;
                      avmm_slv_write = {NUM_SLAVE{avmm_mst_write}} & slave_sel;
                      avmm_slv_read = {NUM_SLAVE{avmm_mst_read}} & slave_sel;
                      avmm_mst_waitrequest = waitrequest_w;
                    end
       STATE_READ:  begin
                      avmm_slv_address = address_reg; 
                      avmm_slv_writedata = writedata_reg;
                      avmm_slv_write = {NUM_SLAVE{1'b0}};
                      avmm_slv_read = slave_sel;
                      avmm_mst_waitrequest = time_out ? 1'b0 : waitrequest_w;
                    end
       STATE_WRITE: begin
                      avmm_slv_address = address_reg; 
                      avmm_slv_writedata = writedata_reg;
                      avmm_slv_write = slave_sel;
                      avmm_slv_read = {NUM_SLAVE{1'b0}};
                      avmm_mst_waitrequest = time_out ? 1'b0 : waitrequest_w;
                    end
       STATE_WAIT:  begin
                      avmm_slv_address = address_reg; 
                      avmm_slv_writedata = writedata_reg;
                      avmm_slv_write = {NUM_SLAVE{1'b0}};
                      avmm_slv_read = {NUM_SLAVE{1'b0}};
                      avmm_mst_waitrequest = time_out ? 1'b0 : waitrequest_w;
                    end
    endcase
  end

  always @(*)
    avmm_mst_readdata_valid = readdata_valid_w;

  always @(*)
    avmm_mst_readdata = ({32{slave_sel[12]}} & avmm_slv_readdata[(32*12)+31:(32*12)+0]) |
                        ({32{slave_sel[11]}} & avmm_slv_readdata[(32*11)+31:(32*11)+0]) |
                        ({32{slave_sel[10]}} & avmm_slv_readdata[(32*10)+31:(32*10)+0]) |
                        ({32{slave_sel[9]}}  & avmm_slv_readdata[(32*9) +31:(32*9) +0]) |
                        ({32{slave_sel[8]}}  & avmm_slv_readdata[(32*8) +31:(32*8) +0]) |
                        ({32{slave_sel[7]}}  & avmm_slv_readdata[(32*7) +31:(32*7) +0]) |
                        ({32{slave_sel[6]}}  & avmm_slv_readdata[(32*6) +31:(32*6) +0]) |
                        ({32{slave_sel[5]}}  & avmm_slv_readdata[(32*5) +31:(32*5) +0]) |
                        ({32{slave_sel[4]}}  & avmm_slv_readdata[(32*4) +31:(32*4) +0]) |
                        ({32{slave_sel[3]}}  & avmm_slv_readdata[(32*3) +31:(32*3) +0]) |
                        ({32{slave_sel[2]}}  & avmm_slv_readdata[(32*2) +31:(32*2) +0]) |
                        ({32{slave_sel[1]}}  & avmm_slv_readdata[(32*1) +31:(32*1) +0]) |
                        ({32{slave_sel[0]}}  & avmm_slv_readdata[(32*0) +31:(32*0) +0]) ;

`endif
endmodule
