# (C) 2001-2022 Intel Corporation. All rights reserved.
# Your use of Intel Corporation's design tools, logic functions and other 
# software and tools, and its AMPP partner logic functions, and any output 
# files from any of the foregoing (including device programming or simulation 
# files), and any associated documentation or information are expressly subject 
# to the terms and conditions of the Intel Program License Subscription 
# Agreement, Intel FPGA IP License Agreement, or other applicable 
# license agreement, including, without limitation, that your use is for the 
# sole purpose of programming logic devices manufactured by Intel and sold by 
# Intel or its authorized distributors.  Please refer to the applicable 
# agreement for further details.


# ---------------------------------------------------------------------------------------------------------------#
# Script Name  : main_script.tcl
# First Version : 01/01/2016 
# Latest version : 15/05/2018 
# Description: This script run all the TCL scripts that use to test CRETE3 on hardware. It will dump all tests 
#              results in "main_script.log" and detailed results in corresponding test log (*.log file).
#
# -------------------------------------------------------------------------------------------------------------- #
# These are some global variables used in TCL scripting and declared in /src/common_inc.tcl. 
# We must have to initialize these variables first for new test scripts in this framework.
# 1: global error_count : Initialize to 0 in start of every test script and it use to count ERRORs in test.
# 2: global fileid : Identify the output channel whether to store output in file or console, default is file.
# 3: global jtag_port_id : It depends on JTAG connection whether it connected to port 0 or 1 default is 0.
# 4: global service_on : Default should be 0.
# 5: totalChannel: Set variable for multi-channels in /tests/*.tcl scripts, default set to 1
# 
# ./main_script.tcl <number of total channels>  <JTAG Port ID number>
#
# NOTE: Its better to use USB Blaster (USB-1) for system, in case if it has multiple USB Blaster connected, 
#       otherwise it is necessary to change default values of variables in all TCL (/tests/*.tcl) test scripts, 
#       specially for JTAG port numbers. 
# -------------------------------------------------------------------------------------------------------------- #

set local_dir [pwd]

file delete -force "$local_dir/main_script.log"
#file delete -force "$local_dir/log"

source src/parameter.tcl

if {[info exists totalChannel]} {
   set totalChannel $totalChannel
   puts "Info: Number of Channels = $totalChannel"
} 

if {[info exists jtag_port_id]} {
   set jtag_port_id $jtag_port_id
   puts "Info: JTAG Port ID       = $jtag_port_id"
} 

if {[info exists speed]} {
   set speed $speed
   puts "Info: Speed              = $speed"
} 

if { [info exists totalChannel] && [info exists jtag_port_id] && [info exists speed]} {
  #file mkdir "$local_dir/log"
  set filename "$local_dir/main_script.log"                                                                   
  set lfileid [open $filename w]                                                                                
  set systemTime [clock seconds]                                                                               
  puts $lfileid "\n*************************************************************************************"   
  puts $lfileid   "** CRETE3 CPRI PHY HARDWARE TESTS                                                 ***"  
  puts $lfileid   "*************************************************************************************\n" 
  close $lfileid    
                                                                                         
  # Send/Receive packets and measure latency
  source   tests/c3_cpri_test.tcl

}


if {[info exists totalChannel] == 0} {
   set totalChannel 1
   puts "Warning: Number of (default) Channels                         : set totalChannel $totalChannel"
}

if {[info exists jtag_port_id] == 0} {
   set jtag_port_id 0
   puts "Warning: JTAG (default) Port ID                               : set jtag_port_id $jtag_port_id"
}

if {[info exists speed] == 0} {
   set speed 24G
   puts "Warning: Speed (10G, 24G, 24G_fec) (default) : set speed        $speed"
}


