# (C) 2001-2022 Intel Corporation. All rights reserved.
# Your use of Intel Corporation's design tools, logic functions and other 
# software and tools, and its AMPP partner logic functions, and any output 
# files from any of the foregoing (including device programming or simulation 
# files), and any associated documentation or information are expressly subject 
# to the terms and conditions of the Intel Program License Subscription 
# Agreement, Intel FPGA IP License Agreement, or other applicable 
# license agreement, including, without limitation, that your use is for the 
# sole purpose of programming logic devices manufactured by Intel and sold by 
# Intel or its authorized distributors.  Please refer to the applicable 
# agreement for further details.


source src/common_inc.tcl
source src/system_base_addr_map.tcl
source src/c3_cpri_cfgcsr_reg_map.tcl

 proc write_cpri_mc_cfgcsr_reg {channel offset wdata} {
    global fileid
    set register_name [dict get $::cpri_user_reg $offset]
    reg_write [expr $::ADDR_C3_ELANE_RECONFIG_INT + [expr $channel<<23]] [expr $offset<<2] $wdata 
    puts $fileid "\tSuccessfully Write CPRI Channel $channel Register $register_name, offset = [format "%#x" $offset], data = [format "%#x" $wdata] " 
 }

 proc read_cpri_mc_cfgcsr_reg {channel offset compflag comprdata} {
    global fileid
    global error_count
    set register_name [dict get $::cpri_user_reg $offset]
    set rdata [reg_read [expr $::ADDR_C3_ELANE_RECONFIG_INT + [expr $channel<<23]] [expr $offset<<2]]        
    if {$compflag} {
          if {$rdata == $comprdata} {
             puts $fileid "\tSuccessfully Read CPRI Channel $channel Register $register_name, offset = [format "%#x" $offset], data = [format "%#x" $rdata] "
          } else {
             set error_count [expr $error_count + 1]
             puts $fileid "\tERROR: Read CPRI Channel $channel Register $register_name, offset = [format "%#x" $offset], data = [format "%#x" $rdata] but expected data = [format "%#x" $comprdata] "
          }           
    } else {
          puts $fileid "\tSuccessfully Read CPRI Channel $channel Register $register_name, offset = [format "%#x" $offset], data = [format "%#x" $rdata] " 
    }
    return $rdata
 }
 
 proc polling_for_bit {channel offset bit_offset comprdata} {
    global fileid
    global error_count
    global nibble0
    global nibble1
    
    set register_name [dict get $::cpri_user_reg $offset]
    set timeout 10000
    set i 0
    puts $fileid "\tINFO: Polling For CPRI Channel $channel Register $register_name, offset = [format "%#x" $offset] bit $bit_offset ... "
    
    while {1} {
       set rdata [reg_read [expr $::ADDR_C3_ELANE_RECONFIG_INT + [expr $channel<<23]] [expr $offset<<2]]  
       word_split $rdata
       set octet0 [concat [hex2bin 0x$nibble1][hex2bin 0x$nibble0]]
       set bit_index [expr 7 - $bit_offset]
       set bit_value [lindex [split $octet0 {}] $bit_index]
       if {$bit_value == $comprdata || $i >= $timeout } break
       incr i
    }
    if {$bit_value == $comprdata} {
        puts $fileid "\tSuccessfully Read CPRI Channel $channel Register $register_name, offset = [format "%#x" $offset] bit $bit_offset, data = [format "%#x" $bit_value] "
        return 1
    } else {
        set error_count [expr $error_count + 1]
        puts $fileid "\tERROR! Read CPRI Channel $channel Register $register_name, offset = [format "%#x" $offset] bit $bit_offset, data = [format "%#x" $bit_value] but expected data = [format "%#x" $comprdata] "
        return 0
    }
 }

