# (C) 2001-2022 Intel Corporation. All rights reserved.
# Your use of Intel Corporation's design tools, logic functions and other 
# software and tools, and its AMPP partner logic functions, and any output 
# files from any of the foregoing (including device programming or simulation 
# files), and any associated documentation or information are expressly subject 
# to the terms and conditions of the Intel Program License Subscription 
# Agreement, Intel FPGA IP License Agreement, or other applicable 
# license agreement, including, without limitation, that your use is for the 
# sole purpose of programming logic devices manufactured by Intel and sold by 
# Intel or its authorized distributors.  Please refer to the applicable 
# agreement for further details.


source src/common_inc.tcl
source src/system_base_addr_map.tcl
source src/c3_ehiplane_user_cfgcsr_reg_map.tcl

 proc write_ehiplane_user_cfgcsr_reg {offset wdata} {
        global fileid
        set register_name [dict get $::ehip_user_reg $offset]
        reg_write $::ADDR_C3_ELANE_RECONFIG_INT [expr $offset<<2] $wdata 
        puts $fileid "\tSuccessfully Write EHIPLANE User Register $register_name, offset = [format "%#x" $offset], data = [format "%#x" $wdata] " 
 }

 proc read_ehiplane_user_cfgcsr_reg {offset compflag comprdata} {
        global fileid
        global error_count
		set register_name [dict get $::ehip_user_reg $offset]
        set rdata [reg_read $::ADDR_C3_ELANE_RECONFIG_INT [expr $offset<<2]]		
        if {$compflag} {
		      if {$rdata == $comprdata} {
                 puts $fileid "\tSuccessfully Read  EHIPLANE User Register $register_name, offset = [format "%#x" $offset], data = [format "%#x" $rdata] "
              } else {
			     set error_count [expr $error_count + 1]
                 puts $fileid "\tERROR: Read EHIPLANE User Register $register_name, offset = [format "%#x" $offset], data = [format "%#x" $rdata] but expected data = [format "%#x" $comprdata] "
              }			  
        } else {
              puts $fileid "\tSuccessfully Read  EHIPLANE User Register $register_name, offset = [format "%#x" $offset], data = [format "%#x" $rdata] " 
        }
        return $rdata
 }

 proc write_ehiplane_mc_user_cfgcsr_reg {channel offset wdata} {
        global fileid
        set register_name [dict get $::ehip_user_reg $offset]
        reg_write [expr $::ADDR_C3_ELANE_RECONFIG_INT + [expr $channel<<23]] [expr $offset<<2] $wdata 
        puts $fileid "\tSuccessfully Write EHIPLANE Channel $channel, User Register $register_name, offset = [format "%#x" $offset], data = [format "%#x" $wdata] " 
 }

 proc read_ehiplane_mc_user_cfgcsr_reg {channel offset compflag comprdata} {
        global fileid
        global error_count
        set register_name [dict get $::ehip_user_reg $offset]
        set rdata [reg_read [expr $::ADDR_C3_ELANE_RECONFIG_INT + [expr $channel<<23]] [expr $offset<<2]]		
        if {$compflag} {
		      if {$rdata == $comprdata} {
                 puts $fileid "\tSuccessfully Read  EHIPLANE Channel $channel, User Register $register_name, offset = [format "%#x" $offset], data = [format "%#x" $rdata] "
              } else {
			     set error_count [expr $error_count + 1]
                 puts $fileid "\tERROR: Read EHIPLANE Channel $channel, User Register $register_name, offset = [format "%#x" $offset], data = [format "%#x" $rdata] but expected data = [format "%#x" $comprdata] "
              }			  
        } else {
              puts $fileid "\tSuccessfully Read  EHIPLANE Channel $channel, User Register $register_name, offset = [format "%#x" $offset], data = [format "%#x" $rdata] " 
        }
        return $rdata
 }
 
 
 proc c3_ehiplane_mac_loopback_mode { compresult err_cnt } {
    global error_count
    global fileid
    global jtag_port_id
    global SEL_FPGA1

    set error_count $err_cnt

    # Overwrite for MAC loop-back configuration
    write_ehiplane_user_cfgcsr_reg    $::phy_ehip_mode_muxes               0x098000 
    read_ehiplane_user_cfgcsr_reg     $::phy_ehip_mode_muxes  $compresult  0x098000
    
    if {$error_count > $err_cnt} {
       puts $fileid "\n\tERROR: C3 ELANE MAC Loopback mode isn't enabled \n" 
    } else {
       puts $fileid "\n\tC3 ELANE MAC Loopback mode is successfully enabled \n"
    }
 }

 proc c3_ehiplane_pcs_loopback_mode { channel compresult err_cnt } {
    global error_count
    global fileid
    global jtag_port_id
    global SEL_FPGA1

    set error_count $err_cnt

    # Overwrite for MAC+PCS loop-back configuration
    write_ehiplane_mc_user_cfgcsr_reg   $channel   $::phy_ehip_mode_muxes               0x018000 
    read_ehiplane_mc_user_cfgcsr_reg    $channel   $::phy_ehip_mode_muxes  $compresult  0x018000
    
    if {$error_count > $err_cnt} {
       puts $fileid "\n\tERROR: C3 ELANE PCS Loopback mode isn't enabled \n" 
    } else {
       puts $fileid "\n\tC3 ELANE PCS Loopback mode is successfully enabled \n"
    }
 }
 
 proc c3_ehip_soft_system_reset { compresult err_cnt } {
    global error_count
    global fileid
    global jtag_port_id
    global SEL_FPGA1

    set error_count $err_cnt

    # Reset Assertion
    write_ehiplane_user_cfgcsr_reg  $::phy_ehip_csr_soft_reset  0x0
    write_ehiplane_user_cfgcsr_reg  $::phy_ehip_csr_soft_reset  0x1
    write_ehiplane_user_cfgcsr_reg  $::phy_ehip_csr_soft_reset  0x3
    write_ehiplane_user_cfgcsr_reg  $::phy_ehip_csr_soft_reset  0x7
    read_ehiplane_user_cfgcsr_reg   $::phy_ehip_csr_soft_reset  $compresult 0x7
    # Reset De-assertion
    write_ehiplane_user_cfgcsr_reg  $::phy_ehip_csr_soft_reset  0x6
    write_ehiplane_user_cfgcsr_reg  $::phy_ehip_csr_soft_reset  0x4
    write_ehiplane_user_cfgcsr_reg  $::phy_ehip_csr_soft_reset  0x0
    read_ehiplane_user_cfgcsr_reg   $::phy_ehip_csr_soft_reset  $compresult 0x0    
    
    if {$error_count > $err_cnt} {
       puts $fileid "\n\tERROR: C3 ELANE System Reset isn't successfully \n" 
    } else {
       puts $fileid "\n\tC3 ELANE System Reset is successfully \n"
    }
 }

 proc c3_ehip_mc_soft_system_reset { channel compresult err_cnt } {
    global error_count
    global fileid
    global jtag_port_id
    global SEL_FPGA1

    set error_count $err_cnt

    # Reset Assertion
    write_ehiplane_mc_user_cfgcsr_reg  $channel $::phy_ehip_csr_soft_reset  0x0
    write_ehiplane_mc_user_cfgcsr_reg  $channel $::phy_ehip_csr_soft_reset  0x1
    write_ehiplane_mc_user_cfgcsr_reg  $channel $::phy_ehip_csr_soft_reset  0x3
    write_ehiplane_mc_user_cfgcsr_reg  $channel $::phy_ehip_csr_soft_reset  0x7
    read_ehiplane_mc_user_cfgcsr_reg   $channel $::phy_ehip_csr_soft_reset  $compresult 0x7
    # Reset De-assertion
    write_ehiplane_mc_user_cfgcsr_reg  $channel $::phy_ehip_csr_soft_reset  0x6
    write_ehiplane_mc_user_cfgcsr_reg  $channel $::phy_ehip_csr_soft_reset  0x4
    write_ehiplane_mc_user_cfgcsr_reg  $channel $::phy_ehip_csr_soft_reset  0x0
    read_ehiplane_mc_user_cfgcsr_reg   $channel $::phy_ehip_csr_soft_reset  $compresult 0x0    
    
    if {$error_count > $err_cnt} {
       puts $fileid "\n\tERROR: C3 ELANE Channel $channel System Reset isn't successfully \n" 
    } else {
       puts $fileid "\n\tC3 ELANE Channel $channel System Reset is successfully \n"
    }
 }
 
 proc c3_ehip_soft_tx_rx_reset { compresult err_cnt } {
    global error_count
    global fileid
    global jtag_port_id
    global SEL_FPGA1

    set error_count $err_cnt

    # Reset Assertion
    write_ehiplane_user_cfgcsr_reg  $::phy_ehip_csr_soft_reset  0x0
    write_ehiplane_user_cfgcsr_reg  $::phy_ehip_csr_soft_reset  0x2
    write_ehiplane_user_cfgcsr_reg  $::phy_ehip_csr_soft_reset  0x6
    read_ehiplane_user_cfgcsr_reg   $::phy_ehip_csr_soft_reset  $compresult 0x6
    # Reset De-assertion
    write_ehiplane_user_cfgcsr_reg  $::phy_ehip_csr_soft_reset  0x4
    write_ehiplane_user_cfgcsr_reg  $::phy_ehip_csr_soft_reset  0x0
    read_ehiplane_user_cfgcsr_reg   $::phy_ehip_csr_soft_reset  $compresult 0x0
    
    if {$error_count > $err_cnt} {
       puts $fileid "\n\tERROR: C3 ELANE TX/RX Reset isn't successfully \n" 
    } else {
       puts $fileid "\n\tC3 ELANE TX/RX Reset is successfully \n"
    }
 }

 
