# (C) 2001-2022 Intel Corporation. All rights reserved.
# Your use of Intel Corporation's design tools, logic functions and other 
# software and tools, and its AMPP partner logic functions, and any output 
# files from any of the foregoing (including device programming or simulation 
# files), and any associated documentation or information are expressly subject 
# to the terms and conditions of the Intel Program License Subscription 
# Agreement, Intel FPGA IP License Agreement, or other applicable 
# license agreement, including, without limitation, that your use is for the 
# sole purpose of programming logic devices manufactured by Intel and sold by 
# Intel or its authorized distributors.  Please refer to the applicable 
# agreement for further details.


source src/common_inc.tcl
source src/system_base_addr_map.tcl
source src/c3_xcvr_cfgcsr_reg_map.tcl

 proc write_xcvr_reconfig_csr_reg {offset wdata} {
        global fileid
        #set register_name [dict get $::ehip_xcvr_reg $offset]
        reg_write $::ADDR_C3_XCVR_RECONFIG_INT [expr $offset<<2] $wdata 
        #puts $fileid "\tSuccessfully Write EHIPLANE User Register $register_name, offset = [format "%#x" $offset], data = [format "%#x" $wdata] " 
        puts $fileid "\tSuccessfully Write XCVR CSR Register offset = [format "%#x" $offset], data = [format "%#x" $wdata] "
 }

 proc read_xcvr_reconfig_csr_reg {offset compflag comprdata} {
        global fileid
        global error_count
        #set register_name [dict get $::ehip_xcvr_reg $offset]
        set rdata [reg_read $::ADDR_C3_XCVR_RECONFIG_INT [expr $offset<<2]]     
        if {$compflag} {
              if {$rdata == $comprdata} {
                 #puts $fileid "\tSuccessfully Read  EHIPLANE User Register $register_name, offset = [format "%#x" $offset], data = [format "%#x" $rdata] "
                 puts $fileid "\tSuccessfully Read  XCVR CSR Register offset = [format "%#x" $offset], data = [format "%#x" $rdata] "
              } else {
                 set error_count [expr $error_count + 1]
                 #puts $fileid "\tERROR: Read EHIPLANE User Register $register_name, offset = [format "%#x" $offset], data = [format "%#x" $rdata] but expected data = [format "%#x" $comprdata] "
                 puts $fileid "\tERROR: Read XCVR CSR Register offset = [format "%#x" $offset], data = [format "%#x" $rdata] but expected data = [format "%#x" $comprdata] "
              }           
        } else {
              #puts $fileid "\tSuccessfully Read  EHIPLANE User Register $register_name, offset = [format "%#x" $offset], data = [format "%#x" $rdata] " 
              puts $fileid "\tSuccessfully Read  XCVR CSR Register offset = [format "%#x" $offset], data = [format "%#x" $rdata] " 
        }
        return $rdata
 }

 proc write_xcvr_mc_reconfig_csr_reg {channel offset wdata} {
        global fileid
        #set register_name [dict get $::ehip_xcvr_reg $offset]
        reg_write [expr $::ADDR_C3_XCVR_RECONFIG_INT + [expr $channel<<23]] [expr $offset<<2] $wdata 
        #puts $fileid "\tSuccessfully Write EHIPLANE User Register $register_name, offset = [format "%#x" $offset], data = [format "%#x" $wdata] " 
        puts $fileid "\tSuccessfully Write XCVR Channel $channel, CSR Register offset = [format "%#x" $offset], data = [format "%#x" $wdata] "
 }

 proc read_xcvr_mc_reconfig_csr_reg {channel offset compflag comprdata} {
        global fileid
        global error_count
        #set register_name [dict get $::ehip_xcvr_reg $offset]
        set rdata [reg_read [expr $::ADDR_C3_XCVR_RECONFIG_INT + [expr $channel<<23]] [expr $offset<<2]]        
        if {$compflag} {
              if {$rdata == $comprdata} {
                 #puts $fileid "\tSuccessfully Read  EHIPLANE User Register $register_name, offset = [format "%#x" $offset], data = [format "%#x" $rdata] "
                 puts $fileid "\tSuccessfully Read  XCVR Channel $channel, CSR Register offset = [format "%#x" $offset], data = [format "%#x" $rdata] "
              } else {
                 set error_count [expr $error_count + 1]
                 #puts $fileid "\tERROR: Read EHIPLANE User Register $register_name, offset = [format "%#x" $offset], data = [format "%#x" $rdata] but expected data = [format "%#x" $comprdata] "
                 puts $fileid "\tERROR: Read XCVR  Channel $channel, CSR Register offset = [format "%#x" $offset], data = [format "%#x" $rdata] but expected data = [format "%#x" $comprdata] "
              }           
        } else {
              #puts $fileid "\tSuccessfully Read  EHIPLANE User Register $register_name, offset = [format "%#x" $offset], data = [format "%#x" $rdata] " 
              puts $fileid "\tSuccessfully Read  XCVR  Channel $channel, CSR Register offset = [format "%#x" $offset], data = [format "%#x" $rdata] " 
        }
        return $rdata
 }

 # ===================================================================================================================
 # Reconfiguration Steps (Partial):
 # 4. Verify that the PMA register read/write is sent to the PMA by verifying that 0x8A[7] is asserted.
 #    Addr: 8A,bit7: cfg_core_int_in_prog_assert: Expect 1
 # 5. Verify that 0x8B[0] de-asserts to indicate that the PMA register read/write transaction completed.
 #    Addr: 8B,bit0: cfg_core_int_in_progress: Expect 0             
 # ===================================================================================================================
  proc polling_for_cfg_bit {channel offset location expected_value} {
 
        global fileid
        global error_count
        global nibble0
        global nibble1
        global nibble2
        global nibble3
        global nibble4
        global nibble5
        global nibble6
        global nibble7
    
        global bit0
        global bit1
        global bit2
        global bit3
        global bit4
        global bit5
        global bit6
        global bit7
        
        set iteration 1000
        set i 0
        set j 0
        set rdata [reg_read [expr $::ADDR_C3_XCVR_RECONFIG_INT + [expr $channel<<23]] [expr $offset<<2]]        
        word_split $rdata
        puts "\tINFO: Polling For PMA Register: Read  XCVR CSR Register offset = [format "%#x" $offset], data = [format "%#x" $rdata] "
        set octet0 [concat [hex2bin 0x$nibble1][hex2bin 0x$nibble0]]
        byte_split $octet0
        #puts "\t\tBit0 = [format "%x" $bit0], Bit1 = [format "%x" $bit1], Bit2 = [format "%x" $bit2], Bit3 = [format "%x" $bit3], Bit4 = [format "%x" $bit4], Bit5 = [format "%x" $bit5], Bit6 = [format "%x" $bit6], Bit7 = [format "%x" $bit7]"
        #puts "\tTargeted Bit: $location\n"     
        
        if {$location == 7} {
            while {$bit7 != $expected_value && $i < $iteration } {
               set rdata [reg_read $::ADDR_C3_XCVR_RECONFIG_INT [expr $offset<<2]]  
               word_split $rdata
               set octet0 [concat [hex2bin 0x$nibble1][hex2bin 0x$nibble0]]
               byte_split $octet0
               puts "\t\tINSIDE Iteration: Bit7 = [format "%x" $bit7], Iteration:$i, readdata = [format "%x" $rdata]\n"
               incr i
            }
            if {$bit7 != $expected_value} {
                set error_count [expr $error_count + 1]
                puts "\tWARNING: Bit 7 of $offset is not expected"                      
                return 1
            } else {
                return 0
            }
        }
        
        if {$location == 0} {
            while {$bit0 != $expected_value && $j < $iteration } {
               set rdata [reg_read $::ADDR_C3_XCVR_RECONFIG_INT [expr $offset<<2]]  
               word_split $rdata
               set octet0 [concat [hex2bin 0x$nibble1][hex2bin 0x$nibble0]]
               byte_split $octet0
               puts "\t\tINSIDE Iteration: Bit0 = [format "%x" $bit0], Iteration:$j, readdata = [format "%x" $rdata]\n"
               incr j
            }
            if {$bit0 != $expected_value} {
                set error_count [expr $error_count + 1]
                puts "\tWARNING: Bit 0 of $offset is not expected"                      
                return 1
            } else {
                return 0
            }
        }
 }

 # ===================================================================================================================
 # Reconfiguration Steps (Partial):
 # 4. Verify that the PMA register read/write is sent to the PMA by verifying that 0x8A is asserted.
 #    Addr: 8A, cfg_core_int_in_prog_assert: Expect 0x80        
 # ===================================================================================================================
  proc polling_for_cfg {channel offset expected_value} {
 
    global fileid
    global error_count
        
    set iteration 20
    set delay_ms 1000
    set i 0
    # set j 0
    # set rdata [reg_read [expr $::ADDR_C3_XCVR_RECONFIG_INT + [expr $channel<<23]] [expr $offset<<2]]
    # puts "\tINFO: Polling For PMA Register: Read  XCVR CSR Register offset = [format "%#x" $offset], data = [format "%#x" $rdata] "     
    # set octet0 [expr $rdata & 0xFF]
    ## puts "\t\tINSIDE \$octet0 = $octet0\n"

    # while { $octet0 != $expected_value && $i < $iteration } {
        # set rdata [reg_read $::ADDR_C3_XCVR_RECONFIG_INT [expr $offset<<2]]
        ##puts "\t\tINSIDE \$expected_value = $expected_value, \$rdata = $rdata\n"
        # puts "\t\tINSIDE Expected = [format "%#x" $expected_value], Iteration:$i, data = [format "%#x" $rdata]\n"
        # set octet0 [expr $rdata & 0xFF]
        ## puts "\t\tINSIDE \$octet0 = $octet0\n"
        # after $delay_ms
        # incr i
    # }
    # if {$octet0 != $expected_value} {
        # set error_count [expr $error_count + 1]
        # puts "\tWARNING: Value of $offset is not expected"                      
    # }

    set rdata [format 0x%x [read_xcvr_mc_reconfig_csr_reg $channel $offset 0 0]]
    puts "\tINFO: Polling For PMA Register: Read XCVR CSR Register offset = [format "%#x" $offset], data = [format "%#x" $rdata] "     

    while { $rdata != $expected_value && $i < $iteration } {
        after $delay_ms
        incr i
        set rdata [format 0x%x [read_xcvr_mc_reconfig_csr_reg $channel $offset 0 0]]
        puts "\t\tINSIDE Expected = [format "%#x" $expected_value], Iteration:$i, data = [format "%#x" $rdata]\n"
    }

    if {$rdata != $expected_value} {
        set error_count [expr $error_count + 1]
        puts "\tWARNING: Value of $offset is not expected"                      
    }
 }

 proc c3_ehiplane_xcvr_mc_loopback_mode { channel compresult err_cnt } {
    global error_count
    global fileid
    global jtag_port_id
    global SEL_FPGA1

    set error_count $err_cnt

    puts "\n\tINFO: Start of C3 ELANE XCVR Channel $channel Loopback mode\n"
    puts $fileid "\n\tINFO: Start of C3 ELANE XCVR Channel $channel Loopback mode\n"

    # Overwrite for MAC+PCS loop-back configuration
    write_xcvr_mc_reconfig_csr_reg $channel 0x84 0x1
    write_xcvr_mc_reconfig_csr_reg $channel 0x85 0x1
    write_xcvr_mc_reconfig_csr_reg $channel 0x86 0x8
    write_xcvr_mc_reconfig_csr_reg $channel 0x87 0x0

    # read_xcvr_mc_reconfig_csr_reg $channel 0x84 $compresult 0x1
    # read_xcvr_mc_reconfig_csr_reg $channel 0x85 $compresult 0x1
    # read_xcvr_mc_reconfig_csr_reg $channel 0x86 $compresult 0x8
    # read_xcvr_mc_reconfig_csr_reg $channel 0x87 $compresult 0x0

    write_xcvr_mc_reconfig_csr_reg $channel 0x90 0x1
#    write_xcvr_mc_reconfig_csr_reg $channel 0x8A 0x80

#    read_xcvr_mc_reconfig_csr_reg $channel 0x88 0 0x0
    # Polling PMA register 
    polling_for_cfg_bit $channel 0x8a 7 1
    polling_for_cfg_bit $channel 0x8B 0 0
        
    write_xcvr_mc_reconfig_csr_reg $channel 0x8a 0x8

    read_xcvr_mc_reconfig_csr_reg $channel 0x89 0 0x0
    read_xcvr_mc_reconfig_csr_reg $channel 0x88 0 0x8
    
    if {$error_count > $err_cnt} {
       puts "\n\tERROR: C3 ELANE XCVR Channel $channel Loopback mode isn't enabled \n"
       puts $fileid "\n\tERROR: C3 ELANE XCVR Channel $channel Loopback mode isn't enabled \n" 
    } else {
       puts "\n\tINFO: C3 ELANE XCVR Channel $channel Loopback mode is successfully enabled \n"
       puts $fileid "\n\tINFO: C3 ELANE XCVR Channel $channel Loopback mode is successfully enabled \n"
    }
 } 
 
  proc c3_ehiplane_xcvr_mc_mission_mode { channel compresult err_cnt } {
    global error_count
    global fileid
    global jtag_port_id
    global SEL_FPGA1

    set error_count $err_cnt

    puts "\n\tINFO: Start of C3 ELANE XCVR Channel $channel mission mode\n"
    puts $fileid "\n\tINFO: Start of C3 ELANE XCVR Channel $channel mission mode\n"

    # Overwrite for MAC+PCS loop-back configuration
    write_xcvr_mc_reconfig_csr_reg $channel 0x84 0x0
    write_xcvr_mc_reconfig_csr_reg $channel 0x85 0x1
    write_xcvr_mc_reconfig_csr_reg $channel 0x86 0x8
    write_xcvr_mc_reconfig_csr_reg $channel 0x87 0x0

    # read_xcvr_mc_reconfig_csr_reg $channel 0x84 $compresult 0x1
    # read_xcvr_mc_reconfig_csr_reg $channel 0x85 $compresult 0x1
    # read_xcvr_mc_reconfig_csr_reg $channel 0x86 $compresult 0x8
    # read_xcvr_mc_reconfig_csr_reg $channel 0x87 $compresult 0x0

    write_xcvr_mc_reconfig_csr_reg $channel 0x90 0x1
#    write_xcvr_mc_reconfig_csr_reg $channel 0x8A 0x80

#    read_xcvr_mc_reconfig_csr_reg $channel 0x88 0 0x0
    # Polling PMA register 
    polling_for_cfg_bit $channel 0x8a 7 1
    polling_for_cfg_bit $channel 0x8B 0 0
        
    write_xcvr_mc_reconfig_csr_reg $channel 0x8a 0x8

    read_xcvr_mc_reconfig_csr_reg $channel 0x89 0 0x0
    read_xcvr_mc_reconfig_csr_reg $channel 0x88 0 0x8
    
    if {$error_count > $err_cnt} {
       puts "\n\tERROR: C3 ELANE XCVR Channel $channel mission mode isn't enabled \n"
       puts $fileid "\n\tERROR: C3 ELANE XCVR Channel $channel mission mode isn't enabled \n" 
    } else {
       puts "\n\tINFO: C3 ELANE XCVR Channel $channel mission mode is successfully enabled \n"
       puts $fileid "\n\tINFO: C3 ELANE XCVR Channel $channel mission mode is successfully enabled \n"
    }
 } 
 
 proc c3_ehiplane_xcvr_loopback_mode { compresult err_cnt } {
    global error_count
    global fileid
    global jtag_port_id
    global SEL_FPGA1

    set error_count $err_cnt

    # Overwrite for MAC+PCS loop-back configuration
    write_xcvr_reconfig_csr_reg 0x84 0x1
    write_xcvr_reconfig_csr_reg 0x85 0x1
    write_xcvr_reconfig_csr_reg 0x86 0x8
    write_xcvr_reconfig_csr_reg 0x87 0x0

    read_xcvr_reconfig_csr_reg 0x84 $compresult 0x1
    read_xcvr_reconfig_csr_reg 0x85 $compresult 0x1
    read_xcvr_reconfig_csr_reg 0x86 $compresult 0x8
    read_xcvr_reconfig_csr_reg 0x87 $compresult 0x0

    write_xcvr_reconfig_csr_reg 0x90 0x1
    write_xcvr_reconfig_csr_reg 0x8A 0x80

    read_xcvr_reconfig_csr_reg 0x88 0 0x0
    
    if {$error_count > $err_cnt} {
       puts $fileid "\n\tERROR: C3 ELANE XCVR Loopback mode isn't enabled \n" 
    } else {
       puts $fileid "\n\tC3 ELANE XCVR Loopback mode is successfully enabled \n"
    }
 } 
 
 proc c3_ehiplane_xcvr_run_calibration { channel err_cnt loopback_mode rcp_load_enable } {
    global error_count
    global fileid
    
    set error_count $err_cnt
    
    puts "\n\tINFO: Running calibration for channel $channel\n"
    puts $fileid "\n\tINFO: Running calibration for channel $channel\n"

    ########################
    # 1. set_operation_mode
    ########################
    if {$loopback_mode == 1} {
        # Internal loopback, disable PRBS
        write_xcvr_mc_reconfig_csr_reg $channel 0x200 0x1F
    } else {
        # External loopback, disable PRBS
        write_xcvr_mc_reconfig_csr_reg $channel 0x200 0x1E
    }
    write_xcvr_mc_reconfig_csr_reg $channel 0x201 0x00
    write_xcvr_mc_reconfig_csr_reg $channel 0x202 0x00
    write_xcvr_mc_reconfig_csr_reg $channel 0x203 0x93
    puts "\n\tINFO: SET_OPERATION_MODE status check start...\n"
    puts $fileid "\n\tINFO: SET_OPERATION_MODE status check start...\n"
    polling_for_cfg $channel 0x207 0x80
    
    if {$channel == 0 && $rcp_load_enable == 1} {
        #########################
        # 2. run_load_pma_config
        #########################
        write_xcvr_mc_reconfig_csr_reg $channel 0x40143 0x80
        
        ###################################
        # 3. run_read_load_pma_config_stat
        ###################################
        puts "\n\tINFO: Load PMA configuration status check start...\n"
        puts $fileid "\n\tINFO: Load PMA configuration status check start...\n"
        polling_for_cfg_bit $channel 0x40144 0 1
    }
    
    ######################
    # 4. start_adaptation
    ######################
    #write_xcvr_mc_reconfig_csr_reg $channel 0x200 0xF2
    if {$loopback_mode == 1} {
        # Initial adaptation
        puts "\tINFO: Initial adaptation \n"
        write_xcvr_mc_reconfig_csr_reg $channel 0x200 0xF2

        # Internal loopback, disable PRBS
        puts "\tINFO: Internal loopback \n"
        write_xcvr_mc_reconfig_csr_reg $channel 0x201 0x03

        # Initial adaptation flow
        puts "\tINFO: Enable initial adaptation \n"
        write_xcvr_mc_reconfig_csr_reg $channel 0x202 0x01
    } else {
        # Initial and Continuous adaptation
        puts "\tINFO: Initial and Continuous adaptation \n"
        write_xcvr_mc_reconfig_csr_reg $channel 0x200 0xF6

        # External loopback, disable PRBS
        puts "\tINFO: External loopback \n"
        write_xcvr_mc_reconfig_csr_reg $channel 0x201 0x01

        # Full Flow (initial and continuous adaptation)
        puts "\tINFO: Enable initial and continuous adaptation\n"
        write_xcvr_mc_reconfig_csr_reg $channel 0x202 0x03
    }
    #write_xcvr_mc_reconfig_csr_reg $channel 0x202 0x01
    write_xcvr_mc_reconfig_csr_reg $channel 0x203 0x96
    puts "\n\tINFO: START_ADAPTATION status check start...\n"
    puts $fileid "\n\tINFO: START_ADAPTATION status check start...\n"
    after 20000
    polling_for_cfg $channel 0x207 0x80
    
    ####################
    # 5. check_cal_stat
    ####################
    write_xcvr_mc_reconfig_csr_reg $channel 0x200 0x01
    write_xcvr_mc_reconfig_csr_reg $channel 0x201 0x00
    write_xcvr_mc_reconfig_csr_reg $channel 0x202 0x00
    write_xcvr_mc_reconfig_csr_reg $channel 0x203 0x97
    puts "\n\tINFO: CHECK_CAL_STAT status check start...\n"
    puts $fileid "\n\tINFO: CHECK_CAL_STAT status check start...\n"
    polling_for_cfg $channel 0x207 0x80
    if {$loopback_mode == 1} {
        # Initial adaptation done: 0x80
        polling_for_cfg $channel 0x204 0x80
    } else {
        # Running continuous adaptation: 0xE2
        polling_for_cfg $channel 0x204 0xE2
    }
    
    puts "\n\tINFO: Calibration for channel $channel is done\n"
    puts $fileid "\n\tINFO: Calibration for channel $channel is done\n"
 }


 proc disable_serdes { channel } {
    # interupt_sl_request $channel 0x1 0x0
    write_xcvr_mc_reconfig_csr_reg $channel 0x84 0x0
    write_xcvr_mc_reconfig_csr_reg $channel 0x85 0x0
    write_xcvr_mc_reconfig_csr_reg $channel 0x86 0x1
    write_xcvr_mc_reconfig_csr_reg $channel 0x87 0x0
    
    write_xcvr_mc_reconfig_csr_reg $channel 0x90 0x1
    
    polling_for_cfg_bit $channel 0x8a 7 1
    polling_for_cfg_bit $channel 0x8B 0 0
    
    write_xcvr_mc_reconfig_csr_reg $channel 0x8A 0x80
 }

 proc enable_serdes { channel } {
    # interupt_sl_request $channel 0x1 0x7
    write_xcvr_mc_reconfig_csr_reg $channel 0x84 0x7
    write_xcvr_mc_reconfig_csr_reg $channel 0x85 0x0
    write_xcvr_mc_reconfig_csr_reg $channel 0x86 0x1
    write_xcvr_mc_reconfig_csr_reg $channel 0x87 0x0
    
    write_xcvr_mc_reconfig_csr_reg $channel 0x90 0x1
    
    polling_for_cfg_bit $channel 0x8a 7 1
    polling_for_cfg_bit $channel 0x8B 0 0
    
    write_xcvr_mc_reconfig_csr_reg $channel 0x8A 0x80
 }

 proc initial_adaptation { channel } {
    # interupt_sl_request $channel 0xA 0x1
    write_xcvr_mc_reconfig_csr_reg $channel 0x84 0x1
    write_xcvr_mc_reconfig_csr_reg $channel 0x85 0x0
    write_xcvr_mc_reconfig_csr_reg $channel 0x86 0xA
    write_xcvr_mc_reconfig_csr_reg $channel 0x87 0x0
    
    write_xcvr_mc_reconfig_csr_reg $channel 0x90 0x1
    
    polling_for_cfg_bit $channel 0x8a 7 1
    polling_for_cfg_bit $channel 0x8B 0 0
    
    write_xcvr_mc_reconfig_csr_reg $channel 0x8A 0x80
    
 }

 proc continuous_adaptation { channel } {
    # interupt_sl_request $channel 0xA 0x6
    write_xcvr_mc_reconfig_csr_reg $channel 0x84 0x6
    write_xcvr_mc_reconfig_csr_reg $channel 0x85 0x0
    write_xcvr_mc_reconfig_csr_reg $channel 0x86 0xA
    write_xcvr_mc_reconfig_csr_reg $channel 0x87 0x0
    
    write_xcvr_mc_reconfig_csr_reg $channel 0x90 0x1
    
    polling_for_cfg_bit $channel 0x8a 7 1
    polling_for_cfg_bit $channel 0x8B 0 0
    
    write_xcvr_mc_reconfig_csr_reg $channel 0x8A 0x80
 }

 proc initial_adaptation_status {channel  {time_out 20000} } {
    global fileid
    global error_count
      
    # Read :code[8] =1 
    set start_time [clock milliseconds]
    set interval 0
    set busy 1
    while {($busy != 0) && ($interval < $time_out) } {
        set curr_time [clock milliseconds]
        write_xcvr_mc_reconfig_csr_reg $channel 0x84 0x0
        write_xcvr_mc_reconfig_csr_reg $channel 0x85 0xB
        write_xcvr_mc_reconfig_csr_reg $channel 0x86 0x26
        write_xcvr_mc_reconfig_csr_reg $channel 0x87 0x1
        
        write_xcvr_mc_reconfig_csr_reg $channel 0x90 0x1
        
        polling_for_cfg_bit $channel 0x8a 7 1
        polling_for_cfg_bit $channel 0x8B 0 0
        
        write_xcvr_mc_reconfig_csr_reg $channel 0x8A 0x80
        
        set busy [polling_for_cfg_bit $channel 0x88 0 0]
        # revert flag error here, error will be flagged by timeout
        if {$busy == 1} {
            set error_count [expr $error_count - 1]
        }
        # set busy [expr $return & 0x1]
        set interval [expr $curr_time - $start_time]
    }
    
    if {$interval >= $time_out } {
        puts "\tERROR: Initial adaptation on channel $channel timed out"
        puts $fileid "\tERROR: Initial adaptation on channel $channel timed out"
        set error_count [expr $error_count + 1]
        # error "initialAdaptation timed out"
        return 1
    } else {
        # initialAdaptation is done successful
        puts "\tINFO: Initial adaptation is done successfully on channel $channel"
        puts $fileid "\tINFO: Initial adaptation is done successfully on channel $channel"
        return 0
    }
 }

 proc reset_pma { channel } {
    global fileid
    global error_count
    set timeout 1000
    
    #reset PMA
    write_xcvr_mc_reconfig_csr_reg $channel 0x200 0x0
    write_xcvr_mc_reconfig_csr_reg $channel 0x201 0x0
    write_xcvr_mc_reconfig_csr_reg $channel 0x202 0x0
    write_xcvr_mc_reconfig_csr_reg $channel 0x203 0x81
    set rst_done0 [read_xcvr_mc_reconfig_csr_reg $channel  0x207  0  0]
    set rst_done1 [read_xcvr_mc_reconfig_csr_reg $channel  0x204  0  0]
    set i 0
    while {(($rst_done0 != 0) || ($rst_done1 != 0)) && ($i <= $timeout)} {
        puts "\tINFO: Waiting PMA reset . . ."
        puts $fileid "\tINFO: Waiting PMA reset . . ."
        set rst_done0 [read_xcvr_mc_reconfig_csr_reg $channel  0x207  0  0]
        set rst_done1 [read_xcvr_mc_reconfig_csr_reg $channel  0x204  0  0]
        incr i
    }
    after 500

    # Load initial/last PMA setting
    write_xcvr_mc_reconfig_csr_reg $channel 0x91 1

    set init_time [clock clicks -milliseconds]
    set r [read_xcvr_mc_reconfig_csr_reg $channel  0x8b 0 0]
    set tx_ready [expr ($r >> 2) & 1]
    set rx_ready [expr ($r >> 3) & 1]
    set j 0
    while {(($tx_ready == 0) || ($rx_ready == 0)) && ($j <= $timeout)} {
      set r [read_xcvr_mc_reconfig_csr_reg $channel  0x8b 0 0]
      set tx_ready [expr ($r >> 2) & 1]
      set rx_ready [expr ($r >> 3) & 1]
      set end_time [clock clicks -milliseconds]
      set wait_time [expr $end_time - $init_time]
      puts "\tINFO: Waiting $wait_time" 
      puts $fileid "\tINFO: Waiting $wait_time" 
      incr j      
    }
    if { ($i > $timeout) || ($j > $timeout) } {
       puts "\tERROR: PMA reset isn't triggered properly"
       puts $fileid "\tERROR: PMA reset isn't triggered properly"
       set  error_count [expr $error_count + 1]      
    }
    puts "" 
    puts $fileid "" 
 }
 
 proc run_calibration_direct_interrupt { channel loopback_mode} {
    global fileid
    global error_count

    if {$loopback_mode == 1} {
        puts "\tINFO: Internal loopback \n"
        c3_ehiplane_xcvr_mc_loopback_mode $channel 1 $error_count
    } else {
        puts "\tINFO: External loopback \n"
        c3_ehiplane_xcvr_mc_loopback_mode $channel 1 $error_count
        initial_adaptation $channel
        initial_adaptation_status $channel
        c3_ehiplane_xcvr_mc_mission_mode $channel 1 $error_count
    }
    puts "\tINFO: Channel $channel initial adaptation \n"
    initial_adaptation $channel
    puts "\tINFO: Channel $channel initial adaptation status"
    puts $fileid "\tINFO: Channel $channel initial adaptation status"    
    initial_adaptation_status $channel

    if {$loopback_mode == 0} {
        puts "\tINFO: Channel $channel continuous adaptation \n"
        continuous_adaptation $channel
    }

    puts ""
    puts $fileid ""
    
    after 1000
 }
