# (C) 2001-2022 Intel Corporation. All rights reserved.
# Your use of Intel Corporation's design tools, logic functions and other 
# software and tools, and its AMPP partner logic functions, and any output 
# files from any of the foregoing (including device programming or simulation 
# files), and any associated documentation or information are expressly subject 
# to the terms and conditions of the Intel Program License Subscription 
# Agreement, Intel FPGA IP License Agreement, or other applicable 
# license agreement, including, without limitation, that your use is for the 
# sole purpose of programming logic devices manufactured by Intel and sold by 
# Intel or its authorized distributors.  Please refer to the applicable 
# agreement for further details.



global jtag_master
global error_count
global fileid
global port_id
global jtag_port_id
global service_on
global totalChannel
global speed

global SEL_FPGA1
global SEL_FPGA2
    
#==============================================================================
#                       JTAG PATH               
#==============================================================================
proc path_jtag {value} {
    global jtag_master
    global fileid   
    set jtag_master [lindex [get_service_paths master] $value];
    puts $fileid "\nInfo: Set JTAG Master Service Path\n"
}

#==============================================================================
#                       Open JTAG               
#==============================================================================
proc open_jtag {} {
    global jtag_master
    global fileid
    global service_on   
    open_service master $jtag_master;
    set service_on 1
    puts $fileid "\nInfo: Opened JTAG Master Service\n"
}

#==============================================================================
#                       Close JTAG               
#==============================================================================
proc close_jtag {} {
    global jtag_master
    global fileid
    global service_on
    close_service master $jtag_master;
    set service_on 0
    puts $fileid "\nInfo: Closed JTAG Master Service\n\n";
}

#==============================================================================
# Write 32                
#==============================================================================
proc wr32 {base_address sub_base_address offset value} {
    global jtag_master
    
    set address [expr $base_address + $sub_base_address + $offset];
    set address_hex [format "%#x" $address];
    master_write_32 $jtag_master $address_hex $value;
}

#==============================================================================
# Read 32                
#==============================================================================
proc rd32 {base_address sub_base_address offset} {
    global jtag_master
    
    set address [expr $base_address + $sub_base_address + $offset];
    set address_hex [format "%#x" $address];
    set data [master_read_32 $jtag_master $address_hex 1] 
    return $data
}

#==============================================================================
# Read 64                
#==============================================================================
proc rd64 {base_address sub_base_address offset} {
    global jtag_master

    set address [expr $base_address + $sub_base_address + $offset];
    set address_hex [format "%#x" $address];
    set data1 [master_read_32 $jtag_master $address_hex 1]
    set data2 [master_read_32 $jtag_master [expr $address_hex + 0x04] 1]
    regsub {0x} $data1 {} data1  
    return $data2$data1
}

#==============================================================================
# write : 32-bit data 
#==============================================================================
 proc reg_write {base offset wdata} {
    global port_id
    global jtag_port_id
    global service_on
    if { $service_on } { 
       set rdata [wr32 $base 0 $offset $wdata]
puts "\t w-base =$base "
       set port_id [lindex [get_service_paths master] 0];
       open_service master $port_id
set address [format "%#x" [expr $base + $offset]];
       master_write_32 $port_id $address $wdata;
       
close_service master $port_id
       
 puts "\t w-address  $address"

    } else {
       set port_id [lindex [get_service_paths master] $jtag_port_id];
       open_service master $port_id;
       set address [format "%#x" [expr $base + $offset]]; 
       master_write_32 $port_id $address $wdata;
       close_service master $port_id;                                                       
    }
 }

#==============================================================================
# read : 32-bit data 
#==============================================================================
 proc reg_read {base offset} {
    global port_id
    global jtag_port_id
    global service_on
 
    if { $service_on } {
       set rdata [rd32 $base 0 $offset]
puts "\t r-base =$base"
       set port_id [lindex [get_service_paths master] 0];
       open_service master $port_id
set address [format "%#x" [expr $base + $offset]];
#master_read_32 $port_id 0x1803004 1
set rdata [master_read_32 $port_id $address 1]
       
close_service master $port_id
       
 puts "\t r-address $address"
    } else {
       set port_id [lindex [get_service_paths master] $jtag_port_id];
       open_service master $port_id;

       set address [format "%#x" [expr $base + $offset]]; 
       set rdata [master_read_32 $port_id $address 1] 
       close_service master $port_id;
    }
    return $rdata
 }

#==============================================================================
# Sleep : ms calculation for delay insertion
#==============================================================================
 proc uniqkey { } {
    set key   [ expr { pow(2,31) + [ clock clicks ] } ]
    set key   [ string range $key end-8 end-3 ]
    set key   [ clock seconds ]$key
    return $key
 }
 
 proc sleep { ms } {
    set uniq [ uniqkey ]
    set ::__sleep__tmp__$uniq 0
    after $ms set ::__sleep__tmp__$uniq 1
    vwait ::__sleep__tmp__$uniq
    unset ::__sleep__tmp__$uniq
 }
    
#==============================================================================
# rand : generate random integer b/w min and max
#============================================================================== 
 proc myRand { min max } {
    set maxFactor [expr [expr $max + 1] - $min]
    set value [expr int([expr rand() * 100])]
    set value [expr [expr $value % $maxFactor] + $min]
    return $value
 }

 # ========================================================================
 # split 32-bit word into 8 nibbles
 proc word_split {word} {
   global nibble0
   global nibble1
   global nibble2
   global nibble3
   global nibble4
   global nibble5
   global nibble6
   global nibble7   
   set nibble0 [lindex [split $word {}] 9]
   set nibble1 [lindex [split $word {}] 8]
   set nibble2 [lindex [split $word {}] 7]
   set nibble3 [lindex [split $word {}] 6]
   set nibble4 [lindex [split $word {}] 5]
   set nibble5 [lindex [split $word {}] 4]
   set nibble6 [lindex [split $word {}] 3]
   set nibble7 [lindex [split $word {}] 2]
   return $nibble0
   return $nibble1
   return $nibble2
   return $nibble3
   return $nibble4
   return $nibble5
   return $nibble6
   return $nibble7
}

# split a byte into 8 bits
proc byte_split {byte} {
   global bit0
   global bit1
   global bit2
   global bit3
   global bit4
   global bit5
   global bit6
   global bit7
   set bit0 [lindex [split $byte {}] 7]
   set bit1 [lindex [split $byte {}] 6]
   set bit2 [lindex [split $byte {}] 5]
   set bit3 [lindex [split $byte {}] 4]
   set bit4 [lindex [split $byte {}] 3]
   set bit5 [lindex [split $byte {}] 2]
   set bit6 [lindex [split $byte {}] 1]
   set bit7 [lindex [split $byte {}] 0]
   return $bit0
   return $bit1
   return $bit2
   return $bit3
   return $bit4
   return $bit5
   return $bit6
   return $bit7
}

# convert hexadecimal nibble to binary
proc hex2bin {hex} {
   global bin
   set number [expr $hex]
   set bin ""
   for {set i 0} {$i<4} {incr i} {
      set digit [expr $number & 1]
      set bin "$digit$bin"
      set number [expr $number >> 1]
   }
   return $bin
}

