# (C) 2001-2022 Intel Corporation. All rights reserved.
# Your use of Intel Corporation's design tools, logic functions and other 
# software and tools, and its AMPP partner logic functions, and any output 
# files from any of the foregoing (including device programming or simulation 
# files), and any associated documentation or information are expressly subject 
# to the terms and conditions of the Intel Program License Subscription 
# Agreement, Intel FPGA IP License Agreement, or other applicable 
# license agreement, including, without limitation, that your use is for the 
# sole purpose of programming logic devices manufactured by Intel and sold by 
# Intel or its authorized distributors.  Please refer to the applicable 
# agreement for further details.


global claimed_issp

proc open_issp {index} {
    global claimed_issp
    
#    set claimed_issp [lindex [get_service_paths issp] $index]
    set claimed_issp [lsearch -inline [get_service_paths issp] *probe_$index*]

    open_service issp $claimed_issp
}

proc close_issp {} {
    global claimed_issp

    close_service issp $claimed_issp
}

# Active high
proc set_reconfig_reset {} {
    global claimed_issp

    open_issp 0    
    # Assert bit 0
    puts "\tINFO: Set Reconfig Reset"
    set current_value [issp_read_probe_data $claimed_issp]
    issp_write_source_data $claimed_issp [expr 0x1 | $current_value]
    after 100

    close_issp
}

proc release_reconfig_reset {} {
    global claimed_issp

    open_issp 0    
    # Deassert bit 0
    puts "\tINFO: Release Reconfig Reset"
    set current_value [issp_read_probe_data $claimed_issp]
    issp_write_source_data $claimed_issp [expr ~0x1 & $current_value]
    after 100

    close_issp
}

# Active low
proc set_csr_reset {ch} {
    global claimed_issp

    open_issp 0    
    # Deassert bit ($ch*3 + 3)
    puts "\tINFO: Channel $ch: Set CSR Reset"
    set current_value [issp_read_probe_data $claimed_issp]
    issp_write_source_data $claimed_issp [expr ~(0x1 << $ch*3+3) & $current_value]
    after 100
    
    close_issp
}

proc release_csr_reset {ch} {
    global claimed_issp

    open_issp 0    
    # Assert bit ($ch*3 + 3)
    puts "\tINFO: Channel $ch: Release CSR Reset"
    set current_value [issp_read_probe_data $claimed_issp]
    issp_write_source_data $claimed_issp [expr (0x1 << $ch*3+3) | $current_value]
    after 100

    close_issp
}

# Active low
proc set_tx_reset {ch} {
    global claimed_issp

    open_issp 0    
    # Deassert bit ($ch*3 + 2)
    puts "\tINFO: Channel $ch: Set TX Reset"    
    set current_value [issp_read_probe_data $claimed_issp]
    issp_write_source_data $claimed_issp [expr ~(0x1 << $ch*3+2) & $current_value]
    after 100

    close_issp
}

proc release_tx_reset {ch} {
    global claimed_issp

    open_issp 0    
    # Assert bit ($ch*3 + 2)
    puts "\tINFO: Channel $ch: Release TX Reset"    
    set current_value [issp_read_probe_data $claimed_issp]
    issp_write_source_data $claimed_issp [expr (0x1 << $ch*3+2) | $current_value]
    after 100

    close_issp
}

# Active low
proc set_rx_reset {ch} {
    global claimed_issp

    open_issp 0    
    # Deassert bit ($ch*3 + 1)
    puts "\tINFO: Channel $ch: Set RX Reset"    
    set current_value [issp_read_probe_data $claimed_issp]
    issp_write_source_data $claimed_issp [expr ~(0x1 << $ch*3+1) & $current_value]
    after 100
    
    close_issp
}

proc release_rx_reset {ch} {
    global claimed_issp

    open_issp 0    
    # Assert bit ($ch*3 + 1)
    puts "\tINFO: Channel $ch: Release RX Reset"    
    set current_value [issp_read_probe_data $claimed_issp]
    issp_write_source_data $claimed_issp [expr (0x1 << $ch*3+1) | $current_value]
    after 100

    close_issp
}

proc get_pll_lock_status {} {
    global claimed_issp

    set timeout 1000000
    set i 0
    open_issp 1    
    puts "\tINFO: Checking PLL lock status..."
    
    while {1} {
        set rdata0 [issp_read_probe_data $claimed_issp]
        set iopll_sclk_locked [expr ($rdata0 >> 0) & 0x1]
        set channel_pll_locked [expr ($rdata0 >> 1) & 0x1]
        puts "\tiopll_sclk_locked $iopll_sclk_locked, channel_pll_locked $channel_pll_locked"
        if {($iopll_sclk_locked == 1 && $channel_pll_locked == 1 ) || $i >= $timeout } break
        incr i
    }
    if {$iopll_sclk_locked == 1 && $channel_pll_locked == 1 } {
        puts "\tINFO: PLL is locked"
    } else {
        puts "\tERROR: PLL is not locked"
    }

    close_issp
}

proc get_checker_pass_status {compflag comprdata ch} {
    global claimed_issp
    global fileid    
    global error_count

    open_issp 2
    puts "\tChannel $ch : Get checker_pass status:"
    set rdata0 [issp_read_probe_data $claimed_issp]
    # puts "\tDebug: rdata0 = $rdata0"
    after 1000  
    
    set rdata [expr ($rdata0 >> $ch) & 0x1]
    if {$compflag} {
        puts "\tChecker value = $rdata"
        puts $fileid "\tChannel $ch : Checker value = $rdata"
        if {$rdata == $comprdata} {
            puts "\tChecker status = Passed!"
            puts $fileid "\tChannel $ch : Checker status = Passed!"
        } else {
            puts "\tChecker status = Failed!"   
            puts $fileid "\tChannel $ch : Checker status = Failed!"
            set error_count [expr $error_count + 1]
        }
    } else {
        puts "\tChecker value = $rdata"
        puts $fileid "\tChannel $ch : Checker value = $rdata"
    }

    close_issp
}

proc instance_info {} {
    global claimed_issp

    open_issp 0   
    array set instance_info [issp_get_instance_info $claimed_issp]
    set source_width $instance_info(source_width)
    set probe_width $instance_info(probe_width)
    
    puts "\tsource_width = $source_width, probe_width = $probe_width"

    close_issp    
}

proc read_issp_dl {ch} {
    global claimed_issp

    open_issp 3
    puts "\tChannel $ch : Read Determenistic latency counts"
    set rdata0 [issp_read_probe_data $claimed_issp]
    set shift_bit [expr $ch * 20]
    set rdata1  [expr ($rdata0 >> $shift_bit) & 0xFFFFF]

    close_issp

    return $rdata1
}

proc get_rx_cdr_locked {ch} {
    global claimed_issp

    open_issp 4
    puts "\tINFO: Channel $ch : Checking RX CDR lock status..."

    set iteration 15
    set delay_ms 1000

    set i 0
    while {$i < $iteration} {
        set rdata0 [issp_read_probe_data $claimed_issp]
        set shift_bit [expr $ch * 20 + 0]
        set rdata1 [expr ($rdata0 >> $shift_bit) & 0x1]

        if {$rdata1 == 1} break

        set i [expr $i + 1]
        after $delay_ms
        puts "\t\t INSIDE Iteration: $i"
    }

    close_issp

    if { $rdata1 == 0 && $i == $iteration } {
        puts "\t\t ERROR: Timeout exceeded"
    }    

    return $rdata1
}

proc get_tx_pll_locked {ch} {
    global claimed_issp

    open_issp 4
    puts "\tINFO: Channel $ch : Checking TX PLL lock status..."

    set iteration 15
    set delay_ms 1000

    set i 0
    while {$i < $iteration} {
        set rdata0 [issp_read_probe_data $claimed_issp]
        set shift_bit [expr $ch * 20 + 1]
        set rdata1 [expr ($rdata0 >> $shift_bit) & 0x1]

        if {$rdata1 == 1} break

        set i [expr $i + 1]
        after $delay_ms
        puts "\t\t INSIDE Iteration: $i"
    }

    close_issp

    if { $rdata1 == 0 && $i == $iteration } {
        puts "\t\t ERROR: Timeout exceeded"
    }    

    return $rdata1
}

proc get_tx_lanes_stable {ch} {
    global claimed_issp

    open_issp 4
    puts "\tINFO: Channel $ch : Checking TX lane stable status..."

    set iteration 15
    set delay_ms 1000

    set i 0
    while {$i < $iteration} {
        set rdata0 [issp_read_probe_data $claimed_issp]
        set shift_bit [expr $ch * 20 + 5]
        set rdata1 [expr ($rdata0 >> $shift_bit) & 0x1]

        if {$rdata1 == 1} break

        set i [expr $i + 1]
        after $delay_ms
        puts "\t\t INSIDE Iteration: $i"
    }

    close_issp

    if { $rdata1 == 0 && $i == $iteration } {
        puts "\t\t ERROR: Timeout exceeded"
    }    

    return $rdata1
}

proc get_rx_pcs_aligned {ch} {
    global claimed_issp

    open_issp 4
    puts "\tINFO: Channel $ch : Checking RX PCS align status..."
    
    set iteration 15
    set delay_ms 1000

    set i 0
    while {$i < $iteration} {
        set rdata0 [issp_read_probe_data $claimed_issp]
        set shift_bit [expr $ch * 20 + 4]
        set rdata1 [expr ($rdata0 >> $shift_bit) & 0x1]

        if {$rdata1 == 1} break

        set i [expr $i + 1]
        after $delay_ms
        puts "\t\t INSIDE Iteration: $i"
    }

    close_issp

    if { $rdata1 == 0 && $i == $iteration } {
        puts "\t\t ERROR: Timeout exceeded"
    }    

    return $rdata1
}

proc get_ehip_ready {ch} {
    global claimed_issp

    open_issp 4
    puts "\tINFO: Channel $ch : Checking EHIP ready status..."

    set iteration 15
    set delay_ms 1000

    set i 0
    while {$i < $iteration} {
        set rdata0 [issp_read_probe_data $claimed_issp]
        set shift_bit [expr $ch * 20 + 3]
        set rdata1 [expr ($rdata0 >> $shift_bit) & 0x1]

        if {$rdata1 == 1} break

        set i [expr $i + 1]
        after $delay_ms
        puts "\t\t INSIDE Iteration: $i"
    }

    close_issp

    if { $rdata1 == 0 && $i == $iteration } {
        puts "\t\t ERROR: Timeout exceeded"
    }    

    return $rdata1
}

proc get_rx_block_lock {ch} {
    global claimed_issp

    open_issp 4
    puts "\tINFO: Channel $ch : Checking RX block lock status..."

    set iteration 15
    set delay_ms 1000

    set i 0
    while {$i < $iteration} {
        set rdata0 [issp_read_probe_data $claimed_issp]
        set shift_bit [expr $ch * 20 + 2]
        set rdata1 [expr ($rdata0 >> $shift_bit) & 0x1]

        if {$rdata1 == 1} break

        set i [expr $i + 1]
        after $delay_ms
        puts "\t\t INSIDE Iteration: $i"
    }

    close_issp

    if { $rdata1 == 0 && $i == $iteration } {
        puts "\t\t ERROR: Timeout exceeded"
    }    

    return $rdata1
}

proc get_rx_ready {ch} {
    global claimed_issp

    open_issp 4
    puts "\tINFO: Channel $ch : Checking RX ready status..."

    set iteration 15
    set delay_ms 1000

    set i 0
    while {$i < $iteration} {
        set rdata0 [issp_read_probe_data $claimed_issp]
        set shift_bit [expr $ch * 20 + 6]
        set rdata1 [expr ($rdata0 >> $shift_bit) & 0x1]

        if {$rdata1 == 1} break

        set i [expr $i + 1]
        after $delay_ms
        puts "\t\t INSIDE Iteration: $i"
    }

    close_issp

    if { $rdata1 == 0 && $i == $iteration } {
        puts "\t\t ERROR: Timeout exceeded"
    }    

    return $rdata1
}

proc get_tx_ready {ch} {
    global claimed_issp

    open_issp 4
    puts "\tINFO: Channel $ch : Checking TX ready status..."

    set iteration 15
    set delay_ms 1000

    set i 0
    while {$i < $iteration} {
        set rdata0 [issp_read_probe_data $claimed_issp]
        set shift_bit [expr $ch * 20 + 7]
        set rdata1 [expr ($rdata0 >> $shift_bit) & 0x1]

        if {$rdata1 == 1} break

        set i [expr $i + 1]
        after $delay_ms
        puts "\t\t INSIDE Iteration: $i"
    }

    close_issp

    if { $rdata1 == 0 && $i == $iteration } {
        puts "\t\t ERROR: Timeout exceeded"
    }    

    return $rdata1
}

