# (C) 2001-2022 Intel Corporation. All rights reserved.
# Your use of Intel Corporation's design tools, logic functions and other 
# software and tools, and its AMPP partner logic functions, and any output 
# files from any of the foregoing (including device programming or simulation 
# files), and any associated documentation or information are expressly subject 
# to the terms and conditions of the Intel Program License Subscription 
# Agreement, Intel FPGA IP License Agreement, or other applicable 
# license agreement, including, without limitation, that your use is for the 
# sole purpose of programming logic devices manufactured by Intel and sold by 
# Intel or its authorized distributors.  Please refer to the applicable 
# agreement for further details.


source src/c3_ehiplane_user_cfgcsr_inc.tcl
source src/c3_cpri_cfgcsr_inc.tcl
source src/c3_rsfec_cfgcsr_inc.tcl
source src/c3_xcvr_cfgcsr_inc.tcl
source src/issp.tcl
source src/parameter.tcl

set fileid stdout
set service_on 0

set systemTime [clock seconds]
set local_dir [pwd]
set filename "$local_dir/c3_cpri_test.log"
set fileid [open $filename w]

set dllog [open "$local_dir/det_latency_${speed}.log" w]
puts $dllog "\titeration\tchannel\ttx_delay\trx_delay\trt_count\tbit_slip\n"

path_jtag $jtag_port_id

open_jtag

set error_count 0
set fail_count 0
set pass_count 0

set compresult       1

set rsfec_lane_rx_stat_0_val        0 
                  
set rsfec_corr_cw_cnt_0_lo_val      0
set rsfec_corr_cw_cnt_0_hi_val      0
                  
set rsfec_uncorr_cw_cnt_0_lo_val    0
set rsfec_uncorr_cw_cnt_0_hi_val    0

set pkt_num_ch0      0x347821
set pkt_num_ch0_25g  0x349398
set pkt_num_ch0_10g  0x349392

set systemTime [clock seconds]
puts $fileid "\tTest Start time is: [clock format $systemTime -format %H:%M:%S]"
puts $fileid "\tTest Start date is: [clock format $systemTime -format %D]\n"

puts $fileid "\nInfo: Start of c3_cpri_test \n"  
puts "\nInfo: Start of c3_cpri_test \n"

set NUM_CHANNELS $totalChannel

puts "INFO: Basic CPRI test \n"

get_pll_lock_status

puts ""

set ch_reset_queue(0)  $ch_reset_queue_0
set ch_reset_queue(1)  $ch_reset_queue_1
set ch_reset_queue(2)  $ch_reset_queue_2
set ch_reset_queue(3)  $ch_reset_queue_3

set loop_error_count 0

for {set i 0} {$i < 1} {incr i} {
    puts $fileid "Loop $i"
    puts "Loop $i"

    set loop_error_count $error_count
   
    # Set all reset at the beginning
    set_reconfig_reset
    for {set ch 0} {$ch < $NUM_CHANNELS} {incr ch} {
        set_csr_reset $ch
        set_tx_reset $ch
        set_rx_reset $ch
    }

    # Release reconfig reset
    release_reconfig_reset

    # Release master channel reset first
    release_csr_reset $ch_reset_queue(0)
    release_tx_reset $ch_reset_queue(0)
    release_rx_reset $ch_reset_queue(0)
    puts "INFO: Wait for master channel to stable"
    after 5000

    # Release slave channel reset
    for {set ch 1} {$ch < $NUM_CHANNELS} {incr ch} {
        release_csr_reset $ch_reset_queue($ch)
        release_tx_reset $ch_reset_queue($ch)
        release_rx_reset $ch_reset_queue($ch)
    }
    puts "INFO: Release Reset Done!"

    if {$loopback_mode == 1} {
        # Turn on serial loopback
        puts "INFO: Turn on serial loopback"
        for {set ch 0} {$ch < $NUM_CHANNELS} {incr ch} {
            c3_ehiplane_xcvr_mc_loopback_mode $ch 1 $error_count
        }
    } else {
        # Turn on external loopback
        puts "INFO: Turn on mission mode"
        for {set ch 0} {$ch < $NUM_CHANNELS} {incr ch} {
            c3_ehiplane_xcvr_mc_mission_mode $ch 1 $error_count
        }
    }

    for {set ch 0} {$ch < $NUM_CHANNELS} {incr ch} {
        puts "INFO: Running calibration..."
        puts "INFO: Channel $ch \n"

        set ATTEMPT_LIMIT $recal_attempt_limit
        set fail_status 0
        for {set attempt 1} {$attempt <= $ATTEMPT_LIMIT} {incr attempt} {

            puts "INFO: Attempt $attempt \n"

            # Step 1 - Assert TX RX Digital Reset
            puts "INFO: Assert TX RX Digital Reset \n"
            set_tx_reset $ch
            set_rx_reset $ch
            after 100
            puts ""

            # Step 2 - Reset PMA
            puts "INFO: Reset PMA \n"
            reset_pma $ch
            after 100
            puts ""

            # Step 3 - De-assert TX Digital Reset
            puts "INFO: De-assert TX Digital Reset \n"
            release_tx_reset $ch
            after 100
            puts ""
            
            if { ($rcp_load_enable == 1) } {
                # Step 4 - Load new PMA configuration (mailbox method) -> initialAdaptation (ILB/ELB)
                puts "INFO: Load new PMA configuration \n"
                c3_ehiplane_xcvr_run_calibration $ch $error_count $loopback_mode $rcp_load_enable
            } else {
                # Step 4 - Direct interrupt method
                run_calibration_direct_interrupt $ch $loopback_mode
            }
            after 100
            puts ""

            # Step 5 - De-assert RX Digital Reset
            puts "INFO: De-assert RX Digital Reset \n"
            release_rx_reset $ch
            after 2000
            puts ""

            # Check status registers
            puts "INFO: Check status registers \n"

            set tx_pll_locked [get_tx_pll_locked $ch]
            puts "\tINFO: tx_pll_locked   = $tx_pll_locked"

            set rx_cdr_locked [get_rx_cdr_locked $ch]
            puts "\tINFO: rx_cdr_locked   = $rx_cdr_locked"

            if {$ehip_mode_sl_0 != "ehip_disable"} {
                set tx_lanes_stable [get_tx_lanes_stable $ch]
                puts "\tINFO: tx_lanes_stable = $tx_lanes_stable"

                set rx_pcs_aligned [get_rx_pcs_aligned $ch]
                puts "\tINFO: rx_pcs_aligned  = $rx_pcs_aligned"

                set ehip_ready [get_ehip_ready $ch]
                puts "\tINFO: ehip_ready      = $ehip_ready"

                set rx_block_locked [get_rx_block_lock $ch]
                puts "\tINFO: rx_block_locked = $rx_block_locked"

                set rx_ready 1
                set tx_ready 1
            } else { 
                set tx_ready [get_tx_ready $ch]
                puts "\tINFO: tx_ready = $tx_ready"

                set rx_ready [get_rx_ready $ch]
                puts "\tINFO: rx_ready = $rx_ready"
                
                set tx_lanes_stable 1
                set rx_pcs_aligned 1
                set ehip_ready 1
                set rx_block_locked 1
            }

            puts ""

            if { $tx_pll_locked & $rx_cdr_locked & $tx_lanes_stable & $rx_pcs_aligned & $rx_block_locked & $ehip_ready & $tx_ready & $rx_ready } {
                set fail_status 0
                puts "\tINFO: Status check successful \n"
                break
            } else {
                set fail_status 1
                puts "\tERROR: Status check failed. \n"
            }
        }

        puts ""
        set error_count [expr $error_count + $fail_status]
    }

    after 1000
   
    # Send transactions on channel 0
    for {set ch 0} {$ch < $NUM_CHANNELS} {incr ch} {
        puts "Channel $ch : Wait for measure_valid to assert"
        set measure_valid [polling_for_bit $ch 0xC01 0 1]
        if {$measure_valid} {
            puts "\tmeasure_valid is asserted"
        } else {
            puts "\tERROR: measure_valid is not asserted"
        }
        get_checker_pass_status 0x1 0x1 $ch
        
        # deterministic latency measure
        set ch_offset [expr $ch * 20]
        set rt_count [read_issp_dl $ch ]
        set o_tx_delay_sclk [read_cpri_mc_cfgcsr_reg $ch 0xC02 0 0]
        set o_rx_delay_sclk [read_cpri_mc_cfgcsr_reg $ch 0xC03 0 0]
        if {$ehip_mode_sl_0 != "ehip_disable"} {
            if {$enable_rsfec == 1} {
                set bit_slip [read_xcvr_mc_reconfig_csr_reg $ch 0x29 0 0]
                set bit_slip_dec    [expr $bit_slip & 0x1f]
            } else {
                set bit_slip [read_xcvr_mc_reconfig_csr_reg $ch 0x28 0 0]
                set bit_slip_dec    [expr $bit_slip & 0x7f]
            }
        }
        if {$ehip_mode_sl_0 == "ehip_disable"} {
            set bit_slip [read_cpri_mc_cfgcsr_reg $ch 0xC00 0 0]
            set bit_slip_dec    [expr $bit_slip >> 5]
        }
        puts $dllog   "\t$i\t$ch\t$o_tx_delay_sclk\t$o_rx_delay_sclk\t$rt_count\t$bit_slip_dec"

    }

    sleep 1000

    if {$loop_error_count < $error_count} {
        puts $fileid "ERROR: Loop $i failed"
        puts "ERROR: Loop $i failed"
        set fail_count [expr $fail_count + 1]
    } else {
        puts $fileid "Info: Loop $i passed"
        puts "Info: Loop $i passed"
        set pass_count [expr $pass_count + 1]
    }

    puts $fileid "End of loop $i\n\n"
    puts "End of loop $i\n\n"

}

puts $fileid "\nInfo: End of c3_cpri_test \n"
puts "\nInfo: End of c3_cpri_test \n" 

set systemTime [clock seconds]
puts $fileid "\n\tTest End time is: [clock format $systemTime -format %H:%M:%S]"
puts $fileid "\tTest End date is: [clock format $systemTime -format %D]"

close_jtag

if {$error_count == 0} {
  puts $fileid "\nInfo: Test <c3_cpri_test> Passed \n" 
  } else {
  puts $fileid "\nERROR: Test <c3_cpri_test> Failed \n"
}
 
set total_count [expr $fail_count + $pass_count]
puts $fileid "\nInfo: Total loop passed = $pass_count/$total_count \n"
puts "\nInfo: Total loop passed = $pass_count/$total_count \n" 

close $fileid

set local_dir [pwd]
set filename "$local_dir/main_script.log"
set fileid [open $filename a]
if {$error_count == 0} {
  puts "\nInfo: Test <c3_cpri_test> Passed \n"
  puts $fileid "\nInfo: Test <c3_cpri_test> Passed \n" 
} else {
  puts "\nERROR: Test <c3_cpri_test> Failed \n"
  puts $fileid "\nERROR: Test <c3_cpri_test> Failed \n"
}
close $dllog
close $fileid
