// (C) 2001-2021 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.



//`define USE_CH_4567 // ch 0/1/2/3 is used if not defied


`timescale 1 ps / 1 ps
module Ethern
   (

      ///////// CLOCK /////////
      input              CLK_100,
      input              CLK_50,

      ///////// Buttons /////////
      input              CPU_RESET_n,

      ///////// Swtiches /////////
      input    [ 1: 0]   SW,

      ///////// LED /////////
      output   [ 3: 0]   LED, //LED is Low-Active

      ///////// QSFPDDA /////////

      output   [ 3: 0]   QSFP_TX_p,
      output   [ 3: 0]   QSFP_TX_n,
      input    [ 3: 0]   QSFP_RX_p,
      input    [ 3: 0]   QSFP_RX_n,
      input              QSFP_REFCLK_p,
      output             QSFP_INITMODE,
      input              QSFP_INTERRUPT_n,
      input              QSFP_MOD_PRS_n,
      output             QSFP_MOD_SEL_n,
      output             QSFP_RST_n,
      inout              QSFP_SCL,
      inout              QSFP_SDA

);


//////////////////////////////////////
// pin assignment

wire 			clk100;

wire 			qsfp_lowpwr;
wire 			qsfp_rstn;
wire        qsfp_mod_sel_n;

wire 			i_clk_ref;
wire [3:0]  o_tx_serial;
wire [3:0]  i_rx_serial;
wire [3:0]  o_tx_serial_n;
wire [3:0]  i_rx_serial_n;

wire 			tx_serial_data;
wire [1:0]  user_led;

assign clk100         = CLK_100;
assign LED            = user_led;

//--refclk wire
assign i_clk_ref      = QSFP_REFCLK_p;

//--control wire
assign QSFP_RST_n     = qsfp_rstn;
assign QSFP_INITMODE  = qsfp_lowpwr;
assign QSFP_MOD_SEL_n = qsfp_mod_sel_n;

//----tx data wire 
assign QSFP_TX_p      = {  o_tx_serial[3],  o_tx_serial[2],  o_tx_serial[1],  o_tx_serial[0]};
assign QSFP_TX_n      = {o_tx_serial_n[3],o_tx_serial_n[2],o_tx_serial_n[1],o_tx_serial_n[0]};

//----rx data wire 
assign {  i_rx_serial[3],  i_rx_serial[2],  i_rx_serial[1],  i_rx_serial[0]} = QSFP_RX_p;
assign {i_rx_serial_n[3],i_rx_serial_n[2],i_rx_serial_n[1],i_rx_serial_n[0]} = QSFP_RX_n;
  
//---reset
wire por_reset;

power_on_reset power_on_reset_inst(
	.clk(CLK_50),
	.reset(por_reset)
);

//////////////////////////////////////


    assign qsfp_rstn      = 1'b1;
    assign qsfp_lowpwr    = SW[0]; // 1'b1; 1(SW UP): low power, 0(SW DOWN): not low power
	 assign qsfp_mod_sel_n = SW[1]; // 1'b0: enable i2c                                    
    assign user_led       =  CPU_RESET_n?~SW :2'b00; 


    localparam DEVICE_FAMILY = "Agilex";
    localparam WORDS = 8;
    localparam WIDTH = 64;
    localparam SOP_ON_LANE0 = 1'b1;
    localparam SIM_NO_TEMP_SENSE = 1'b0;


    /////////////////////////
    // dev_clr sync-reset
    /////////////////////////
    //clk100=i_reconfig_clk
    //wire user_mode_sync;
    wire arst, iopll_locked, clk100;
    //alt_aeuex_user_mode_det dev_clr( .ref_clk(clk100), .user_mode_sync(user_mode_sync));

    wire source_reset;

 //Instantiate S10 IP reset:

      wire ninit_done ;
      reset_ip reset (
	.ninit_done (ninit_done)
	  );

      // Reset delay will start counting after ninit_done, and will keep
      // arst asserted until it finishes
      wire rd1_ready;
      alt_reset_delay rd1 (
          .clk(clk100),
          .ready_in(~ninit_done),
          .ready_out(rd1_ready)
      );
      defparam rd1.CNTR_BITS = 16;
	  
	  
// ARST conditional upon ninit_done from S10 IP reset. Ninit_done is active low. ninit_done=0 => s10 is good to go	  
    //assign arst = ~user_mode_sync | ~iopll_locked | source_reset | ninit_done ;
    
    assign arst = ~rd1_ready | ~iopll_locked | source_reset ;

    wire [20:0] i_eth_reconfig_addr;
    wire [10:0] i_rsfec_reconfig_addr;
    wire [31:0] o_eth_reconfig_readdata;
    wire [7:0] o_rsfec_reconfig_readdata;
    wire        i_eth_reconfig_read, i_eth_reconfig_write,o_eth_reconfig_readdata_valid;
    wire        i_rsfec_reconfig_read, i_rsfec_reconfig_write,o_rsfec_reconfig_readdata_valid;


    wire [31:0] i_eth_reconfig_writedata;
    wire [7:0] i_rsfec_reconfig_writedata;
    wire [31:0] i_xcvr_reconfig_writedata;

    wire        i_reconfig_clk = clk100;
    wire        o_clk_pll_div64;

    wire [15:0] khz_clk_rx_rs;
    wire [15:0] khz_clk_tx_rs;
    wire [15:0]   khz_rx_clk_i;
    wire [15:0]   khz_tx_clk_i;
    wire clk_rx_rs;
    wire clk_tx_rs;

    // input domain (from user logic toward pins)
    wire                   clk_din = o_clk_pll_div64;
    wire [WORDS*WIDTH-1:0] din;               // payload to send, left to right
    wire       [WORDS-1:0] din_start;         // start pos, first of every 8 bytes
    wire     [WORDS*8-1:0] din_end_pos;       // end position, any byte
    wire                   din_ack;           // payload is accepted

    // output domain (from pins toward user logic)
    wire                   clk_dout = o_clk_pll_div64;

    wire    [WORDS*64-1:0] dout_d;            // 5 word out stream, left to right
    wire       [WORDS-1:0] dout_first_data;
    wire     [WORDS*8-1:0] dout_last_data;
    wire                   dout_valid;

    wire    [5:0]   i_tx_empty;
    wire    [511:0]    i_tx_data;
    wire                       i_tx_endofpacket;
    wire                       o_tx_ready;
    wire                       i_tx_startofpacket;
    wire                       i_tx_valid;
    wire    [511:0]    o_rx_data;
    wire    [5:0]   o_rx_empty;
    wire                       o_rx_endofpacket;
    wire                       o_rx_startofpacket;
    wire                       o_rx_valid;
    wire    [5:0]   o_rx_error;

    //--- functions
        `include "common/alt_aeuex_wide_l8if_functions.iv"
    assign i_tx_data          = din;
    //assign i_tx_valid         = 1'b1;
    assign din_ack            = o_tx_ready;
    assign dout_d             = o_rx_data;
    assign dout_valid         = o_rx_valid;
    assign dout_first_data    = {o_rx_startofpacket, {WORDS-1{1'b0}}};
    assign dout_last_data     = o_rx_endofpacket ? alt_aeuex_wide_decode6to64(o_rx_empty) : 64'h0;

    assign iopll_locked = 1'b1;

    wire                 status_read;
    wire                 status_write;
    wire                 status_waitrequest;
    wire                 status_readdata_valid;
    wire     [31:0]      status_readdata;
    wire     [31:0]      status_writedata;
    wire     [23:0]      status_addr;

    wire                 status_read_r = status_read;
    wire                 status_write_r = status_write;
    wire     [31:0]      status_writedata_r = status_writedata;
    wire     [23:0]      status_addr_r = status_addr;
    wire                select_waitrequest;
    wire [3:0]   o_xcvr_reconfig_waitrequest;
    wire         o_eth_reconfig_waitrequest;
    wire         o_rsfec_reconfig_waitrequest;
    wire [31:0]  o_xcvr_reconfig_readdata;

    wire [31:0] status_readdata_pc;
    wire status_readdata_valid_pc;


    assign select_waitrequest = ((status_addr_r >= 24'h1_00000) && (status_addr_r <= 24'h1_FFFFF)) ? o_xcvr_reconfig_waitrequest[0] :
        ((status_addr_r >= 24'h2_00000) && (status_addr_r <= 24'h2_FFFFF)) ? o_xcvr_reconfig_waitrequest[1] :
        ((status_addr_r >= 24'h3_00000) && (status_addr_r <= 24'h3_FFFFF)) ? o_xcvr_reconfig_waitrequest[2] :
        ((status_addr_r >= 24'h4_00000) && (status_addr_r <= 24'h4_FFFFF)) ? o_xcvr_reconfig_waitrequest[3] :
        ((status_addr_r >= 24'h0_10000) && (status_addr_r <= 24'h0_107FF)) ? o_rsfec_reconfig_waitrequest :
        ((status_addr_r >= 24'h0_00000) && (status_addr_r <= 24'h0_00FFF)) ? o_eth_reconfig_waitrequest :
        1'b0;

    assign status_waitrequest = select_waitrequest;

//Decoding for avmm address map registers
    wire [3:0]   xcvr_reconfig_readdata_valid;
    wire [3:0]   i_xcvr_reconfig_read;
    wire [3:0]   i_xcvr_reconfig_write;
    wire [75:0]  i_xcvr_reconfig_address;

    assign status_readdata_valid = status_readdata_valid_pc | o_eth_reconfig_readdata_valid | o_rsfec_reconfig_readdata_valid | (|xcvr_reconfig_readdata_valid[3:0]);

//Reconfig Decoding XCVR 0
    wire    xcvr0_cs    = ((status_addr_r >= 24'h1_00000) && (status_addr_r <= 24'h1_FFFFF));
    wire    xcvr0_read = status_read_r && xcvr0_cs;
    wire    xcvr0_write = status_write_r && xcvr0_cs;
    assign i_xcvr_reconfig_address[18:0]   = (xcvr0_read || xcvr0_write) ? status_addr_r[18:0] : 18'b0;
    assign xcvr_reconfig_readdata_valid[0] = xcvr0_read && !o_xcvr_reconfig_waitrequest[0];
//Reconfig Decoding XCVR 1
    wire    xcvr1_cs    = ((status_addr_r >= 24'h2_00000) && (status_addr_r <= 24'h2_FFFFF));
    wire    xcvr1_read = status_read_r && xcvr1_cs;
    wire    xcvr1_write = status_write_r && xcvr1_cs;
    assign i_xcvr_reconfig_address[37:19]  = (xcvr1_read || xcvr1_write) ? status_addr_r[18:0] : 18'b0;
    assign xcvr_reconfig_readdata_valid[1] = xcvr1_read  && !o_xcvr_reconfig_waitrequest[1];

    //Reconfig Decoding XCVR 2
    wire    xcvr2_cs    = ((status_addr_r >= 24'h3_00000) && (status_addr_r <= 24'h3_FFFFF));
    wire    xcvr2_read = status_read_r && xcvr2_cs;
    wire    xcvr2_write = status_write_r && xcvr2_cs;
    assign i_xcvr_reconfig_address[56:38]  = (xcvr2_read || xcvr2_write) ? status_addr_r[18:0] : 18'b0;
    assign xcvr_reconfig_readdata_valid[2] = xcvr2_read  && !o_xcvr_reconfig_waitrequest[2];
    //Reconfig Decoding XCVR 3
    wire    xcvr3_cs    = ((status_addr_r >= 24'h4_00000) && (status_addr_r <= 24'h4_FFFFF));
    wire    xcvr3_read = status_read_r && xcvr3_cs;
    wire    xcvr3_write = status_write_r && xcvr3_cs;
    assign i_xcvr_reconfig_address[75:57]  = (xcvr3_read ||xcvr3_write) ? status_addr_r[18:0] : 18'b0;
    assign xcvr_reconfig_readdata_valid[3] = xcvr3_read && !o_xcvr_reconfig_waitrequest[3];

    //Reconfig Decoding Eth
    wire    eth_cs    = ((status_addr_r >= 24'h0_00000) && (status_addr_r <= 24'h0_00FFF));
    wire    eth_read = status_read_r && eth_cs;
    wire    eth_write = status_write_r && eth_cs;
    assign i_eth_reconfig_addr = (eth_read|| eth_write) ? status_addr_r[20:0] : 21'b0;

    //Reconfig Decoding RSFEC
    wire    rsfec_cs    = ((status_addr_r >= 24'h0_10000) && (status_addr_r <= 24'h0_107FF));
    wire    rsfec_read = status_read_r && rsfec_cs;
    wire    rsfec_write = status_write_r && rsfec_cs;
    assign i_rsfec_reconfig_addr           = (rsfec_read|| rsfec_write) ? status_addr_r[10:0] : 11'b0;
    assign o_rsfec_reconfig_readdata_valid = rsfec_read && !o_rsfec_reconfig_waitrequest;
    assign i_eth_reconfig_writedata   = status_writedata_r;
    assign i_rsfec_reconfig_writedata = status_writedata_r[7:0];
    assign i_xcvr_reconfig_writedata  = {4{status_writedata_r[7:0]}};

    wire [39:0] o_rxstatus_data;
    wire [7:0] o_rx_pfc;
    wire [7:0] i_tx_pfc;
    wire i_stats_snapshot;
    assign i_stats_snapshot       = 1'b0;
    assign i_xcvr_reconfig_read   = {xcvr3_read,xcvr2_read,xcvr1_read,xcvr0_read};
    assign i_xcvr_reconfig_write  = {xcvr3_write,xcvr2_write,xcvr1_write,xcvr0_write};
    assign i_eth_reconfig_read    = eth_read;
    assign i_eth_reconfig_write   = eth_write;
    assign i_rsfec_reconfig_read  = rsfec_read;
    assign i_rsfec_reconfig_write = rsfec_write;

    assign status_readdata =
        status_readdata_valid_pc        ?  status_readdata_pc[31:0] :
        o_eth_reconfig_readdata_valid   ?  o_eth_reconfig_readdata  :
        o_rsfec_reconfig_readdata_valid ?  {24'b0, o_rsfec_reconfig_readdata}  :
        xcvr_reconfig_readdata_valid[0] ?  {24'b0, o_xcvr_reconfig_readdata[7:0]} :
        xcvr_reconfig_readdata_valid[1] ?  {24'b0, o_xcvr_reconfig_readdata[15:8]} :
        xcvr_reconfig_readdata_valid[2] ?  {24'b0, o_xcvr_reconfig_readdata[23:16]} :
        xcvr_reconfig_readdata_valid[3] ?  {24'b0, o_xcvr_reconfig_readdata[31:24]} : 32'hdead_c0de;

/*** to merge all avmm interface to one source  -- end ***/

    ex_100G av_top (


                .i_stats_snapshot(i_stats_snapshot),

                .o_cdr_lock(o_cdr_lock),

                .o_tx_pll_locked(o_tx_pll_locked),

                .i_eth_reconfig_addr(i_eth_reconfig_addr),

                .i_eth_reconfig_read(i_eth_reconfig_read),

                .i_eth_reconfig_write(i_eth_reconfig_write),

                .o_eth_reconfig_readdata(o_eth_reconfig_readdata),

                .o_eth_reconfig_readdata_valid(o_eth_reconfig_readdata_valid),

                .i_eth_reconfig_writedata(i_eth_reconfig_writedata),

                .o_eth_reconfig_waitrequest(o_eth_reconfig_waitrequest),

                .i_rsfec_reconfig_addr(i_rsfec_reconfig_addr),

                .i_rsfec_reconfig_read(i_rsfec_reconfig_read),

                .i_rsfec_reconfig_write(i_rsfec_reconfig_write),

                .o_rsfec_reconfig_readdata(o_rsfec_reconfig_readdata),

                .i_rsfec_reconfig_writedata(i_rsfec_reconfig_writedata),

                .o_rsfec_reconfig_waitrequest(o_rsfec_reconfig_waitrequest),

                .o_tx_lanes_stable(o_tx_lanes_stable),

                .o_rx_pcs_ready(o_rx_pcs_ready),

                .o_ehip_ready(o_ehip_ready),

                .o_rx_block_lock(o_rx_block_lock),

                .o_rx_am_lock(o_rx_am_lock),

                .o_rx_hi_ber(o_rx_hi_ber),

                .o_local_fault_status(o_local_fault_status),

                .o_remote_fault_status(o_remote_fault_status),

                .i_clk_ref(i_clk_ref),

                .i_clk_tx(o_clk_pll_div64),

                .i_clk_rx(o_clk_pll_div64),

                .o_clk_pll_div64(o_clk_pll_div64),

                .o_clk_pll_div66(o_clk_pll_div66),

                .o_clk_rec_div64(o_clk_rec_div64),

                .o_clk_rec_div66(o_clk_rec_div66),

                .i_csr_rst_n(~arst),

                .i_tx_rst_n(1'b1),

                .i_rx_rst_n(1'b1),

                .o_tx_serial(o_tx_serial),

                .i_rx_serial(i_rx_serial),

                .o_tx_serial_n(o_tx_serial_n),
                .i_rx_serial_n(i_rx_serial_n),

                .i_reconfig_clk(i_reconfig_clk),

                .i_reconfig_reset(arst),

                .i_xcvr_reconfig_address(i_xcvr_reconfig_address),

                .i_xcvr_reconfig_read(i_xcvr_reconfig_read),

                .i_xcvr_reconfig_write(i_xcvr_reconfig_write),

                .o_xcvr_reconfig_readdata(o_xcvr_reconfig_readdata),

                .i_xcvr_reconfig_writedata(i_xcvr_reconfig_writedata),

                .o_xcvr_reconfig_waitrequest(o_xcvr_reconfig_waitrequest),

                .o_tx_ready(o_tx_ready),

                .i_tx_valid(i_tx_valid),

                .i_tx_data(i_tx_data),

                .i_tx_error(i_tx_error),

                .i_tx_startofpacket(i_tx_startofpacket),

                .i_tx_endofpacket(i_tx_endofpacket),

                .i_tx_empty(i_tx_empty),

                .i_tx_skip_crc(i_tx_skip_crc),

                .o_rx_valid(o_rx_valid),

                .o_rx_data(o_rx_data),

                .o_rx_startofpacket(o_rx_startofpacket),

                .o_rx_endofpacket(o_rx_endofpacket),

                .o_rx_empty(o_rx_empty),

                .o_rx_error(o_rx_error),

                .o_rxstatus_data(o_rxstatus_data),

                .o_rxstatus_valid(o_rxstatus_valid),

                .i_tx_pfc(i_tx_pfc),

                .o_rx_pfc(o_rx_pfc),

                .i_tx_pause(i_tx_pause),

                .o_rx_pause(o_rx_pause)
    );



    // _____________________________________________________________
    // generate and check some simple data transfers
    // _____________________________________________________________
    alt_e100s10_packet_client pc (
        .i_arst                         (arst),

        // TX to Ethernet
        .i_clk_tx                       (clk_din),
        .i_tx_ready                     (o_tx_ready),
        .o_tx_valid                     (i_tx_valid),
        .o_tx_data                      (din),
        .o_tx_sop                       (i_tx_startofpacket),
        .o_tx_eop                       (i_tx_endofpacket),
        .o_tx_empty                     (i_tx_empty),
        // RX from Ethernet
        .i_clk_rx                       (clk_dout),
        .i_rx_sop                       (o_rx_startofpacket),
        .i_rx_eop                       (o_rx_endofpacket),
        .i_rx_empty                     (o_rx_empty),
        .i_rx_data                  	(o_rx_data),
        .i_rx_valid                 	(o_rx_valid),
        // status register bus
        .i_clk_status                   (i_reconfig_clk),
        .i_status_addr                  (status_addr[15:0]),
        .i_status_read              	(status_read & ~|status_addr[19:16]),
        .i_status_write             	(status_write & ~|status_addr[19:16]),
        .i_status_writedata             (status_writedata),
        .o_status_readdata              (status_readdata_pc),
        .o_status_readdata_valid        (status_readdata_valid_pc)
    );
    
    
    defparam pc.WORDS = WORDS;
    defparam pc.WIDTH = WIDTH;
    


    // _______________________________________________________________________________________________________________
    // jtag_avalon
    // _______________________________________________________________________________________________________________
    wire [31:0] av_addr;
    assign status_addr = av_addr[25:2];
    wire [3:0] byteenable;

    alt_ehipc3_fm_jtag_avalon jtag_master (
        .clk_clk                (i_reconfig_clk),

        .clk_reset_reset        (arst),
        .master_address         (av_addr),
        .master_readdata        (status_readdata),
        .master_read            (status_read),
        .master_write           (status_write),
        .master_writedata       (status_writedata),
        .master_waitrequest     (status_waitrequest),
        .master_readdatavalid   (status_readdata_valid),
        .master_byteenable      (byteenable),
        .master_reset_reset     ()
    );
    // ___________________________________________________________

    // Sources and probes instance

    reg [5:0] clk100_cnt           = 6'b0;
    reg       o_rx_startofpacketD  = 0;
    reg       o_rx_endofpacketD    = 0;
    reg        i_rx_endofpacketD   = 0;
    reg        i_tx_startofpacketD = 0;
    reg        i_tx_endofpacketD   = 0;
    reg        o_rx_errorD         = 0;


    wire [7:0] system_status;
    assign system_status[0] = arst;

    assign system_status[1] = 0;
    assign system_status[2] = 0;
    assign system_status[3] = 0;

    assign system_status[4] = 0;
    assign system_status[5] = 0;
    assign system_status[6] = 0;
//      assign system_status[7] = tx_pll_locked_01;

//    assign user_io[0]       = 0;
//    assign user_io[1]       = 0;
//    assign user_io[2]       = 0;
//    assign user_io[3]       = 0;
//    assign user_io[4]       = 0;
//    assign user_io[5]       = 0;
//    assign user_io[6]       = 0;
//    assign user_io[7]       = 0;
//    assign user_io[8]       = 0;
//    assign user_io[9]       = 0;


    probe8 prb (
        .source  (source_reset),
        .probe   (system_status)
    );
	 
endmodule

