// (C) 2001-2021 Intel Corporation. All rights reserved.
// Your use of Intel Corporation's design tools, logic functions and other 
// software and tools, and its AMPP partner logic functions, and any output 
// files from any of the foregoing (including device programming or simulation 
// files), and any associated documentation or information are expressly subject 
// to the terms and conditions of the Intel Program License Subscription 
// Agreement, Intel FPGA IP License Agreement, or other applicable 
// license agreement, including, without limitation, that your use is for the 
// sole purpose of programming logic devices manufactured by Intel and sold by 
// Intel or its authorized distributors.  Please refer to the applicable 
// agreement for further details.



 module alt_ehipc3_fm_sl_pio_avmm_if (
     input  wire         clk,
     input  wire         reset_n,

     // EHIP datapath signals
     input  wire         i_tx_valid,
     input  wire         i_tx_startofpacket,
     input  wire         i_rx_valid,
     input  wire         i_rx_startofpacket,
    
     // User TX 2-step timestamp request
     output  wire        o_ptp_ts_req,
     output  wire [7:0]  o_ptp_fp,
    
     // EHIP TX 2-step timestamp
     input  wire         i_ptp_ets_valid,
     input  wire [95:0]  i_ptp_ets,
     input  wire [7:0]   i_ptp_ets_fp,
    
     // EHIP RX timestamp
     input  wire [95:0]  i_ptp_rx_its,
    
     input  wire [31:0]  pio_status0,
     input  wire [31:0]  pio_status1,
     input  wire [31:0]  pio_status2,
     input  wire [31:0]  pio_status3,
     
     input  wire         i_avl_mm_clk,       
     input  wire         i_avl_mm_read,
     input  wire         i_avl_mm_write,
     output wire         o_avl_mm_waitrequest,
     input  wire [11:0]  i_avl_mm_baddress,
     output reg  [31:0]  o_avl_mm_readdata,
     input  wire [31:0]  i_avl_mm_writedata
 );

  // ________________________________________________
  //  PIO Control
  reg  [31:0] pio_ctl0;
  reg  [31:0] pio_ctl1;
  reg  [31:0] pio_ctl2;
  reg  [31:0] pio_ctl3;

  reg         ptp_ts_req_edge;
  reg         ptp_rx_its_edge;
  reg  [95:0] ptp_rx_its;
  reg  [95:0] ptp_rx_its_r;
  reg  [95:0] ptp_ets;
  reg   [7:0] ptp_ets_fp;
  reg  [95:0] ptp_ets_r;
  reg   [7:0] ptp_ets_fp_r;

  wire reset_n_sync_clk;
  reg  reset_n_sync_clk_r;
  reg  reset_n_sync_clk_rr;
  wire reset_sync_clk;

  alt_xcvr_resync_std #(
      .SYNC_CHAIN_LENGTH  (3),
      .WIDTH              (1),
      .INIT_VALUE         (1)
  ) ehip_reset_n_sync_clk (
      .clk                (clk),
      .reset              (!reset_n),
      .d                  (1'b0),
      .q                  (reset_n_sync_clk)
  );
 
  always @ (posedge clk)
  begin
      reset_n_sync_clk_r    <= reset_n_sync_clk;
      reset_n_sync_clk_rr   <= reset_n_sync_clk_r;
  end

  assign reset_sync_clk = ~reset_n_sync_clk_rr;
 
  always @ (posedge clk or negedge reset_sync_clk)
  begin
      if (!reset_sync_clk) begin
         ptp_ts_req_edge  <= 0;
      end
      else if (i_tx_valid & i_tx_startofpacket) begin
         ptp_ts_req_edge  <= pio_ctl0[8];
      end
      else if (~pio_ctl0[8]) begin
         ptp_ts_req_edge  <= 0;
      end     
  end
 
  assign o_ptp_ts_req = (~ptp_ts_req_edge & pio_ctl0[8]) ? 1'b1 : 1'b0;
  assign o_ptp_fp = pio_ctl0[7:0];

  always @ (posedge clk or negedge reset_sync_clk)
  begin
      if (!reset_sync_clk) begin
         ptp_rx_its_edge  <= 0;
      end
      else if (i_rx_valid & i_rx_startofpacket) begin
         ptp_rx_its_edge  <= pio_ctl0[8];
      end
      else if (~pio_ctl0[8]) begin
         ptp_rx_its_edge  <= 0;
      end     
  end

  always @ (posedge clk or negedge reset_sync_clk)
  begin
      if (!reset_sync_clk) begin
         ptp_rx_its <= 0;
      end
      else if (~ptp_rx_its_edge & i_rx_valid & i_rx_startofpacket) begin
         ptp_rx_its <= i_ptp_rx_its;
      end    
  end
  
  always @ (posedge clk)
  begin
      ptp_rx_its_r <= ptp_rx_its;
  end
 
  always @ (posedge clk or negedge reset_sync_clk)
  begin
      if (!reset_sync_clk) begin
         ptp_ets    <= 0;
         ptp_ets_fp <= 0;
      end
      else if (i_ptp_ets_valid) begin
         ptp_ets    <= i_ptp_ets;
         ptp_ets_fp <= i_ptp_ets_fp;
      end    
  end     

  always @ (posedge clk)
  begin
      ptp_ets_r <= ptp_ets;
      ptp_ets_fp_r <= ptp_ets_fp;
  end

  reg [31:0] pio_status0_r;
  always @ (posedge clk or negedge reset_sync_clk)
  begin
      if (!reset_sync_clk) begin
         pio_status0_r  <= 0;
      end
      else begin 
         pio_status0_r  <= pio_status0;
      end    
  end 
  
  // ________________________________________________
  //  AVMM interface
  reg         avl_mm_read_r;
  reg         avl_mm_write_r;
  reg  [11:0] avl_mm_baddress_r;
  reg  [31:0] avl_mm_writedata_r;

  reg         avl_mm_read_r1;
  reg         avl_mm_write_r1;
  reg  [11:0] avl_mm_baddress_r1;
  reg  [31:0] avl_mm_writedata_r1;

  wire [9:0]  avl_mm_address = avl_mm_baddress_r[11:2]; // byte to word address
  //wire        blk_sel_pio = (avl_mm_address[9:8] == 2'd3);

  wire        waitrequest_pio;
  reg  [31:0] avl_mm_readdata_pio1;
  reg  [31:0] avl_mm_readdata_pio2;
  reg  [31:0] avl_mm_readdata_pio3;
  reg  [31:0] avl_mm_readdata_pio4;
  reg  [31:0] avl_mm_readdata_pio1_r;
  reg  [31:0] avl_mm_readdata_pio2_r;
  reg  [31:0] avl_mm_readdata_pio3_r;
  reg  [31:0] avl_mm_readdata_pio4_r;
  reg  [1:0]  avl_mm_address_msb;
  
  reg         avl_mm_waitrequest;
  wire        avl_mm_read = avl_mm_read_r;
  wire        avl_mm_write = avl_mm_write_r;
  wire [31:0] avl_mm_writedata = avl_mm_writedata_r;

  reg [31:0]  avl_mm_readdata;

  always @ (posedge clk or negedge reset_sync_clk)
   begin
      if (!reset_sync_clk) begin
         avl_mm_readdata <= 0;
         avl_mm_waitrequest <= 0;
      end
      else begin
         avl_mm_readdata    <= avl_mm_address_msb == 2'b00 ? avl_mm_readdata_pio1_r : 
                               avl_mm_address_msb == 2'b01 ? avl_mm_readdata_pio2_r :
                               avl_mm_address_msb == 2'b10 ? avl_mm_readdata_pio3_r :
                                                             avl_mm_readdata_pio4_r ;
         avl_mm_waitrequest <= waitrequest_pio;
      end
   end

// __________________________________________________________________________________
// AVMM Clock crossing bridge
    reg rddly, wrdly;
    reg [7:0] rddly1;
    reg [5:0] wrdly1;   
    wire rdempty;
    wire rd_empty;
    wire [71:0] pkt_wr_fifo_read;  
    wire [71:0] pkt_rd_fifo_read; 
    reg avl_mm_write_int_r;
    reg avl_mm_waitrequest_r;
    reg rdreq_wr_fifo;
    reg rdreq_rd_fifo;
    wire wredge = i_avl_mm_write& ~wrdly;
    wire rdedge = i_avl_mm_read & ~rddly;

    wire reset_n_sync_avl_mm_clk;
    wire reset_sync_avl_mm_clk;

    alt_xcvr_resync_std #(
        .SYNC_CHAIN_LENGTH  (3),
        .WIDTH              (1),
        .INIT_VALUE         (1)
    ) ehip_reset_n_sync_avl_mm_clk (
        .clk                (i_avl_mm_clk),
        .reset              (!reset_n),
        .d                  (1'b0),
        .q                  (reset_n_sync_avl_mm_clk)
    );
    
    assign reset_sync_avl_mm_clk = ~reset_n_sync_avl_mm_clk;

    always@(posedge i_avl_mm_clk or negedge reset_sync_avl_mm_clk)
    begin
      if(!reset_sync_avl_mm_clk) 
      begin 
        avl_mm_write_int_r <= 1'b0;
      end 
      else 
      begin 
        avl_mm_write_int_r <= (i_avl_mm_write | i_avl_mm_read);
      end 
    end
 
    assign o_avl_mm_waitrequest = (wredge|rdedge) || (|wrdly1) || (|rddly1); // Design done with transaction when this goes down 
    
    always@(posedge i_avl_mm_clk or negedge reset_sync_avl_mm_clk)
    begin
      if(!reset_sync_avl_mm_clk) 
      begin 
        wrdly <= 1'b0; 
        rddly <= 1'b0; 
      end 
      else 
      begin 
        wrdly <= i_avl_mm_write; 
        rddly <= i_avl_mm_read; 
      end 
    end
    
    always@(posedge i_avl_mm_clk or negedge reset_sync_avl_mm_clk)
    begin
      if(!reset_sync_avl_mm_clk) 
      begin 
        wrdly1 <= 6'h0; 
        rddly1 <= 8'h0; 
      end 
      else 
      begin 
        wrdly1 <= {wrdly1[4:0],wredge}; 
        rddly1 <= {rddly1[6:0],rdedge}; 
      end 
    end

    always@(posedge clk or negedge reset_sync_clk)
    begin
      if(!reset_sync_clk) 
      begin 
        avl_mm_read_r <= 1'b0;
        avl_mm_write_r <= 1'b0;
        avl_mm_baddress_r <= 12'h0;
        avl_mm_writedata_r <= 32'h0;
      end 
      else 
      begin 
        avl_mm_read_r <= avl_mm_read_r1;
        avl_mm_write_r <= avl_mm_write_r1;
        avl_mm_baddress_r <= avl_mm_baddress_r1;
        avl_mm_writedata_r <= avl_mm_writedata_r1;
      end 
    end
 
 //flop the output of ram for timing
    reg rdreq_wr_fifo_r;
    reg [71:0] pkt_wr_fifo_read_r;
    always@(posedge clk or negedge reset_sync_clk)
    begin
      if(!reset_sync_clk) 
      begin 
        rdreq_wr_fifo_r <= 1'b0;
        pkt_wr_fifo_read_r <= 1'b0;
      end 
      else 
      begin 
        rdreq_wr_fifo_r <= rdreq_wr_fifo;
        pkt_wr_fifo_read_r <= pkt_wr_fifo_read;
      end 
    end
 
    always@(posedge clk or negedge reset_sync_clk)
    begin
      if(!reset_sync_clk) 
      begin 
        avl_mm_read_r1 <= 1'b0;
        avl_mm_write_r1 <= 1'b0;
        avl_mm_baddress_r1 <= 12'h0;
        avl_mm_writedata_r1 <= 32'h0;
      end 
      else //if (rdreq_wr_fifo)
      begin 
        avl_mm_read_r1 <= rdreq_wr_fifo_r ? pkt_wr_fifo_read_r[45] : avl_mm_waitrequest ? 1'b0 : avl_mm_read_r1;
        avl_mm_write_r1 <= rdreq_wr_fifo_r ? pkt_wr_fifo_read_r[44] : 1'b0;
        avl_mm_baddress_r1 <= rdreq_wr_fifo_r ? pkt_wr_fifo_read_r[43:32] : avl_mm_baddress_r1;
        avl_mm_writedata_r1 <= rdreq_wr_fifo_r ? pkt_wr_fifo_read_r[31:0] : avl_mm_writedata_r1;
      end 
    end
   
    always@(posedge clk or negedge reset_sync_clk)
    begin
      if(!reset_sync_clk) 
      begin 
        rdreq_wr_fifo <= 1'b0;
      end 
      else 
      begin 
        rdreq_wr_fifo <= (~rdempty & ~rdreq_wr_fifo) ? 1'b1 : 1'b0;
      end 
    end

   
    avalon_st_avmm_if_fifo pkt_avmm_fifo_wrdata (
      .aclr    (~reset_sync_avl_mm_clk),
      .data    ({26'h0,i_avl_mm_read,i_avl_mm_write,i_avl_mm_baddress,i_avl_mm_writedata}),    
      .wrreq   ((i_avl_mm_write | i_avl_mm_read) & ~avl_mm_write_int_r),   
      .rdreq   (rdreq_wr_fifo),  
      .wrclk   (i_avl_mm_clk),  
      .rdclk   (clk),
      .q       (pkt_wr_fifo_read),  
      .rdempty (rdempty), 
      .wrfull  ()   
    );

    always@(posedge i_avl_mm_clk or negedge reset_sync_avl_mm_clk)
    begin
      if(!reset_sync_avl_mm_clk) 
      begin 
        o_avl_mm_readdata <= 32'h0;
      end 
      else if (rdreq_rd_fifo)
      begin 
        o_avl_mm_readdata <= pkt_rd_fifo_read[31:0];
      end 
    end
    
    always@(posedge i_avl_mm_clk or negedge reset_sync_avl_mm_clk)
    begin
      if(!reset_sync_avl_mm_clk) 
      begin 
        rdreq_rd_fifo <= 1'b0;
      end 
      else 
      begin 
        rdreq_rd_fifo <= (~rd_empty & ~rdreq_rd_fifo) ? 1'b1 : 1'b0;
      end 
    end

    always@(posedge clk or negedge reset_sync_clk)
    begin
      if(!reset_sync_clk) 
      begin 
        avl_mm_waitrequest_r <= 1'b0;
      end 
      else
      begin 
        avl_mm_waitrequest_r <= avl_mm_waitrequest;
      end 
    end
    
    avalon_st_avmm_if_fifo pkt_avmm_fifo_rddata (
      .aclr    (reset_n_sync_clk),
      .data    ({40'h0,avl_mm_readdata}),    
      .wrreq   (avl_mm_read_r & avl_mm_waitrequest_r),   
      .rdreq   (rdreq_rd_fifo),  
      .wrclk   (clk),  
      .rdclk   (i_avl_mm_clk),
      .q       (pkt_rd_fifo_read),  
      .rdempty (rd_empty), 
      .wrfull  ()   
    );
    
// __________________________________________________________________________________

// ____________________________________________________________________________
// Control register 0
// ____________________________________________________________________________
//for timing purpose
reg wrpio0;
reg wrpio1;
reg wrpio2;
reg wrpio3;

always @ (posedge clk or negedge reset_sync_clk) begin
    if (!reset_sync_clk) begin
        wrpio0 <= 1'b0;
        wrpio1 <= 1'b0;
        wrpio2 <= 1'b0;
        wrpio3 <= 1'b0;
    end else begin
        wrpio0 <= avl_mm_write_r1 & (avl_mm_baddress_r1[5:2] == 4'hC);
        wrpio1 <= avl_mm_write_r1 & (avl_mm_baddress_r1[5:2] == 4'hD);
        wrpio2 <= avl_mm_write_r1 & (avl_mm_baddress_r1[5:2] == 4'hE);
        wrpio3 <= avl_mm_write_r1 & (avl_mm_baddress_r1[5:2] == 4'hF);
    end
end

always @ (posedge clk or negedge reset_sync_clk)
   begin
      if (!reset_sync_clk) pio_ctl0 <= 32'h0;
      else if (wrpio0) pio_ctl0 <= avl_mm_writedata;
   end

// ____________________________________________________________________________
// Control register 1
// ____________________________________________________________________________

always @ (posedge clk or negedge reset_sync_clk)
   begin
      if (!reset_sync_clk) pio_ctl1 <= 32'h0;
      else if (wrpio1) pio_ctl1 <= avl_mm_writedata;
   end
   
// ____________________________________________________________________________
// Control register 2
// ____________________________________________________________________________

always @ (posedge clk or negedge reset_sync_clk)
   begin
      if (!reset_sync_clk) pio_ctl2 <= 32'h0;
      else if (wrpio2) pio_ctl2 <= avl_mm_writedata;
   end
   
// ____________________________________________________________________________
// Control register 3
// ____________________________________________________________________________

always @ (posedge clk or negedge reset_sync_clk)
   begin
      if (!reset_sync_clk) pio_ctl3 <= 32'h0;
      else if (wrpio3) pio_ctl3 <= avl_mm_writedata;
   end

// Output MUX of registers into readdata bus
   always@(posedge clk)
   begin
      if(!reset_sync_clk) begin
         avl_mm_readdata_pio1 <= 32'h0;
         avl_mm_readdata_pio2 <= 32'h0;
         avl_mm_readdata_pio3 <= 32'h0;
         avl_mm_readdata_pio4 <= 32'h0;
      end
      else begin
         case (avl_mm_baddress_r1[3:2])
  /*4'h0*/  2'h0  : avl_mm_readdata_pio1 <= pio_status0_r;
  /*4'h1*/  2'h1  : avl_mm_readdata_pio1 <= pio_status1;
  /*4'h2*/  2'h2  : avl_mm_readdata_pio1 <= pio_status2;
  /*4'h3*/  2'h3  : avl_mm_readdata_pio1 <= pio_status3;
            default:avl_mm_readdata_pio1 <= 32'h0;
         endcase

         case (avl_mm_baddress_r1[3:2])
  /*4'h4*/  2'h0  : avl_mm_readdata_pio2 <= ptp_ets_r[31:0];
  /*4'h5*/  2'h1  : avl_mm_readdata_pio2 <= ptp_ets_r[63:32];
  /*4'h6*/  2'h2  : avl_mm_readdata_pio2 <= ptp_ets_r[95:64];
  /*4'h7*/  2'h3  : avl_mm_readdata_pio2 <= {24'h0,ptp_ets_fp_r[7:0]};
            default:avl_mm_readdata_pio2 <= 32'h0;
         endcase

         case (avl_mm_baddress_r1[3:2])
  /*4'h8*/  2'h0  : avl_mm_readdata_pio3 <= ptp_rx_its_r[31:0];
  /*4'h9*/  2'h1  : avl_mm_readdata_pio3 <= ptp_rx_its_r[63:32];
  /*4'hA*/  2'h2  : avl_mm_readdata_pio3 <= ptp_rx_its_r[95:64];
  /*4'hB*/  2'h3  : avl_mm_readdata_pio3 <= 32'hB;
            default:avl_mm_readdata_pio3 <= 32'h0;
         endcase

         case (avl_mm_baddress_r1[3:2])
  /*4'hC*/  2'h0  : avl_mm_readdata_pio4 <= pio_ctl0;
  /*4'hD*/  2'h1  : avl_mm_readdata_pio4 <= pio_ctl1;
  /*4'hE*/  2'h2  : avl_mm_readdata_pio4 <= pio_ctl2;
  /*4'hF*/  2'h3  : avl_mm_readdata_pio4 <= pio_ctl3;
            default:avl_mm_readdata_pio4 <= 32'h0;
         endcase
      end
   end

   always@(posedge clk)
   begin
      avl_mm_address_msb <= avl_mm_address[3:2];
      avl_mm_readdata_pio1_r <= avl_mm_readdata_pio1;
      avl_mm_readdata_pio2_r <= avl_mm_readdata_pio2;
      avl_mm_readdata_pio3_r <= avl_mm_readdata_pio3;
      avl_mm_readdata_pio4_r <= avl_mm_readdata_pio4;
   end

 // ____________________________________________________________
 //
   reg rddly_pio, wrdly_pio;
   always@(posedge clk or negedge reset_sync_clk)
   begin
      if(!reset_sync_clk) 
      begin 
        wrdly_pio <= 1'b0; 
        rddly_pio <= 1'b0; 
      end 
      else 
      begin 
        wrdly_pio <= avl_mm_write; 
        rddly_pio <= avl_mm_read; 
      end 
   end
  
   wire wredge_pio = avl_mm_write& ~wrdly_pio;
   wire rdedge_pio = avl_mm_read & ~rddly_pio;

   assign waitrequest_pio = (wredge_pio|rdedge_pio); // your design is done with transaction when this goes down
   

endmodule
