# (C) 2001-2021 Intel Corporation. All rights reserved.
# Your use of Intel Corporation's design tools, logic functions and other 
# software and tools, and its AMPP partner logic functions, and any output 
# files from any of the foregoing (including device programming or simulation 
# files), and any associated documentation or information are expressly subject 
# to the terms and conditions of the Intel Program License Subscription 
# Agreement, Intel FPGA IP License Agreement, or other applicable 
# license agreement, including, without limitation, that your use is for the 
# sole purpose of programming logic devices manufactured by Intel and sold by 
# Intel or its authorized distributors.  Please refer to the applicable 
# agreement for further details.


# ===================================================== 
# 	Top level register map: Base Addresses
# ===================================================== 
source [file join [file dirname [info script]] "../sval_top/reg_map_inc.tcl"]

# ================================================================== 
# 	PHY Registers
# ==================================================================
#
set ADDR_PHY_REVID    0x00
set ADDR_PHY_SCRTCH   0x01
set ADDR_PHY_IPNAME2  0x02
set ADDR_PHY_IPNAME1  0x03
set ADDR_PHY_IPNAME0  0x04
           
# config bits, Alvin added 0x12
set ADDR_PHY_PMACFG   0x10
set ADDR_PHY_WORDLCK  0x12
set ADDR_PHY_PMALOOP  0x13
set ADDR_PHY_PHYFSEL  0x14
set ADDR_PHY_PHYFLAG  0x15
      
# status flags, alvin added x28 and x29, removed x20
set ADDR_PHY_FREQLOCK 0x21
set ADDR_PHY_CLKMACOK 0x22
set ADDR_PHY_FRMERROR 0x23
set ADDR_PHY_CLRFRMER 0x24
set ADDR_PHY_SFTRESET 0x25
set ADDR_PHY_FALIGNED 0x26
set ADDR_PHY_ERINJRST 0x27
set ADDR_PHY_AMLOCK   0x28
set ADDR_PHY_LNDSKWED 0x29
# Pcs_vlane info
set ADDR_PHY_PCSVLAN0 0x30
set ADDR_PHY_PCSVLAN1 0x31
set ADDR_PHY_PCSVLAN2 0x32
set ADDR_PHY_PCSVLAN3 0x33

# clock frequencies, alvin modified x43 and x44
set ADDR_PHY_RFCLKHZ  0x40
set ADDR_PHY_RXCLKHZ  0x41
set ADDR_PHY_TXCLKHZ  0x42
set ADDR_PHY_TXRSKHZ  0x43
set ADDR_PHY_RXRSKHZ  0x44

# reconfig controller
set ADDR_PHY_RECOADDR 0x50
set ADDR_PHY_RECORDWR 0x51
set ADDR_PHY_RECOWDAT 0x52
set ADDR_PHY_RECORDAT 0x53

#RSFEC EHIP registers
set RSFEC_LANES_RX_STAT_EHIP 0x180


# ___________________________________________________________________________
#
#

proc chkphy_revid {{device 0}} {
    global BASE_RXPHY
    global ADDR_PHY_REVID
    global BASE_NDEVICE
    puts "RX PHY Register Access: RevID\n"
    reg_read  [expr ($BASE_NDEVICE * $device) + $BASE_RXPHY] $ADDR_PHY_REVID
}

#For E-Tile PMA access
proc pma_int {ch num code data {device 0}} {
    #kr_pause will stop the KR cpu from issuing ints, or noop if AN/LT is not enabled
    kr_pause
    #load interrupt data
    reconfig_write $ch 0x84 [expr $data & 0xff] $device
    reconfig_write $ch 0x85 [expr $data >> 8] $device
    reconfig_write $ch 0x86 $num $device
    reconfig_write $ch 0x87 $code $device
    #trigger interrupt
    reconfig_write $ch 0x90 1 $device

    #wait for transaction bit
    while {![expr [reconfig_read $ch 0x8A $device] & 0x80]} {}

    #wait for interrupt to finish
    while {[expr [reconfig_read $ch 0x8B $device] & 0x01]} {}

    #read out data
    set rdata [expr [reconfig_read $ch 0x88 $device] | ([reconfig_read $ch 0x89 $device] << 8)]

    #clear transaction bit (write 1 to clear)
    reconfig_write $ch 0x8A 0x80 $device

    kr_unpause
    return 0x[format %0x $rdata]
}

proc pma_int_all {num code data {device 0}} {
    pma_int 0 $num $code $data $device
    pma_int 1 $num $code $data $device
    pma_int 2 $num $code $data $device
    pma_int 3 $num $code $data $device
}

proc pma_init {ch {device 0}} {
    pma_int $ch 0x01 0x00 0x0000 $device
    pma_int $ch 0x05 0x00 0x10a5 $device
    pma_int $ch 0x06 0x00 0x00a5 $device
    pma_int $ch 0x14 0x00 0x0055 $device
    pma_int $ch 0x15 0x00 0x0000 $device
    pma_int $ch 0x15 0x00 0x3000 $device
    pma_int $ch 0x15 0x00 0x4000 $device
    pma_int $ch 0x15 0x00 0x7000 $device
    pma_int $ch 0x15 0x00 0x8000 $device
    pma_int $ch 0x01 0x00 0x0003 $device
    pma_int $ch 0x01 0x00 0x0007 $device
}

proc pma_dis {ch {device 0}} {
    pma_int $ch 0x01 0x00 0x0000 $device
}

proc an_reco {ch {device 0}} {
    pma_int $ch 0x05 0x00 0x9008 $device
    pma_int $ch 0x14 0x00 0x0011 $device
    # pma_int $ch 0x01 0x00 0x0007 $device
    pma_int $ch 0x01 0x00 0x0003 $device
}

proc an_start {ch {device 0}} {
    pma_int $ch 0x29 0x00 0x0c01 $device
    pma_int $ch 0x29 0x01 0x2000 $device
    pma_int $ch 0x29 0x02 0x0000 $device
    pma_int $ch 0x07 0x00 0x000d $device
    pma_int $ch 0x01 0x00 0x0007 $device
}

proc pma_stat {ch {device 0}} {
    puts "FW Revision: [pma_int $ch 0 0 0 $device]"
    puts "Build ID: [pma_int $ch 0x3f 0 0 $device]"
    puts ""
    set r [reconfig_read $ch 0x8b $device]
    puts "RX Ready: [expr ($r >> 2) & 1]"
    puts "TX Ready: [expr ($r >> 3) & 1]"
    puts "Signal OK: [expr ($r >> 1) & 1]"
    puts "Int in progress: [expr $r & 1]"
    puts ""
    set r [reconfig_read $ch 0x8e $device]
    puts "AN Link good: [expr ($r >> 4) & 1]"
    puts "AN Complete: [expr ($r >> 3) & 1]"
    puts "AN SCAN: [expr ($r >> 5) & 1]"
    puts "AN ENTX: [expr ($r >> 6) & 1]"
    puts "AN State: [pma_int $ch 0x7 0x8 0x0001 $device]"
    puts "AN SHist: [pma_int $ch 0x7 0x8 0x0002 $device]"
    puts ""
    set r [reconfig_read $ch 0x8c $device]
    puts "LT Training: [expr ($r >> 1) & 1]"
    puts "LT RX Trained: [expr ($r >> 2) & 1]"
    puts "LT Frame lock: [expr ($r >> 3) & 1]"
    puts "LT Failure: [expr $r & 1]"
}

proc ssm_msg {ch cmd data {device 0}} {
    #clear message receive bit
    reconfig_read $ch 0x207 $device
    #send message
    reconfig_write $ch 0x200 [expr $data & 0xff] $device
    reconfig_write $ch 0x201 [expr ($data >> 8) & 0xff] $device
    reconfig_write $ch 0x202 [expr ($data >> 16) & 0xff] $device
    reconfig_write $ch 0x203 [expr $cmd | 0x80] $device
    #wait until message response
    while {![expr [reconfig_read $ch 0x207 $device] & 0x80]} {}
}

proc pma_anlg_rst {ch {device 0}} {
    kr_pause
    reconfig_read $ch 0x207 $device
    reconfig_write $ch 0x203 0x81 $device
    while {![expr [reconfig_read $ch 0x207 $device] & 0x80]} {}
    kr_unpause
}

proc get_rx_eq {ch input type {device 0}} {
    return [pma_int $ch 0x26 0x01 [expr ($input << 8) | ($type <<12)] $device]
}

proc set_rx_eq {ch input type val {sign 0} {device 0}} {
    return [pma_int $ch 0x26 0x00 [expr ($input << 8) | ($type <<12) | $val | ($sign <<15)] $device]
}

proc dump_rx_eq {ch {device 0}} {
    puts "Input\t0x0\t0x1\t0x2\t0x3\t0x4\t0x5"
    for {set i 0} {$i <= 0xf} {incr i} {
        puts -nonewline "0x[format %X $i]\t"
        for {set c 0} {$c <= 0x5} {incr c} {
            puts -nonewline "[get_rx_eq $ch $i $c $device]\t"
        }
        puts ""
    }
}

proc get_hal {ch table fn {device 0}} {
    return [pma_int $ch 0x2C 0x00 [expr ($table << 8) | $fn] $device]
}

proc dump_hal_table {ch table end {start 0} {device 0}} {
    for {set i $start} {$i <= $end} {incr i} {
        puts "0x[format %0X $i]\t[get_hal $ch $table $i $device]"
    }
}


proc pma_loop {ch on {device 0}} {
    pma_int $ch 0x08 0x00 [expr 0x0100 | $on] $device
}

proc setphy_lpon {{device 0}} {
    pma_loop 3 1 $device
    pma_loop 2 1 $device
    pma_loop 1 1 $device
    pma_loop 0 1 $device
}

# for the 2nd set of 100G IP(the xcvr offset address for lane0 is 24'h5_00000, lane1 is 24'h6_00000, lane3 is 24'h8_00000
# proc setphy_lpon_2 {} {
#     pma_loop 4 1
#     pma_loop 5 1
#     pma_loop 6 1
#     pma_loop 7 1
# }
# proc setphy_lpoff_2 {} {
#     pma_loop 4 0
#     pma_loop 5 0
#     pma_loop 6 0
#     pma_loop 7 0
# }
proc loop_on_pam4 {{device 0}} {
    pma_loop 0 1 $device 
    pma_loop 2 1 $device
}

proc loop_off_pam4 {{device 0}} {
    pma_loop 0 0 $device
    pma_loop 2 0 $device
}


proc settmp_rst_on {{device 0}} {
    global BASE_RXPHY
    global ADDR_PHY_ERINJRST
    global BASE_NDEVICE
    # puts "RX PHY Register Access: Setting Rst=1 on Top level RXSOP RXEOP TXSOP TXEOP CNTERS \n"
    reg_write [expr ($BASE_NDEVICE * $device) + $BASE_RXPHY] $ADDR_PHY_ERINJRST 0x0003
    reg_read  [expr ($BASE_NDEVICE * $device) + $BASE_RXPHY] $ADDR_PHY_ERINJRST
}

proc settmp_rst_off {{device 0}} {
    global BASE_RXPHY
    global ADDR_PHY_ERINJRST
    global BASE_NDEVICE
    # puts "RX PHY Register Access: Setting Rst=0 on Top level RXSOP RXEOP TXSOP TXEOP CNTERS \n"
    reg_write [expr ($BASE_NDEVICE * $device) + $BASE_RXPHY] $ADDR_PHY_ERINJRST 0x0000
    reg_read  [expr ($BASE_NDEVICE * $device) + $BASE_RXPHY] $ADDR_PHY_ERINJRST
}

proc rxsft_rst_on {{device 0}} {
    global BASE_RXPHY
    global ADDR_PHY_PMACFG
    global BASE_NDEVICE
    variable eth_offset 
    # puts "RX PHY Register Access: Enable Soft Reset on the Rx \n"
    reg_write [expr ($BASE_NDEVICE * $device) + $BASE_RXPHY] $ADDR_PHY_PMACFG 0x0004
    reg_read  [expr ($BASE_NDEVICE * $device) + $BASE_RXPHY] $ADDR_PHY_PMACFG
}

proc rxsft_rst_off {{device 0}} {
    global BASE_RXPHY
    global ADDR_PHY_PMACFG
    global BASE_NDEVICE
    variable eth_offset 
    # puts "RX PHY Register Access: Disable Soft Reset on the Rx \n"
    reg_write [expr ($BASE_NDEVICE * $device) + $BASE_RXPHY] $ADDR_PHY_PMACFG 0x0000
    reg_read  [expr ($BASE_NDEVICE * $device) + $BASE_RXPHY] $ADDR_PHY_PMACFG
}


proc txsft_rst_on {{device 0}} {
    global BASE_RXPHY
    global ADDR_PHY_PMACFG
    global BASE_NDEVICE
    variable eth_offset 
    # puts "TX PHY Register Access: Enable Soft Reset on the Tx \n"
    reg_write [expr ($BASE_NDEVICE * $device) + $BASE_RXPHY] $ADDR_PHY_PMACFG 0x0002
    set pmacfg_b2 [ reg_read [expr ($BASE_NDEVICE * $device) + $BASE_RXPHY] $ADDR_PHY_PMACFG ]
    return $pmacfg_b2
}

proc txsft_rst_off {{device 0}} {
    global BASE_RXPHY
    global ADDR_PHY_PMACFG
    global BASE_NDEVICE
    variable eth_offset 
    # puts "TX PHY Register Access: Disable Soft Reset on the Tx \n"
    reg_write [expr ($BASE_NDEVICE * $device) + $BASE_RXPHY] $ADDR_PHY_PMACFG 0x0000
    set pmacfg_b2 [ reg_read [expr ($BASE_NDEVICE * $device) + $BASE_RXPHY] $ADDR_PHY_PMACFG ]
    return $pmacfg_b2
}


proc eiosys_rst_on {{device 0}} {
    global BASE_RXPHY
    global ADDR_PHY_PMACFG
    global BASE_NDEVICE
    # puts "EIO SYS Register Access: Enable Eio Sys Reset on the Xcvr and CSR \n"
    reg_write [expr ($BASE_NDEVICE * $device) + $BASE_RXPHY] $ADDR_PHY_PMACFG 0x0001
    reg_read  [expr ($BASE_NDEVICE * $device) + $BASE_RXPHY] $ADDR_PHY_PMACFG
}

proc eiosys_rst_off {{device 0}} {
    global BASE_RXPHY
    global ADDR_PHY_PMACFG
    global BASE_NDEVICE
    # puts "EIO SYS Register Access: Disable Eio Sys Reset on the Xcvr and CSR \n"
    reg_write [expr ($BASE_NDEVICE * $device) + $BASE_RXPHY] $ADDR_PHY_PMACFG 0x0000
    reg_read  [expr ($BASE_NDEVICE * $device) + $BASE_RXPHY] $ADDR_PHY_PMACFG
}


proc setphy_lpoff {{device 0}} {
    pma_loop 3 0 $device
    pma_loop 2 0 $device
    pma_loop 1 0 $device
    pma_loop 0 0 $device
}

proc chkphy_clk_speeds {{device 0}} {
    global BASE_RXPHY 
    global ADDR_PHY_RFCLKHZ
    global ADDR_PHY_TXCLKHZ
    global ADDR_PHY_RXCLKHZ
    global ADDR_PHY_TXRSKHZ
    global ADDR_PHY_RXRSKHZ
    global BASE_NDEVICE

    puts " RX PHY Register Access: Checking Clock Frequencies (KHz) \n"
    set refclk  [ expr [ reg_read [expr ($BASE_NDEVICE * $device) +  $BASE_RXPHY] $ADDR_PHY_RFCLKHZ ]]
    set txclk   [ expr [ reg_read [expr ($BASE_NDEVICE * $device) +  $BASE_RXPHY] $ADDR_PHY_TXCLKHZ ]]
    set rxclk   [ expr [ reg_read [expr ($BASE_NDEVICE * $device) +  $BASE_RXPHY] $ADDR_PHY_RXCLKHZ ]]
    set txrsclk [ expr [ reg_read [expr ($BASE_NDEVICE * $device) +  $BASE_RXPHY] $ADDR_PHY_TXRSKHZ ]]
    set rxrsclk [ expr [ reg_read [expr ($BASE_NDEVICE * $device) +  $BASE_RXPHY] $ADDR_PHY_RXRSKHZ ]]

    puts "	REFCLK 		:$refclk (KHZ) "
    puts "	TXCLK 		:$txclk  (KHZ) "
    puts "	RXCLK 		:$rxclk  (KHZ) "
    puts "	TXRSCLK 	:$txrsclk  (KHZ) "
    puts "	RXRSCLK 	:$rxrsclk  (KHZ) "
}

proc setphy_clear_frame_error {{device 0}} {
    global BASE_RXPHY 
    global ADDR_PHY_CLRFRMER 
    global BASE_NDEVICE
    reg_write [expr ($BASE_NDEVICE * $device) + $BASE_RXPHY] $ADDR_PHY_CLRFRMER 0x01
    reg_write [expr ($BASE_NDEVICE * $device) + $BASE_RXPHY] $ADDR_PHY_CLRFRMER 0x00
}

proc chkphy_frame_error {{device 0}} {
    global 	BASE_RXPHY 
    global 	ADDR_PHY_FRMERROR 
    global BASE_NDEVICE
    set rxfrm_err [ reg_read [expr ($BASE_NDEVICE * $device) + $BASE_RXPHY] $ADDR_PHY_FRMERROR ]
    setphy_clear_frame_error 
    return $rxfrm_err
}

proc read_phy_frame_error {{device 0}} {
    global 	BASE_RXPHY 
    global 	ADDR_PHY_FRMERROR 
    global BASE_NDEVICE
    set rxfrm_err [ reg_read [expr ($BASE_NDEVICE * $device) + $BASE_RXPHY] $ADDR_PHY_FRMERROR ]
    return $rxfrm_err
}

proc chkphy_io_status {{device 0}} {
    global BASE_RXPHY
    global ADDR_PHY_FREQLOCK
    global ADDR_PHY_CLKMACOK
    global ADDR_PHY_FRMERROR
    global ADDR_PHY_CLRFRMER
    global ADDR_PHY_SFTRESET
    global ADDR_PHY_FALIGNED
    global ADDR_PHY_AMLOCK
    global ADDR_PHY_LNDSKWED
    global BASE_NDEVICE

    #if res, then rsfec is enabled. 
    # puts [expr ($BASE_NDEVICE * $device) + 0x10000]
    set res [expr [reg_read [expr ($BASE_NDEVICE * $device) + 0x10000] 0x014] != 0x0]

    puts " RX PHY Status Polling \n"
    puts " Rx Frequency Lock Status     [ reg_read [expr ($BASE_NDEVICE * $device) + $BASE_RXPHY] $ADDR_PHY_FREQLOCK ] \n"
    puts " Mac Clock in OK Condition?   [ reg_read [expr ($BASE_NDEVICE * $device) + $BASE_RXPHY] $ADDR_PHY_CLKMACOK ] \n"
    puts " Rx Frame Error               [ chkphy_frame_error $device ] \n"
    if {!$res} {
        puts " Rx PHY Fully Aligned?        [ reg_read [expr ($BASE_NDEVICE * $device) + $BASE_RXPHY] $ADDR_PHY_FALIGNED ] \n"
    }
    
    if {$res} {
        puts " Rx AM LOCK Condition?        [format 0x%08X [rsfec_locked $device]] \n"
    }  else {
        puts " Rx AM LOCK Condition?        [ reg_read [expr ($BASE_NDEVICE * $device) + $BASE_RXPHY] $ADDR_PHY_AMLOCK   ] \n"
    }
    puts " Rx Lanes Deskewed Condition? [ reg_read [expr ($BASE_NDEVICE * $device) + $BASE_RXPHY] $ADDR_PHY_LNDSKWED ] \n"
}


# ______________________________________________________________________________
#
#
proc chkphy_status {{device 0}} {
    chkphy_clk_speeds $device
    chkphy_io_status $device
}


proc loop_on {{device 0}} {setphy_lpon $device}
proc loop_off {{device 0}} {setphy_lpoff $device}

proc tmpcntrst_on {{device 0}} {settmp_rst_on $device}
proc tmpcntrst_off {{device 0}} {settmp_rst_off $device}
# ___________________________________________________________________________
#

proc phy_locked {{device 0}} {
    global BASE_RXPHY 
    global ADDR_PHY_FALIGNED
    global BASE_NDEVICE
    set locked [ reg_read [expr ($BASE_NDEVICE * $device) + $BASE_RXPHY] $ADDR_PHY_FALIGNED ]
    return $locked
}

proc wait_for_phy_lock { { timeout 2 } {device 0} } {
    set num_tries [ expr {$timeout * 10}]
    for {set i 0} {$i < $num_tries} {incr i} {
        set locked [phy_locked $device]
           puts " wait for phy lock=$num_tries, locked=$locked\n"
        if {$locked == 1} {return 0}
        after 100
    }
    return -1
}

proc wait_for_phy_unlock { { timeout 2 } {device 0}} {
    set num_tries [ expr {$timeout * 10}]
    for {set i 0} {$i < $num_tries} {incr i} {
        set locked [phy_locked $device]
        if {$locked == 0} {return 0}
        after 100
    }
    return -1
}

proc rsfec_locked {{device 0}} {
    global BASE_RSFEC_EHIP 
    global RSFEC_LANES_RX_STAT_EHIP
    global BASE_NDEVICE

    set not_locked [ reg_read [expr ($BASE_NDEVICE * $device) + $BASE_RSFEC_EHIP] $RSFEC_LANES_RX_STAT_EHIP ]
    set locked [expr !$not_locked]
    return $locked
}

proc wait_for_rsfec_lock { { timeout 2 } {device 0}} {
    set num_tries [ expr {$timeout * 10}]
    for {set i 0} {$i < $num_tries} {incr i} {
        set locked [rsfec_locked $device]
           puts " wait for phy lock=$num_tries, locked=$locked\n"
        if {$locked == 1} {return 0}
        after 100
    }
    return -1
}

proc wait_for_rsfec_unlock { { timeout 2 } {device 0}} {
    set num_tries [ expr {$timeout * 10}]
    for {set i 0} {$i < $num_tries} {incr i} {
        set locked [rsfec_locked $device]
        if {$locked == 0} {return 0}
        after 100
    }
    return -1
}

proc cycle_loopback {count} {
    loop_off
    wait_for_unlock

    for {set i 1} {$i <= $count} {incr i} {
        puts "Loopback cycle count = $i"

        loop_on
        set res [ wait_for_lock ]
        if {$res != 0} {return -1}

        loop_off
        set res [ wait_for_unlock ]
        if {$res != 0} {return -1}
    }
    return 0
}

proc cycle_reset {count} {
    loop_on
    wait_for_lock

    for {set i 1} {$i <= $count} {incr i} {
        puts "Reset cycle = $i"
        reg_write 0x326 1
        reg_write 0x326 0
        set res [ wait_for_lock ]
        if {$res != 0} {return -1}
    }
    return 0
}

proc recovery_test {count} {
    set res [ cycle_loopback $count ]
    if {$res != 0} {
        puts "Failed loopback test"
        return -1;
    }

    set res [ cycle_reset $count ]
    if {$res != 0} {
        puts "Failed reset test"
        return -1;
    }

    puts "Recovery test passed"
    return 0
}

proc chk_rsfec_lanes { {device 0} } {
    global BASE_RSFEC_EHIP 
    global BASE_NDEVICE
    set lane0 [reg_read [expr ($BASE_NDEVICE * $device) + $BASE_RSFEC_EHIP] 0x150]
    set lane1 [reg_read [expr ($BASE_NDEVICE * $device) + $BASE_RSFEC_EHIP] 0x154]
    set lane2 [reg_read [expr ($BASE_NDEVICE * $device) + $BASE_RSFEC_EHIP] 0x158]
    set lane3 [reg_read [expr ($BASE_NDEVICE * $device) + $BASE_RSFEC_EHIP] 0x15c]

    set lane0skew [reg_read [expr ($BASE_NDEVICE * $device) + $BASE_RSFEC_EHIP] 0x1b0]
    set lane1skew [reg_read [expr ($BASE_NDEVICE * $device) + $BASE_RSFEC_EHIP] 0x1b4]
    set lane2skew [reg_read [expr ($BASE_NDEVICE * $device) + $BASE_RSFEC_EHIP] 0x1b8]
    set lane3skew [reg_read [expr ($BASE_NDEVICE * $device) + $BASE_RSFEC_EHIP] 0x1bc]

    puts "---------------------------------------\n"
    puts "Rx lanes register reads\n"
    puts "Rx lane 0 = $lane0, skew = $lane0skew\n"
    puts "Rx lane 1 = $lane1, skew = $lane1skew\n"
    puts "Rx lane 2 = $lane2, skew = $lane2skew\n"
    puts "Rx lane 3 = $lane3, skew = $lane3skew\n"
    puts "---------------------------------------\n"
}

proc disable_serdes {{device 0}} {
   pma_int 0 0x1 0x0 0x0 $device 
   pma_int 1 0x1 0x0 0x0 $device
   pma_int 2 0x1 0x0 0x0 $device
   pma_int 3 0x1 0x0 0x0 $device
}
 
proc disable_serdes_pam4 {{device 0}} {
   pma_int 0 0x1 0x0 0x0 $device 
   pma_int 2 0x1 0x0 0x0 $device
}   

proc enable_serdes {{device 0}} {
   pma_int 0 0x1 0x0 0x7 $device 
   pma_int 1 0x1 0x0 0x7 $device 
   pma_int 2 0x1 0x0 0x7 $device 
   pma_int 3 0x1 0x0 0x7 $device
}
 
proc enable_serdes_pam4 {{device 0}} {
   pma_int 0 0x1 0x0 0x7 $device
   pma_int 2 0x1 0x0 0x7 $device
}

proc enable_prbs_gen {{device 0}} {
   pma_int 0 0x02 0x0 0x105 $device
   pma_int 1 0x02 0x0 0x105 $device
   pma_int 2 0x02 0x0 0x105 $device
   pma_int 3 0x02 0x0 0x105 $device
}

proc enable_prbs_gen_pam4 {{device 0}} {
   pma_int 0 0x02 0x0 0x105 $device 
   pma_int 2 0x02 0x0 0x105 $device 
}

proc disable_prbs_gen {{device 0}} {
   pma_int 0 0x02 0x0 0x3ff $device
   pma_int 1 0x02 0x0 0x3ff $device
   pma_int 2 0x02 0x0 0x3ff $device
   pma_int 3 0x02 0x0 0x3ff $device
}
 
proc disable_prbs_gen_pam4 {{device 0}} {
   pma_int 0 0x02 0x0 0x3ff $device
   pma_int 2 0x02 0x0 0x3ff $device
}

# FW init_adaptation for NRZ variant
proc start_pma_init_adaptation_old { {device 0} } {
    global BASE_NDEVICE
    global BASE_RSFEC_EHIP

    # Perform init_adaptation multiple times if init_adaptation fails on first try.
    set tries 0
    set res 1
    
    while {$res && $tries < 5} {
 
        # The below function is to perform PMA adaptaion with internal loopback with initial adaptation only (No continuous adaptation) for NRZ using default values(No configuration Load).
        # Please refer to user guide for more options
        # send mailbox commands to perform init_adaptation 
        init_adaptation_16_NoPrbsNoLdILB03 $device
        

        # check init_adaptation status
        chk_init_adaptation_status $device
      
        # Apply tx and rx reset
        tx_rx_rst $device
       
        #if rsfec, then rsfec is enabled. 
        set rsfec [expr [reg_read [expr $BASE_NDEVICE * $device + $BASE_RSFEC_EHIP] 0x014] != 0x0]
        if {$rsfec} {
          set res [wait_for_rsfec_lock 5 $device] 
        } else {
          set res [wait_for_phy_lock 5 $device]
        }
        
        if {$res} {
            puts "--- Iteration:$tries - No PHY lock. Retrying ---"
        } else {
            puts "----Iteration:$tries - PMA Adaptaion is Successful---- \n"
            puts "------------------------------------------------------ \n"
            puts "------------------------------------------------------ \n"
        }
        incr tries        
    }
}


# Example function to show FW init_adaptation for NRZ variant with external loopback or connected to external device with init_adaptation+cont_adaptation using default values(No configuration Load).
proc start_pma_init_adaptation_ex_old { {device 0} } {
    global BASE_NDEVICE
    global BASE_RSFEC_EHIP

    # Perform init_adaptation multiple times if init_adaptation fails on first try.
    set tries 0
    set res 1
    
    while {$res && $tries < 5} {
        
        # XCVR reset
        start_pma_anlg_rst02 $device
	
        ## Exmaple for configuration load and check
        ## Load the configuration when using init_adaptation functions with configuration load
        # ld_pma_config $device
        ## check configuration load status
        # chk_pma_config_status $device
 
        # The below function is to perform PMA adaptaion with internal loopback with initial adaptation only (No continuous adaptation) for NRZ using default values(No configuration Load).
        # Please refer to user guide for more options
        # send mailbox commands to perform init_adaptation 
        init_adaptation_16_NoPrbsNoLdEL03 $device
        

        # check init_adaptation status
        chk_init_adaptation_status $device
      
        # Apply tx and rx reset
        tx_rx_rst $device
       
        #if rsfec, then rsfec is enabled. 
        set rsfec [expr [reg_read [expr $BASE_NDEVICE * $device + $BASE_RSFEC_EHIP] 0x014] != 0x0]
        if {$rsfec} {
          set res [wait_for_rsfec_lock 5 $device] 
        } else {
          set res [wait_for_phy_lock 5 $device]
        }
        
        if {$res} {
            puts "--- Iteration:$tries - No PHY lock. Retrying ---"
        } else {
            puts "----Iteration:$tries - PMA Adaptaion is Successful---- \n"
            puts "------------------------------------------------------ \n"
            puts "------------------------------------------------------ \n"
        }
        incr tries        
    }
}

# FW init_adaptation for PAM4 variant
proc start_pma_02_init_adaptation_old {{device 0}} {

    # Perform init_adaptation multiple times if init_adaptation fails on first try.
    set tries 0
    set res 1
    
    while {$res && $tries < 5} {

        # The below function is to perform PMA adaptaion with internal loopback with initial adaptation only (No continuous adaptation) for PAM4 using default values(No configuration Load).
        # Please refer to user guide for more options        
        # send mailbox commands to perform init_adaptation 
        init_adaptation_16_NoPrbsNoLdILB02 $device

        # check init_adaptation status
        chk_init_adaptation_status_02 $device

        # Apply tx and rx reset
        tx_rx_rst $device

        set res [wait_for_rsfec_lock 5 $device] 
        if {$res} {
            puts "--- Iteration:$tries - No PHY lock. Retrying ---"
        } else {
            puts "----Iteration:$tries - PMA Adaptaion is Successful---- \n"
            puts "------------------------------------------------------ \n"
            puts "------------------------------------------------------ \n"
        }
        incr tries        
    }
}

# Example function to show FW init_adaptation for PAM4 variant with external loopback or connected to external device with init_adaptation+cont_adaptation using default values(No configuration Load).
proc start_pma_02_init_adaptation_ex_old {{device 0}} {

    # Perform init_adaptation multiple times if init_adaptation fails on first try.
    set tries 0
    set res 1
    
    while {$res && $tries < 5} {

        # XCVR reset
        start_pma_anlg_rst02 $device
	
        ## Exmaple for configuration load and check
        ## Load the configuration when using init_adaptation functions with configuration load
        # ld_pma_config $device
        ## check configuration load status
        # chk_pma_config_status $device
        
        # send mailbox commands to perform init_adaptation
        init_adaptation_16_NoPrbsNoLdELCntPC02 $device

        # check init_adaptation status
        chk_init_adaptation_status_02 $device

        # Apply tx and rx reset
        tx_rx_rst $device

        after 4000
        set res [wait_for_rsfec_lock 5 $device] 
        if {$res} {
            puts "--- Iteration:$tries - No PHY lock. Retrying ---"
        } else {
            puts "--Iteration:$tries - PMA Adaptaion is Successful--"
        }
        incr tries        
    }
}

################################################################################################################### NEW PMA ADAPTATION PROC (October 14th 2019) --START--

# FW init_adaptation for NRZ variant (Internal Loopback)
proc start_pma_init_adaptation { {device 0} } {
    global BASE_NDEVICE
    global BASE_RSFEC_EHIP

    # Perform init_adaptation multiple times if init_adaptation fails on first try.
    set tries 0
    set res 1
    
    while {$res && $tries < 5} {
    ######################################## PPE Golden Calibration Flow ##############################################
        ## Assert RX digital and TX digital resets (MAC+PCS reset)

        reg_write 0x300 0x10 0x6

        ## Toggle PMA Analog Reset

        start_pma_anlg_rst 0x0 $device
        start_pma_anlg_rst 0x1 $device
        start_pma_anlg_rst 0x2 $device
        start_pma_anlg_rst 0x3 $device

        ## Re-Load Serdes settings (call interrupt sequencer) for static IPs

        reconfig_write 0x0 0x91 0x1 
        reconfig_write 0x1 0x91 0x1
        reconfig_write 0x2 0x91 0x1        
        reconfig_write 0x3 0x91 0x1

        ## De-Assert TX digital reset ############################### modification, apply EIOSYS Reset at this time as well and deassert EIOSYS Reset

        reg_write 0x300 0x10 0x5

        reg_write 0x300 0x10 0x4

        ## Disable ILB or switch to mission mode (interrupt 0x08-0x100). If this step is missed, calibration won't go through. Alternatively, issue ILB + iCal then ELB + iCal. This step is automatically done if using mailbox. *

          ##### loop_off (don't need this) done by the mailbox command below #####

        ## If Recipe used --> Call Recipe Load from SIP

        ##### not applicable #####

        loop_on

        sleep 2

        ## Call initAdapt + contAdapt from mailbox flow or use direct interrupts -- For mailbox message, refer to slide 7. The below function is to perform PMA adaptaion with internal loopback with initial adaptation only (No continuous adaptation) for NRZ using default values(No configuration Load).

        init_adaptation_16_NoPrbsNoLdILB03 $device

        after 1000

        ## Deassert all resets
        reg_write 0x300 0x10 0x0  

        ## Check link status

        chk_init_adaptation_status

 ######################################## PPE Golden Calibration Flow ##############################################
       
        #if rsfec, then rsfec is enabled. 
        set rsfec [expr [reg_read [expr $BASE_NDEVICE * $device + $BASE_RSFEC_EHIP] 0x014] != 0x0]
        if {$rsfec} {
          set res [wait_for_rsfec_lock 5 $device] 
        } else {
          set res [wait_for_phy_lock 5 $device]
        }
        
        if {$res} {
            puts "----Iteration:$tries - No PHY lock. Retrying --------- \n"
        } else {
            puts "----Iteration:$tries - PMA Adaptaion is Successful---- \n"
            puts "------------------------------------------------------ \n"
            puts "------------------------------------------------------ \n"
        }
        incr tries        
    }
}


# FW init_adaptation for NRZ variant (External Loopback)
proc start_pma_init_adaptation_ex { {device 0} } {
    global BASE_NDEVICE
    global BASE_RSFEC_EHIP

    # Perform init_adaptation multiple times if init_adaptation fails on first try.
    set tries 0
    set res 1
    
    while {$res && $tries < 5} {
    ######################################## PPE Golden Calibration Flow ##############################################
        ## Assert RX digital and TX digital resets (MAC+PCS reset)

        reg_write 0x300 0x10 0x6

        ## Toggle PMA Analog Reset

        start_pma_anlg_rst 0x0 $device
        start_pma_anlg_rst 0x1 $device
        start_pma_anlg_rst 0x2 $device
        start_pma_anlg_rst 0x3 $device

        ## Re-Load Serdes settings (call interrupt sequencer) for static IPs

        reconfig_write 0x0 0x91 0x1 
        reconfig_write 0x1 0x91 0x1
        reconfig_write 0x2 0x91 0x1        
        reconfig_write 0x3 0x91 0x1

        ## De-Assert TX digital reset ############################### modification, apply EIOSYS Reset at this time as well and deassert EIOSYS Reset

        reg_write 0x300 0x10 0x5

        reg_write 0x300 0x10 0x4

        ## Disable ILB or switch to mission mode (interrupt 0x08-0x100). If this step is missed, calibration won't go through. Alternatively, issue ILB + iCal then ELB + iCal. This step is automatically done if using mailbox. *

          ##### loop_off (don't need this) done by the mailbox command below #####

        ## If Recipe used --> Call Recipe Load from SIP

          ##### not applicable #####

        ## Call initAdapt + contAdapt from mailbox flow or use direct interrupts -- For mailbox message, refer to slide 7. 

        init_adaptation_16_NoPrbsNoLdEL03 $device

        after 1000

        ## Deassert all resets
        reg_write 0x300 0x10 0x0  

        ## Check link status

        chk_init_adaptation_status

    ######################################## PPE Golden Calibration Flow ##############################################
       
        #if rsfec, then rsfec is enabled. 
        set rsfec [expr [reg_read [expr $BASE_NDEVICE * $device + $BASE_RSFEC_EHIP] 0x014] != 0x0]
        if {$rsfec} {
          set res [wait_for_rsfec_lock 5 $device] 
        } else {
          set res [wait_for_phy_lock 5 $device]
        }
        
        if {$res} {
            puts "----Iteration:$tries - No PHY lock. Retrying --------- \n"
        } else {
            puts "----Iteration:$tries - PMA Adaptaion is Successful---- \n"
            puts "------------------------------------------------------ \n"
            puts "------------------------------------------------------ \n"
        }
        incr tries        
    }
}


# FW init_adaptation for PAM4 variant (Internal Loopback)
proc start_pma_02_init_adaptation {{device 0}} {
    global BASE_NDEVICE
    global BASE_RSFEC_EHIP

    # Perform init_adaptation multiple times if init_adaptation fails on first try.
    set tries 0
    set res 1
    
    while {$res && $tries < 5} {
    ######################################## PPE Golden Calibration Flow ##############################################
        ## Assert RX digital and TX digital resets (MAC+PCS reset)

        reg_write 0x300 0x10 0x6

        ## Toggle PMA Analog Reset

        start_pma_anlg_rst 0x0 $device
        start_pma_anlg_rst 0x2 $device

        ## Re-Load Serdes settings (call interrupt sequencer) for static IPs

        reconfig_write 0x0 0x91 0x1 
        reconfig_write 0x2 0x91 0x1        

        ## De-Assert TX digital reset ############################### modification, apply EIOSYS Reset at this time as well and deassert EIOSYS Reset

        reg_write 0x300 0x10 0x5

        reg_write 0x300 0x10 0x4

        ## Disable ILB or switch to mission mode (interrupt 0x08-0x100). If this step is missed, calibration won't go through. Alternatively, issue ILB + iCal then ELB + iCal. This step is automatically done if using mailbox. *

          ##### loop_off (don't need this) done by the mailbox command below #####

        ## If Recipe used --> Call Recipe Load from SIP

        ##### not applicable #####

        ## Call initAdapt + contAdapt from mailbox flow or use direct interrupts -- For mailbox message, refer to slide 7. 

        loop_on_pam4

        sleep 2
	
        init_adaptation_16_NoPrbsNoLdILB02 $device

        after 1000

        ## Deassert all resets
        reg_write 0x300 0x10 0x0  

        ## Check link status

        chk_init_adaptation_status_02 $device

        sleep 5

    ######################################## PPE Golden Calibration Flow ##############################################

        set res [wait_for_rsfec_lock 5 $device] 

        if {$res} {
            puts "----Iteration:$tries - No PHY lock. Retrying --------- \n"
        } else {
            puts "----Iteration:$tries - PMA Adaptaion is Successful---- \n"
            puts "------------------------------------------------------ \n"
            puts "------------------------------------------------------ \n"
        }
        incr tries        
    }
}


# FW init_adaptation for PAM4 variant (External Loopback)
proc start_pma_02_init_adaptation_ex {{device 0}} {
    global BASE_NDEVICE
    global BASE_RSFEC_EHIP

    # Perform init_adaptation multiple times if init_adaptation fails on first try.
    set tries 0
    set res 1
    
    while {$res && $tries < 5} {
    ######################################## PPE Golden Calibration Flow ##############################################
        ## Assert RX digital and TX digital resets (MAC+PCS reset)

        reg_write 0x300 0x10 0x6

        ## Toggle PMA Analog Reset

        start_pma_anlg_rst 0x0 $device
        start_pma_anlg_rst 0x2 $device

        ## Re-Load Serdes settings (call interrupt sequencer) for static IPs

        reconfig_write 0x0 0x91 0x1 
        reconfig_write 0x2 0x91 0x1        

        ## De-Assert TX digital reset ############################### modification, apply EIOSYS Reset at this time as well and deassert EIOSYS Reset

        reg_write 0x300 0x10 0x5

        reg_write 0x300 0x10 0x4

        ## Disable ILB or switch to mission mode (interrupt 0x08-0x100). If this step is missed, calibration won't go through. Alternatively, issue ILB + iCal then ELB + iCal. This step is automatically done if using mailbox. *

          ##### loop_off (don't need this) done by the mailbox command below #####

        ## If Recipe used --> Call Recipe Load from SIP

        ##### not applicable #####

        ## Call initAdapt + contAdapt from mailbox flow or use direct interrupts -- For mailbox message, refer to slide 7. 

        init_adaptation_16_NoPrbsNoLdELCntPC02 $device

        after 1000

        ## Deassert all resets
        reg_write 0x300 0x10 0x0  

        ## Check link status

        chk_init_adaptation_status_02 $device

        sleep 5

    ######################################## PPE Golden Calibration Flow ##############################################
        
        set res [wait_for_rsfec_lock 5 $device] 

        if {$res} {
            puts "----Iteration:$tries - No PHY lock. Retrying --------- \n"
        } else {
            puts "----Iteration:$tries - PMA Adaptaion is Successful---- \n"
            puts "------------------------------------------------------ \n"
            puts "------------------------------------------------------ \n"
        }
        incr tries        
    }
}


################################################################################################################### NEW PMA ADAPTATION PROC (October 14th 2019) --END--




##########################################
################# Resets #################
##########################################

# TX and RX reset
proc tx_rx_rst {{device 0}} {
    global BASE_NDEVICE
    puts "------------ Applying TX and RX Reset ----------"
    reg_write [expr ($BASE_NDEVICE * $device) + 0x300] 0x10 0x7
    after 1000 
    reg_write [expr ($BASE_NDEVICE * $device) + 0x300] 0x10 0x0
    after 200  
}  

# PMA reset For NRZ
proc start_pma_anlg_rst03 { {device 0} } {
    puts "------------- Starting PMA reset ------------- \n"
    # Reset each channel 
    start_pma_anlg_rst 0x0 $device
    start_pma_anlg_rst 0x1 $device
    start_pma_anlg_rst 0x2 $device
    start_pma_anlg_rst 0x3 $device

    # restart sequencer SM (XCVR reg)
    reconfig_write 0x0 0x91 0x1 $device
    reconfig_write 0x1 0x91 0x1 $device
    reconfig_write 0x2 0x91 0x1 $device
    reconfig_write 0x3 0x91 0x1 $device

}

# PMA reset for PAM4
proc start_pma_anlg_rst02 { {device 0} } {
    global BASE_NDEVICE
    puts "------------- Starting PMA reset ------------- \n"
    reg_write [expr ($BASE_NDEVICE * $device) + 0x300] 0x10 0x7
    after 2000
    start_pma_anlg_rst 0x0 $device
    start_pma_anlg_rst 0x2 $device
    reconfig_write 0x0 0x91 0x1 $device
    reconfig_write 0x2 0x91 0x1 $device

    # set init_time [clock clicks -milliseconds]
    set r0 [reconfig_read 0x0 0x8b $device]
    set tx_ready0 [expr ($r0 >> 3) & 1]
    set rx_ready0 [expr ($r0 >> 2) & 1]
    while {($tx_ready0 == 0) || ($rx_ready0 == 0)} {        
        set r0 [reconfig_read 0x0 0x8b $device]
        set tx_ready0 [expr ($r0 >> 3) & 1]
        set rx_ready0 [expr ($r0 >> 2) & 1]
    }

    set r2 [reconfig_read 0x2 0x8b $device]
    set tx_ready2 [expr ($r2 >> 3) & 1]
    set rx_ready2 [expr ($r2 >> 2) & 1]

    while {($tx_ready2 == 0) || ($rx_ready2 == 0)} {        
        set r2 [reconfig_read 0x2 0x8b $device]
        set tx_ready2 [expr ($r2 >> 3) & 1]
        set rx_ready2 [expr ($r2 >> 2) & 1]
    }
    after 2000
    reg_write [expr ($BASE_NDEVICE * $device) + 0x300] 0x10 0x0
    after 2000
}


# Use Mailbox Bit dedicated for SerDes PMA reset
proc start_pma_anlg_rst { ch {device 0} } {
    reconfig_write  $ch 0x200 0x00 $device
    reconfig_write  $ch 0x201 0x00 $device
    reconfig_write  $ch 0x202 0x00 $device
    reconfig_write  $ch 0x203 0x81 $device
    chk_pma_anlg_rst_stat $ch $device

}

# check PMA reset status
proc chk_pma_anlg_rst_stat { ch {device 0} } {
    set tries 0	 
    set outbox_207 0

    while {$outbox_207 != 0x80 && $tries < 5} {
    after 1000
    set outbox_207 [format 0x%x [reconfig_read $ch 0x207 $device]]
    incr tries
    }

    if {$outbox_207 != 0x80 } {
        puts "---------- PMA reset failed for ch$ch ----------"
    } 
}
##########################################
################# Resets #################
##########################################


# Load PMA Configuration from GUI
proc ld_pma_config { {device 0} } {
    puts "--------------- Loading Configuration ----------------- \n"
    # Load PMA Configuration 0 from GUI
    reconfig_write 0 0x40143 0x80 $device

    # Load PMA Configuration 1 from GUI
    # reconfig_write 0 0x40143 0x81 $device
    
    # Load PMA Configuration 2 from GUI
    # reconfig_write 0 0x40143 0x82 $device
    
    # Load PMA Configuration 3 from GUI
    # reconfig_write 0 0x40143 0x83 $device
    
    # Load PMA Configuration 4 from GUI
    # reconfig_write 0 0x40143 0x84 $device
    
    # Load PMA Configuration 5 from GUI
    # reconfig_write 0 0x40143 0x85 $device
    
    # Load PMA Configuration 6 from GUI
    # reconfig_write 0 0x40143 0x86 $device
    
    # Load PMA Configuration 7 from GUI
    # reconfig_write 0 0x40143 0x87 $device
    
}

# Check configuration load status and retry till timeout
proc chk_pma_config_status { {device 0} } {
    set outbox_40144 0x00
    set tries 0

    while {$outbox_40144 != 0x01 && $tries < 10} {
        after 2000

        # check status
        set outbox_40144 [format 0x%x [reconfig_read 0 0x40144 $device]]

        incr tries
    }

    if  {$outbox_40144 != 0x01 } {
        puts "-------------- Configuration Load Failed -------------- \n"
    } else {
        puts "--------------- Configuration Load Done --------------- \n"
    }

}

#################################################
#### Perform init_adaptation on each channel ####
#################################################

# For NRZ variant (External Loopback)
proc init_adaptation_16_NoPrbsNoLdEL03 {{device 0}} {
    puts "------------ Starting PMA Adaptation ------------ \n"
    init_adaptation_16_NoPrbsNoLdEL 0x0 $device
    get_calib_status 0x0 $device
    init_adaptation_16_NoPrbsNoLdEL 0x1 $device
    get_calib_status 0x1 $device
    init_adaptation_16_NoPrbsNoLdEL 0x2 $device
    get_calib_status 0x2 $device
    init_adaptation_16_NoPrbsNoLdEL 0x3 $device
    get_calib_status 0x3 $device
    
}

# For NRZ variant (Internal Loopback)
proc init_adaptation_16_NoPrbsNoLdILB03 {{device 0}} {
    puts "------------ Starting PMA Adaptation ------------ \n"
    init_adaptation_16_NoPrbsNoLdILB 0x0 $device 
    get_calib_status 0x0 $device
    init_adaptation_16_NoPrbsNoLdILB 0x1 $device
    get_calib_status 0x1 $device
    init_adaptation_16_NoPrbsNoLdILB 0x2 $device
    get_calib_status 0x2 $device
    init_adaptation_16_NoPrbsNoLdILB 0x3 $device
    get_calib_status 0x3 $device
    
}

# to get calibration status on return mailbox
proc get_calib_status { ch {device 0}} {

    reconfig_write  $ch 0x200 0x01 $device
    after  200
    reconfig_write  $ch 0x201 0x00 $device
    after  200
    reconfig_write  $ch 0x202 0x00 $device
    after  200
    reconfig_write  $ch 0x203 0x97 $device
    after  200

}

# For PAM4 variant 
proc init_adaptation_16_NoPrbsNoLdILB02 {{device 0}} {
    puts "------------ Starting PMA Adaptation ------------ \n"
    init_adaptation_16_NoPrbsNoLdILB 0x0 $device
    get_calib_status 0x0 $device
    init_adaptation_16_NoPrbsNoLdILB 0x2 $device
    get_calib_status 0x2 $device
    after 200
}

# For PAM4 variant 
proc init_adaptation_16_NoPrbsNoLdELCntPC02 {{device 0}} {
    puts "------------ Starting PMA Adaptation ------------ \n"
    init_adaptation_16_NoPrbsNoLdELCntPC 0x0 $device
    get_calib_status 0x0 $device
    init_adaptation_16_NoPrbsNoLdELCntPC 0x2 $device
    get_calib_status 0x2 $device
    after 200
}

## Perform PMA adaptation with - Disable Prbs , External LB, init_adaptation only(cont_adaptation Disabled), No configuration Load, 10_effort
proc init_adaptation_16_NoPrbsNoLdEL { ch {device 0}} {
    reconfig_write  $ch 0x200 0xe2 $device
    reconfig_write  $ch 0x201 0x01 $device
    reconfig_write  $ch 0x202 0x01 $device
    reconfig_write  $ch 0x203 0x96 $device
}

## Perform PMA adaptation with - Disable Prbs , internal LB, init_adaptation only(cont_adaptation Disabled), No configuration load, 05_effort
proc init_adaptation_16_NoPrbsNoLdILB { ch {device 0}} {
    reconfig_write  $ch 0x200 0xe1 $device
    reconfig_write  $ch 0x201 0x03 $device
    reconfig_write  $ch 0x202 0x01 $device
    reconfig_write  $ch 0x203 0x96 $device
    after 800
}

## Perform PMA adaptation with - Disable Prbs , External LB, full flow (init_adaptation+cont_adaptation), No configuration, continuous cont_adaptation, 10_effort
proc init_adaptation_16_NoPrbsNoLdELCntPC { ch {device 0}} {
    reconfig_write  $ch 0x200 0xe6 $device
    reconfig_write  $ch 0x201 0x01 $device
    reconfig_write  $ch 0x202 0x03 $device
    reconfig_write  $ch 0x203 0x96 $device
    after 800
}
#################################################
#### Perform init_adaptation on each channel ####
#################################################


#########################################################################
######## Check mailbox for init_adaptation status of each channel #######
#########################################################################

# NRZ variant
proc chk_init_adaptation_status {{device 0}} {
    puts "------- Checking PMA Adaptation Status------- \n"
    chk_init_adaptationstat_207 0x0 $device
    chk_init_adaptationstat_207 0x1 $device
    chk_init_adaptationstat_207 0x2 $device
    chk_init_adaptationstat_207 0x3 $device

}

# PAM4 Variant
proc chk_init_adaptation_status_02 {{device 0}} {
    puts "------- Checking PMA Adaptation Status------- \n"
    chk_init_adaptationstat_207 0x0 $device
    chk_init_adaptationstat_207 0x2 $device
}


# Check init_adaptation status per channel
proc chk_init_adaptationstat_207 { ch {device 0}} {

    set tries 0	 
    set outbox_207 0

    while {$outbox_207 != 0x80 && $tries < 10} {
    after 2000
    set outbox_207 [format 0x%x [reconfig_read $ch 0x207 $device]]
    incr tries
    }

    if {$outbox_207 != 0x80 } {
        puts "------ PMA Adaptation failed for ch$ch ------"
    } else {
        puts "------- PMA Adaptation Done for ch$ch -------"
    }
}
#########################################################################
######## Check mailbox for init_adaptation status of each channel #######
#########################################################################



#################################################################################
####################### Interrupt way for init_adaptation #######################
#################################################################################

proc pma_1ch_init_adaptation {ch {device 0}} {
    pma_int $ch 0x0a 0x00 0x0001 $device
}
proc pma_03_init_adaptation {{device 0}} {
    pma_int 0 0x0a 0x00 0x0001 $device
    pma_int 1 0x0a 0x00 0x0001 $device
    pma_int 2 0x0a 0x00 0x0001 $device
    pma_int 3 0x0a 0x00 0x0001 $device
}

proc pma_02_init_adaptation {{device 0}} {
    pma_int 0 0x0a 0x00 0x0001 $device
    pma_int 2 0x0a 0x00 0x0001 $device
}

proc pma_47_init_adaptation {{device 0}} {
    pma_int 4 0x0a 0x00 0x0001 $device
    pma_int 5 0x0a 0x00 0x0001 $device
    pma_int 6 0x0a 0x00 0x0001 $device
    pma_int 7 0x0a 0x00 0x0001 $device
}

# Might need timeout
proc chk_pma_02_init_adaptation_status {{device 0}} {
    #check init_adaptation status
    set init_adaptation_status_chan0 [expr [get_rx_eq 0 0xB 0x0 $device] & 0x1]
    set init_adaptation_status_chan2 [expr [get_rx_eq 2 0xB 0x0 $device] & 0x1]
    #wait init_adaptation completed
    while {($init_adaptation_status_chan0 != 0x0) || ($init_adaptation_status_chan2 != 0x0)} {
        after 1000
        set init_adaptation_status_chan0 [expr [get_rx_eq 0 0xB 0x0 $device] & 0x1]
        puts "$init_adaptation_status_chan0"
        set init_adaptation_status_chan2 [expr [get_rx_eq 2 0xB 0x0 $device] & 0x1]
        puts "$init_adaptation_status_chan2"
       }
    puts "channel 0 init_adaptation status is $init_adaptation_status_chan0"
    puts "channel 2 init_adaptation status is $init_adaptation_status_chan2"
}

# Might need timeout
proc chk_pma_03_init_adaptation_status {{device 0}} {
    #check init_adaptation status
    set init_adaptation_status_chan0 [expr [get_rx_eq 0 0xB 0x0 $device] & 0x1]
    set init_adaptation_status_chan1 [expr [get_rx_eq 1 0xB 0x0 $device] & 0x1]
    set init_adaptation_status_chan2 [expr [get_rx_eq 2 0xB 0x0 $device] & 0x1]
    set init_adaptation_status_chan3 [expr [get_rx_eq 3 0xB 0x0 $device] & 0x1]
    #wait init_adaptation completed
    puts  "-------- checking init_adaptation status ---------"
    puts  "--------------------------------------"
    while {($init_adaptation_status_chan0 != 0x0) || ($init_adaptation_status_chan1 != 0x0) || ($init_adaptation_status_chan2 != 0x0) || ($init_adaptation_status_chan3 != 0x0)} {
        after 1000
        set init_adaptation_status_chan0 [expr [get_rx_eq 0 0xB 0x0 $device] & 0x1]
        set init_adaptation_status_chan1 [expr [get_rx_eq 1 0xB 0x0 $device] & 0x1]
        set init_adaptation_status_chan2 [expr [get_rx_eq 2 0xB 0x0 $device] & 0x1]
        set init_adaptation_status_chan3 [expr [get_rx_eq 3 0xB 0x0 $device] & 0x1]
       }
    puts "channel 0 init_adaptation status is $init_adaptation_status_chan0"
    puts "channel 1 init_adaptation status is $init_adaptation_status_chan1"
    puts "channel 2 init_adaptation status is $init_adaptation_status_chan2"
    puts "channel 3 init_adaptation status is $init_adaptation_status_chan3"
}
###############################################################################
#################### END-Interrupt way for init_adaptation ####################
###############################################################################


set frame_type_list { "incr" "constant"  "random"} 
proc pcs_only_traffic_test { { iterations 1 } {device 0}} {
    variable    frame_type_list

    global BASE_NDEVICE
    set test_name [lindex [info level [info level]] 0]
    puts "Running ${test_name} test"
    after 100
    # init_ehip_loop_on
    after 100
    loop_on $device
    pma_03_init_adaptation $device
    chkphy_status $device
    # global $CLIENT_BASE
    set PCS_pkt_client_base [expr ($BASE_NDEVICE * $device) + 0xf000]
    for {set i 0} {$i < $iterations } {incr i} {
        ##randomize some stuff. Like number of Frames
        set res [random 30 9999]
        #set res [random 500 502]
        puts "Setting Number of frames to $res\n"
        set_pcs_gen_num_frames $res
        set res [random 64 600]
        #set res [random 64 100]
        puts "Setting Size of frames to $res\n"
        set_pcs_gen_frame_size $res
        ##set res [random_element "incr"]
        set res [random_element $frame_type_list]
        puts "Setting Size of frames to $res\n"
        pcs_gen_set_frame_type $res
        puts "-------------------------------------"
        puts "PCS TRAFFIC = ${i}"
        start_pcs_pkt_gen
        after 1000
        #after 1000
        set res [read_pcs_pkt_gen]
        if { $res == 0x53} {
            puts  "${test_name}:pass"
        } else {
            puts  "${test_name}:fail, Got $res instead of 0x53"
            puts "\n"
            puts "------chkphy_status-------"
            chkphy_status $device
            return -1
        }
    }
    return 0
}
#-----------------PCS ONLY TRAFFIC GEN-----------------
proc random_element { list } {
    lindex $list [expr {int (rand()* [llength $list])}]
}
proc random { {min 30 } { max 100 } } \
{
    return [expr {int($min + rand() * $max)}]
}
proc start_pcs_pkt_gen { } \
{
    variable PCS_pkt_client_base
    reg_write $PCS_pkt_client_base 0x0 0x02
    after 500
    reg_write $PCS_pkt_client_base 0x0 0x01
}
proc read_pcs_pkt_gen { } \
{
    variable PCS_pkt_client_base
    set res [reg_read $PCS_pkt_client_base 0x2]
    return $res
}
proc set_pcs_gen_num_frames { { num_frames 200} } \
{
    variable PCS_pkt_client_base
    reg_write $PCS_pkt_client_base 0x6 $num_frames
}
proc get_pcs_gen_num_frames { } \
{
    variable PCS_pkt_client_base
    reg_read $PCS_pkt_client_base 0x6
}
proc set_pcs_gen_frame_size { { frame_size 200} } \
{
    variable PCS_pkt_client_base
    reg_write $PCS_pkt_client_base 0x7  $frame_size
}
proc get_pcs_gen_frame_size {   } \
{
    variable PCS_pkt_client_base
    reg_read $PCS_pkt_client_base 0x7
}
proc get_pcs_gen_match_count {   } \
{
    variable PCS_pkt_client_base
    reg_write $PCS_pkt_client_base 0x8
}
proc get_pcs_gen_mismatch_count {   } \
{
    variable PCS_pkt_client_base
    reg_read $PCS_pkt_client_base 0x9
}
proc pcs_gen_set_frame_type { {frame_type "incr"} } \
{
    variable PCS_pkt_client_base
    if {$frame_type == "incr"} {
        reg_write $PCS_pkt_client_base 0xa 0x1
    } elseif    { $frame_type == "constant_crc" } {
        reg_write $PCS_pkt_client_base 0xa 0x9
    } elseif    { $frame_type == "constant" } {
        reg_write $PCS_pkt_client_base 0xa 0x3
    } elseif    { $frame_type == "random" } {
        reg_write $PCS_pkt_client_base 0xa 0x4
    } else {
        puts "ERROR"
        return -1
    }
    return 0
}
proc pcs_only_traffic_test_pam4 { { iterations 1 } {device 0}} {
    variable    log_file_handle
    variable    frame_type_list
    variable    PCS_pkt_client_base
    global BASE_NDEVICE
    set test_name [lindex [info level [info level]] 0]

	
	    ##################################

    chkphy_status $device
    #set VOD to maximum value
    pma_int 0 0x15 0x0 0x700f $device
    pma_int 2 0x15 0x0 0x700f $device
    #set post 1st pre-emphasis
    pma_int 0 0x15 0x0 0x8005 $device
    pma_int 2 0x15 0x0 0x8005 $device
    loop_off_pam4 $device
    sleep 1 
    loop_on_pam4 $device
    sleep 1 
    #trigger init_adaptation
    pma_02_init_adaptation $device
    chk_pma_02_init_adaptation_status $device
    after 2000
    #reset EHIP RX
    reg_write [expr ($BASE_NDEVICE * $device) + 0x300] 0x10 0x7
    after 1000 
    # approved by PPE runs 1000 times
    reg_write [expr ($BASE_NDEVICE * $device) + 0x300] 0x10 0x0
    sleep 2 
    chkphy_status $device
    set res [wait_for_rsfec_lock 5 $device]
    puts "rsfec locked status is $phy_locked "
    if {$res} {
        puts $log_file_handle "${test_name}:fail no phy lock"
    } else {
        puts $log_file_handle "${test_name}:pass"
    }

    ##################################

    puts "Running ${test_name} test"
    after 100
    after 100
    loop_on_pam4 $device
    pma_02_init_adaptation $device
    chkphy_status $device
    # global $CLIENT_BASE
    set PCS_pkt_client_base [expr ($BASE_NDEVICE * $device) + 0xf000]
    for {set i 0} {$i < $iterations} {incr i} {
        puts "-------------------------------------"
        puts "PCS TRAFFIC = ${i}"
        ##randomize some stuff. Like number of Frames
        set res [random 30 9999]
        puts "Setting Number of frames to $res\n"
        set_pcs_gen_num_frames $res
        set res [random 64 600]
        puts "Setting Size of frames to $res\n"
        set_pcs_gen_frame_size $res
        set res [random_element $frame_type_list]
        puts "Setting Size of frames to $res\n"
        pcs_gen_set_frame_type $res
        start_pcs_pkt_gen
        after 1000
        set res [read_pcs_pkt_gen]
        if { $res == 0x53} {
            puts  "${test_name}:pass"
        } else {
            puts  "${test_name}:fail, Got $res instead of 0x53"
            puts "\n"
            puts "------chkphy_status-------"
            chkphy_status $device
            return -1
        }
    }
    return 0
}
