# (C) 2001-2021 Intel Corporation. All rights reserved.
# Your use of Intel Corporation's design tools, logic functions and other 
# software and tools, and its AMPP partner logic functions, and any output 
# files from any of the foregoing (including device programming or simulation 
# files), and any associated documentation or information are expressly subject 
# to the terms and conditions of the Intel Program License Subscription 
# Agreement, Intel FPGA IP License Agreement, or other applicable 
# license agreement, including, without limitation, that your use is for the 
# sole purpose of programming logic devices manufactured by Intel and sold by 
# Intel or its authorized distributors.  Please refer to the applicable 
# agreement for further details.


source [file join [file dirname [info script]] "altera/sval_top/main.tcl"]

# Function to run a simple series of tests
proc run_test {} {
    puts "--- Turning off packet generation ----"
    puts "--------------------------------------\n"
    stop_pkt_gen

    puts "--------- Enabling loopback ----------"
    puts "--------------------------------------\n"
    loop_on

    ##Example for external loopback or connection to external device.
    #puts "---Starting PMA adaptation Sequence without internal loopback---\n"
    #start_pma_init_adaptation
    #puts "--------------------------------------\n"

    puts "--- Wait for RX clock to settle... ---"
    puts "--------------------------------------\n"
    sleep 2

    puts "-------- Printing PHY status ---------"
    puts "--------------------------------------\n"
    chkphy_status

    puts "---- Clearing MAC stats counters -----"
    puts "--------------------------------------\n"
    clear_all_stats

    puts "--------- Sending packets... ---------"
    puts "--------------------------------------\n"
    start_pkt_gen
    sleep 1
    stop_pkt_gen

    puts "----- Reading MAC stats counters -----"
    puts "--------------------------------------\n"
    chkmac_stats
}

proc run_test_ex {} {
    puts "--- Turning off packet generation ----"
    puts "--------------------------------------\n"
    stop_pkt_gen

    puts "--------- Disabling loopback ----------"
    puts "--------------------------------------\n"
    loop_off

    puts "--------- Set TX Post-Tap=12 for Agilex-Som ----------"
    puts "-------------------------------------------------------\n"
    pma_int 0 0x15 0x00 0x800C 0
    pma_int 1 0x15 0x00 0x800C 0
    pma_int 2 0x15 0x00 0x800C 0
    pma_int 3 0x15 0x00 0x800C 0

    ##Example for external loopback or connection to external device.
    #puts "---Starting PMA adaptation Sequence without internal loopback---\n"
    #start_pma_init_adaptation
    #puts "--------------------------------------\n"

    puts "--- Wait for RX clock to settle... ---"
    puts "--------------------------------------\n"
    sleep 2

    puts "-------- Printing PHY status ---------"
    puts "--------------------------------------\n"
    chkphy_status

    puts "---- Clearing MAC stats counters -----"
    puts "--------------------------------------\n"
    clear_all_stats

    puts "--------- Sending packets... ---------"
    puts "--------------------------------------\n"
    start_pkt_gen
    sleep 1
    stop_pkt_gen

    puts "----- Reading MAC stats counters -----"
    puts "--------------------------------------\n"
    chkmac_stats
}


proc run_test_pam4 {} {
    puts "--- Turning off packet generation ----"
    puts "--------------------------------------\n"
    stop_pkt_gen

    puts "--------- Enabling loopback ----------"
    puts "--------------------------------------\n"
    loop_on_pam4

    puts "--- Performing PMA adaptation... ---"
    puts "--------------------------------------\n"
    #With internal loopback
    start_pma_02_init_adaptation

    puts "--- Wait for RX clock to settle... ---"
    puts "--------------------------------------\n"
    sleep 2

    puts "-------- Printing PHY status ---------"
    puts "--------------------------------------\n"
    chkphy_status

    puts "---- Clearing MAC stats counters -----"
    puts "--------------------------------------\n"
    clear_all_stats

    puts "--------- Sending packets... ---------"
    puts "--------------------------------------\n"
    start_pkt_gen
    sleep 1
    stop_pkt_gen

    puts "----- Reading MAC stats counters -----"
    puts "--------------------------------------\n"
    chkmac_stats
}


proc pcs_only_traffic_test { { iterations 1 } {device 0}} {
    global BASE_NDEVICE
    set PCS_pkt_client_base [expr ($BASE_NDEVICE * $device) + 0xf000]
    set frame_type_list { "incr" "constant"  "random"} 
    set test_name [lindex [info level [info level]] 0]

    puts "Running ${test_name} test"

    after 200

    loop_on $device
    pma_03_init_adaptation $device
    chkphy_status $device

    for {set i 0} {$i < $iterations} {incr i} {
        puts "-------------------------------------"
        puts "PCS TRAFFIC = ${i}"
        set res [expr int (floor(rand()*4060) + 30)]
        puts "Setting Number of frames to $res\n"
        set_pcs_gen_num_frames $res

        set res [random 64 600]
        puts "Setting Size of frames to $res\n"
        set_pcs_gen_frame_size $res

        set res [random_element $frame_type_list]
        puts "Setting Size of frames to $res\n"
        pcs_gen_set_frame_type $res

        start_pcs_pkt_gen

        after 1000

        set res [read_pcs_pkt_gen]
        if { $res == 0x53} {
            puts  "${test_name}:pass"
        } else {
            puts  "${test_name}:fail, Got $res instead of 0x53"
            puts "\n"
            puts "------chkphy_status-------"
            chkphy_status $device
            return -1
        }
    }
    return 0
}


proc pcs_only_traffic_test_pam4 { { iterations 1 } {device 0}} {
    global BASE_NDEVICE
    set PCS_pkt_client_base [expr ($BASE_NDEVICE * $device) + 0xf000]
    set frame_type_list { "incr" "constant"  "random"} 
    set test_name [lindex [info level [info level]] 0]

    puts "Running ${test_name} test"

    after 200

    loop_on_pam4 $device
    start_pma_02_init_adaptation $device

    chkphy_status $device

    for {set i 0} {$i < $iterations} {incr i} {
        puts "-------------------------------------"
        puts "PCS TRAFFIC = ${i}"
        ##randomize some stuff. Like number of Frames
        set res [expr int (floor(rand()*4060) + 30)]
        puts "Setting Number of frames to $res\n"
        set_pcs_gen_num_frames $res

        set res [random 64 600]
        puts "Setting Size of frames to $res\n"
        set_pcs_gen_frame_size $res

        set res [random_element $frame_type_list]
        puts "Setting Size of frames to $res\n"
        pcs_gen_set_frame_type $res

        start_pcs_pkt_gen
        
        after 1000

        set res [read_pcs_pkt_gen]
        if { $res == 0x53} {
            puts  "${test_name}:pass"
        } else {
            puts  "${test_name}:fail, Got $res instead of 0x53"
            puts "\n"
            puts "------chkphy_status-------"
            chkphy_status
            return -1
        }
    }
    return 0
}
