// ============================================================================
// Copyright (c) 2021 by Terasic Technologies Inc.
// ============================================================================
//
// Permission:
//
//   Terasic grants permission to use and modify this code for use
//   in synthesis for all Terasic Development Boards and Altera Development 
//   Kits made by Terasic.  Other use of this code, including the selling 
//   ,duplication, or modification of any portion is strictly prohibited.
//
// Disclaimer:
//
//   This VHDL/Verilog or C/C++ source code is intended as a design reference
//   which illustrates how these types of functions can be implemented.
//   It is the user's responsibility to verify their design for
//   consistency and functionality through the use of formal
//   verification methods.  Terasic provides no warranty regarding the use 
//   or functionality of this code.
//
// ============================================================================
//           
//  Terasic Technologies Inc
//  No.80, Fenggong Rd., Hukou Township, Hsinchu County 303035 Taiwan
//  
//  
//                     web: http://www.terasic.com/  
//                     email: support@terasic.com
//
// ============================================================================
//Date:  Wed Dec 22 14:16:44 2021
// ============================================================================

//`define ENABLE_DDR4A
//`define ENABLE_DDR4B
//`define ENABLE_QSFP28
`define ENABLE_QSFP28RSV
//`define ENABLE_FMC
//`define ENABLE_FMCP
//`define ENABLE_HPS

module Agilex_SOM(

      ///////// CLOCK /////////
      input              CLK_100_B2A,
      input              CLK_30M72,
      input              CLK_50_B2A,
      input              CLK_50_B2D,
      input              CLK_50_B3A,
      input              CLK_50_B3C,

      ///////// Buttons /////////
      input              CPU_RESET_n,
      input    [ 1: 0]   BUTTON, //BUTTON is Low-Active

      ///////// Swtiches /////////
      input    [ 1: 0]   SW,

      ///////// LED /////////
      output   [ 1: 0]   LED, //LED is Low-Active

`ifdef ENABLE_DDR4A
      ///////// DDR4A /////////
      input              DDR4A_REFCLK_p,
      output   [16: 0]   DDR4A_A,
      output   [ 1: 0]   DDR4A_BA,
      output   [ 1: 0]   DDR4A_BG,
      output             DDR4A_CK,
      output             DDR4A_CK_n,
      output             DDR4A_CKE,
      inout    [ 8: 0]   DDR4A_DQS,
      inout    [ 8: 0]   DDR4A_DQS_n,
      inout    [71: 0]   DDR4A_DQ,
      inout    [ 8: 0]   DDR4A_DBI_n,
      output             DDR4A_CS_n,
      output             DDR4A_RESET_n,
      output             DDR4A_ODT,
      output             DDR4A_PAR,
      input              DDR4A_ALERT_n,
      output             DDR4A_ACT_n,
      input              DDR4A_EVENT_n,
      inout              DDR4A_SCL,
      inout              DDR4A_SDA,
      input              DDR4A_RZQ,
`endif /*ENABLE_DDR4A*/

`ifdef ENABLE_DDR4B
      ///////// DDR4B /////////
      input              DDR4B_REFCLK_p,
      output   [16: 0]   DDR4B_A,
      output   [ 1: 0]   DDR4B_BA,
      output   [ 1: 0]   DDR4B_BG,
      output   [ 1: 0]   DDR4B_CK,
      output   [ 1: 0]   DDR4B_CK_n,
      output   [ 1: 0]   DDR4B_CKE,
      inout    [ 8: 0]   DDR4B_DQS,
      inout    [ 8: 0]   DDR4B_DQS_n,
      inout    [71: 0]   DDR4B_DQ,
      inout    [ 8: 0]   DDR4B_DBI_n,
      output   [ 1: 0]   DDR4B_CS_n,
      output             DDR4B_RESET_n,
      output   [ 1: 0]   DDR4B_ODT,
      output             DDR4B_PAR,
      input              DDR4B_ALERT_n,
      output             DDR4B_ACT_n,
      input              DDR4B_EVENT_n,
      inout              DDR4B_SCL,
      inout              DDR4B_SDA,
      input              DDR4B_RZQ,
`endif /*ENABLE_DDR4B*/

      ///////// SI5340A /////////
      inout              SI5340A_I2C_SCL,
      inout              SI5340A_I2C_SDA,
      input              SI5340A_LOL,
      input              SI5340A_LOS_XAXB,
      output             SI5340A_OE_n,
      output             SI5340A_RST_n,

      ///////// GPIO_D /////////
      inout    [ 7: 0]   GPIO_D,

`ifdef ENABLE_FMC
      ///////// FMC /////////
      inout              FMC_CLK2_BIDIR_p,
      inout              FMC_CLK2_BIDIR_n,
      inout              FMC_CLK3_BIDIR_p,
      inout              FMC_CLK3_BIDIR_n,
      output   [ 9: 0]   FMC_DP_C2M_p,
      output   [ 9: 0]   FMC_DP_C2M_n,
      input    [ 9: 0]   FMC_DP_M2C_p,
      input    [ 9: 0]   FMC_DP_M2C_n,
      inout    [23: 0]   FMC_HA_p,
      inout    [23: 0]   FMC_HA_n,
      inout    [21: 0]   FMC_HB_p,
      inout    [21: 0]   FMC_HB_n,
      inout    [33: 0]   FMC_LA_p,
      inout    [33: 0]   FMC_LA_n,
      input    [ 1: 0]   FMC_CLK_M2C_p,
      input    [ 1: 0]   FMC_GBTCLK_M2C_p,
      input              FMC_REFCLK0_p,
      input              FMC_REFCLK1_p,
      inout    [ 1: 0]   FMC_GA,
      inout    [ 0: 0]   FMC_RES,
      inout              FMC_SCL,
      inout              FMC_SDA,
`endif /*ENABLE_FMC*/
      input              FMC_REFCLK0_p,
      input              FMC_REFCLK1_p,
`ifdef ENABLE_FMCP
      ///////// FMCP /////////
      inout              FMCP_CLK2_BIDIR_p,
      inout              FMCP_CLK2_BIDIR_n,
      inout              FMCP_CLK3_BIDIR_p,
      inout              FMCP_CLK3_BIDIR_n,
      output   [15: 0]   FMCP_DP_C2M_p,
      output   [15: 0]   FMCP_DP_C2M_n,
      input    [15: 0]   FMCP_DP_M2C_p,
      input    [15: 0]   FMCP_DP_M2C_n,
      inout    [23: 0]   FMCP_HA_p,
      inout    [23: 0]   FMCP_HA_n,
      inout    [21: 0]   FMCP_HB_p,
      inout    [21: 0]   FMCP_HB_n,
      inout    [33: 0]   FMCP_LA_p,
      inout    [33: 0]   FMCP_LA_n,
      input    [ 1: 0]   FMCP_CLK_M2C_p,
      input    [ 1: 0]   FMCP_GBTCLK_M2C_p,
      output             FMCP_REFCLK_C2M_p,
      input              FMCP_REFCLK_M2C_p,
      inout    [ 1: 0]   FMCP_GA,
      input              FMCP_RES0,
      inout              FMCP_RES1,
      inout              FMCP_SCL,
      inout              FMCP_SDA,
      output             FMCP_SYNC_C2M_p,
      input              FMCP_SYNC_M2C_p,
`endif /*ENABLE_FMCP*/

`ifdef ENABLE_QSFP28
      ///////// QSFP28 /////////
      output   [ 3: 0]   QSFP28_TX_p,
      output   [ 3: 0]   QSFP28_TX_n,
      input    [ 3: 0]   QSFP28_RX_p,
      input    [ 3: 0]   QSFP28_RX_n,
      input              QSFP28_REFCLK_p,
      input              QSFP28_INTERRUPT_n,
      output             QSFP28_LP_MODE,
      input              QSFP28_MOD_PRS_n,
      output             QSFP28_MOD_SEL_n,
      output             QSFP28_RST_n,
      inout              QSFP28_SCL,
      inout              QSFP28_SDA,
`else
      input              QSFP28_REFCLK_p,
`endif /*ENABLE_QSFP28*/

`ifdef ENABLE_QSFP28RSV
      ///////// QSFP28RSV /////////
      input              QSFP28RSV_REFCLK_p,
`endif /*ENABLE_QSFP28RSV*/

      ///////// AG /////////
      output             AG_UART_TX,
      input              AG_UART_RX,
      input              AG_UART_CTS,
      output             AG_UART_RTS,

`ifdef ENABLE_HPS
      ///////// HPS /////////
      output             HPS_ENET_MDC,
      inout              HPS_ENET_MDIO,
      input              HPS_ENET_RX_CLK,
      input              HPS_ENET_RX_CTL,
      input    [ 3: 0]   HPS_ENET_RX_DATA,
      output             HPS_ENET_TX_CLK,
      output             HPS_ENET_TX_CTL,
      output   [ 3: 0]   HPS_ENET_TX_DATA,
      inout    [ 4: 0]   HPS_GPIO,
      inout              HPS_I2C_SCL,
      inout              HPS_I2C_SDA,
      input              HPS_JTAG_TCK,
      input              HPS_JTAG_TDI,
      output             HPS_JTAG_TDO,
      input              HPS_JTAG_TMS,
      inout              HPS_KEY,
      inout              HPS_LED,
      input              HPS_OSC_CLK,
      output             HPS_SD_CLK,
      inout              HPS_SD_CMD,
      inout    [ 3: 0]   HPS_SD_DATA,
      input              HPS_UART_RX,
      output             HPS_UART_TX,
      input              HPS_USB_CLK,
      inout    [ 7: 0]   HPS_USB_DATA,
      input              HPS_USB_DIR,
      input              HPS_USB_NXT,
      output             HPS_USB_STP,
`endif /*ENABLE_HPS*/

      ///////// INFO /////////
      output             INFO_SPI_SCLK,
      input              INFO_SPI_MISO,
      output             INFO_SPI_MOSI,
      output             INFO_SPI_CS_n 

);


//=======================================================
//  REG/WIRE declarations
//=======================================================




//=======================================================
//  Structural coding
//=======================================================



assign reset_n = !ninit_done;
wire ninit_done;
reset_release reset_release_inst (
	.ninit_done(ninit_done)
	);

	


	 
    Qsys Qsys_inst (
        .clk_clk                                  (CLK_50_B2A),                                  //   input,  width = 1,                               clk.clk
        .fmc_refclk0_clk_in_ref_export            (CLK_50_B2A),            //   input,  width = 1,            fmc_refclk0_clk_in_ref.export
        .fmc_refclk0_clk_in_target_export         (FMC_REFCLK0),         //   input,  width = 1,         fmc_refclk0_clk_in_target.export
        .fmc_refclk1_clk_in_ref_export            (CLK_50_B2A),            //   input,  width = 1,            fmc_refclk1_clk_in_ref.export
        .fmc_refclk1_clk_in_target_export         (FMC_REFCLK1),         //   input,  width = 1,         fmc_refclk1_clk_in_target.export
        .qsfp28_refclk_clk_in_ref_export          (CLK_50_B2A),          //   input,  width = 1,          qsfp28_refclk_clk_in_ref.export
        .qsfp28_refclk_clk_in_target_export       (QSFP28_REFCLK),       //   input,  width = 1,       qsfp28_refclk_clk_in_target.export
        .qsfp28rsv_refclk_clk_in_ref_export       (CLK_50_B2A),       //   input,  width = 1,       qsfp28rsv_refclk_clk_in_ref.export
        .qsfp28rsv_refclk_clk_in_target_export    (QSFP28RSV_REFCLK),    //   input,  width = 1,    qsfp28rsv_refclk_clk_in_target.export
        .reset_reset_n                            (reset_n),                            //   input,  width = 1,                             reset.reset_n
        .si5340a_i2c_export_scl_pad_io            (SI5340A_I2C_SCL),            //   inout,  width = 1,                si5340a_i2c_export.scl_pad_io
        .si5340a_i2c_export_sda_pad_io            (SI5340A_I2C_SDA),            //   inout,  width = 1,                                  .sda_pad_io
        .si5340a_oe_n_external_connection_export  (SI5340A_OE_n),  //  output,  width = 1,  si5340a_oe_n_external_connection.export
        .si5340a_rst_n_external_connection_export (SI5340A_RST_n),  //  output,  width = 1, si5340a_rst_n_external_connection.export
		  //Board INFO
		  .info_board_version_external_connection_export   (rBoard_Version),   //   input,  width = 16,   info_board_version_external_connection.export
        .info_maxcode_version_external_connection_export (rMaxCode_Version), //   input,  width = 16, info_maxcode_version_external_connection.export
        .info_power_in_vol_external_connection_export    (rPowerIn_Voltage),    //   input,  width = 16,    info_power_in_vol_external_connection.export
        .info_power_in_cur_external_connection_export    (rPowerIn_Current),    //   input,  width = 16,    info_power_in_cur_external_connection.export
        .info_core_vol_external_connection_export        (rCORE_Voltage),        //   input,  width = 16,        info_core_vol_external_connection.export
        .info_core_cur_external_connection_export        (rCORE_Current),        //   input,  width = 16,        info_core_cur_external_connection.export
        .info_core_vol_2_external_connection_export      (rCORE_Voltage2),      //   input,  width = 16,      info_core_vol_2_external_connection.export
        .info_core_cur_2_external_connection_export      (rCORE_Current2),      //   input,  width = 16,      info_core_cur_2_external_connection.export
        .info_fan_speed_1_external_connection_export     (rFan_Speed),     //   input,  width = 16,     info_fan_speed_1_external_connection.export
        .info_temp_fpga_external_connection_export       (rTemp_FPGA),       //   input,  width = 16,       info_temp_fpga_external_connection.export
        .info_temp_board_external_connection_export      (rTemp_Board),      //   input,  width = 16,      info_temp_board_external_connection.export
        .info_temp_board_2_external_connection_export    (rTemp_Board2),    //   input,  width = 16,    info_temp_board_2_external_connection.export
        .info_temp_sdm_external_connection_export        (rTemp_SDM),        //   input,  width = 16,        info_temp_sdm_external_connection.export
        .info_temp_etile_external_connection_export      (rTemp_ETILE),      //   input,  width = 16,      info_temp_etile_external_connection.export
        .info_temp_ptile_external_connection_export      (rTemp_PTILE)       //   input,  width = 16,      info_temp_ptile_external_connection.export
 
	 );
wire QSFP28RSV_REFCLK;
wire QSFP28_REFCLK;
wire [2: 0] qsfp28_out;

wire FMC_REFCLK0;
wire FMC_REFCLK1;

	//not to do any thing
    xcvr_native_phy_qsfp28 xcvr_fmc_refclk0_inst (
        .pll_locked     (),     //  output,  width = 1,     pll_locked.tx_pma_ready
        .tx_serial_data (), //  output,  width = 1, tx_serial_data.tx_serial_data
        .pll_refclk0    (FMC_REFCLK0_p),    //   input,  width = 1,    pll_refclk0.clk
        .pll_clkout1    (FMC_REFCLK0),    //  output,  width = 1,    pll_clkout1.clk
        .pll_clkout2    ()     //  output,  width = 1,    pll_clkout2.clk
    );
	 
	 //not to do any thing
    xcvr_native_phy xcvr_fmc_refclk1_inst (
        .pll_locked     (),     //  output,  width = 1,     pll_locked.tx_pma_ready
        .tx_serial_data (), //  output,  width = 1, tx_serial_data.tx_serial_data
        .pll_refclk0    (FMC_REFCLK1_p),    //   input,  width = 1,    pll_refclk0.clk
        .pll_clkout1    (FMC_REFCLK1),    //  output,  width = 1,    pll_clkout1.clk
        .pll_clkout2    ()     //  output,  width = 1,    pll_clkout2.clk
    );
	 

    xcvr_native_phy xcvr_qsfpddrsv_inst (
        .pll_locked     (),     //  output,  width = 1,     pll_locked.tx_pma_ready
        .tx_serial_data (), //  output,  width = 1, tx_serial_data.tx_serial_data
        .pll_refclk0    (QSFP28RSV_REFCLK_p),    //   input,  width = 1,    pll_refclk0.clk
        .pll_clkout1    (QSFP28RSV_REFCLK),    //  output,  width = 1,    pll_clkout1.clk
        .pll_clkout2    ()     //  output,  width = 1,    pll_clkout2.clk
    );
	 
	 
    xcvr_native_phy_qsfp28 xcvr_qsfpdd_inst (
        .pll_locked     (),     //  output,  width = 1,     pll_locked.tx_pma_ready
        .tx_serial_data (), //  output,  width = 1, tx_serial_data.tx_serial_data
        .pll_refclk0    (QSFP28_REFCLK_p),    //   input,  width = 1,    pll_refclk0.clk
        .pll_clkout1    (QSFP28_REFCLK),    //  output,  width = 1,    pll_clkout1.clk
        .pll_clkout2    ()     //  output,  width = 1,    pll_clkout2.clk
    );

//System Monitor IP
//IP interface
wire        Info_Valid;
wire [15:0] Board_Version	;//A~F.ex BOARD_VERSION = 16'h000b ,express to revB
wire [15:0] MaxCode_Version;//0~F.ex MAX10_VERSION = 16'h0004 ,express to rev4
wire [15:0] PowerIn_Voltage;//unit : Celsius degree    
wire [15:0] PowerIn_Current;//unit : Celsius degree    
wire [15:0] CORE_Voltage	;//unit : mv   
wire [15:0] CORE_Current	;//unit : mA  
wire [15:0] Fan_Speed	   ;//unit : rpm  
wire [15:0] Fan_Speed2	   ;//unit : rpm 
wire [15:0] Temp_FPGA	   ;//unit : Celsius degree 
wire [15:0] Temp_Board	   ;//unit : Celsius degree 
wire [15:0] PIN_STATUS	   ;//
wire [15:0] Temp_SDM	      ;//unit : Celsius degree 
wire [15:0] Temp_ETILE	   ;//unit : Celsius degree 
wire [15:0] Temp_PTILE	   ;//unit : Celsius degree 
wire [15:0] CORE_Voltage2	;//unit : mv
wire [15:0] CORE_Current2	;//unit : mA                                         
wire [15:0] Temp_Board2	   ;//unit : Celsius degree 
wire [15:0] Shutdown_flag  ;


//----reg
reg [15:0] rBoard_Version	;
reg [15:0] rMaxCode_Version;
reg [15:0] rPowerIn_Voltage;
reg [15:0] rPowerIn_Current;
reg [15:0] rCORE_Voltage	;
reg [15:0] rCORE_Current	;
reg [15:0] rFan_Speed	   ;
reg [15:0] rFan_Speed2	   ;
reg [15:0] rTemp_FPGA	   ; 
reg [15:0] rTemp_Board	   ; 
reg [15:0] rPIN_STATUS	   ; 
reg [15:0] rTemp_SDM	      ;
reg [15:0] rTemp_ETILE	   ;
reg [15:0] rTemp_PTILE	   ;
reg [15:0] rCORE_Voltage2	;
reg [15:0] rCORE_Current2	;
reg [15:0] rTemp_Board2	   ; 
reg [15:0] rShutdown_flag  ;
                
BOARD_INFO BOARD_INFO_i(    
             .RESET_N          (BUTTON[0]    ),  // input
             .CLK_50           (CLK_50_B2A   ),  // input

             // 4-pin SPI master interface
             .SCLK             (INFO_SPI_SCLK), // output
             .MISO             (INFO_SPI_MISO), // input
             .MOSI             (INFO_SPI_MOSI), // output
             .CS_n             (INFO_SPI_CS_n), // output

             // status report (output)
             .Info_Valid       ( Info_Valid     ), //1: following data is valid
             .Board_Version	 ( Board_Version	), //A~F.ex BOARD_VERSION = 16'h000b ,express to revB
             .MaxCode_Version  ( MaxCode_Version), //0~F.ex MAX10_VERSION = 16'h0004 ,express to rev4
             .PowerIn_Voltage  ( PowerIn_Voltage), //unit : Celsius degree    
             .PowerIn_Current  ( PowerIn_Current), //unit : Celsius degree    
             .CORE_Voltage	    ( CORE_Voltage	), //unit : mv   
             .CORE_Current	    ( CORE_Current	), //unit : mA  
				 .Fan_Speed	       ( Fan_Speed	   ), //unit : rpm
				 //.Fan_Speed2	    ( Fan_Speed2	   ), //unit : rpm                                        
             .Temp_FPGA	       ( Temp_FPGA	   ), //unit : Celsius degree 
				 .Temp_Board	    ( Temp_Board	   ), //unit : Celsius degree
				 .PIN_STATUS	    ( PIN_STATUS	   ),  
				 .Temp_SDM	       ( Temp_SDM	      ), //unit : Celsius degree
				 .Temp_ETILE	    ( Temp_ETILE	   ), //unit : Celsius degree 
				 .Temp_PTILE	    ( Temp_PTILE	   ), //unit : Celsius degree 
				 .CORE_Voltage2	 ( CORE_Voltage2	), //unit : mv
				 .CORE_Current2	 ( CORE_Current2	), //unit : mA
				 .Temp_Board2	    ( Temp_Board2	   ),  //unit : Celsius degree 
             .Shutdown_flag	 ( Shutdown_flag  )			 
				 
);





always @(posedge CLK_50_B2A ) begin
    if (Info_Valid) begin
        rBoard_Version	<= Board_Version	;
        rMaxCode_Version<= MaxCode_Version;
        rPowerIn_Voltage<= PowerIn_Voltage;
        rPowerIn_Current<= PowerIn_Current;
        rCORE_Voltage	<= CORE_Voltage	;
        rCORE_Current	<= CORE_Current	;
        rFan_Speed	   <= Fan_Speed	   ;
		 //rFan_Speed2	   <= Fan_Speed2	   ;
		  rTemp_FPGA	   <= Temp_FPGA	   ;
		  rTemp_Board	   <= Temp_Board	   ;
		  rPIN_STATUS	   <= PIN_STATUS	   ;
		  rTemp_SDM	      <= Temp_SDM	      ;
		  rTemp_ETILE	   <= Temp_ETILE	   ;
		  rTemp_PTILE	   <= Temp_PTILE	   ;
		  rCORE_Voltage2	<= CORE_Voltage2	;
		  rCORE_Current2	<= CORE_Current2	;
		  rTemp_Board2	   <= Temp_Board2	   ;
		  rShutdown_flag  <= Shutdown_flag  ;
    end
end

endmodule
