// --------------------------------------------------------------------
// Copyright (c) 2012 by Terasic Technologies Inc.
// --------------------------------------------------------------------
//
// Permission:
//
//   Terasic grants permission to use and modify this code for use
//   in synthesis for all Terasic Development Boards and Altera Development
//   Kits made by Terasic.  Other use of this code, including the selling
//   ,duplication, or modification of any portion is strictly prohibited.
//
// Disclaimer:
//
//   This VHDL/Verilog or C/C++ source code is intended as a design reference
//   which illustrates how these types of functions can be implemented.
//   It is the user's responsibility to verify their design for
//   consistency and functionality through the use of formal
//   verification methods.  Terasic provides no warranty regarding the use
//   or functionality of this code.
//
// --------------------------------------------------------------------
//
//                     Terasic Technologies Inc
//                     E. Rd Sec. 1. JhuBei City,
//                     HsinChu County, Taiwan
//                     302
//
//                     web: http://www.terasic.com/
//                     email: support@terasic.com
//
// --------------------------------------------------------------------

#include "terasic_includes.h"
#include "clock.h"
#include "si5340A_config.h"
#include "I2C_core.h"
#include <stdbool.h>

typedef bool (*LP_VERIFY_FUNC)(void);

bool TEST_Si5340A(void);
bool TEST_Board_Info(void);

typedef struct {
    LP_VERIFY_FUNC func;
    char szName[128];
} FUNC_INFO;

FUNC_INFO szFuncList[] = { { TEST_Si5340A, "Si5340A" },
                           { TEST_Board_Info, "Display Board Info" } };

void GUI_ShowMenu(void)
{
    int nNum, i;

    nNum = sizeof(szFuncList) / sizeof(szFuncList[0]);
    printf("======= Agilex Demo Program =======\n");
    for (i = 0; i < nNum; i++) {
        printf("[%d] %s\n", i, szFuncList[i].szName);
    }
    printf("Input your chioce:");
}

int GUI_QueryUser(void)
{
    int nChoice = 0;
    scanf("%d", &nChoice);
    printf("%d\n", nChoice);
    return nChoice;
}

//===============================================================
int main(void)
{
    int nChoice;
    int nNum;
    bool bPass;

    nNum = sizeof(szFuncList) / sizeof(szFuncList[0]);
    while (1) {
        GUI_ShowMenu();
        nChoice = GUI_QueryUser();
        if (nChoice >= 0 && nChoice < nNum) {
            bPass = szFuncList[nChoice].func();
            printf("%s Test:%s\n", szFuncList[nChoice].szName,
                   bPass ? "PASS" : "NG");
        }
    }

}

bool TEST_Si5340A(void)
{
    typedef struct {
        int FreqID;
        float FREQUENCY;
    } Si5340A_CONFIG;

    Si5340A_CONFIG config_list_xcvr[] =
    		{ { SI5340_644M53125,  644.53125 },
            { SI5340_625M,       625.0 },
			{ SI5340_375M,		 375.0 },
            { SI5340_322M265625, 322.265625},
			{ SI5340_312M5,      312.5 },
			{ SI5340_307M2,      307.2 },
            { SI5340_250M,       250.0 },
            { SI5340_184M32,     184.32 },
            { SI5340_156M25,     156.25 },
			{ SI5340_153M6,      153.6 },
            { SI5340_125M,       125.0 },
            { SI5340_100M,       100.0 },
            { SI5340_NONE,       0 } };
    bool bPass, bThisPass;
    alt_u32 clk1, clk2, ScaledClk2;
    const int nTargetCnt = 1000;
    const int nTol = 3; //tolerance
    int nNum, i;
    int config_num_xcvr;
    config_num_xcvr = sizeof(config_list_xcvr) / sizeof(config_list_xcvr[0]);
    alt_u32 szBase[] = { FMC_REFCLK0_BASE,
    					 FMC_REFCLK1_BASE,
						 QSFP28RSV_REFCLK_BASE ,
						 QSFP28_REFCLK_BASE
    					};
    const int channel_num_xcvr = 4;
    alt_u8 szIndex[][32] = { "FMC REF CLK0", "FMC REF CLK1", "QSFPDDRSV", "QSFP28" };
    int channel_frequency[] = { SI5340_156M25, SI5340_184M32, SI5340_184M32, SI5340_156M25};
    float FREQUENCY[] = { 156.25, 184.32, 184.32, 156.25 };
    nNum = sizeof(szBase) / sizeof(szBase[0]);
    // show menu
    printf("===== Si5340A Programming =====\r\n");
    IOWR(SI5340A_OE_N_BASE, 0, 0x00);
    IOWR(SI5340A_RST_N_BASE, 0, 0x00);
    usleep(10000);
    IOWR(SI5340A_RST_N_BASE, 0, 0x01);
    for (i = 0; i < config_num_xcvr; i++) {
        printf("[%d] %.6f MHz\n", i, config_list_xcvr[i].FREQUENCY);
    }
    printf("[Other] exit\n");
    for (i = 0; i < channel_num_xcvr; i++) {
        printf("please select %s:", szIndex[i]);
        channel_frequency[i] = GUI_QueryUser();
        FREQUENCY[i] = config_list_xcvr[channel_frequency[i]].FREQUENCY;
    }
    usleep(1000);
    bPass = SI5340A_Config_Init(SI5340A_I2C_BASE);
    if (bPass) {
        bPass = SI5340A_Config(SI5340A_I2C_BASE, channel_frequency[0], channel_frequency[1], channel_frequency[2], channel_frequency[3]);
    }

    //alt_u8 szPass[32];
    alt_u32 szClk1[32];
    alt_u32 szClk2[32];

    if (bPass) {
        ////////OSC_50
        for (i = 0; i < nNum; i++) {
            usleep(200 * 1000);
            // printf("i =%d\r\n", i );
            szClk1[i] = -1;
            szClk2[i] = -1;

            bThisPass = TRUE;
            if (CLOCK_Test(szBase[i], nTargetCnt, &clk1, &clk2)) {
                szClk1[i] = clk1;
                szClk2[i] = clk2;
                ScaledClk2 = (int) ((float) clk2 * 50.0 / FREQUENCY[i]);
                if (abs(nTargetCnt - clk1) < nTol && abs(clk1 - ScaledClk2) < nTol) {
                    printf("%s_REFCLK PASS (clk1=%d, clk2=%d)\r\n", szIndex[i], (int) szClk1[i], (int) szClk2[i]);
                } else {
                    printf("%s NG (clk1=%d, clk2=%d)\r\n", szIndex[i], (int) szClk1[i], (int) szClk2[i]);
                    bThisPass = FALSE;

                }
            } else {
                bThisPass = FALSE;
                printf("Failed to read count of %s\r\n", szIndex[i]);
            }
            //szPass[i] = bThisPass;
            if (!bThisPass)
                bPass = FALSE;
        }
    }

    return bPass;

}

bool TEST_Board_Info(void)
{
	uint16_t bPass = 1;
	uint16_t board_version, maxcode_version;
	uint16_t power_in_voltage, power_in_current;
	uint16_t core_voltage1, core_voltage2;
    float core_voltage1_float, core_voltage2_float;
    uint16_t core_current1, core_current2;
    float core_current1_float, core_current2_float;
    uint16_t fan_speed1, fan_speed2;
    uint16_t temperature_fpga, temperature_board1, temperature_board2;
    uint16_t temperature_sdm, temperature_e_tile, temperature_p_tile;
    uint16_t pin_status;


    float core_voltage, core_current;
    board_version = IORD_ALTERA_AVALON_PIO_DATA(INFO_BOARD_VERSION_BASE) & 0xFFFF;
    maxcode_version = IORD_ALTERA_AVALON_PIO_DATA(INFO_MAXCODE_VERSION_BASE) & 0xFFFF;
    power_in_voltage = IORD_ALTERA_AVALON_PIO_DATA(INFO_POWER_IN_VOL_BASE) & 0xFFFF;
    power_in_current = IORD_ALTERA_AVALON_PIO_DATA(INFO_POWER_IN_CUR_BASE) & 0xFFFF;
    core_voltage1 = IORD_ALTERA_AVALON_PIO_DATA(INFO_CORE_VOL_BASE) & 0xFFFF;
    core_current1 = (int16_t)(IORD_ALTERA_AVALON_PIO_DATA(INFO_CORE_CUR_BASE) & 0xFFFF);
    core_voltage2 = IORD_ALTERA_AVALON_PIO_DATA(INFO_CORE_VOL_2_BASE) & 0xFFFF;
    core_current2 = (int16_t)(IORD_ALTERA_AVALON_PIO_DATA(INFO_CORE_CUR_2_BASE) & 0xFFFF);
    fan_speed1 = IORD_ALTERA_AVALON_PIO_DATA(INFO_FAN_SPEED_1_BASE) & 0xFFFF;
    //fan_speed2 = IORD_ALTERA_AVALON_PIO_DATA(INFO_FAN_SPEED_2_BASE) & 0xFFFF;
    temperature_fpga = IORD_ALTERA_AVALON_PIO_DATA(INFO_TEMP_FPGA_BASE) & 0xFFFF;
    temperature_board1 = IORD_ALTERA_AVALON_PIO_DATA(INFO_TEMP_BOARD_BASE) & 0xFFFF;
    temperature_board2 = IORD_ALTERA_AVALON_PIO_DATA(INFO_TEMP_BOARD_2_BASE) & 0xFFFF;
    temperature_sdm = IORD_ALTERA_AVALON_PIO_DATA(INFO_TEMP_SDM_BASE) & 0xFFFF;
    temperature_e_tile = IORD_ALTERA_AVALON_PIO_DATA(INFO_TEMP_ETILE_BASE) & 0xFFFF;
    temperature_p_tile = IORD_ALTERA_AVALON_PIO_DATA(INFO_TEMP_PTILE_BASE) & 0xFFFF;

    core_voltage1_float = core_voltage1 / 1000.0;
    core_voltage2_float = core_voltage2 / 1000.0;
    core_voltage = (core_voltage1_float + core_voltage2_float) / 2.0;
    core_current1_float = core_current1 / 1000.0;
    core_current2_float = core_current2 / 1000.0;
    core_current = core_current1_float + core_current2_float;

  //  printf("==== BOARD Info ====\n");
  //  printf("\tBoard Version: %x\n", board_version);
   // printf("\tMAX Code Version: %d\n", maxcode_version);

    printf("==== Temperature ====\n");
    printf("\tFPGA: %d*C\n", temperature_fpga);
    printf("\tBoard 1: %d*C\n", temperature_board1);
    printf("\tBoard 2: %d*C\n", temperature_board2);
    printf("\tSDM: %d*C\n", temperature_sdm);
    printf("\tE-Tile: %d*C\n", temperature_e_tile);
    printf("\tP-Tile: %d*C\n", temperature_p_tile);
    printf("\n");

    printf("==== Fan ====\n");
    printf("\tFan 1 RPM: %d\n", fan_speed1);
   // printf("\tFan 2 RPM: %d\n", fan_speed2);
    printf("\n");
    printf("==== Power (12V) Monitor ====\n");
    printf("\tVoltage       = %.3f V \n", power_in_voltage / 1000.0);
    printf("\tCurrent       = %.3f A \n", power_in_current / 1000.0);
    printf("\tPower         = %.3f W \n", power_in_voltage / 1000.0 * power_in_current / 1000.0);
    printf("==== Core Power Monitor ====\n");
    printf("\tVoltage       = %.3f V \n", core_voltage);
    printf("\tCurrent       = %.3f A \n", core_current);
    printf("\tPower         = %.3f W \n", core_voltage * core_current);

    return true;
}
