// ============================================================================
// Copyright (c) 2021 by Terasic Technologies Inc.
// ============================================================================
//
// Permission:
//
//   Terasic grants permission to use and modify this code for use
//   in synthesis for all Terasic Development Boards and Altera Development 
//   Kits made by Terasic.  Other use of this code, including the selling 
//   ,duplication, or modification of any portion is strictly prohibited.
//
// Disclaimer:
//
//   This VHDL/Verilog or C/C++ source code is intended as a design reference
//   which illustrates how these types of functions can be implemented.
//   It is the user's responsibility to verify their design for
//   consistency and functionality through the use of formal
//   verification methods.  Terasic provides no warranty regarding the use 
//   or functionality of this code.
//
// ============================================================================
//           
//  Terasic Technologies Inc
//  No.80, Fenggong Rd., Hukou Township, Hsinchu County 303035 Taiwan
//  
//  
//                     web: http://www.terasic.com/  
//                     email: support@terasic.com
//
// ============================================================================
//Date:  Wed Dec 22 14:16:44 2021
// ============================================================================

`define ENABLE_DDR4A
`define ENABLE_DDR4B
//`define ENABLE_QSFP28
//`define ENABLE_QSFP28RSV
//`define ENABLE_FMC
//`define ENABLE_FMCP
//`define ENABLE_HPS

module Agilex_SOM
   #(
`ifdef ENABLE_DDR4B_DUAL_RANK
      parameter RANK = 1
`else
      parameter RANK = 0
`endif
   )

(
      ///////// CLOCK /////////
      input              CLK_100_B2A,
      input              CLK_30M72,
      input              CLK_50_B2A,
      input              CLK_50_B2D,
      input              CLK_50_B3A,
      input              CLK_50_B3C,

      ///////// Buttons /////////
      input              CPU_RESET_n,
      input    [ 1: 0]   BUTTON, //BUTTON is Low-Active

      ///////// Swtiches /////////
      input    [ 1: 0]   SW,

      ///////// LED /////////
      output   [ 1: 0]   LED, //LED is Low-Active

`ifdef ENABLE_DDR4A
      ///////// DDR4A /////////
      input              DDR4A_REFCLK_p,
      output   [16: 0]   DDR4A_A,
      output   [ 1: 0]   DDR4A_BA,
      output   [ 1: 0]   DDR4A_BG,
      output             DDR4A_CK,
      output             DDR4A_CK_n,
      output             DDR4A_CKE,
      inout    [ 8: 0]   DDR4A_DQS,
      inout    [ 8: 0]   DDR4A_DQS_n,
      inout    [71: 0]   DDR4A_DQ,
      inout    [ 8: 0]   DDR4A_DBI_n,
      output             DDR4A_CS_n,
      output             DDR4A_RESET_n,
      output             DDR4A_ODT,
      output             DDR4A_PAR,
      input              DDR4A_ALERT_n,
      output             DDR4A_ACT_n,
      input              DDR4A_EVENT_n,
      inout              DDR4A_SCL,
      inout              DDR4A_SDA,
      input              DDR4A_RZQ,
`endif /*ENABLE_DDR4A*/

`ifdef ENABLE_DDR4B
      ///////// DDR4B /////////
      input              DDR4B_REFCLK_p,
      output   [16: 0]   DDR4B_A,
      output   [ 1: 0]   DDR4B_BA,
      output   [ 1: 0]   DDR4B_BG,
      output   [ RANK: 0]   DDR4B_CK,
      output   [ RANK: 0]   DDR4B_CK_n,
      output   [ RANK: 0]   DDR4B_CKE,
      inout    [ 8: 0]   DDR4B_DQS,
      inout    [ 8: 0]   DDR4B_DQS_n,
      inout    [71: 0]   DDR4B_DQ,
      inout    [ 8: 0]   DDR4B_DBI_n,
      output   [ RANK: 0]   DDR4B_CS_n,
      output             DDR4B_RESET_n,
      output   [ RANK: 0]   DDR4B_ODT,
      output             DDR4B_PAR,
      input              DDR4B_ALERT_n,
      output             DDR4B_ACT_n,
      input              DDR4B_EVENT_n,
      inout              DDR4B_SCL,
      inout              DDR4B_SDA,
      input              DDR4B_RZQ,
`endif /*ENABLE_DDR4B*/

      ///////// SI5340A /////////
      inout              SI5340A_I2C_SCL,
      inout              SI5340A_I2C_SDA,
      input              SI5340A_LOL,
      input              SI5340A_LOS_XAXB,
      output             SI5340A_OE_n,
      output             SI5340A_RST_n,

      ///////// GPIO_D /////////
      inout    [ 7: 0]   GPIO_D,

`ifdef ENABLE_FMC
      ///////// FMC /////////
      inout              FMC_CLK2_BIDIR_p,
      inout              FMC_CLK2_BIDIR_n,
      inout              FMC_CLK3_BIDIR_p,
      inout              FMC_CLK3_BIDIR_n,
      output   [ 9: 0]   FMC_DP_C2M_p,
      output   [ 9: 0]   FMC_DP_C2M_n,
      input    [ 9: 0]   FMC_DP_M2C_p,
      input    [ 9: 0]   FMC_DP_M2C_n,
      inout    [23: 0]   FMC_HA_p,
      inout    [23: 0]   FMC_HA_n,
      inout    [21: 0]   FMC_HB_p,
      inout    [21: 0]   FMC_HB_n,
      inout    [33: 0]   FMC_LA_p,
      inout    [33: 0]   FMC_LA_n,
      input    [ 1: 0]   FMC_CLK_M2C_p,
      input    [ 1: 0]   FMC_GBTCLK_M2C_p,
      input              FMC_REFCLK0_p,
      input              FMC_REFCLK1_p,
      inout    [ 1: 0]   FMC_GA,
      inout    [ 0: 0]   FMC_RES,
      inout              FMC_SCL,
      inout              FMC_SDA,
`endif /*ENABLE_FMC*/

`ifdef ENABLE_FMCP
      ///////// FMCP /////////
      inout              FMCP_CLK2_BIDIR_p,
      inout              FMCP_CLK2_BIDIR_n,
      inout              FMCP_CLK3_BIDIR_p,
      inout              FMCP_CLK3_BIDIR_n,
      output   [15: 0]   FMCP_DP_C2M_p,
      output   [15: 0]   FMCP_DP_C2M_n,
      input    [15: 0]   FMCP_DP_M2C_p,
      input    [15: 0]   FMCP_DP_M2C_n,
      inout    [23: 0]   FMCP_HA_p,
      inout    [23: 0]   FMCP_HA_n,
      inout    [21: 0]   FMCP_HB_p,
      inout    [21: 0]   FMCP_HB_n,
      inout    [33: 0]   FMCP_LA_p,
      inout    [33: 0]   FMCP_LA_n,
      input    [ 1: 0]   FMCP_CLK_M2C_p,
      input    [ 1: 0]   FMCP_GBTCLK_M2C_p,
      output             FMCP_REFCLK_C2M_p,
      input              FMCP_REFCLK_M2C_p,
      inout    [ 1: 0]   FMCP_GA,
      input              FMCP_RES0,
      inout              FMCP_RES1,
      inout              FMCP_SCL,
      inout              FMCP_SDA,
      output             FMCP_SYNC_C2M_p,
      input              FMCP_SYNC_M2C_p,
`endif /*ENABLE_FMCP*/

`ifdef ENABLE_QSFP28
      ///////// QSFP28 /////////
      output   [ 3: 0]   QSFP28_TX_p,
      output   [ 3: 0]   QSFP28_TX_n,
      input    [ 3: 0]   QSFP28_RX_p,
      input    [ 3: 0]   QSFP28_RX_n,
      input              QSFP28_REFCLK_p,
      input              QSFP28_INTERRUPT_n,
      output             QSFP28_LP_MODE,
      input              QSFP28_MOD_PRS_n,
      output             QSFP28_MOD_SEL_n,
      output             QSFP28_RST_n,
      inout              QSFP28_SCL,
      inout              QSFP28_SDA,
`else
      input              QSFP28_REFCLK_p,
`endif /*ENABLE_QSFP28*/

`ifdef ENABLE_QSFP28RSV
      ///////// QSFP28RSV /////////
      input              QSFP28RSV_REFCLK_p,
`endif /*ENABLE_QSFP28RSV*/

      ///////// AG /////////
      output             AG_UART_TX,
      input              AG_UART_RX,
      input              AG_UART_CTS,
      output             AG_UART_RTS,

`ifdef ENABLE_HPS
      ///////// HPS /////////
      output             HPS_ENET_MDC,
      inout              HPS_ENET_MDIO,
      input              HPS_ENET_RX_CLK,
      input              HPS_ENET_RX_CTL,
      input    [ 3: 0]   HPS_ENET_RX_DATA,
      output             HPS_ENET_TX_CLK,
      output             HPS_ENET_TX_CTL,
      output   [ 3: 0]   HPS_ENET_TX_DATA,
      inout    [ 4: 0]   HPS_GPIO,
      inout              HPS_I2C_SCL,
      inout              HPS_I2C_SDA,
      input              HPS_JTAG_TCK,
      input              HPS_JTAG_TDI,
      output             HPS_JTAG_TDO,
      input              HPS_JTAG_TMS,
      inout              HPS_KEY,
      inout              HPS_LED,
      input              HPS_OSC_CLK,
      output             HPS_SD_CLK,
      inout              HPS_SD_CMD,
      inout    [ 3: 0]   HPS_SD_DATA,
      input              HPS_UART_RX,
      output             HPS_UART_TX,
      input              HPS_USB_CLK,
      inout    [ 7: 0]   HPS_USB_DATA,
      input              HPS_USB_DIR,
      input              HPS_USB_NXT,
      output             HPS_USB_STP,
`endif /*ENABLE_HPS*/

      ///////// INFO /////////
      output             INFO_SPI_SCLK,
      input              INFO_SPI_MISO,
      output             INFO_SPI_MOSI,
      output             INFO_SPI_CS_n 

);


//=======================================================
//  REG/WIRE declarations
//=======================================================




//=======================================================
//  Structural coding
//=======================================================

assign SI5340A_RST_n = 1'b1;
assign SI5340A_OE_n = 1'b0;

wire reset_n;
wire ninit_done;

wire ddr4_local_reset_req;
//ddr4a
wire ddr4a_local_reset_done;
wire ddr4a_status_local_cal_fail;
wire ddr4a_status_local_cal_success;
//ddr4b
wire ddr4b_local_reset_done;
wire ddr4b_status_local_cal_fail;
wire ddr4b_status_local_cal_success;


reset_release reset_release (
	.ninit_done(ninit_done)
	);
assign  reset_n = !ninit_done;


wire [5:0] ddr4_status;
assign ddr4_status = {ddr4a_status_local_cal_fail, ddr4a_status_local_cal_success, ddr4a_local_reset_done,
                      ddr4b_status_local_cal_fail, ddr4b_status_local_cal_success, ddr4b_local_reset_done
							 };
     
							 
    Qsys Qsys_Inst (
        .button_external_connection_export               (BUTTON),               //   input,   width = 2,               button_external_connection.export
        .clk_clk                                         (CLK_50_B2A),                                         //   input,   width = 1,                                      clk.clk
        .ddr4_local_reset_req_external_connection_export (ddr4_local_reset_req), //  output,   width = 1, ddr4_local_reset_req_external_connection.export
        .ddr4_status_external_connection_export          (ddr4_status),          //   input,   width = 6,          ddr4_status_external_connection.export
        
		  .emif_ddr4a_local_reset_req_local_reset_req      (ddr4_local_reset_req),      //   input,   width = 1,               emif_ddr4a_local_reset_req.local_reset_req
        .emif_ddr4a_local_reset_status_local_reset_done  (ddr4a_local_reset_done),  //  output,   width = 1,            emif_ddr4a_local_reset_status.local_reset_done
        .emif_ddr4a_pll_ref_clk_clk                      (DDR4A_REFCLK_p),                      //   input,   width = 1,                   emif_ddr4a_pll_ref_clk.clk
        .emif_ddr4a_oct_oct_rzqin                        (DDR4A_RZQ),                        //   input,   width = 1,                           emif_ddr4a_oct.oct_rzqin
        .emif_ddr4a_mem_mem_ck                           (DDR4A_CK),                           //  output,   width = 1,                           emif_ddr4a_mem.mem_ck
        .emif_ddr4a_mem_mem_ck_n                         (DDR4A_CK_n),                         //  output,   width = 1,                                         .mem_ck_n
        .emif_ddr4a_mem_mem_a                            (DDR4A_A),                            //  output,  width = 17,                                         .mem_a
        .emif_ddr4a_mem_mem_act_n                        (DDR4A_ACT_n),                        //  output,   width = 1,                                         .mem_act_n
        .emif_ddr4a_mem_mem_ba                           (DDR4A_BA),                           //  output,   width = 2,                                         .mem_ba
        .emif_ddr4a_mem_mem_bg                           (DDR4A_BG),                           //  output,   width = 2,                                         .mem_bg
        .emif_ddr4a_mem_mem_cke                          (DDR4A_CKE),                          //  output,   width = 1,                                         .mem_cke
        .emif_ddr4a_mem_mem_cs_n                         (DDR4A_CS_n),                         //  output,   width = 1,                                         .mem_cs_n
        .emif_ddr4a_mem_mem_odt                          (DDR4A_ODT),                          //  output,   width = 1,                                         .mem_odt
        .emif_ddr4a_mem_mem_reset_n                      (DDR4A_RESET_n),                      //  output,   width = 1,                                         .mem_reset_n
        .emif_ddr4a_mem_mem_par                          (DDR4A_PAR),                          //  output,   width = 1,                                         .mem_par
        .emif_ddr4a_mem_mem_alert_n                      (DDR4A_ALERT_n),                      //   input,   width = 1,                                         .mem_alert_n
        .emif_ddr4a_mem_mem_dqs                          (DDR4A_DQS),                          //   inout,   width = 9,                                         .mem_dqs
        .emif_ddr4a_mem_mem_dqs_n                        (DDR4A_DQS_n),                        //   inout,   width = 9,                                         .mem_dqs_n
        .emif_ddr4a_mem_mem_dq                           (DDR4A_DQ),                           //   inout,  width = 72,                                         .mem_dq
        .emif_ddr4a_mem_mem_dbi_n                        (DDR4A_DBI_n),                        //   inout,   width = 9,                                         .mem_dbi_n
        .emif_ddr4a_status_local_cal_success             (ddr4a_status_local_cal_success),             //  output,   width = 1,                        emif_ddr4a_status.local_cal_success
        .emif_ddr4a_status_local_cal_fail                (ddr4a_status_local_cal_fail),                //  output,   width = 1,                                         .local_cal_fail
        
		  
		  .emif_ddr4b_local_reset_req_local_reset_req      (ddr4_local_reset_req),      //   input,   width = 1,               emif_ddr4b_local_reset_req.local_reset_req
        .emif_ddr4b_local_reset_status_local_reset_done  (ddr4b_local_reset_done),  //  output,   width = 1,            emif_ddr4b_local_reset_status.local_reset_done
        .emif_ddr4b_pll_ref_clk_clk                      (DDR4B_REFCLK_p),                      //   input,   width = 1,                   emif_ddr4b_pll_ref_clk.clk
        .emif_ddr4b_oct_oct_rzqin                        (DDR4B_RZQ),                        //   input,   width = 1,                           emif_ddr4b_oct.oct_rzqin
        .emif_ddr4b_mem_mem_ck                           (DDR4B_CK),                           //  output,   width = 1,                           emif_ddr4b_mem.mem_ck
        .emif_ddr4b_mem_mem_ck_n                         (DDR4B_CK_n),                         //  output,   width = 1,                                         .mem_ck_n
        .emif_ddr4b_mem_mem_a                            (DDR4B_A),                            //  output,  width = 17,                                         .mem_a
        .emif_ddr4b_mem_mem_act_n                        (DDR4B_ACT_n),                        //  output,   width = 1,                                         .mem_act_n
        .emif_ddr4b_mem_mem_ba                           (DDR4B_BA),                           //  output,   width = 2,                                         .mem_ba
        .emif_ddr4b_mem_mem_bg                           (DDR4B_BG),                           //  output,   width = 2,                                         .mem_bg
        .emif_ddr4b_mem_mem_cke                          (DDR4B_CKE),                          //  output,   width = 1,                                         .mem_cke
        .emif_ddr4b_mem_mem_cs_n                         (DDR4B_CS_n),                         //  output,   width = 1,                                         .mem_cs_n
        .emif_ddr4b_mem_mem_odt                          (DDR4B_ODT),                          //  output,   width = 1,                                         .mem_odt
        .emif_ddr4b_mem_mem_reset_n                      (DDR4B_RESET_n),                      //  output,   width = 1,                                         .mem_reset_n
        .emif_ddr4b_mem_mem_par                          (DDR4B_PAR),                          //  output,   width = 1,                                         .mem_par
        .emif_ddr4b_mem_mem_alert_n                      (DDR4B_ALERT_n),                      //   input,   width = 1,                                         .mem_alert_n
        .emif_ddr4b_mem_mem_dqs                          (DDR4B_DQS),                          //   inout,   width = 9,                                         .mem_dqs
        .emif_ddr4b_mem_mem_dqs_n                        (DDR4B_DQS_n),                        //   inout,   width = 9,                                         .mem_dqs_n
        .emif_ddr4b_mem_mem_dq                           (DDR4B_DQ),                           //   inout,  width = 72,                                         .mem_dq
        .emif_ddr4b_mem_mem_dbi_n                        (DDR4B_DBI_n),                        //   inout,   width = 9,                                         .mem_dbi_n
        .emif_ddr4b_status_local_cal_success             (ddr4b_status_local_cal_success),             //  output,   width = 1,                        emif_ddr4b_status.local_cal_success
        .emif_ddr4b_status_local_cal_fail                (ddr4b_status_local_cal_fail),                //  output,   width = 1,                                         .local_cal_fail
        .iopll_locked_export                             (),                             //  output,   width = 1,                             iopll_locked.export
        .reset_reset_n                                   (reset_n)                                    //   input,   width = 1,                                    reset.reset_n
    );

wire [1:0]heart_led;

heart_beat  heart_beat_0 ( .CLK(DDR4A_REFCLK_p ),     .CLK_FREQ  (33_333_333) , . CK_1HZ (heart_led[0]) );
heart_beat  heart_beat_1 ( .CLK(DDR4B_REFCLK_p ),     .CLK_FREQ  (33_333_333) , . CK_1HZ (heart_led[1]) );

assign LED[0] =  (ddr4a_status_local_cal_success) ?~(1'b1):heart_led[0];
assign LED[1] =  (ddr4b_status_local_cal_success) ?~(1'b1):heart_led[1];


endmodule
