#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <unistd.h>
#include <fcntl.h>
#include "gpio_lib.h"

 /****************************************************************
 * Constants
 ****************************************************************/
#define TEST_PATTENRN1 0x1234f0f0
#define TEST_PATTENRN2 0x43210f0f


/****************************************************************
 * Main
 ****************************************************************/
int main(int argc, char **argv)
{	
	unsigned int GPIO_BASE=get_gpio_base();
	
	unsigned int hps_gpio_0=GPIO_BASE+18, hps_gpio_1=GPIO_BASE+19, hps_gpio_2=GPIO_BASE+20, hps_gpio_3=GPIO_BASE+21;
	unsigned int test_data1=TEST_PATTENRN1, test_data2=TEST_PATTENRN2,recv_data1=0,recv_data2=0;
	unsigned int tmp_data=0;
	int i=0;
	int error = 0;

	// export gpio1
	error = gpio_export(hps_gpio_0);
	if (error == 0)
		error = gpio_export(hps_gpio_1);
	if (error == 0)
		error = gpio_export(hps_gpio_2);
	if (error == 0)
		error = gpio_export(hps_gpio_3);
	
	if (error != 0){
		printf("failed to write/read data\n");
		return 0;	
	}	
	
	// continue to test
			
	printf("\n====================Loopback Test:Start Test1======================\n");
	printf("Test 1 : hps_gpio_0->hps_gpio_1,hps_gpio_2->hps_gpio_3, write data: 0x%x\n", test_data1);
	
	// set direction(0:in 1:out) : 11:in, 10:out, 9:in, 8:out
	gpio_set_dir(hps_gpio_0, 1); // out
	gpio_set_dir(hps_gpio_1, 0); // in
	gpio_set_dir(hps_gpio_2, 1); // out
	gpio_set_dir(hps_gpio_3, 0); // in
	
	// data write and read back
	for (i=0; i<32; i++){
		tmp_data = (unsigned int)((test_data1>>(31-i))&0x1);
		
		gpio_set_value(hps_gpio_0,tmp_data);
		gpio_set_value(hps_gpio_2,tmp_data);
		
		gpio_get_value(hps_gpio_1,&tmp_data);
		recv_data1 = (recv_data1<<1)|tmp_data;
		
		gpio_get_value(hps_gpio_3,&tmp_data);
		recv_data2 = (recv_data2<<1)|tmp_data;
	} // for
	
	printf("Recv Data : hps_gpio_1 = 0x%x\n", recv_data1);
	printf("Recv Data : hps_gpio_3 = 0x%x\n", recv_data2);
	
	// verify1
	if (recv_data1==test_data1){
		printf("Test1 hps_gpio_0->hps_gpio_1 successfully\n");
	}else{
		printf("Test1 hps_gpio_0->hps_gpio_1 failed\n");
	}
	
	if (recv_data2==test_data1){
		printf("Test1 hps_gpio_2->hps_gpio_3 successfully\n");
	}else{
		printf("Test1 hps_gpio_2->hps_gpio_3 failed\n");
	}
	
		
	printf("\n====================Loopback Test:Start Test2======================\n");
	printf("Test 2 : hps_gpio_0<-hps_gpio_1,hps_gpio_2<-hps_gpio_3, wirte data: 0x%x\n", test_data2);
	recv_data1=0;
	recv_data2=0;
	// set direction(0:in 1:out) : 11:out, 10:in, 9:out, 8:in
	gpio_set_dir(hps_gpio_0, 0);
	gpio_set_dir(hps_gpio_1, 1);
	gpio_set_dir(hps_gpio_2, 0);
	gpio_set_dir(hps_gpio_3, 1);
	
	// data write and read back
	for (i=0; i<32; i++){
		tmp_data = (unsigned int)((test_data2>>(31-i))&0x1);
		
		gpio_set_value(hps_gpio_1,tmp_data);
		gpio_set_value(hps_gpio_3,tmp_data);
		
		gpio_get_value(hps_gpio_0,&tmp_data);
		recv_data1 = (recv_data1<<1)|tmp_data;
		
		gpio_get_value(hps_gpio_2,&tmp_data);
		recv_data2 = (recv_data2<<1)|tmp_data;
	}
	
	printf("Recv Data : hps_gpio_0 = 0x%x\n", recv_data1);
	printf("Recv Data : hps_gpio_2 = 0x%x\n", recv_data2);

	// verify2
	if (recv_data1==test_data2){
		printf("Test2 hps_gpio_0<-hps_gpio_1 successfully\n");
	}else
		printf("Test2 hps_gpio_0<-hps_gpio_1 failed\n");
	
	if (recv_data2==test_data2){
		printf("Test2 hps_gpio_2<-hps_gpio_3 successfully\n");
	}else
		printf("Test2 hps_gpio_2<-hps_gpio_3 failed\n");

	gpio_unexport(hps_gpio_0);
	gpio_unexport(hps_gpio_1);
	gpio_unexport(hps_gpio_2);
	gpio_unexport(hps_gpio_3);
	
	return 0;
}
